<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Loginplus
 * @package    Loginplus_Service
 * @subpackage Loginplus_Service_Adapter
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Facebook auth adapter
 *
 * @uses       Zend_Auth_Adapter_Interface
 * @category   Loginplus
 * @package    Loginplus_Service
 * @subpackage Loginplus_Service_Adapter
 */
class Loginplus_Service_Adapter_Facebook implements Zend_Auth_Adapter_Interface
{
    /**
     * The Authentication URI, used to bounce the user to the facebook redirect uri.
     *
     * @var string
     */
    const AUTH_URI = 'https://graph.facebook.com/oauth/authorize?client_id=%s&redirect_uri=%s';

    /**
     * The token URI, used to retrieve the OAuth Token.
     *
     * @var string
     */
    const TOKEN_URI = 'https://graph.facebook.com/oauth/access_token';

    /**
     * The user URI, used to retrieve information about the user.
     *
     * @var string
     */
    const USER_URI = 'https://graph.facebook.com/me';

    /**
     * The application ID
     *
     * @var string
     */
    private $_appId = null;

    /**
     * The application secret
     *
     * @var string
     */
    private $_secret = null;

    /**
     * The authentication scope (advanced options) requested
     *
     * @var string
     */
    private $_scope = null;

    /**
     * The redirect uri
     *
     * @var string
     */
    private $_redirectUri = null;

    /**
     * Constructor
     *
     * @param string $appId the application ID
     * @param string $secret the application secret
     * @param string $scope the application scope
     * @param string $redirectUri the URI to redirect the user to after successful authentication
     */
    public function __construct($appId, $secret, $redirectUri)
    {
        $this->_appId       = $appId;
        $this->_secret      = $secret;
        $this->_scope       = 'email, user_photos';
        $this->_redirectUri = urlencode($redirectUri);
    }

    /**
     * Sets the value to be used as the application ID
     *
     * @param  string $appId The application ID
     * @return Zend_Auth_Adapter_Facebook Provides a fluent interface
     */
    public function setAppId($appId)
    {
        $this->_appId = $appId;
        return $this;
    }

    /**
     * Sets the value to be used as the application secret
     *
     * @param  string $secret The application secret
     * @return Zend_Auth_Adapter_Facebook Provides a fluent interface
     */
    public function setSecret($secret)
    {
        $this->_secret = $secret;
        return $this;
    }

    /**
     * Sets the value to be used as the application scope (array())
     *
     * @param  string $scope The application scope
     * @return Zend_Auth_Adapter_Facebook Provides a fluent interface
     */
    public function setApplicationScope($scope)
    {
        $this->_scope = $scope;
        return $this;
    }

    /**
     * Sets the redirect uri after successful authentication
     *
     * @param  string $redirectUri The redirect URI
     * @return Zend_Auth_Adapter_Facebook Provides a fluent interface
     */
    public function setRedirectUri($redirectUri)
    {
        $this->_redirectUri = $redirectUri;
        return $this;
    }

    /**
     * Authenticates the user against facebook
     * Defined by Zend_Auth_Adapter_Interface.
     *
     * @throws Zend_Auth_Adapter_Exception If answering the authentication query is impossible
     * @return Zend_Auth_Result
     */
    public function authenticate()
    {
        // Get the request object.
        $frontController = Zend_Controller_Front::getInstance();
        $request = $frontController->getRequest();

        // First check to see wether we're processing a redirect response.
        $code = $request->getParam('code');

        if (empty($code)) {
            // Create the initial redirect
            $loginUri = sprintf(self::AUTH_URI , $this->_appId, $this->_redirectUri);

            if (!empty($this->_scope)) {
                $loginUri .= "&scope=" . $this->_scope;
            }

            Zend_Controller_Action_HelperBroker::getStaticHelper('redirector')->gotoUrlAndExit($loginUri);
        } else {
             $tokenUrl = self::TOKEN_URI . "?"
               . "client_id=" . $this->_appId . "&redirect_uri=" . $this->_redirectUri 
               . "&client_secret=" . $this->_secret . "&code=" . $code;
             
             $client = new Zend_Http_Client($tokenUrl);
             $response = $client->request();
             $params = null;
             parse_str($response->getBody(), $params);
             
             $graphUrl = self::USER_URI . "?"
                .  "access_token=" . $params['access_token'];
             
             $client->setUri($graphUrl);
             $data = Zend_Json::decode($client->request()->getBody());

             if ($data["email"]) {
                 $validator = new Zend_Validate_EmailAddress();
                 if ($validator->isValid($data["email"])) {
                     if (!preg_match("|^[^@]*@proxymail.facebook.com$|ui", $data["email"])) {
                         $user['email'] = $data['email'];
                     }
                 }
             }
             $user['facebook_id'] = $data['id'];
             $user['full_name']   = $data['name'];
             $user['photo_url']   = 'http://graph.facebook.com/' . $data['id'] . '/picture';
             $user['photo_name']  = $data['id'];

             return new Zend_Auth_Result(Zend_Auth_Result::SUCCESS, $user['facebook_id'], array('user' => $user, 'token' => $params['access_token']));
        }

        return new Zend_Auth_Result(Zend_Auth_Result::FAILURE, null, array("Authenticaton failed"));
    }
}
