<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * System api controller
 *
 * @uses       System_Controller_Abstract_Api
 * @category   Resources
 * @package    Resources_Controller
 */
class Resources_ApiController extends System_Controller_Abstract_Api
{
    /**
     * Initialization
     * 
     * @return void 
     */
    public function init ()
    {
        parent::init();
        
        // Active comment object
        $commentId = $this->_request->getParam('comment_id');
        $comments = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
        if ($commentId) {
            $comment = $comments->findById($commentId);
            if (!$comment instanceof Resources_Model_ResponseComment) {
                $this->error(self::ERROR_NOT_FOUND);
            }
            $this->activeComment = $comment;
        } else {
            $this->activeComment = $comments->createRow();
        }
        Singular_Runtime::store('activeComment', $this->activeComment);
    }

    /**
     * Move comment to trash
     * 
     * @return void 
     */
    public function commentToTrashAction ()
    {
        if ($this->activeComment->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeComment->canDelete()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        if (!$this->activeComment->trash()) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
    }
    
    /**
     * Recovery comment from trash
     * 
     * @return void 
     */
    public function commentFromTrashAction ()
    {
        if ($this->activeComment->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeComment->canDelete()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        if (!$this->activeComment->recovery()) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
    }
    
    /**
     * Permanently delete comment
     * 
     * @return void
     */
    public function removeCommentAction ()
    {
        if ($this->activeComment->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeComment->canDelete()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        if (!$this->activeComment->delete()) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
    }
    
    /**
     * Set comment as best
     * 
     * @return void 
     */
    public function commentSetBestAction ()
    {
        if ($this->activeComment->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeComment->canEdit()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        $comments = Singular_Loader::get(
            'Resources_Model_DbTable_ResponsesComments'
        )->findBestByResponseId($this->activeComment->response_id);
        foreach ($comments as $comment) {
            $comment->unsetBest();
            $comment->save();
        }
        $this->activeComment->setBest();
        $this->activeComment->save();
    }
    
    /**
     * Add vote to comment
     * 
     * @return void 
     */
    public function voteForCommentAction ()
    {
        if ($this->activeComment->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->loggedUser->canAddVote()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        if ($this->activeComment->isVoted($this->loggedUser->id)) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        
        // Add vote to comment
        $vote = new Resources_Model_Vote();
        $vote->object    = $this->activeComment->getObject();
        $vote->object_id = $this->activeComment->id;
        $vote->value     = 1;
        $vote->save();
    }
    
    /**
     * Show comment details
     * 
     * @return void
     */
    public function commentAction ()
    {
        if ($this->activeComment->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        $this->setData(
            System_Service_Api::commentsListing($this->activeComment)
        );
    }
   
    /**
     * Build comments list
     * 
     * @return void 
     */
    public function commentsAction ()
    {
        $where = array();
        
        // Response filter
        $responseId = $this->_request->getParam('response_id', '');
        if ($responseId) {
            $responses = Singular_Loader::get('System_Model_DbTable_Responses');
            $response = $responses->findById($responseId);
            if (!$response instanceof System_Model_Response) {
                $this->error(self::ERROR_NOT_FOUND);
            }
            $where = array_merge($where, array('response_id' => $responseId));
        }
        Singular_Runtime::store('activeResponse', $response);
        
        // User filter
        $userId = $this->_request->getParam('user_id', '');
        if ($userId) {
            $users = Singular_Loader::get('System_Model_DbTable_Users');
            $user = $users->findById($userId);
            if (!$user instanceof System_Model_User) {
                $this->error(self::ERROR_NOT_FOUND);
            }
            $where = array_merge($where, array('user_id' => $userId));
        }

        // Fetch comments
        $comments = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
        $comments = $comments->getPaginator(
            $where, $this->getPageNumber(), null, $this->getRowCount()
        );
        
        $this->setData(array(
            'count' => $comments->getTotalItemCount(),
            'items' => System_Service_Api::commentsListing($comments->getCurrentItems()),
        ));
    }
    
    /**
     * Add new comment
     * 
     * @return void
     */
    public function addCommentAction ()
    {
        if (!$this->activeComment->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->loggedUser->canAddComment()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        
        // Fetch active response
        $responseId = $this->_request->getParam('response_id', '');
        $responses = Singular_Loader::get('System_Model_DbTable_Responses');
        $response = $responses->findById($responseId);
        if (!$response instanceof System_Model_Response) {
            $this->error(self::ERROR_NOT_FOUND);
        }
        Singular_Runtime::store('activeResponse', $response);
        if (!$response->canCommented()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        
        // Gettimg request
        $post = array(
            'content' => $this->_request->getParam('content', ''),
            'response_id' => $response->id,
            'vote' => $this->_request->getParam('vote', ''),
        );

        // Validate
        $form = new Resources_Form_CommentAdd();
        $form->removeElement('file');
        if (!$form->isValid($post)) {
            $this->error(self::ERROR_BAD_REQUEST, $form);
        }
        
        // Store comment in database
        if (!$this->activeComment->save($form->getValues())) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }

        // Notify listeners
        Singular_Event::dispatch('commentAdded', $this->activeComment);
    }
    
    /**
     * Edit active comment
     * 
     * @return void 
     */
    public function editCommentAction ()
    {
        if ($this->activeComment->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeComment->canEdit()) {
             $this->error(self::ERROR_FORBIDDEN);
        }
        
        // Fetch active response
        $responseId = $this->_request->getParam('response_id', '');
        $responses = Singular_Loader::get('System_Model_DbTable_Responses');
        $response = $responses->findById($responseId);
        if (!$response instanceof System_Model_Response) {
            $this->error(self::ERROR_NOT_FOUND);
        }
        Singular_Runtime::store('activeResponse', $response);

        // Gettimg request
        $post = array(
            'content' => $this->_request->getParam('content', ''),
            'response_id' => $response->id,
            'comment_id' => $this->activeComment->id,
        );
        
        // Validate
        $form = new Resources_Form_CommentEdit();
        $form->removeElement('file');
        if (!$form->isValid($post)) {
            $this->error(self::ERROR_BAD_REQUEST, $form);
        }
        
        // Save!
        if (!$this->activeComment->save($form->getValues())) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
    }
}