<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Model
 * @subpackage Resources_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * response_comments table model.
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   Resources
 * @package    Resources_Model
 * @subpackage Resources_Model_DbTable
 */
class Resources_Model_DbTable_ResponsesComments extends System_Model_DbTable_Abstract
{
    /**
     * Default fetch limit
     */
    const LIMIT = 15;

    /**
     * The table name
     *
     * @var string
     */
    protected $_name            = 'response_comments';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary         = 'id';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass        = 'Resources_Model_ResponseComment';

    /**
     * Rowset class
     *
     * @var string
     */
    protected $_rowsetClass     = 'System_Model_Rowset';

    /**
     * Reference map
     *
     * @var array
     */
    protected $_referenceMap    = array(
        'Response' => array(
            'columns'           => array('response_id'),
            'refTableClass'     => 'System_Model_DbTable_Responses',
            'refColumns'        => array('id'),
            'onDelete'          => self::CASCADE,
        ),
        'CreatedBy' => array(
            'columns'           => array('created_by_id'),
            'refTableClass'     => 'System_Model_DbTable_Users',
            'refColumns'        => array('id'),
            'onDelete'          => self::CASCADE,
        )
    );

    /**
     * Inserts a new response comment row.
     *
     * @see    parent::insert()
     * @param  array  $data  Column-value pairs.
     * @return mixed         The primary key of the row inserted.
     */
    public function insert(array $data)
    {
        $data = $this->_setDate($data, "created_at");
        $data = $this->_setActor($data, "created_by_id");

        return parent::insert($data);
    }

    /**
     * Updates existing response comment rows.
     *
     * @param  array        $data  Column-value pairs.
     * @param  array|string $where An SQL WHERE clause, or an array of SQL WHERE clauses.
     * @return int          The number of rows updated.
     */
    public function update(array $data, $where)
    {
        $data = $this->_setDate($data, "updated_at");
        $data = $this->_setActor($data, "updated_by_id");

        return parent::update($data, $where);
    }

    /**
     * Returns response comment by id
     *
     * @param  int $id
     * @return Resources_Model_ResponseComment
     */
    public function findById($id)
    {        
        $select = $this->select();
        $select->where('id=?', $id);
        $comment = $this->fetchRow($select);
        if ($comment instanceof Resources_Model_ResponseComment) {
            $loggedUser = Singular_Runtime::extract('loggedUser');
            if ($loggedUser->canSeeComment($comment)) {
                return $comment;
            }
        }
        return;
    }

    /**
     * Returns response comments by ids
     *
     * @param  array $ids
     * @return System_Model_Rowset
     */
    public function findByIds($ids)
    {       
        $select = $this->select();
        $select->where('id IN (?)', $ids);
        $comments = $this->fetchAll($select);
        if ($comments->valid()) {
            $loggedUser = Singular_Runtime::extract('loggedUser');
            $data = array();
            foreach ($comments as $comment) {
                if ($loggedUser->canSeeComment($comment)) {
                    $data[] = $comment->toArray();
                }
            }
            $comments = new System_Model_Rowset(array(
                'table'    => $this,
                'rowClass' => 'Resources_Model_ResponseComment',
                'readOnly' => $select->isReadOnly(),
                'data'     => $data,
                'stored'   => true
            ));
        }
        return $comments;
    }

    /**
     * Returns last response comment by user_id
     *
     * @param  int $userId
     * @return Resources_Model_ResponseComment
     */
    public function findLastByUserId($userId)
    {
        $select = $this->getSelect();
        $select->where("{$this->getTableName()}.created_by_id=?", $userId);
        $select->reset('order');
        $select->order("{$this->getTableName()}.created_at DESC");
        $select->limit(1);

        return $this->fetchRow($select);
    }

    /**
     * Returns last response comment by response_id
     *
     * @param  int $responseId
     * @return Resources_Model_ResponseComment
     */
    public function findLastByResponseId($responseId)
    {
        $select = $this->getSelect();
        $select->where("{$this->getTableName()}.response_id=?", $responseId);
        $select->reset('order');
        $select->order("{$this->getTableName()}.created_at DESC");
        $select->limit(1);

        return $this->fetchRow($select);
    }

    /**
     * Returns response comments by given criteria
     *
     * @param  array|null $where
     * @return System_Model_Rowset
     */
    public function findAll($where = null)
    {
        $select = $this->getSelect();
        $tableName = $this->getTableName();
        if (is_array($where)) {
            if (!empty($where['response_id'])) {
                $select->where("$tableName.response_id=?", $where['response_id']);
            }
            if (!empty($where['date']) && $where['date'] == 'new') {
                $select->where("$tableName.created_at > ?", date("Y-m-d H:i:s", strtotime("-7 day last Monday")));
                $select->reset("order");
                $select->order("created_at DESC");
            }
        }
        return $this->fetchAll($select);
    }

    /**
     * Return fetched paginator rows.
     *
     * @param  array $where
     * @param  int $page
     * @param  mixed $order
     * @param  int $limit
     * @return Zend_Db_Table_Rowset|null
     */
    public function getPaginator($where, $page, $order = null, $limit = null)
    {
        $select = $this->getSelect();
        $tableName = $this->getTableName();
        if (!empty($where['response_id'])) {
            $select->where("{$this->getTableName()}.response_id=?", $where['response_id']);
            if (isset($where['votes']) && $where['votes'] != 'all') {
                if ($where['votes'] == 'positive') {
                    $select->where("{$this->getTableName()}.is_negative=0");
                } else {
                    $select->where("{$this->getTableName()}.is_negative=1");
                }
            }
        }
        if (!empty($where['user_id'])) {
            $select->where("{$this->getTableName()}.created_by_id = ?", $where['user_id']);
        }
        if (!empty($where['type'])) {
            $rTableName = Singular_Loader::get('System_Model_DbTable_Responses')
                            ->getTableName();
            $from = $select->getPart("from");
            $finded = false;
            foreach ($from as $key=>$param) {
                if ($param['tableName'] == $rTableName) {
                    $select->where("{$key}.type=?", $where['type']);
                    $finded = true;
                }
            }
            if (!$finded) {
                $select->joinLeft(array('r' => $rTableName),
                                  "{$tableName}.response_id=r.id",
                                  array());
                $select->where("r.type=?", $where['type']);
            }
        }
        if (isset($order)) {
            if (is_array($order)) {
                if (isset($order["order"]) && $order["order"]) {
                    if (isset($order["sort"]) && $order['sort']) {
                        $sort = "ASC";
                    } else {
                        $sort = "DESC";
                    }
                }
                if (in_array($order["order"], $this->getCols())) {
                    $select->reset("order");
                    $select->order("$tableName.{$order["order"]} $sort");
                }
            } else {
                $select->order($order);
            }
        }
        $paginator = $this->getPaginatorRows(
            $select, 'id', null, $page, $limit ? $limit : self::LIMIT
        );
        return $paginator;
    }

    /**
     * Returns all response comments by response_id
     *
     * @param  int $responseId
     * @return System_Model_Rowset
     */
    public function findByResponseId($responseId, $vote = 'all')
    {
        $select = $this->getSelect();
        if ($responseId) {
            $select->where("{$this->getTableName()}.response_id=?", $responseId);
            if ($vote != 'all') {
                if ($vote == 'positive') {
                    $select->where("{$this->getTableName()}.is_negative=0");
                } else {
                    $select->where("{$this->getTableName()}.is_negative=1");
                }
            }
        }
        return $this->fetchAll($select);
    }

    /**
     * Returns all best comments by response_id
     *
     * @param  int $responseId
     * @return System_Model_Rowset
     */
    public function findBestByResponseId($responseId)
    {
        $select = $this->getSelect();
        if ($responseId) {
            $select->where("{$this->getTableName()}.response_id=?", $responseId);
        }
        $select->where("{$this->getTableName()}.is_best=1");
        return $this->fetchAll($select);
    }
    
    /**
     * Returns total number of response comments by response_id
     *
     * @param  int $responseId
     * @return int
     */
    public function countByResponseId($responseId)
    {
        return $this->count(array('response_id' => $responseId));
    }

    /**
     * Returns total number of response comments by created_by_id
     *
     * @param  int $userId
     * @return int
     */
    public function countByUserId($userId)
    {
        return $this->count(array('created_by_id' => $userId));
    }

    /**
     * Returns total number of response comments by given criteria
     *
     * @param  int $where
     * @return System_Model_Rowset
     */
    public function count($where = null)
    {
        $select = $this->getSelect();
        $select->reset('columns')
               ->reset('order')
               ->reset('group');
        if (System_Service_Environment::getInstance()->isAdmin()) {
            $select->reset('where');
        } else {
            $from = $select->getPart('from');
            $responseDbName = Singular_Loader::get("System_Model_DbTable_Responses")
                                ->getTableName();

            if ( !(isset($from['r']) && $from['r']['tableName'] == $responseDbName) ) {
                $select->setIntegrityCheck(false);
                $select->joinLeft(array('r' => $responseDbName), "r.id={$this->getTableName()}.response_id", array());
            }
            $select->where("r.deleted_by_id=0");
            
            //types
            $types = Singular_Runtime::extract('types')->findValuesBySlug();
            $select->where('r.type IN (?)', $types);
        }
        
        if(is_array($where)){
            foreach($where as $cond => $value){
                if($value !== null){
                    $select->where("{$this->getTableName()}.$cond=?", $value);
                }
            }
        } else if (!empty($where) && is_string($where)) {
            $select->where($where);
        }
        
        $select->columns(array('count' => new Zend_Db_Expr("COUNT(DISTINCT({$this->getTableName()}.id))")));
        $result = $this->getAdapter()->fetchRow($select);
        
        return $result['count'];
    }
    
    /**
     * Counts comments (by one selected response type) by given criteria
     *
     * @param  string|array $where
     * @return int
     */
    public function countByType($where = null, $type = null)
    {
        $select = $this->getSelect();
        $select->reset('where');
        
        $from = $select->getPart('from');
        $responseDbName = Singular_Loader::get("System_Model_DbTable_Responses")
                            ->getTableName();

        if ( !(isset($from['r']) && $from['r']['tableName'] == $responseDbName) ) {
            $select->setIntegrityCheck(false);
            $select->joinLeft(array('r' => $responseDbName), "r.id={$this->getTableName()}.response_id", array());
        }
        $select->where("r.deleted_by_id=0");
        
        if(!empty($type)) {
            $select->where('r.type=?', $type);
        } else {
            $types = Singular_Runtime::extract('types')->findValuesBySlug();
            $select->where('r.type IN (?)', $types);
        }
        if(!empty($where)) {
            $where = $this->_addResponseDbAlias($where);   
            $select->where($where);
        }
        $select->reset('columns')
               ->reset('order')
               ->reset('group');

        $select->columns(array('count' => new Zend_Db_Expr("COUNT(DISTINCT({$this->getTableName()}.id))")));
        $result = $this->fetchRow($select);

        return $result['count'];
    }
    
    /**
     * Add to expression responseDb alias for column names
     * 
     * @param string $expression
     * @return string
     */
    protected function _addResponseDbAlias($expression)
    {
        $responseDbCols = array (
                'created_at'
            );
        foreach ($responseDbCols as $responseDbCol) {
            if (strpos($expression, $responseDbCol)) {

            } 
            $expression = str_replace($responseDbCol, 'r.'.$responseDbCol, $expression);
        }
        return $expression;
    }        

    /**
     * Returns prepared select object
     *
     * @see    parent::getSelect()
     * @return Zend_Db_Select
     */
    public function getSelect()
    {
        $select = parent::getSelect();
        if (System_Service_Environment::getInstance()->isAdmin()) {
            $select->where("{$this->getTableName()}.deleted_by_id <> 0");
        } else {
            $select->where("{$this->getTableName()}.deleted_by_id=?", 0);
        }
        if (Singular_Core::_("settings")->getSetting("show_comments")) {
            $select->order("{$this->getTableName()}.created_at ASC");
        } else {
            $select->order("{$this->getTableName()}.created_at DESC");
        }
        return $select;
    }
}
