<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Model
 * @subpackage Resources_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * votes table model.
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   Resources
 * @package    Resources_Model
 * @subpackage Resources_Model_DbTable
 */
class Resources_Model_DbTable_Votes extends System_Model_DbTable_Abstract
{
    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'votes';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'id';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass = 'Resources_Model_Vote';

    /**
     * Reference map
     *
     * @var array
     */
    protected $_referenceMap    = array(
        'CreatedBy' => array(
            'columns'           => array('created_by_id'),
            'refTableClass'     => 'System_Model_DbTable_Users',
            'refColumns'        => array('id'),
            'onDelete'          => self::CASCADE,
        )
    );

    /**
     * Returns all votes by object data
     *
     * @param  Zend_Db_table_Row $object
     * @return Zend_Db_table_Rowset
     */
    public function findByObject($object)
    {
        $select = $this->getSelect();
        $select->where('object=?', $object->getObject());
        $select->where('object_id=?', $object->id);
        $votes = $this->fetchAll($select);
        return $votes;
    }

    /**
     * Inserts a new vote row.
     *
     * @see    parent::insert()
     * @param  array  $data  Column-value pairs.
     * @return mixed         The primary key of the row inserted.
     */
    public function insert(array $data)
    {
        $data = $this->_setActor($data, 'created_by_id');
        $data = $this->_setDate($data, 'created_at');

        /* correct value */
        if ($data['value'] > 0) {
            $data['value'] = 1;
        } else {
            $data['value'] = -1;
        }
        return parent::insert($data);
    }

    /**
     * Returns rating for given object
     *
     * @param  Zend_Db_Table_Row $object
     * @return object
     */
    public function getRatingByObject($object)
    {
        $select = $this->getAdapter()->select();
        $select->from($this->_name, array(
                   'type'   => new Zend_Db_Expr('CASE WHEN value<0 THEN "negative" ELSE "positive" END'),
                   'rating' => 'sum(value)',
               ))
               ->where('object=?', $object->getObject())
               ->where('object_id = ?', $object->id)
               ->group('type');
        
        $params = (object) array(
            'select' => $select,
            'object' => $object
        );
        Singular_Event::dispatch('prepareSelectForRating', $params);
        
        
        $votes = $this->getAdapter()->fetchPairs($select);

        $votes['all'] = 0;
        if (isset($votes['negative'])) {
            $votes['negative'] = -$votes['negative'];
            $votes['all'] += $votes['negative'];
        } else {
            $votes['negative'] = 0;
        }
        if (isset($votes['positive'])) {
            $votes['all'] += $votes['positive'];
        } else {
            $votes['positive'] = 0;
        }

        return (object) $votes;
    }

    /**
     * Returns rating for specific user
     *
     * @param  System_Model_User $user
     * @return mixed
     */
    public function getRatingForUser($user)
    {
        $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
        $select = $responsesDb->getSelect();
        $select->setIntegrityCheck(false)
               ->reset('columns')
               ->columns(array(
                       'rating' => 'count(v.id)',
               ))
               ->joinLeft(array('v' => $this->getTableName()),
                          "{$responsesDb->getTableName()}.id=v.object_id",
                          array()
               )
               ->where("{$responsesDb->getTableName()}.created_by_id=?", $user->id)
               ->where("{$responsesDb->getTableName()}.type=v.object")
               ->where('v.created_by_id<>?', $user->id);
        $votesResponses = $this->getAdapter()->fetchOne($select);

        $responsesCommentsDb = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
        $select = $responsesCommentsDb->getSelect();
        $select->setIntegrityCheck(false)
               ->reset('columns')
               ->columns(array(
                       'rating' => 'count(v.id)',
               ))
               ->joinLeft(array('v' => $this->getTableName()),
                          "{$responsesCommentsDb->getTableName()}.id=v.object_id",
                          array()
               )
               ->where("{$responsesCommentsDb->getTableName()}.created_by_id=?", $user->id)
               ->where("v.object='comment'")
               ->where('v.created_by_id<>?', $user->id);
        $votesComments = $this->getAdapter()->fetchOne($select);
         
        return $votesResponses + $votesComments;
    }
    
    /**
     * Returns Number of votes for the responses
     *
     * @param  int $userId
     * @return int
     */
    public function findByResponsesAndUserId($userId)
    {
        $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
        $select = $this->getSelect();
        $select->reset('columns')
               ->setIntegrityCheck(false)
               ->columns(array(
                       'votes' => "count({$this->getTableName()}.id)",
               ))
               ->joinLeft(array('r' => $responsesDb->getTableName()),
                          "{$this->getTableName()}.object_id=r.id AND {$this->getTableName()}.object=r.type",
                          array()
               )
               ->where("{$this->getTableName()}.created_by_id=?", $userId)
               ->where('r.created_by_id<>?', $userId);
               
        return $this->getAdapter()->fetchOne($select);
    }

    /**
     * Returns votes by given object(field:object_id)
     * and user id (field:created_by_id)
     *
     * @param  Zend_Db_Table_Row $object
     * @param  int $userId
     * @param  int $pole
     * @return Resources_Model_Vote
     */
    public function findByObjectAndUserId($object, $userId, $pole = null)
    {
        $select = $this->getSelect()
                       ->where('created_by_id=?', $userId)
                       ->where('object=?', $object->getObject())
                       ->where('object_id=?', $object->id);
        if (isset($pole)) {
            if ($pole > 0) {
                $select->where("value > 0");
            } elseif($pole < 0) {
                $select->where("value < 0");
            }
        }
        return $this->fetchRow($select);
    }
    
    /**
     * Returns array (object_id=>value) by user
     *
     * @param  array $objects
     * @param  int $userId
     * @return array
     */
    public function findByObjectIdsAndUserId($objects, $userId)
    {
        $select = $this->getSelect()
                       ->where("created_by_id=?", $userId)
                       ->setIntegrityCheck(false)
                       ->reset("columns")
                       ->columns(array("object_id", "value"));
        $sql =  array();
        foreach ($objects as $object) {
            $object['type'] = $this->getAdapter()->quote($object['type']);
            $object['id'] = $this->getAdapter()->quote($object['id']);
            $sql[] = "(object={$object['type']} AND object_id={$object['id']})";
        }
        $select->where(implode(" OR ", $sql));

        return $this->getAdapter()->fetchPairs($select);
    }

    /**
     * Returns rating for specific user (field:created_by_id)
     *
     * @param  int $userId
     * @return Zend_Db_Table_Rowset
     */
    public function getByUserId($userId)
    {
        $where = $this->getAdapter()->quoteInto('created_by_id=?', $userId);
        return $this->getAdapter()-fetchAll($where);
    }
    
    /**
     * Returns total number of votes by given condition
     *
     * @param  array|null $where
     * @return int
     */
    public function count ($where = null)
    {
        $select = $this->getAdapter()->select();

        if (is_array($where)) {
            foreach ($where as $cond => $value) {
                if ($value) {
                    $select->where('v.' . $cond . '=?', $value);
                }
            }
        } else if (!empty($where) && is_string($where)) {
            $select->where($where);
        }
        $select->from(array('v' => $this->getTableName()), array('count' => new Zend_Db_Expr('COUNT(v.id)')));

        return $this->getAdapter()->fetchOne($select);
    }
}
