<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Model
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * email_templates row model
 *
 * @uses       System_Model_Row
 * @category   Resources
 * @package    Resources_Model
 */
class Resources_Model_ResponseComment extends System_Model_Row
{
    /**
     * Table class
     *
     * @var string
     */
    protected $_tableClass = 'Resources_Model_DbTable_ResponsesComments';

    /**
     * @var int
     */
    protected $_votesSum = null;

    /**
     * @var string
     */
    public $type = 'comment';

    /**
     * Returns ISO-formatted created_at date
     *
     * @return Zend_Date
     */
    public function getCreatedAt()
    {
        return new Zend_Date($this->created_at, Zend_Date::ISO_8601);
    }

    /**
     * Returns ISO-formatted updated_at date
     *
     * @return Zend_Date
     */
    public function getUpdatedAt()
    {
        return new Zend_Date($this->updated_at, Zend_Date::ISO_8601);
    }

    /**
     * Allows post-delete logic to be applied to row.
     * Subclasses may override this method.
     *
     * @see    parent::_postDelete()
     * @return void
     */
    protected function _postDelete()
    {
        parent::_postDelete();

        // Remove relative votes
        $votesDb = Singular_Loader::get("Resources_Model_DbTable_Votes");
        $votes   = $votesDb->fetchAll("object_id={$this->id} AND object='{$this->type}'");
        if ($votes->valid()) {
            foreach ($votes as $vote) {
                $vote->delete();
            }
        }

        // Remove relative attachments
        if (Singular_Module::getInstance()->moduleExists('files')) {
            $attachments = Singular_Loader::get('Files_Model_DbTable_Files')->findByObjectId($this);
            /** @var Files_Model_File $attachment */
            foreach ($attachments as $attachment) {
                $attachment->delete();
            }
        }

        $this->_cleanDefaultCache();
    }

    /**
     * Allows post-update logic to be applied to row.
     * Subclasses may override this method.
     *
     * @return void
     */
    protected function _postUpdate()
    {
        parent::_postUpdate();
        
        $this->_cleanDefaultCache();
    }
    
    /**
     * Allows post-insert logic to be applied to row.
     * Subclasses may override this method.
     *
     * @see    parent::_postInsert()
     * @return void
     */
    public function _postInsert()
    {
        $vote = new Resources_Model_Vote();
        $vote->object = $this->getObject();
        $vote->object_id = $this->id;
        $vote->value = 1;
        $vote->save();

        //log
        $log = new System_Model_Log();
        $log->save(array(
            'object'    => $this->getObject(),
            'object_id' => $this->id,
            'action'    => System_Model_DbTable_Logs::COMMENTED,
            'title'     => $this->getResponse()->title,
            'comment'   => $this->content
        ));

        parent::_postInsert();
        
        $this->_cleanDefaultCache();
    }

    /**
     * Clean default cache
     *
     * @return void
     */
    protected function _cleanDefaultCache()
    {       
        $activeResponse = Singular_Runtime::extract('activeResponse');
        if ( !($activeResponse instanceof System_Model_Response) ) {
            $activeResponse = $this->getResponse();
        }
        $cacheTags = array('breadcrumbs', 'comment_' . $this->id, 'comments',
                           $activeResponse->type . '_' . $activeResponse->id, 'homepage');
        $cache     = Singular_Runtime::extract('cachemanager')->getCache('default');
        $cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG, $cacheTags);
        if (class_exists('Supercache', false)) {
            SuperCache::getInstance()->getCache()->clean(Zend_Cache::CLEANING_MODE_ALL);
        }
    }
    
    /**
     * Returns response author
     *
     * @return System_Model_User
     */
    public function getAuthor()
    {
        if(!isset($this->_properties['CreatedBy'])){
            $users = Singular_Loader::get('System_Model_DbTable_Users');
            $result = $users->findById($this->created_by_id);
            if($result instanceof System_Model_User){
                $this->_properties['CreatedBy'] = $result;
            }
        }
        return $this->_properties['CreatedBy'];
    }

    /**
     * Returns response by response_id
     *
     * @return System_Model_Response
     */
    public function getResponse()
    {
        if(!isset($this->_properties['Response'])) {
            $responses = Singular_Loader::get('System_Model_DbTable_Responses');
            $result = $responses->find($this->response_id)->current();
            if($result instanceof System_Model_Response){
                $this->_properties['Response'] = $result;
            }
            
        }
        return $this->_properties['Response'];
    }

    /**
     * Removes best status for current comment
     *
     * @return void
     */
    public function unsetBest()
    {
        $this->is_best = 0;
        $this->best_at = null;
    }

    /**
     * Sets best status for current comment
     *
     * @return void
     */
    public function setBest()
    {
        $this->is_best = 1;
        $this->best_at = date('Y-m-d H:i:s');
    }

    /**
     * Checks if current comment is negative
     *
     * @return int
     */
    public function isNegative()
    {
        return $this->is_negative;
    }

    /**
     * Returns total number of votes for current comment
     *
     * @return int
     */
    public function getVotesSum()
    {
        if(is_null($this->_votesSum)){
            $votes = Singular_Loader::get('Resources_Model_DbTable_Votes');
            $this->_votesSum = $votes->getRatingByObject($this);
        }
        return $this->_votesSum;
    }

    /**
     * Checks if given user (field:user_id)
     * is already voted for current comment
     *
     * @param  int|null $userId
     * @return bool
     */
    public function isVoted($userId = null)
    {
        if(is_null($userId)){
            $auth = Zend_Auth::getInstance();
            if ($auth->hasIdentity()) {
                $userId = $auth->getIdentity()->id;
            } elseif(Singular_Core::_("Settings")->getSetting("anonymous_votes")) {
                $cookie = Zend_Controller_Front::getInstance()->getRequest()
                            ->getCookie('votes', '{}');
                $cookie = Zend_Json_Decoder::decode($cookie);
                if (isset($cookie['comment'][$this->id])) {
                    return true;
                } else {
                    return false;
                }
                return true;
            } else {
                return true;
            }
        }
        $votes = Singular_Loader::get('Resources_Model_DbTable_Votes');
        $vote = $votes->findByObjectAndUserId($this, $userId);
        
        if($vote instanceof Resources_Model_Vote){
            return true;
        }else{
            return false;
        }
    }

    /**
     * Checks if logged user can edit comments
     *
     * @return bool
     */
    public function canEdit()
    {
        $result = $this->isAllowed('manage_responses') || $this->isAllowed('can_access_admin');
        if (!$result) {
            $loggedUser = Singular_Runtime::extract("loggedUser");
            if ($loggedUser->id == $this->created_by_id) {
                $createdAt = strtotime($this->created_at);
                if ($createdAt > (time() - 60*60)) {
                    $result = true;
                }
            }
        }
        
        $data = new stdClass();
        $data->comment = $this;
        $data->result = $result;
        Singular_Event::dispatch('canEditComment', $data);

        return $data->result;
    }

    /**
     * Checks if logged user can delete comments
     *
     * @return bool
     */
    public function canDelete()
    {
        $result = false;
        if($this->isAllowed('manage_responses') || $this->isAllowed('can_access_admin')) {
            $result = true;
        }
        
        $data = new stdClass();
        $data->comment = $this;
        $data->result = $result;
        Singular_Event::dispatch('canDeleteComment', $data);

        return $data->result;
    }

    /**
     * Recoveries current comment from trash
     *
     * @return mixed The primary key value(s), as an associative array if the
     *     key is compound, or a scalar if the key is single-column.
     */
    public function recovery()
    {
        $this->deleted_by_id = 0;
        $this->deleted_at = null;
        $result = $this->save();
        if ($result) {
            $log = new System_Model_Log();
            $log->save(array(
                'object'    => $this->getObject(),
                'object_id' => $this->id,
                'action'    => System_Model_DbTable_Logs::RECOVERY,
                'title'     => $this->getResponse()->title,
                'comment'   => $this->content
            ));
        }
        return $result;
    }

    /**
     * Moves current comment to trash
     *
     * @return mixed
     */
    public function trash()
    {
        $this->deleted_by_id = Singular_Runtime::extract("loggedUser")->id;
        $this->deleted_at = date("Y-m-d H:i:s");
        $result = $this->save();
        if ($result) {
            $log = new System_Model_Log();
            $log->save(array(
                'object'    => $this->getObject(),
                'object_id' => $this->id,
                'action'    => System_Model_DbTable_Logs::MOVED_TO_TRASH,
                'title'     => $this->getResponse()->title,
                'comment'   => $this->content
            ));
        }
        return $result;
    }

    /**
     * Returns comment url
     *
     * @return string
     */
    public function getUrl()
    {
        $response = $this->getResponse();
        if ($response instanceof System_Model_Response) {
            return $response->getUrl() . "#comment-" . $this->id;
        } else {
            return '#';
        }
    }
    
    /**
     * Returns user ids from reply block
     *
     * @return array
     */
    public function getReplyUserIds()
    {
        preg_match_all("|\[reply user_id=([0-9]*)\]|i", $this->content, $matchesarray);
        if (!empty($matchesarray)) {
            return array_unique($matchesarray[1]);
        }
        return array();
    }
}
