<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Service
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * ApplicationMailer service
 *
 * @uses       Zend_Auth_Adapter_Interface
 * @category   Resources
 * @package    Resources_Service
 */
class Resources_Service_ApplicationMailer
{
    /**
     * Sends templated email
     *
     * @static
     * @param  System_Model_User|System_Model_Rowset $to    Recipients
     * @param  string $tpl  Selected template
     * @param  array $replacements  Replacement tokens (key=>val)
     * @return int
     * @throws Singular_Exception
     */
    static public function send($to, $tpl, $replacements = null)
    {
        if ($to instanceof System_Model_User) {
            $users = new System_Model_Rowset(array(
                'table' => Singular_Loader::get('System_Model_DbTable_Users'),
                'rowClass' => 'System_Model_User'
            ));
            $users->add($to);
        } elseif ($to instanceof System_Model_Rowset) {
            $users = $to;
        } else {
            return 0;
        }

        $params = new stdClass();
        $params->to = $users;
        $params->tpl = $tpl;
        $params->vars = $replacements;
        
        Singular_Event::dispatch("beforeEmailPrepare", $params);

        $count = 0;
        if ($users->valid()) {
            //find templates body
            $templates = Singular_Loader::get('Resources_Model_DbTable_Templates');
            $template = $templates->findBySlug($tpl);
            if (!($template instanceof Resources_Model_Template)) {
                throw new Singular_Exception('Template not found');
            }
            $mail = self::prepareEmail();

            //get vars
            $variables = $template->getVariables();

            //make body
            $body = $template->getBody();
            foreach ($variables as $var) {
                $body = str_replace(":$var", $replacements[$var], $body);
            }

            // Licensing mail body
            $license = Singular_Runtime::extract('license');
            if ((bool)!$license['branding_removed']) {
                $body .= '---<br/><p><strong>Powered by: </strong><a href="http://www.useresponse.com">UseResponse</a></p>';
            }

            $mail->setBodyHtml($body);

            // Make subject
            $subject = $template->getSubject();
            foreach ($variables as $var) {
                $subject = str_replace(":$var", $replacements[$var], $subject);
            }
            $mail->setSubject($subject);

            $params->mail = $mail;
            Singular_Event::dispatch("beforeEmailSend", $params);

            foreach ($users as $user) {
                if (Zend_Auth::getInstance()->hasIdentity()) {
                    $loggedUser = Singular_Runtime::extract("loggedUser");
                    if ($loggedUser->id == $user->id) {
                        continue;
                    }
                }
                if (preg_match('/localhost$/', $user->email)) {
                    continue;
                }
                $config = $user->getConfig(System_Model_DbTable_UsersConfigs::SEND_NOTIFICATIONS);
                if ($config == System_Model_UserConfig::ONCE
                    && $tpl != 'forgot_password'
                ) {
                    $emailRow = new Resources_Model_Email();
                    $time = $user->getConfig(System_Model_DbTable_UsersConfigs::SEND_NOTIFICATIONS_ONCE_TIME);
                    $timestamp = strtotime(date('Y-m-d ') . $time);
                    if ($timestamp > time()) {
                        $emailRow->time = date("Y-m-d H:i:s", $timestamp);
                    } else {
                        $timestamp = strtotime(date('Y-m-d ') . $time . ' +1 day');
                        $emailRow->time = date("Y-m-d H:i:s", $timestamp);
                    }
                    $emailRow->email = $user->email;
                    $emailRow->subject = $subject;
                    $emailRow->body = $mail->getBodyHtml()->getRawContent();
                    $emailRow->save();
                } else {
                    $mail->addTo($user->email);
                }
            }

            $emails = new ArrayObject($mail->getRecipients());
            Singular_Event::dispatch("beforeEmailDelivery", $emails);            
            if (!empty($emails)) {
                
                foreach ($emails as $email) {
                    $mail->clearRecipients();
                    $mail->addTo($email);
                    try {
                        $mail->send();
                        $count++;
                    } catch (Exception $e) {

                    }
                }
            }
        }
        return $count;
    }

    /**
     * Sends custom email
     *
     * @static
     * @param  string|array $emails
     * @param  string $subject
     * @param  string $body
     * @return int
     */
    public static function sendEmail($emails, $subject, $body)
    {
        $mail = self::prepareEmail();
        $mail->setSubject($subject);
        $mail->setBodyHtml($body);
        $mail->addTo($emails);
        $count = 0;
        $emails = $mail->getRecipients();
        foreach ($emails as $email) {
            $mail->clearRecipients();
            $mail->addTo($email);
            try {
                $mail->send();
                $count++;
            } catch (Exception $e) {

            }
        }
        return $count;
    }

    /**
     * Prepares email sender according to current mailing settings
     *
     * @static
     * @return Zend_Mail
     */
    static function prepareEmail()
    {
        $settings = Singular_Core::_('settings')->getModuleSettings('system', 'mailing');
        $mail = new Zend_Mail('UTF-8');
        if ($settings->email_connection_type->value == 'smtp') {
            if ($settings->email_smtp_authentication->value) {
                $config = array(
                    'auth' => 'login',
                    'username' => $settings->email_smtp_username->value,
                    'password' => $settings->email_smtp_password->value
                );
                if (!empty($settings->email_smtp_security->value)
                        && $settings->email_smtp_security->value != 'off') {
                    $config['ssl'] = $settings->email_smtp_security->value;
                }
            }
            $config['port'] = $settings->email_smtp_port->value;

            $transport = new Zend_Mail_Transport_Smtp($settings->email_smtp_host->value, $config);
        } else {
            $transport = new Zend_Mail_Transport_Sendmail("-oi -f " . $settings->send_email_from_email->value);
        }
        
        $params =  array(
          "utctime"     => gmstrftime("%Y%m%d%H%M%S"),
          "pid"         => getmypid(),
          "randint"     => mt_rand(),
          "customstr"   => "swift",
          "hostname"    => (isset($_SERVER["SERVER_NAME"]) ? $_SERVER["SERVER_NAME"] : php_uname("n")),
        );
        $mail->setMessageId(vsprintf("%s.%d.%d.%s@%s", $params));
        
        $mail->setDefaultTransport($transport);
        
        $mail->setFrom($settings->send_email_from_email->value, $settings->send_email_from_name->value);

        $mail->setReplyTo($settings->send_email_from_email->value);
        $mail->setReturnPath($settings->send_email_from_email->value);

        if ($settings->email_is_bulk->value) {
            $mail->addHeader('Auto-Submitted', 'auto-generated');
            $mail->addHeader('Precedence', 'bulk');
        }
        
        return $mail;
    }
}
