<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Abstract api controller
 *
 * @uses       System_Controller_Abstract_Action
 * @category   System
 * @package    System_Controller
 */
class System_Controller_Abstract_Api extends System_Controller_Abstract_Action
{
    /**
     * Response
     * 
     * @var array 
     */
    protected $_data = array();
    
    /**
     * Default page number
     * 
     * @var int
     */
    protected $_pageNumber = null;
    
    /**
     * Default row count value
     * 
     * @var int
     */
    protected $_rowCount = null;
    
    /**
     * Bad request error http code
     */
    const ERROR_BAD_REQUEST = 400;
    
    /**
     * Unauthorized error http code
     */
    const ERROR_UNAUTHORIZED = 401;
    
    /**
     * Forbidded error http code 
     */
    const ERROR_FORBIDDEN = 403;
    
    /**
     * Not found error http code 
     */
    const ERROR_NOT_FOUND = 404;
    
    /**
     * Operation failed http error code 
     */
    const ERROR_OPERATION_FAILED = 500;
    
    /**
     * Service unavailable http error code 
     */
    const ERROR_SERVICE_UNAVAILABLE = 503;
    
    /**
     * Success HTTP-code
     */
    const STATUS_OK = 200;
    
    /**
     * Default error messages
     * 
     * @var array
     */
    protected $_messages = array(
        self::ERROR_BAD_REQUEST         => 'Bad Request',
        self::ERROR_FORBIDDEN           => 'Forbidden',
        self::ERROR_NOT_FOUND           => 'Not Found',
        self::ERROR_OPERATION_FAILED    => 'Operation Failed',
        self::ERROR_UNAUTHORIZED        => 'Unauthorized',
        self::ERROR_SERVICE_UNAVAILABLE => 'Service Unavailable',
        self::STATUS_OK                 => 'OK',
    );

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @param void
     * @return void
     */
    public function init ()
    {
        parent::init();
        
        // No rendering
        $this->_helper->viewRenderer->setNoRender(true);
        $this->_helper->layout->disableLayout(true);
        
        // Api is enabled?
        if (!Singular_Core::_('settings')->getSetting('api_is_enabled')) {
            $this->error(self::ERROR_SERVICE_UNAVAILABLE);
        }
        
        // If not authenticated
        if (!Zend_Auth::getInstance()->hasIdentity()) {
            $this->error(self::ERROR_UNAUTHORIZED);
        }
        
        // Getting page number
        $this->setPageNumber($this->getRequest()->getParam('page', 1));
        
        // Getting row count
        $limitRows = $this->getRequest()->getParam('limit', Singular_Core::_('settings')->getSetting('api_limit_rows'));
        $this->setRowCount($limitRows);
    }
    
    /**
     * Setter for response data
     * 
     * @param array $data 
     * @return void
     */
    public function setData ($data)
    {
        $this->_data = (array) $data;
    }
    
    /**
     * Getter for response data
     * 
     * @param void
     * @return array
     */
    public function getData ()
    {
        return $this->_data;
    }
    
    /**
     * Setter for page number
     * 
     * @param int $page 
     * @return void
     */
    public function setPageNumber ($page)
    {
        $this->_pageNumber = (int) $page;
    }
    
    /**
     * Getter for page number
     * 
     * @param void
     * @return int
     */
    public function getPageNumber ()
    {
        return $this->_pageNumber;
    }
    
    /**
     * Setter for row count
     * 
     * @param int $count
     * @return void
     */
    public function setRowCount ($count)
    {
        $this->_rowCount = (int) $count;
    }
    
    /**
     * Getter for row count
     * 
     * @param void
     * @return int
     */
    public function getRowCount ()
    {
        return $this->_rowCount;
    }
    
    /**
     * Render response
     * 
     * @param void
     * @return void
     */
    public function postDispatch ()
    {
        if (empty($this->_data)) {
            $this->_data = array($this->_messages[self::STATUS_OK]);
        }
        $this->_request->setParam('response', $this->_data);
    }

    /**
     * Generate http error
     * 
     * @param int $code
     * @param string|null|Singular_Form $data
     * @throws System_Controller_Exception_Abstract 
     */
    public function error ($code = self::ERROR_OPERATION_FAILED, $data = null)
    {
        // Default error message
        if (null === $data) {
            $this->_data = array($this->_messages[$code]);
            
        // Form errors
        } else if ($data instanceof Singular_Form) {
            $elements = $data->getErrors();
            if (is_array($elements)) {
                foreach ($elements as $element => $errors) {
                    if (is_array($errors)) {
                        foreach ($errors as $error) {
                            $messages = $data->getElement($element)->getMessages();
                            $this->_data[$element] = $messages[$error];
                        }
                    }
                }
            }
            
        // Simple string error
        } else {
            $this->_data = array($data);
        }
        
        // Save params to request and throw exception
        $this->_request->setParam('response', $this->_data);

        switch ($code) {
            case 400:
                $msg = 'Invalid request and/or request parameters';
                break;
            case 401:
                $msg = 'Invalid API Key';
                break;
            case 403:
                $msg = 'API request is prohibited due to lack of permissions';
                break;
            case 404:
                $msg = 'Module, module method, or resources not found for API request';
                break;
            case 500:
                $msg = 'Internal server error';
                break;
            case 503:
                $msg = 'API interface unavailable or disabled';
                break;
            default:
                $msg = 'Unknown API error';
        }

        throw new System_Controller_Exception_Abstract($msg, $code);
    }
}
