<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Admin-analytics controller
 *
 * @uses       System_Controller_Abstract_Admin
 * @category   System
 * @package    System_Controller
 */
class AdminAnalyticsController extends System_Controller_Abstract_Admin
{
    /**
     * Initialize object.
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @return void
     */
    public function init()
    {
        parent::init();
        if ($this->isAjax()) $this->_helper->layout->disableLayout();
        $this->view->headLink()->prependStylesheet($this->view->stylesheetSrc('admin.css', 'system'));
    }

    /**
     * Index action
     *
     * @return void
     */
    public function indexAction()
    {
        $this->_initReports();
        $this->view->scriptsForm = new System_Form_ExternalScripts($this->settings->system->analytics);
        $this->view->analyticsFilterForm = new System_Form_AnalyticsFilter();
        $formValues  = $this->_getRequestFormValues();
        if (!empty($formValues)) {
            if(isset($formValues['system-form-analyticsfilter'])) {
                $this->_processFilterForm();
                return;
            }
            $this->_processScriptsForm();
            //return;
        }
        $this->view->analyticsDiagram = $this->_getDiagram();
    }
    
    /**
     * Return base64 encoded diagram image.
     * 
     * @return string
     */
    protected function _getDiagram ()
    {
        $params = $this->_getDiagramParams();
        $reportConfig = $this->_getDiagramReportConfig();
        
        switch ($params['group_by']) {
            case 'none':
                $column = new System_Lib_Analytics_View_Diagram_Svg_DateRange( array( 'reportConfig' => $reportConfig ));
                return $column->render();
            case 'weekday':
                $report = new System_Lib_Analytics_Model_Report_Time_WeekDays($reportConfig);
                $round = new System_Lib_Analytics_View_Diagram_Round();
                $round->addDataSet($report->getReport())->addLegendDataSet( array(
                    $this->view->t('Monday'),
                    $this->view->t('Tuesday'),
                    $this->view->t('Wednesday'),
                    $this->view->t('Thursday'),
                    $this->view->t('Friday'),
                    $this->view->t('Saturday'),
                    $this->view->t('Sunday')
                ));
                return $round->render();
            case 'type':
                $report = new System_Lib_Analytics_Model_Report_Time_ResponseTypes($this->_getDiagramReportConfig($reportConfig));
                $round = new System_Lib_Analytics_View_Diagram_Round();
                $legend = array();
                foreach (Singular_Runtime::extract('types') as $type) {
                    $legend[] = $this->view->t($type->title);
                }
                $round->addDataSet($report->getReport())->addLegendDataSet($legend);
                return $round->render();
        }
    }        

    /**
     * Return request data for any form.
     *
     * @return array
     */
    protected function _getRequestFormValues()
    {
        if (!$this->getRequest()->isPost()) {
            return array();
        }
        return $this->getRequest()->getPost();
    }
    
    /**
     * Processes the Scripts Form 
     *
     * @return void
     */
    protected function _processScriptsForm()
    {
        $form = $this->view->scriptsForm;
        if (!isset($form) || empty($form)) {
            return;
        }
        
        $requestValues = $this->_getRequestFormValues();
        if (!$form->isValid($requestValues)) {
            return; 
        }
        
        $formValues = $form->getValues();
        foreach ($formValues as $name => $value) {
            $setting = Singular_Core::_('settings')->getSetting($name,true); 
            if (!isset($setting) || empty($setting)) {
                continue;
            }
            Singular_Core::_('settings')->changeSetting($name, $value);
        }
        $this->view->notify()->success('External scripts settings are changed!');
    }
    
    /**
     * Processes the Filter Form 
     *
     * @return void
     */
    protected function _processFilterForm ()
    {
        $form = $this->view->analyticsFilterForm;
        if (!isset($form) || empty($form)) {
            return;
        }
        
        $requestValues = $this->_getRequestFormValues();
        if (!$form->isValid($requestValues)) {
            return; 
        }
        
        if ($this->isAjax()) {
            $params = $this->_getDiagramParams();
            $reports = $form->getReports($params['group_by']);
            
            $this->autoresponder->setParams(array(
                'report_type' => $form->getElement('report_type')
                    ->setMultioptions($reports)
                    ->removeDecorator('Zend_Form_Decorator_ViewScript')
                    ->setValue($params['report_type'])
                    ->render(),
                'diagram_img' => $this->_getDiagram(),
            ));
        }
    }

     /**
     * Initialize AnalyticsReports config for view. 
     *
     * @return void
     */
    protected function _initReports ()
    {
        $reportsConf = new stdClass();
        $reportsConf->reports = array();
        Singular_Event::dispatch('afterAnalyticsReportsInit', $reportsConf);
        Singular_Runtime::store('analytics_report_config', $reportsConf);
    }
    
    /**
     * Get initializing parameters for diagrams. 
     * 
     * @return array
     */
    protected function _getDiagramParams()
    {
        $formValues  = $this->_getRequestFormValues();
        if (isset($formValues['system-form-analyticsfilter'])) {
            if (!$this->isAjax()) {
                return $formValues['system-form-analyticsfilter'];
            }
            $form = $this->view->analyticsFilterForm;
            $params = $formValues['system-form-analyticsfilter'];
            $reports = $form->getReports($params['group_by']);
            //set current reportType value
            $hasValue = false;
            foreach ($reports as $report) {
                if ($report['key'] == $params['report_type']) {
                    $hasValue = true;
                }
            }
            $value = ($hasValue) ? $params['report_type'] : $form->getDefault('report_type');
            $params['report_type'] = $value;
            return $params;
        }
        if (isset($this->view->analyticsFilterForm)) {
            $params = array();
            foreach ($this->view->analyticsFilterForm->getElements() as $element) {
                $params[$element->getName()] = $element->getValue(); 
            }
            return $params;
        }
    }
    
    /**
     * Return date range from request.
     * 
     * @return array
     */
    protected function _getDateRange()
    {
        $params = $this->_getDiagramParams();
        $date = $params['date_range'];
        $format = 'Y-m-d';
        if( empty($date)) {
            return array(
                'from' => Zend_Date::now()->subMonth(1)->get($format),
                'to'   => Zend_Date::now()->get($format)
            );
        } else {
           return $this->_parseDate($date);
        }
    }
    
    /**
     * Parse requested date.
     * 
     * @param array $date
     * @return array
     */
    protected function _parseDate($date) {
        if (!strpos($date,'-')) {
            $dates = array(0 => $date, 1 =>'');
        } else {
            $dates = explode('-', $date);
        }        
        $range = array();
        foreach ( $dates as $key => $datePart) {
            $datePart = trim($datePart);
            $datePart = str_replace('.','-', $datePart);
            if (!empty($datePart)) {
                $range[$key] = '20'.$datePart;
            }
        }
        return array(
            'from' => $range[0],
            'to'   => (isset($range[1]))?$range[1]:'',
        );
    }
    
    /**
     * Return config for diagram dataset.
     * 
     * @return boolean|array
     */
    protected function _getDiagramReportConfig()
    {
        $params = $this->_getDiagramParams();
        $reportConfigs = Singular_Runtime::extract('analytics_report_config')->reports;
        $reportType = $params['report_type'];
        foreach( $reportConfigs as $config) {
            if ($config['id'] == $reportType) {
                $reportConfig = $config;
                break;
            }
        }
        if (!isset($reportConfig) || empty($reportConfig)) {
            return false;
        }
        
        $date = $this->_getDateRange();
        $reportConfig['dateRangeData'] = array(
            'dateFrom' => $date['from'],
            'dateTo'   => $date['to'],
            'precision'=> $params['precision'],
        );
        return $reportConfig;
    }        

}
