<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * System api controller
 *
 * @uses       System_Controller_Abstract_Api
 * @category   System
 * @package    System_Controller
 */
class ApiController extends System_Controller_Abstract_Api
{
    const RESPONSES_LIST_DEFAULT = 'default';
    const RESPONSES_LIST_SIMPLE  = 'simple';

    /**
     * Initialization
     * 
     * @return void 
     */
    public function init ()
    {
        parent::init();
        
        // Active user object
        $userId = $this->_request->getParam('user_id');
        $users = Singular_Loader::get('System_Model_DbTable_Users');
        if ($userId) {
            $user = $users->findById($userId);
            if (!$user instanceof System_Model_User) {
                $this->error(self::ERROR_NOT_FOUND);
            }
            $this->activeUser = $user;
        } else {
            $this->activeUser = $this->loggedUser;
        }
        $this->view->activeUser = $this->activeUser;
        
        // Active response object
        $responseId = $this->_request->getParam('response_id');
        $responses = Singular_Loader::get('System_Model_DbTable_Responses');
        if ($responseId) {
            $response = $responses->findById($responseId);
            if (!$response instanceof System_Model_Response) {
                $this->error(self::ERROR_NOT_FOUND);
            }
            $this->activeResponse = $response;
        } else {
            $this->activeResponse = $responses->createRow();
        }
        $this->view->activeResponse = $this->activeResponse;
        Singular_Runtime::store('activeResponse', $this->activeResponse);
    }
    
    /**
     * System statistic for comments, users, problems,
     * thanks, ideas and questions
     * 
     * @return void 
     */
    public function statisticAction ()
    {
        // Getting models
        $comments = Singular_Loader::get(
            'Resources_Model_DbTable_ResponsesComments'
        );
        $users = Singular_Loader::get('System_Model_DbTable_Users');
        
        // Main statistic
        $data = array(
            'comments' => $comments->count(),
            'users' => $users->count(),
        );
        
        // Modules statistic
        $responses = Singular_Loader::get('System_Model_DbTable_Responses');
        if (Singular_Module::getInstance()->moduleExists('thanks')) {
            $data['thanks'] = $responses->count(array('type' => 'thanks'));
        }
        if (Singular_Module::getInstance()->moduleExists('questions')) {
            $data['questions'] = $responses->count(array('type' => 'question'));
        }
        if (Singular_Module::getInstance()->moduleExists('ideas')) {
            $data['ideas'] = $responses->count(array('type' => 'idea'));
        }
        if (Singular_Module::getInstance()->moduleExists('problems')) {
            $data['problems'] = $responses->count(array('type' => 'problem'));
        }
        
        // Send output
        $this->setData($data);
    }
    
    /**
     * Users listing
     * 
     * @return void 
     */
    public function usersListAction ()
    {
        // Getting models
        $users = Singular_Loader::get('System_Model_DbTable_Users');
        
        // Getting params and users list
        $where = $filter = null;
        if ($this->loggedUser->canManagePeople()
            || $this->loggedUser->isAdmin()) {
            $role = $this->_request->getParam('role', '');
            $sort = $this->_request->getParam('sort', '');
            $filter = array('role' => $role);
            $where = array_merge($filter, array('sort' => $sort));
        }
        
        // Fetching users list
        $list = $users->getPaginator(
            $where, $this->getPageNumber(), $this->getRowCount()
        );

        // Send response!
        $this->setData(array(
            'count' => $list->getTotalItemCount(),
            'items' => System_Service_Api::usersListing($list->getCurrentItems()),
        ));
    }
    
    /**
     * Show user details
     * 
     * @return void 
     */
    public function showUserAction ()
    {
        if ($this->activeUser->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        $this->setData(
            System_Service_Api::usersListing($this->activeUser)
        );
    }
    
    /**
     * Edit active user
     * 
     * @return void
     */
    public function changeUserAction ()
    {
        // Can access?
        if ($this->loggedUser->id != $this->activeUser->id) {
            if (!$this->loggedUser->canManagePeople($this->activeUser)) {
                $this->error(self::ERROR_FORBIDDEN);
            }
            $form = new System_Form_AdminUser();
        } else {
            $form = new System_Form_User();
        }
        $form->removeElement('role');
        $form->removeElement('old_password');
        $form->removeElement('password');
        $form->removeElement('password_again');
        $form->removeElement('photo');
        $form->removeElement('gravatar_email');
        $form->removeElement(System_Model_DbTable_UsersConfigs::PHOTO_TYPE);
        $form->removeElement(System_Model_DbTable_UsersConfigs::SEND_NOTIFICATIONS_ONCE_TIME);
        $form->removeElement(System_Model_DbTable_UsersConfigs::SEND_NOTIFICATIONS);
        $form->removeElement('notifications_created');
        $form->removeElement('notifications_posted_comments');
        
        // Gettimg request
        $post = $this->activeUser->toArray();
        if ($this->_getParam('full_name')) {
            $post['full_name'] = $this->_getParam('full_name');
        }
        if ($this->_getParam('email')) {
            $post['email'] = $this->_getParam('email');
        }
        
        // Validation process
        if (!$form->isValid($post)) {
            $this->error(self::ERROR_BAD_REQUEST, $form);
        }
        
        // Update user in database
        if (!$this->activeUser->save($form->getValues())) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
    }
    
    /**
     * Remove user avatar
     * 
     * @return void 
     */
    public function removeUserAvatarAction ()
    {
        // Can access?
        if ($this->loggedUser->id != $this->activeUser->id) {
            if (!$this->loggedUser->canManagePeople($this->activeUser)) {
                $this->error(self::ERROR_FORBIDDEN);
            }
        }
        
        // Deleting user avatar
        $this->loggedUser->deletePhoto();
    }
    
    /**
     * Delete user
     * 
     * @return void 
     */
    public function removeUserAction ()
    {
        // Can access?
        if ($this->loggedUser->id != $this->activeUser->id) {
            if (!$this->loggedUser->canManagePeople($this->activeUser)) {
                $this->error(self::ERROR_FORBIDDEN);
            }
        }
        
        // Reassign responses, answers, comments, votes and delete user
        $reassign = (bool) $this->_request->getParam('reassign');
        $this->activeUser->delete($reassign);
    }
    
    /**
     * Enable / Disable user
     * 
     * @return void
     */
    public function switchUserStatusAction ()
    {
        // Can access?
        if (!$this->loggedUser->canManagePeople($this->activeUser)) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        
        // Getting params
        $status = $this->_request->getParam('status');
        $statuses = array('enable' => 1, 'disable' => 0);
        if (!array_key_exists($status, $statuses)) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        
        // Switch status!
        $this->activeUser->switchStatus($statuses[$status]);
    }
    
    /**
     * Change-password action
     *
     * @return void
     */
    public function changeUserPasswordAction ()
    {
        // Can access?
        if ($this->loggedUser->id != $this->activeUser->id) {
            if (!$this->loggedUser->canManagePeople($this->activeUser)) {
                $this->error(self::ERROR_FORBIDDEN);
            }
        }
        
        // Check current password
        $currentPassword = Singular_Core::_('Auth')->makePassword(
            $this->_request->getParam('current_password')
        );
        if ($currentPassword != $this->activeUser->password) {
            $this->error(self::ERROR_BAD_REQUEST, 'Current Password is incorrect!');
        }
        
        // Check new password
        $newPassword = $this->_request->getParam('new_password');
        if (empty($newPassword)) {
            $this->error(self::ERROR_BAD_REQUEST, 'New Password is incorrect!');
        }
        
        // Change password
        $newPassword = Singular_Core::_('Auth')->makePassword($newPassword);
        $this->activeUser->password = $newPassword;
        $this->activeUser->save();
    }
    
    /**
     * Create new user
     * 
     * @return void 
     */
    public function createUserAction ()
    {
        // Build request
        $request = array(
            'full_name' => $this->_request->getParam('full_name'),
            'email'     => $this->_request->getParam('email'),
        );

        // Validation process
        $form = new System_Form_Registration();
        if (!$form->isValid($request)) {
            $this->error(self::ERROR_BAD_REQUEST, $form);
        }
        
        // Registration process
        $user = new System_Model_User();
        if (!$user->register($form->getValues())) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
    }
    
    /**
     * Getting participants by response
     * 
     * @return void
     */
    public function participantsAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        $users = $this->activeResponse->getParticipants();
        $this->setData(System_Service_Api::usersListing($users));
    }
    
    /**
     * Getting subscribers by response
     * 
     * @return void
     */
    public function subscribersAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        $subscribers = Singular_Loader::get(
            'System_Model_DbTable_ResponsesSubscribers'
        );
        $users = $subscribers->findUsersByResponse($this->activeResponse);
        $this->setData(System_Service_Api::usersListing($users));
    }
    
    /**
     * Build activity list
     * 
     * @return void 
     */
    public function activityListAction ()
    {
        // Can access?
        if (!$this->loggedUser->canUseActivityLog()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        
        // Fetch rows
        $logs = Singular_Loader::get('System_Model_DbTable_Logs');
        $where = array('date' => $this->_request->getParam('date'));
        $items = $logs->getPaginator(
            $where, $this->getPageNumber(), $this->getRowCount()
        );
        $this->setData(array(
            'count' => $items->getTotalItemCount(),
            'items' => $items->getCurrentItems()->toArray(),
        ));
    }

    /**
     * Move response to trash
     * 
     * @return void 
     */
    public function responseToTrashAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeResponse->canDelete()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        if (!$this->activeResponse->trash()) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
    }
    
    /**
     * Recovery response from trash
     * 
     * @return void 
     */
    public function responseFromTrashAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeResponse->canDelete()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        if (!$this->activeResponse->recovery()) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
    }
    
    /**
     * Permanently delete response
     * 
     * @return void
     */
    public function removeResponseAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeResponse->canDelete()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        if (!$this->activeResponse->delete()) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
    }
    
    /**
     * Change response status
     * 
     * @return void
     */
    public function switchResponseStatusAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeResponse->canChangeStatus()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        $status = $this->_request->getParam('status', '');
        $statuses = Singular_Loader::get('System_Model_DbTable_ResponsesStatuses');
        $validator = new Zend_Validate_Db_RecordExists(array(
            'table' => $statuses->getTableName(),
            'field' => 'slug',
        ));
        $validator->getSelect()->where('type = ?', $this->activeResponse->getType()->slug);
        if (!$validator->isValid($status)) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeResponse->switchStatus($status)) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
    }
    
    /**
     * Subscribe on response
     * 
     * @return void
     */
    public function subscribeAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeResponse->isSubscribed($this->loggedUser->id)) {
            $this->activeResponse->subscribe($this->loggedUser->id);
        }
    }
    
    /**
     * Unsubscribe from response
     * 
     * @return void 
     */
    public function unsubscribeAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        $this->activeResponse->unSubscribe($this->loggedUser->id);
    }
    
    /**
     * Lock comments for active response
     * 
     * @return void 
     */
    public function lockCommentsAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeResponse->canLockComments()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        $this->activeResponse->lock_comments = 1;
        if (!$this->activeResponse->save()) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
    }
    
    /**
     * Unlock comments for active response
     * 
     * @return void 
     */
    public function unlockCommentsAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeResponse->canLockComments()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        $this->activeResponse->lock_comments = 0;
        if (!$this->activeResponse->save()) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
    }
    
    /**
     * Getting response types
     * 
     * @return void 
     */
    public function responseTypesAction ()
    {
        $data = array();
        $types = Singular_Runtime::extract('types');
        if ($types instanceof Zend_Db_Table_Rowset_Abstract) {
            foreach ($types as $type) {
                $data[] = array(
                    'image'  => $type->getImage(),
                    'module' => (string)$type->getModule(),
                    'title'  => $type->title,
                    'name'   => $type->slug,
                );
            }
        }
        $this->setData($data);
    }
    
    /**
     * Getting response statuses by type
     * 
     * @return void 
     */
    public function responseStatusesAction ()
    {
        $type = Singular_Loader::get(
            'System_Model_DbTable_ResponsesTypes'
        )->findBySlug($this->_request->getParam('type', ''));

        if (!$type instanceof System_Model_ResponseType) {
            $this->error(self::ERROR_BAD_REQUEST);
        }

        $statuses = Singular_Loader::get(
            'System_Model_DbTable_ResponsesStatuses'
        )->findByType($type->slug);
        
        $data = array();
        if ($statuses instanceof Zend_Db_Table_Rowset_Abstract) {
            foreach ($statuses as $status) {
                $data[] = array(
                    'title' => $status->title,
                    'name'  => $status->slug,
                    'color' => $status->color,
                );
            }
        }
        $this->setData($data);
    }
    
    /**
     * Show active response
     * 
     * @return void 
     */
    public function showResponseAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        $this->setData(
            System_Service_Api::responsesListing($this->activeResponse)
        );
    }
    
    /**
     * Responses listing
     * 
     * @return void 
     */
    public function responsesAction ()
    {
        // Build query to database
        /** @var $responses System_Model_DbTable_Responses */
        $responses = Singular_Loader::get('System_Model_DbTable_Responses');
        $where = array(
            'status' => $this->_request->getParam('status', 'popular'),
            'date'   => $this->_request->getParam('date'),
            'text'   => $this->_request->getParam('text'),
        );
        $limit   = $this->_request->getParam('limit', $this->getRowCount());
        $private = $this->_request->getParam('private', false);
        if ($private && 'false' != $private) {
            $this->getRequest()->setParam('privacy_type', 'privacy-filter-private');
        }

        // Add type filter
        $type = Singular_Loader::get('System_Model_DbTable_ResponsesTypes')
            ->findBySlug(
            $this->_request->getParam('type', '')
        );
        if ($type instanceof System_Model_ResponseType) {
            $where['type'] = $type->slug;
        }

        $total = 0;
        $data  = array();

        $list = $this->_request->getParam('list', self::RESPONSES_LIST_DEFAULT);
        if (self::RESPONSES_LIST_SIMPLE == $list) {
            $select = $responses->getSelect();
            $select = $responses->prepareSelect($select, $where);
            $items = $responses->fetchAll($select);
            if ($items instanceof System_Model_Rowset) {
                $total = $items->count();
                $data = $items->toArray();
            }
        } else {
            // Add status filter
            $items = $responses->getPaginator(
                $where, $this->getPageNumber(), null, $limit
            );
            $total = $items->getTotalItemCount();
            $data  = System_Service_Api::responsesListing($items->getCurrentItems());
        }

        $this->setData(
            array(
                 'total' => $total,
                 'items' => $data,
            )
        );
    }
    
    /**
     * Add new response
     * 
     * @return void
     */
    public function createResponseAction ()
    {
        if (!$this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }

        // Gettimg request
        $post = array(
            'type'       => $this->_request->getParam('type', ''),
            'title'      => $this->_request->getParam('title', ''),
            'content'    => $this->_request->getParam('content', ''),
            'tags'       => $this->_request->getParam('tags', ''),
            'is_private' => $this->_request->getParam('private', 0),
        );
        
        if (!Singular_Module::getInstance()->moduleExists('private')) {
            unset($post['is_private']);
        } else {
            $post['is_private'] = (int) $post['is_private'];
        }

        // Validate
        $form = new System_Form_Response();
        $form->removeElement('file');
        if (!$form->isValid($post)) {
            $this->error(self::ERROR_BAD_REQUEST, $form);
        }

        // Store response in database
        if (!$this->activeResponse->save($form->getValues())) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }

        // Notify listeners
        $data = (object) array(
            'response' => $this->activeResponse,
            'redirect' => null
        );
        Singular_Event::dispatch('responseAdded', $data);
    }
    
    /**
     * Edit active response
     * 
     * @return void
     */
    public function changeResponseAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeResponse->canEdit()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        
        // Validate
        $form = new System_Form_Response();
        $form->removeElement('file');
        
        // Gettimg request
        $post = $this->activeResponse->toArray();
        if ($this->_getParam('type')) {
            $post['type'] = $this->_getParam('type');
        }
        if ($this->_getParam('title')) {
            $post['title'] = $this->_getParam('title');
        }
        if ($this->_getParam('content')) {
            $post['content'] = $this->_getParam('content');
        }
        if ($this->_getParam('tags')) {
            $post['tags'] = $this->_getParam('tags');
        }
        if (!$form->isValid($post)) {
            $this->error(self::ERROR_BAD_REQUEST, $form);
        }
        
        // Update response in database
        if (!$this->activeResponse->save($form->getValues())) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
    }
    
    /**
     * Add answer for active response
     * 
     * @return void
     */
    public function createAnswerAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeResponse->canAddOfficialAnswer()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        if ($this->activeResponse->hasAnswer()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }

        // Gettimg request
        $post = array(
            'answer'      => $this->_request->getParam('answer', ''),
            'response_id' => $this->activeResponse->id,
        );
        
        // Validate
        $form = new System_Form_Answer();
        $form->removeElement('file');
        if (!$form->isValid($post)) {
            $this->error(self::ERROR_BAD_REQUEST, $form);
        }
        
        // Store answer in database
        $answer = new System_Model_ResponseAnswer();
        if (!$answer->save($form->getValues())) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }

        // Notify all listeners
        $params = (object) array(
            'answer'   => $answer,
            'response' => $this->activeResponse,
        );
        Singular_Event::dispatch('answerAdded', $params);
    }
    
    /**
     * Edit answer for active response
     * 
     * @return void
     */
    public function changeAnswerAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeResponse->canAddOfficialAnswer()) {
            $this->error(self::ERROR_FORBIDDEN);
        }
        if (!$this->activeResponse->hasAnswer()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        
        // Gettimg request
        $post = array(
            'answer'      => $this->_request->getParam('answer', ''),
            'response_id' => $this->activeResponse->id,
        );
        
        // Validate
        $form = new System_Form_AnswerEdit();
        $form->removeElement('file');
        if (!$form->isValid($post)) {
            $this->error(self::ERROR_BAD_REQUEST, $form);
        }
        
        // Update answer in database
        $answer = $this->activeResponse->getAnswer();
        $data = $form->getValues();
        $data['id'] = $answer->id;
        if (!$answer->save($data)) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
        
        // Notify subscribers
        if (isset($data['notify']) && $data['notify']) {
            $params = (object) array(
                'answer'   => $answer,
                'response' => $this->activeResponse,
            );
            Singular_Event::dispatch('answerAdded', $params);
        }
    }
    
    /**
     * Add vote for active response
     * 
     * @return void 
     */
    public function voteForResponseAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }

        $value   = $this->_request->getParam('value');
        $comment = $this->_request->getParam('comment');
        $isVoted = $this->activeResponse->isVoted($this->loggedUser->id);
        
        $forceToCommentNegativeVote = $this->settings->system->responses
            ->force_to_comment_negative_vote->value;
        
        $useNegativeVotes = $this->settings->system->responses
            ->use_negative_votes->value;
        
        if ($isVoted && !$useNegativeVotes) {
            $this->error(self::ERROR_BAD_REQUEST, 'You have already voted for this response!');
        }
        
        if ($isVoted) {
            if ($value < 0 && $forceToCommentNegativeVote && !$this->activeResponse->lock_comments) {
                if (!$comment) {
                    $this->error(self::ERROR_BAD_REQUEST, 'Comment is required for negative vote!');
                }
                // TODO: require comment
            }
            $vote = $this->activeResponse->getUserVote($this->loggedUser->id);
            if ($value > 0 && $vote->value < 0) {
                $vote->value = 1;
            } elseif ($value < 0 && $vote->value > 0) {
                $vote->value = -1;
            } else {
                $this->error(self::ERROR_BAD_REQUEST, 'You have already voted for this response!');
            }
            if (!$vote->save()) {
                $this->error(self::ERROR_OPERATION_FAILED);
            }
        } else {
            if ($value < 0 && $forceToCommentNegativeVote) {
                $this->error(self::ERROR_BAD_REQUEST, 'Comment is required for negative vote!');
            }
            $vote = new Resources_Model_Vote();
            $vote->object    = $this->activeResponse->getObject();
            $vote->object_id = $this->activeResponse->id;
            $vote->value     = $value;
            if (!$vote->save()) {
                $this->error(self::ERROR_OPERATION_FAILED);
            }
        }
    }
    
    /**
     * Modify position for response
     * 
     * @return void 
     */
    public function setResponseOrderAction ()
    {
        if ($this->activeResponse->isNew()) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        $order = (int) $this->_getParam('order', 1);
        if (!in_array($order, array(0,1,2))) {
            $this->error(self::ERROR_BAD_REQUEST);
        }
        if (!$this->activeResponse->canDelete()) {
             $this->error(self::ERROR_FORBIDDEN);
        }
        $this->activeResponse->order = $order;
        if (!$this->activeResponse->save()) {
            $this->error(self::ERROR_OPERATION_FAILED);
        }
    }
}
