<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_View
 * @subpackage System_View_Helper
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Slice view helper.
 *
 * Process and render reusable piece of template, code, text.
 *
 * @uses       Zend_View_Helper_Abstract
 * @category   System
 * @package    System_View
 * @subpackage System_View_Helper
 */
class System_View_Helper_Slice extends Zend_View_Helper_Abstract
{
    /**
     * Global options.
     *
     * @var array
     */
    protected static $_options = array(
        'tpl_ext'          => 'phtml',
        'slices_dir'       => 'slices',
        'throw_exceptions' => false,
        'cache'            => null,
    );

    private static $_initiated = false;

    protected static function _init()
    {
        if (self::$_initiated) return;
        self::setOption('cache', Singular_Runtime::extract('cachemanager')->getCache('default'));
        self::$_initiated = true;
    }

    /**
     * Sets global option.
     *
     * @param  string $name
     * @param  mixed  $value
     *
     * @throws Zend_View_Exception
     */
    public static function setOption($name, $value)
    {
        if (!self::hasOption($name)) {
            throw new Zend_View_Exception(
                "Invalid option '{$name}' for slice helper"
            );
        }

        switch ($name) {
            case 'tpl_ext':
                $value = ltrim($value, '.');
                break;
            case 'slices_dir':
                $value = trim($value, '/\\');
                break;
            case 'throw_exceptions':
                $value = (bool) $value;
                break;
        }

        self::$_options[$name] = $value;
    }

    /**
     * Returns global option.
     *
     * @param  string $name
     *
     * @return mixed
     */
    public static function getOption($name)
    {
        return self::hasOption($name) ? self::$_options[$name] : null;
    }

    /**
     * Returns true if option exists.
     *
     * @param  string $name
     *
     * @return bool
     */
    public static function hasOption($name)
    {
        return array_key_exists($name, self::$_options);
    }

    /**
     * Returns cache.
     *
     * @return Zend_Cache_Core
     */
    public static function getCache()
    {
        return self::getOption('cache');
    }

    /**
     * Renders slice.
     *
     * @param  string|array $name
     * @param  array        $params
     * @param  bool         $cache
     *
     * @return string|System_View_Helper_Slice
     * @throws Zend_View_Exception
     */
    public function slice($name = null, $params = array(), $cache = false)
    {
        self::_init();

        if (is_array($name)) {
            $vars   = $name;
            $module = null;
            if (isset($vars['name'])) {
                $name = $vars['name'];
                if (isset($vars['module'])) {
                    $module = $vars['module'];
                }
            } elseif (!empty($vars) && isset($vars[0])) {
                $name = $vars[0];
                if (isset($vars[1])) {
                    $module = $vars[1];
                }
            } else {
                throw new Zend_View_Exception('Unable to find slice name in array');
            }

            if (is_string($module)) {
                $this->view->additionalModulePath($module);
            }
        }

        if (null === $name) {
            return $this;

        } else if (is_array($name) && !empty($name)) {
            $output = '';
            foreach ($name as $iName => $iParams) {
                if (is_array($iParams)) {
                    $iParams = array_merge($params, $iParams);
                }
                $output .= $this->render($iName, $iParams, $cache);
            }
            return $output;

        } elseif (is_string($name)) {
            return $this->render($name, $params, $cache);

        } else {
            throw new Zend_View_Exception('Invalid slice name type');
        }
    }

    public function render($name, $params = array(), $cache = false)
    {
        $tplPath  = self::_getTplPath($name);
        $useCache = is_array($cache) || true === $cache;
        if ($useCache && !is_array($cache)) {
            $cache = array();
        }

        if ($useCache) {
            $cacheId  = "slice_{$name}";
            if (!empty($params)) {
                $cacheId .= '_' . md5(var_export($params, true));
            }
            $output = self::getCache()->load($cacheId);
        }

        if (!isset($output) || false === $output) {
            try {
                if (is_array($params) && !empty($params)) {
                    $output = $this->view->partial($tplPath, $params);
                } else {
                    $output = $this->view->render($tplPath);
                }
            } catch (Exception $e) {
                if (self::getOption('throw_exceptions')) {
                    throw $e;
                }
                $output = null;
            }

            if (null !== $output && $useCache) {
                self::getCache()->save($output, $cacheId, $cache);
            }
        }

        return $output;
    }

    private function _getTplPath($name)
    {
        return self::getOption('slices_dir') . DS . "{$name}." . self::getOption('tpl_ext');
    }
}
