<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Lib
 * @copyright  USWebStyle Inc., 2011-2013 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Analytics time report class
 * 
 * Splits time period on a parts (with the required precission) and returns a report of quantity for each part.
 *
 * @uses       System_Lib_Analytics_Model_Report_Abstract
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Analytics
 */
class System_Lib_Analytics_Model_Report_Time_DateRange extends System_Lib_Analytics_Model_Report_Time_Abstract
{
    /**
     * Report precission
     * 
     * @var array
     */
    protected $_precision = array();
    
    /**
     * Maximum number of database queries
     * 
     * @var int 
     */
    protected $_maxValuesQuantity = null;
    
    /**
     * Report data
     * 
     * @var array 
     */
    protected $_report = array(); 

    /**
     * Difference between the start and end date 
     * 
     * @var array 
     */
    protected $_dateDiff;
    
    /**
     * 
     * @return System_Lib_Analytics_Model_Report_Time_DateRange 
     */
    protected function _createReport ()
    {
        $fromDate = $this->_createDate($this->_dateFrom);
        $toDate = $this->_createDate($this->_dateTo);
        if(!empty($this->_data['precision'])) {
            $queries = $this->_getQueries();
        } else {
            $queries = $this->_getQueryFull($fromDate, $toDate);
            return $this->_getQuantity($queries);
        }
        foreach ($queries as $key => $query) {
            $this->_report[$key]['value'] = (int)$this->_getQuantity($query);
        }
        return $this;  
    } 

    /**
     * Return values (quantity) of generated report 
     * 
     * @return array
     */
    public function getDataSet ()
    {
        return $this->_getReportPart('value');
    }
    
    /**
     * Return precission of generated report
     * 
     * @return type
     */
    public function getPrecision ()
    {
        if (empty($this->_precision)) {
            $this->_createReport();
        }
        return $this->_precision;
    }
    
    /**
     * Return date ranges for each value of report
     * 
     * @return array
     */
    public function getDateRanges ()
    {
        return $this->_getReportPart('dateRange');
    }        
    
    /**
     * 
     * @return array
     */
    public function getReport ()
    {
        if (empty($this->_report)) {
            $this->_createReport();
        }
        return $this->_report;  
    }
    
    /**
     * Return data form report by type
     * 
     * @param type $partName
     * @return array
     */
    protected function _getReportPart($partName)
    {
        if (empty($this->_report)) {
            $this->_createReport();
        }
        $report = array();
        foreach ($this->_report as $key => $reportPart) {
            $report[$key] = $reportPart[$partName];
        }
        return $report;
    }        

    /**
     * Counts the number of queries to database for a given precision
     * 
     * @param array $precision
     * @return int
     */
    public function getReportQuantity ($precision) 
    {
        $quantity = 0;
        $startDate = $this->_createDate($this->_dateFrom);
        while ($this->_dateTo->get() > $startDate->get()) {
            $endDate = $this->getIncrasedDate($this->_createDate($startDate), $precision);
            $startDate = $endDate;
            $quantity++;
        }
        return $quantity;
    } 

    /**
     * Retrieve optimal report precision focusing on the maximum available number of queries.
     * 
     * @param array $precision
     * @return string
     */
    protected function _getPrecision ()
    {
        $precision = array('type'=>$this->_data['precision'],'value'=>'1');
        
        $dateDiff = $this->getDateDiff();
        
        if ( ($precision['type'] == 'week' || $precision['type'] == 'month') && $dateDiff['year'] == 0 && $dateDiff['day'] <= 1) {
            $precision['type'] = 'day';
        }
        
        $reportQuantity = $this->getReportQuantity($precision);
        if (!isset($this->_data['maxValueQuantity']) || ($reportQuantity < $this->_data['maxValueQuantity']) ) {
            return $precision;
        }
        
        $obj = $this;
        $getRatio = function ($precision, $maxValueQuantity) use ($obj) {
            $reportQuantity = $obj->getReportQuantity($precision);
            return ceil ((float) ( $reportQuantity / $maxValueQuantity));
        };

        $ratio = $getRatio($precision, $this->_data['maxValueQuantity']);
        switch (true) {
            case ($precision['type'] == 'hour' && $ratio < 24 ):
                $precision['value'] = $ratio;
                return $precision;
            case ($precision['type'] == 'hour' && $ratio > 24 ):
                $precision['type'] == 'day';
                $ratio = $getRatio($precision, $this->_data['maxValueQuantity']); 
            case ($precision['type'] == 'day' && $ratio < 7 ):
                $precision['value'] = $ratio;
                return $precision;
            case ($precision['type'] == 'day' && $ratio > 7 ):
                $precision['type'] == 'week';
                $ratio = $getRatio($precision, $this->_data['maxValueQuantity']);
            case ($precision['type'] == 'week' && $ratio < 5 ):
                $precision['value'] = $ratio;
                return $precision;
            case ($precision['type'] == 'week' && $ratio > 5 ):
                $precision['type'] == 'month';
                $ratio = $getRatio($precision, $this->_data['maxValueQuantity']);
            case ($precision['type'] == 'month' && $ratio < 12):
                $precision['value'] = $ratio;
                return $precision;
        }
    }        

    /**
     * Return db queries
     * 
     * @return array
     */
    protected function _getQueries()
    {
        $toDate   = $this->getDateTo(); 
        $queries = array();
        
        $precision = $this->_getPrecision();
        $this->_precision = $precision;
        
        $startDate = $this->getDateFrom();
        $i = 0;
        while ($toDate->get() >= $startDate->get()) {
            $endDate = $this->getIncrasedDate($this->_createDate($startDate), $precision);
            $queries[$i] = $this->_getRangeQueryPart($startDate, $endDate);
            $this->_report[$i]['dateRange'] = array('dateFrom' => $startDate, 'dateTo' => $endDate);
            $startDate = $endDate;
            $i++;
        }
        return $queries;
    }        
    
    /**
     * Return query for retriving report of full date range
     * 
     * @param Zend_Date $fromDate
     * @param Zend_Date $toDate
     * @return string
     */
    protected function _getQueryFull($fromDate, $toDate)
    {
        $model = $this->_getModel()->getAdapter();
        return $this->actionType.'  >='.$model->quote($fromDate->get($this->_dateFormat)).' AND '.$this->actionType.' <='.$model->quote($toDate->get($this->_dateFormat));
    }        

    /**
     *  Fetch qantity for date range
     * @param type $query
     * @return type
     */
    protected function _getQuantity($query)
    {
        $query = $this->_prepareQuery($query);
        return parent::_getQuantity($query);
    }
    
    /**
     * 
     * @param array $query
     * @return array
     */
    protected function _prepareQuery($query)
    {
        if (isset($this->responseType)&& !empty($this->responseType)) {
            return array($query, $this->responseType);
        }
        return array($query);
    }
    
    /**
     * Increases date to increment value.
     * 
     * @param Zend_Date $date
     * @param array $dateIncrement
     * @return Zend_Date
     */
    public function getIncrasedDate (Zend_Date $date , array $dateIncrement) 
    {
        
        switch ($dateIncrement['type']) {
            case 'hour':
                $endDate = $date->addHour($dateIncrement['value']);
                break;
            case 'day' :
                $endDate = $date->addDay($dateIncrement['value']);
                break;
            case 'week':
                $endDate = $date->addWeek($dateIncrement['value']);
                break;
            case 'month':
                $endDate = $date->addMonth($dateIncrement['value']);
                break;
        }
        return $date;
    }
    
    /**
     * Return difference between the start and end date
     * 
     * @return array
     */
    public function getDateDiff ()
    {
        if (!isset($this->_dateDiff) || empty($this->_dateDiff))
        {
            $year = $this->getDateFrom()->get('Y') - $this->getDateTo()->get('Y');
            $dateDiff = $this->getDateTo()->sub($this->getDateFrom());
            $day  = (int) $dateDiff->get('z');
            
            $this->_dateDiff = array (
                'day'  => $day,
                'year' => $year,
            );
        }
        return $this->_dateDiff;
    }  
}
