<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Lib
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Number-based captcha adapter
 *
 * Generates random word which user should recognise
 *
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Captcha
 * @copyright  Copyright (c) 2005-2011 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 * @version    $Id: Word.php 23775 2011-03-01 17:25:24Z ralph $
 */
abstract class System_Lib_Captcha_Number extends Zend_Captcha_Base
{
    /**
     * Random session ID
     *
     * @var string
     */
    protected $_id;

    /**
     * Generated first summand
     *
     * @var string
     */
    protected $_firstSummand;
    
    /**
     * Generated second summand
     *
     * @var string
     */
    protected $_secondSummand;

    /**
     * Generated equation
     *
     * @var string
     */
    protected $_equation;
    
    /**
     * Session
     *
     * @var Zend_Session_Namespace
     */
    protected $_session;

    /**
     * Class name for sessions
     *
     * @var string
     */
    protected $_sessionClass = 'Zend_Session_Namespace';

    /**
     * Session lifetime for the captcha data
     *
     * @var integer
     */
    protected $_timeout = 300;

    /**
     * Should generate() keep session or create a new one?
     *
     * @var boolean
     */
    protected $_keepSession = false;

    /**#@+
     * Error codes
     */
    const MISSING_VALUE = 'missingValue';
    const MISSING_ID    = 'missingID';
    const BAD_CAPTCHA   = 'badCaptcha';
    /**#@-*/

    /**
     * Error messages
     * @var array
     */
    protected $_messageTemplates = array(
        self::MISSING_VALUE => 'Empty captcha value',
        self::MISSING_ID    => 'Captcha ID field is missing',
        self::BAD_CAPTCHA   => 'Captcha value is wrong',
    );

    /**
     * Length of the first summand to generate
     *
     * @var integer
     */
    protected $_firstSummandlen = 9;
    
    /**
     * Length of the second summand to generate
     *
     * @var integer
     */
    protected $_secondSummandlen = 9;
    
    /**
     * Retrieve session class to utilize
     *
     * @return string
     */
    public function getSessionClass()
    {
        return $this->_sessionClass;
    }

    /**
     * Set session class for persistence
     *
     * @param  string $_sessionClass
     * @return Zend_Captcha_Word
     */
    public function setSessionClass($_sessionClass)
    {
        $this->_sessionClass = $_sessionClass;
        return $this;
    }

    /**
     * Retrieve word length to use when genrating captcha
     *
     * @return integer
     */
    public function getFirstSummandlen()
    {
        return $this->_firstSummandlen;
    }
    
    /**
     * Retrieve word length to use when genrating captcha
     *
     * @return integer
     */
    public function getSecondSummandlen()
    {
        return $this->_secondSummandlen;
    }

    /**
     * Set first summand length of captcha
     *
     * @param integer $firstSummandlen
     * @return Zend_Captcha_Word
     */
    public function setFirstSummandlen($firstSummandlen)
    {
        $this->_firstSummandlen = $firstSummandlen;
        return $this;
    }

    /**
     * Retrieve captcha ID
     *
     * @return string
     */
    public function getId ()
    {
        if (null === $this->_id) {
            $this->_setId($this->_generateRandomId());
        }
        return $this->_id;
    }

    /**
     * Set captcha identifier
     *
     * @param string $id
     * return Zend_Captcha_Word
     */
    protected function _setId ($id)
    {
        $this->_id = $id;
        return $this;
    }

    /**
     * Set timeout for session token
     *
     * @param  int $ttl
     * @return Zend_Captcha_Word
     */
    public function setTimeout($ttl)
    {
        $this->_timeout = (int) $ttl;
        return $this;
    }

    /**
     * Get session token timeout
     *
     * @return int
     */
    public function getTimeout()
    {
        return $this->_timeout;
    }

    /**
     * Sets if session should be preserved on generate()
     *
     * @param bool $keepSession Should session be kept on generate()?
     * @return Zend_Captcha_Word
     */
    public function setKeepSession($keepSession)
    {
        $this->_keepSession = $keepSession;
        return $this;
    }
    
    /**
     * Get session object
     *
     * @return Zend_Session_Namespace
     */
    public function getSession()
    {
        if (!isset($this->_session) || (null === $this->_session)) {
            $id = $this->getId();
            if (!class_exists($this->_sessionClass)) {
                Zend_Loader::loadClass($this->_sessionClass);
            }
            $this->_session = new $this->_sessionClass('Zend_Form_Captcha_' . $id);
            $this->_session->setExpirationHops(1, null, true);
            $this->_session->setExpirationSeconds($this->getTimeout());
        }
        return $this->_session;
    }

    /**
     * Set session namespace object
     *
     * @param  Zend_Session_Namespace $session
     * @return Zend_Captcha_Word
     */
    public function setSession(Zend_Session_Namespace $session)
    {
        $this->_session = $session;
        if($session) {
            $this->_keepSession = true;
        }
        return $this;
    }

    /**
     * Get first summand
     *
     * @return string
     */
    public function getFirstSummand()
    {
        if (empty($this->_firstSummand)) {
            $session     = $this->getSession();
            $this->_firstSummand = $session->firstSummand;
        }
        return $this->_firstSummand;
    }
    
    /**
     * Get second summand
     *
     * @return string
     */
    public function getSecondSummand()
    {
        if (empty($this->_secondSummand)) {
            $session     = $this->getSession();
            $this->_secondSummand = $session->secondSummand;
        }
        return $this->_secondSummand;
    }

    /**
     * Get equation
     *
     * @return string
     */
    public function getEquation()
    {
        if (empty($this->_equation)) {
            $session     = $this->getSession();
            $this->_equation = $session->equation;
        }
        return $this->_equation;
    }
    
    /**
     * Set captcha equation
     *
     * @param  string $equation
     * @return Systen_Lib_Captcha_Number
     */
    protected function _setEquation($equation)
    {
        $session       = $this->getSession();
        $session->equation = $equation;
        $session->firstSummand = $this->_firstSummand;
        $session->secondSummand = $this->_secondSummand;
        $this->_equation   = $equation;
        return $this;
    }

    /**
     * Generate new random word
     *
     * @return string
     */
    protected function _generateEquation()
    {
        $equation         = '';
        $firstSummandLen  = $this->getFirstSummandlen();
        $secondSummandLen = $this->getSecondSummandlen();
        
        $this->_firstSummand = rand(0, $firstSummandLen);
        $this->_secondSummand = rand(0, $secondSummandLen);

        return "{$this->_firstSummand} + {$this->_secondSummand}";
    }

    /**
     * Generate new session ID and new word
     *
     * @return string session ID
     */
    public function generate()
    {
        if(!$this->_keepSession) {
            $this->_session = null;
        }
        $id = $this->_generateRandomId();
        $this->_setId($id);
        $equation = $this->_generateEquation();
        $this->_setEquation($equation);
        return $id;
    }

    protected function _generateRandomId()
    {
        return md5(mt_rand(0, 1000) . microtime(true));
    }

    /**
     * Validate the word
     *
     * @see    Zend_Validate_Interface::isValid()
     * @param  mixed $value
     * @return boolean
     */
    public function isValid($value, $context = null)
    {
        if (!is_array($value) && !is_array($context)) {
            $this->_error(self::MISSING_VALUE);
            return false;
        }
        if (!is_array($value) && is_array($context)) {
            $value = $context;
        }

        $name = $this->getName();

        if (isset($value[$name])) {
            $value = $value[$name];
        }

        if (!isset($value['input'])) {
            $this->_error(self::MISSING_VALUE);
            return false;
        }
        $input = $value['input'];
        $this->_setValue($input);

        if (!isset($value['id'])) {
            $this->_error(self::MISSING_ID);
            return false;
        }

        $this->_id = $value['id'];
        $firstSummand = $this->getFirstSummand();
        $secondSummand = $this->getSecondSummand();

        if ((int)$input !== $firstSummand + $secondSummand) {
            $this->_error(self::BAD_CAPTCHA);
            return false;
        }

        return true;
    }

    /**
     * Get captcha decorator
     *
     * @return string
     */
    public function getDecorator()
    {
        return "Captcha_Word";
    }
}
