<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Zend_Controller_Plugin_Abstract
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class System_Lib_Controller_Plugin_CrashReport extends Zend_Controller_Plugin_Abstract
{
    const ERROR_EXCEPTION = 100000;
    /**
     * Information display
     */
    const DISPLAY_ON_DESCRIPTION = 1;
    const DISPLAY_ON_SYSTEM_INFO = 2;
    const DISPLAY_ON_PRODUCT     = 3;
    /**
     * Error parameters
     *
     * @var array
     */
    private static $_error = array(
        'level'   => '',
        'message' => '',
        'file'    => '',
        'line'    => '',
        'context' => '',
        'trace'   => '',
        'info'    => array()
    );
    private static $_sendReport = true;
    private static $_shutdownFunctionTouched = false;
    private static $_errorHandlerTouched = false;

    /**
     * Exception handler
     *
     * Catches throwed exceptions anywhere
     *
     * @static
     * @param Exception $exception
     * @param null|array $shutdown
     */
    public static function exceptionHandler (Exception $exception, $shutdown = array())
    {
        if (404 == $exception->getCode() || $exception instanceof Singular_Acl_Exception) self::setSendReport(false);

        $trace = $exception->getTrace();

        /* Is exception was thrown from error handler? */
        if (self::$_errorHandlerTouched) {
            $trace                 = $trace[0];
            $file                  = (isset($trace[0]['file'])) ? $trace[0]['file'] : $exception->getFile();
            $line                  = (isset($trace[0]['line'])) ? $trace[0]['line'] : $exception->getLine();
            self::$_error['level'] = $exception->getCode();

        /* Is exception was thrown from shutdown function? */
        } elseif (self::$_shutdownFunctionTouched) {
            $trace = (isset($trace[1])) ? $trace[1] : $trace[0];
            $file = (isset($shutdown['file'])) ? $shutdown['file'] : $trace['file'];
            $line = (isset($shutdown['line'])) ? $shutdown['line'] : $trace['line'];
            self::$_error['level'] = $exception->getCode();

        /* External exception */
        } else {
            self::$_error['level'] = self::ERROR_EXCEPTION;
            $file = $exception->getFile();
            $line = $exception->getLine();
        }
        self::$_error['message'] = $exception->getMessage();
        self::$_error['file']    = $file;
        self::$_error['line']    = $line;
        self::$_error['trace']   = $exception->getTrace();

        /* Detecting database exceptions */
        if ($exception instanceof Zend_Db_Exception) {
            $trace = $exception->getTrace();
            /** @var $select Zend_Db_Table_Select */
            $select = $trace[2]['args'][0]; unset($trace);
            if ($select instanceof Zend_Db_Table_Select) {
                self::_addInfo(
                    'Corrupted query:',
                    $select->assemble(),
                    self::DISPLAY_ON_DESCRIPTION
                );
            }
        }

        self::_compileError();
    }

    public static function setSendReport ($flag = true)
    {
        self::$_sendReport = (bool) $flag;
    }

    public static function getSendReport ()
    {
        return self::$_sendReport;
    }

    /**
     * Returns true if error level needs to be processed
     *
     * @static
     * @param  int $level
     * @return bool
     */
    protected static function _errorLevelIsAllowed ($level)
    {
        // do not process WARNINGS. ex: @fopen('file.txt');
        if (
            (E_WARNING == $level) ||
            (('development' != APPLICATION_ENV) &&
            (E_WARNING == $level ||
            E_CORE_WARNING == $level ||
            E_COMPILE_WARNING == $level ||
            E_USER_WARNING == $level ||
            E_NOTICE == $level ||
            E_USER_NOTICE == $level ||
            E_STRICT == $level)
            )
        ) {
            restore_error_handler();
            return false;
        }

        return true;
    }

    /**
     * Adds info block to specified displayOn section
     *
     * @static
     * @param string $title
     * @param string $text
     * @param int $displayOn
     */
    protected static function _addInfo ($title, $text, $displayOn = SELF::DISPLAY_ON_SYSTEM_INFO)
    {
        self::$_error['info'][$displayOn][] = array('title' => $title, 'text' => $text);
    }

    /**
     * Detects error type and return formatted string
     *
     * @static
     * @return string
     */
    protected static function _prepareError ()
    {
        $errno   = self::$_error['level'];
        $errstr  = self::$_error['message'];
        $errfile = self::$_error['file'];
        $errline = self::$_error['line'];
        $errcont = self::$_error['context'];

        $str = '';
        switch ($errno) {
            case E_ERROR:
            case E_CORE_ERROR:
            case E_COMPILE_ERROR:
            case E_PARSE:
            case E_USER_ERROR:
            case E_RECOVERABLE_ERROR:
                $str .= "<h2>FATAL ERROR ($errno) $errstr</h2>\n";
                $str .= "Fatal error on";
                break;
            case E_WARNING:
            case E_CORE_WARNING:
            case E_COMPILE_WARNING:
            case E_USER_WARNING:
                $str .= "<h2>WARNING ($errno) $errstr</h2>\n";
                $str .= "Warning on";
                break;
            case E_NOTICE:
            case E_USER_NOTICE:
                $str .= "<h2>NOTICE ($errno) $errstr</h2>\n";
                $str .= "Notice on";
                break;
            case E_STRICT:
                $str .= "<h2>DEBUG ERROR ($errno) $errstr</h2>\n";
                $str .= "Debug error on";
                break;
            case self::ERROR_EXCEPTION:
                $str .= "<h2>EXCEPTION: $errstr</h2>\n";
                $str .= "Exception on";
                break;
            default:
                $str .= "<h2>Unknown error type: ($errno) $errstr</h2>\n";
                $str .= "See";
                break;
        }

        $str .= " line <span style='border-radius:4px; padding:3px 5px; background-color:#0E52E1; color:#ffffff; font-weight:bold;'>$errline</span> in file <span style='border-radius:4px; padding:3px 5px; background-color:#5BCC5D; color:#ffffff; font-weight:bold;'>$errfile</span>";
        $str .= ", <strong>PHP " . PHP_VERSION . " (" . PHP_OS . ")</strong>\n";

        return $str;
    }

    /**
     * Compiles error
     *
     * Steps:
     * #1 - ddds info blocks
     * #2 - prepares error text
     * #3 - decorates errror with html
     *
     * @static
     * @return void
     */
    protected static function _compileError ()
    {
        /** @var $info Singular_Core_Info */
        $info = Singular_Core::_('Info');

        // front controller
        $front = Zend_Controller_Front::getInstance();

        /* PHP version */
        self::_addInfo(
            'PHP version:',
            PHP_VERSION . ' on ' . PHP_OS . ' runned as ' . php_sapi_name(),
            self::DISPLAY_ON_SYSTEM_INFO
        );

        /* Loaded extensions */
        self::_addInfo(
            'Loaded extensions:',
            implode(', ', get_loaded_extensions()),
            self::DISPLAY_ON_SYSTEM_INFO
        );

        /* MySQL server */
        self::_addInfo(
            'MySQL server:',
            'MySQL ' . $info->getParam('mysqlVersion') . ' via ' . $info->getParam('dbAdapter'),
            self::DISPLAY_ON_SYSTEM_INFO
        );

        /* Requested page */
        $addr = (0 === strpos($_SERVER['SERVER_PROTOCOL'], 'HTTP')) ? 'http://' : 'https://';
        $addr .= $_SERVER['SERVER_NAME'] . $_SERVER['REQUEST_URI'];
        $routeInfo  = '<table style="color:#6E6C6C; font-size:12px;"><tr><th style="text-align:left;">Module name:</th><td style="padding-left:10px;">' . $front->getRequest()->getModuleName() . "</td></tr>";
        $routeInfo .= '<tr><th style="text-align:left;">Controller name:</th><td style="padding-left:10px;">' . $front->getRequest()->getControllerName() . "</td></tr>";
        $routeInfo .= '<tr><th style="text-align:left;">Action name:</th><td style="padding-left:10px;">' . $front->getRequest()->getActionName() . "</td></tr>";
        $params = $front->getRequest()->getParams();

        // remove repeated parameters
        unset($params['module']); unset($params['controller']) ; unset($params['action']);

        $formattedParams = '';
        foreach ($params as $pn => $pv) {
            $formattedParams .= ($pn . ' = (' . gettype($pv) . ') ');
            if (is_object($pv)) {
                $formattedParams .= get_class($pv);
            } elseif (is_array($pv)) {
                $formattedParams .= count($pv);
            } elseif (is_string($pv)) {
                $formattedParams .= htmlentities($pv);
            } else {
                $formattedParams .= $pv;
            }
            if ($pv != end($params)) $formattedParams .= "\n<br />";
        }
        if (!empty($formattedParams)) {
            $routeInfo .= '<tr><th style="text-align:left; vertical-align:top;">Parameters:</th><td style="padding-left:10px;">' . $formattedParams . "</td></tr>";
        }
        $routeInfo .= '</table>';

        self::_addInfo(
            'Requested page:',
            '<a href="' . $addr . '">' . $addr . "</a>\n<br /><br />" . $routeInfo,
            self::DISPLAY_ON_DESCRIPTION
        );

        /* Final steps */
        self::_urInfo();
        $errorText = self::_prepareError();
        $decorated = self::_decorateError($errorText);

        $config = Singular_Runtime::extract('config');
        if ('production' == APPLICATION_ENV && $config->show_errors) {
            die($decorated);
        } elseif ('production' == APPLICATION_ENV && !$config->show_errors) {
            /* Detecting that request was making from localhost subnet */
            if (0 !== strpos($_SERVER['REMOTE_ADDR'], '127.0.0') && self::getSendReport()) {
                self::_sendReport($decorated);
            }
        } elseif ('development' == APPLICATION_ENV) {
            die($decorated);
        }

        /* Send error report to administrator */
        //self::_sendReport($errorText);

        /* Make it silent if possible */
        /*$silentCases = array(
            E_WARNING, E_CORE_WARNING, E_COMPILE_WARNING,
            E_USER_WARNING, E_NOTICE, E_USER_NOTICE, E_STRICT
        );
        if (in_array(self::$_error['level'], $silentCases)) {
        }*/

        /* Store exception message in session */
        $errNs = new Zend_Session_Namespace('error');
        $errNs->message = self::$_error['message'];

        /* Make redirect to erorr page */
        $errorUrl = Zend_Layout::getMvcInstance()->getView()->baseUrl('error');
        /** @var $redirector Zend_Controller_Action_Helper_Redirector */
        $redirector = Zend_Controller_Action_HelperBroker::getStaticHelper('Redirector');
        $redirector->gotoUrl($errorUrl);
    }

    /**
     * Collects information about UsResponse
     * and adds it to separate section
     *
     * @static
     * @return void
     */
    protected static function _urInfo ()
    {
        $module = Singular_Module::getInstance();
        $modules = $module->getInstalledModules();

        $product = $module->getModule('system')->getContainer();
        self::_addInfo(
            'Product version:',
            $product->getVersion() . ' released on ' . $product->getReleaseDate() . ' by ' . $product->getAuthor(),
            self::DISPLAY_ON_PRODUCT
        );
        unset($product);

        $installed = '<table style="color:#6E6C6C; font-size:12px;">';
        foreach ($modules as $mod) {
            $installed .= '<tr><th style="text-align:left;">' . $mod->getContainer()->getName() . '</th><td style="padding-left:10px;">v. '
                    . $mod->getContainer()->getVersion()
                    . ' on ' . $mod->getContainer()->getReleaseDate() . ' by '
                    . '<a href="' . $mod->getContainer()->getUrl() . '">' . $mod->getContainer()->getAuthor() . '</a>'
                    . "</td></tr>";
        }
        $installed .= '</table>';

        self::_addInfo(
            'Installed modules:',
            $installed,
            self::DISPLAY_ON_PRODUCT
        );
    }

    /**
     * Final step of error compilation
     *
     * @static
     * @param  string $error
     * @return string
     */
    protected static function _decorateError ($error)
    {
        $descriptiomInfo = self::_getDecoratedInfo(self::DISPLAY_ON_DESCRIPTION);
        $systemInfo      = self::_getDecoratedInfo(self::DISPLAY_ON_SYSTEM_INFO);
        $productInfo     = self::_getDecoratedInfo(self::DISPLAY_ON_PRODUCT);
        $title           = str_replace("\n", '', htmlentities(strip_tags($error)));
        $get     = Singular_Stdlib_Dump::dump($_GET, 'GET', true);
        $post    = Singular_Stdlib_Dump::dump($_POST, 'POST', true);
        $cookies = Singular_Stdlib_Dump::dump($_COOKIE, 'COOKIE', true);
        $session = Singular_Stdlib_Dump::dump($_SESSION, 'SESSION', true);
        $trace   = self::_formatTrace(self::$_error['trace']);
        $html = <<<ERROR
<html>
<head>
<title>{$title}</title>
<style type="text/css">
    * {white-space:pre-wrap; word-wrap:break-word;}
    a {text-decoration: underline;color: #494D4C;}
    pre {width:744px;}
</style>
</head>
<body>
<table cellpadding="10" style="width:998px; border:1px solid black; border-radius:4px; margin:0 auto; font-family:'Trebuchet MS',Arial,Helvetica,sans-serif;">
    <tr>
        <th colspan="2" style="border-bottom:1px solid #000000; border-radius:4px; background-color:#C20303; color:white; text-transform:uppercase; height:40px; text-align:center;">Error has occured!</th>
    </tr>
    <tr>
        <td style="color:#494D4C; background-color:#EEEEFF; border-bottom:1px solid #999999; vertical-align:top; font-size:17px; width:200px; font-weight:bold;">Error description:</td>
        <td style="border-bottom:1px solid #999999; color:#06318B; font-size:12px;">{$error}<br /><br />{$descriptiomInfo}</td>
    </tr>
    <tr>
        <td style="color:#494D4C; background-color:#EEEEFF; vertical-align:top; font-size:17px; width:200px; font-weight:bold;">Stack trace:</td>
        <td style="font-size:12px;">{$trace}</td>
    </tr>
    <tr>
        <td style="color:#494D4C; background-color:#EEEEFF; vertical-align:top; font-size:17px; width:200px; font-weight:bold;">System information:</td>
        <td style="font-size:12px;">{$systemInfo}</td>
    </tr>
    <tr>
        <td style="color:#494D4C; background-color:#EEEEFF; vertical-align:top; font-size:17px; width:200px; font-weight:bold;">UseResponse:</td>
        <td style="font-size:12px;">{$productInfo}</td>
    </tr>
    <tr>
        <td style="color:#494D4C; background-color:#EEEEFF; vertical-align:top; font-size:17px; width:200px; font-weight:bold;">Autoglobal variables:</td>
        <td style="font-size:12px;">{$get}<br />{$post}<br />{$cookies}<br />{$session}</td>
    </tr>
</table>
</body>
</html>
ERROR;
        return $html;
    }

    /**
     * Decorates section data with html
     *
     * @static
     * @param  array $type
     * @return array|null
     */
    protected static function _getDecoratedInfo ($type)
    {
        if (!isset(self::$_error['info'][$type])) {
            return null;
        }

        $out   = '';
        $info  = self::$_error['info'][$type];
        $count = count($info); $i = 1;
        foreach ($info as $item) {
            $out .= '<table style="width:100%; font-size:13px; color:#6e6c6c; border:1px solid #555555; border-radius:4px;"><tr><td style="background-color:#EEEEEE; border-bottom:1px solid #CCC9C9; border-radius:4px; padding:7px; text-align:left; color:#111111">';
            $out .= '<strong>' . $item['title'] . '</strong></td></tr>';
            $out .= '<tr><td style="padding:7px;">' . $item['text'] . '</td></tr></table>';
            if ($i < $count) $out .= '<br />';
            $i++;
        }

        return $out;
    }

    protected static function _formatTrace ($info)
    {
        $out = array();
        $out[] = '<style type="text/css">
                table.trace {
                        width : 100%;
                        border : solid 1px black;
                        font-size: 12px;
                        border-radius: 4px 4px 4px 4px;
                }
                table.trace thead td {
                        color: #ffffff;
                        font-weight: bold;
                        background : #636363;
                        border-radius: 4px;
                }
                table.trace tr.odd {
                        background : white;
                }
                table.trace tr.even {
                        background : #EEEEEE;
                }
                table.trace td {
                        padding: 7px;
                }
                table.trace td span {
                        border-bottom: 1px dotted #bab9b6;
                        cursor: alias;
                }
        </style>
        <table class="trace">
            <thead>
            <tr>
                <td>In file</td>
                <td>On line</td>
                <td>Called class</td>
                <td>Called function</td>
                <td>With arguments</td>
            </tr>
            </thead>
            <tbody>';
        foreach ($info as $i => $trace) {
            $out[] = '<tr class="' . ($i % 2 == 0 ? 'even' : 'odd') . '">';
            $out[] = '<td>' . ((isset($trace['file'])) ? basename($trace['file']) : '') . '</td>';
            $out[] = '<td>' . ((isset($trace['line'])) ? $trace['line'] : '') . '</td>';
            $out[] = '<td>' . ((isset($trace['class'])) ? $trace['class'] : '') . '</td>';
            $out[] = '<td>' . ((isset($trace['function'])) ? $trace['function'] : '') . '</td>';
            $out[] = '<td>';
            if (isset($trace['args'])) {
                foreach ($trace['args'] as $i => $arg) {
                    $out[] = '<span title="' . /*. var_export($arg, true) .*/ '">' . gettype($arg) . '</span>';
                    $out[] = (($i < count($trace['args']) - 1) ? ', ' : '');
                }
            } else {
                $out[] = 'NULL';
            }
            $out[] = '</td>';
            $out[] = '</tr>';
        }
        $out[] = '</tbody>';
        $out[] = '</table>';

        return implode('', $out);
    }

    protected static function _sendReport ($report)
    {
        $users = new System_Model_DbTable_Users();
        $admins = $users->findByRoles('administrator');
        /** @var $admin System_Model_User) */
        $admin = $admins[0]; unset($admins);

        /* send report */
        Resources_Service_ApplicationMailer::sendEmail(
            $admin->email,
            'Crash report from ' . Singular_Core::_('Settings')->getSetting('site_title'),
            $report
        );
    }

    /**
     * Called before Zend_Controller_Front begins evaluating the
     * request against its routes.
     *
     * @param Zend_Controller_Request_Abstract $request
     * @return void
     */
    public function routeStartup (Zend_Controller_Request_Abstract $request)
    {
        if (defined('CRASH_REPORT_DISABLE') && CRASH_REPORT_DISABLE) return;
        /* Do not handle if xdebug extension is loaded */
        if (extension_loaded('xdebug')) return;
        if (version_compare(PHP_VERSION, '5.2', '<')) ini_set('track_errors', 1);
        set_error_handler(array($this, 'errorHandler'));
        Zend_Controller_Front::getInstance()
                ->unregisterPlugin('Zend_Controller_Plugin_ErrorHandler')
                ->throwExceptions(true);
        set_exception_handler(array(__CLASS__, 'exceptionHandler'));
        register_shutdown_function(array($this, 'shutdownFunction'));
    }

    /**
     * Shutdown function handler
     *
     * @return mixed
     */
    public function shutdownFunction ()
    {
        if (function_exists('error_get_last')) {
            $error = error_get_last();
        } elseif (isset($php_errormsg)) {
            $error = $php_errormsg;
        } else {
            return;
        }

        // Is error level needs to be processed?
        if (!self::_errorLevelIsAllowed($error['type'])) return;

        if (empty($error)) return;

        $shutdownParams = (isset($error['file']) && isset($error['line']))
                ? array(
                    'file' => $error['file'],
                    'line' => $error['line']
                )
                : null;

        self::$_shutdownFunctionTouched = true;

        /* Exception handler HOT calling */
        self::exceptionHandler(
            new Exception(
                (isset($error['message'])) ? $error['message'] : $error,
                (isset($error['type'])) ? $error['type'] : 0
            ),
            $shutdownParams
        );
    }

    /**
     * Error handler
     *
     * Globally processes errors and throw Exceptions
     * In another words converts errors to exceptions
     *
     * @param  int $error_level
     * @param  string $error_message
     * @param  string $error_file
     * @param  int $error_line
     * @param  array|null $error_context
     * @throws Exception
     */
    public function errorHandler ($error_level, $error_message, $error_file, $error_line, $error_context)
    {
        // Is error level needs to be processed?
        if (!self::_errorLevelIsAllowed($error_level)) return;

        self::$_errorHandlerTouched = true;
        self::$_error['context'] = $error_context;
        $exception = new Exception($error_message, $error_level);
        throw $exception;
    }
}
