<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Form_Element
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * System_Lib_Form_Element_Sortable
 *
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Form_Element
 */
class System_Lib_Form_Element_Sortable extends Zend_Form_Element_Multi
{
    /**
     * Sortable stack
     *
     * @var ArrayObject
     */
    protected $_sortableItems;

    /**
     * Initialize object; used by extending classes
     *
     * @return void
     */
    public function init()
    {
        $this->getView()->headScript()->appendFile(
            $this->getView()->javascriptSrc('jquery-ui-sortable.min.js', null, false)
        );
        if (!$this->_sortableItems instanceof ArrayObject) {
            $this->_initEmptyList();
        }
        $this->setIsArray(true);
    }

    /**
     * Initializes an empty sortable list
     */
    protected function _initEmptyList()
    {
        $this->_sortableItems = new ArrayObject();
    }

    /**
     * Sets sortable items
     *
     * @example:
     * array(
     *     'ideas'     => array('title' => 'Ideas', 'icon' => '<path>/ideas.png'),
     *     'questions' => array('title' => 'Questions', 'icon' => '<path>/questions.png'),
     *     'problems'  => array('title' => 'Problems', 'icon' => '<path>/problems.png'),
     * )
     * @param  array $items
     *
     * @return System_Lib_Form_Element_Sortable
     * @throws Singular_Form_Element_Exception
     */
    public function setSortable(array $items)
    {
        $this->_initEmptyList();
        foreach ($items as $alias => $data) {
            if (!is_numeric($alias) || !is_array($data)) {
                throw new Singular_Form_Element_Exception(
                    'Invalid sortable format'
                );
            }
            $this->addSortable($alias, $data);
        }
        return $this;
    }

    /**
     * Adds new sortable
     *
     * @param string $alias
     * @param array  $data
     *
     * @return       Singular_Form_Element_Exception
     * @throws       Singular_Form_Element_Exception
     */
    public function addSortable($alias, array $data)
    {
        if (!isset($data['title'])) {
            throw new Singular_Form_Element_Exception(
                "Attribute 'title' is missing for sortable '{$alias}'"
            );
        }
        $data = array(
            'title' => $data['title'],
            'icon'  => (isset($data['icon'])) ? $data['icon'] : null
        );
        $this->_sortableItems->offsetSet($alias, $data);
        return $this;
    }

    /**
     * Returns sortable by given alias
     *
     * @param string $alias
     *
     * @return mixed|null
     */
    public function getSortable($alias)
    {
        if (!$this->_sortableItems->offsetExists($alias)) return null;
        return $this->_sortableItems->offsetGet($alias);
    }

    /**
     * Returns all sortable items
     *
     * @param  bool  $natsort
     * @return array
     */
    public function getAll($natsort = false)
    {
        if ($natsort) {
            $this->_sortableItems->natcasesort();
        }
        return $this->_sortableItems->getArrayCopy();
    }

    /**
     * Returns sortable items number
     *
     * @return int
     */
    public function count()
    {
        return $this->_sortableItems->count();
    }

    /**
     * Returns true if sortable list is an empty
     *
     * @return bool
     */
    public function isEmpty()
    {
        return (0 == $this->count());
    }

    /**
     * Returns sortable storage object
     *
     * @return ArrayObject
     */
    public function getStorage()
    {
        return $this->_sortableItems;
    }

    /**
     * Is the value provided valid?
     *
     * Autoregisters InArray validator if necessary.
     *
     * @param  string $value
     * @param  mixed $context
     * @return bool
     */
    public function isValid($value, $context = null)
    {
        $sortable = $this->getAll();
        if (is_array($value)) {
            foreach ($value as $val) {
                if (array_key_exists($val, $sortable)) {
                    $this->addMultiOption($val);
                }
            }
        }

        if ($this->registerInArrayValidator()) {
            if (!$this->getValidator('InArray')) {
                $multiOptions = $this->getMultiOptions();
                $options      = array();

                foreach ($multiOptions as $opt_value => $opt_label) {
                    // optgroup instead of option label
                    if (is_array($opt_label)) {
                        $options = array_merge($options, array_keys($opt_label));
                    }
                    else {
                        $options[] = $opt_value;
                    }
                }

                $this->addValidator(
                    'InArray',
                    true,
                    array($options)
                );
            }
        }
        return parent::isValid($value, $context);
    }
}
