<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * privileges table model
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 */
class System_Model_DbTable_Responses extends System_Model_DbTable_Abstract
{
    /**
     * Default fetch limit
     */
    const LIMIT = 10;

    /**
     * Type 'any' means all of response types
     */
    const RESPONSE_TYPE_ANY = 'any';

    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'responses';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'id';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass = 'System_Model_Response';

    /**
     * Dependent Tables
     *
     * @var array
     */
    protected $_dependentTables = array(
        'System_Model_DbTable_ResponsesAnswers',
        'System_Model_DbTable_ResponsesSubscribers',
        'Resources_Model_DbTable_ResponsesComments',
    );

    /**
     * Reference map
     *
     * @var array
     */
    protected $_referenceMap    = array(
        'Status' => array(
            'columns'           => array('status'),
            'refTableClass'     => 'System_Model_DbTable_ResponsesStatuses',
            'refColumns'        => array('slug'),
            'onDelete'          => self::CASCADE,
        ),
        'CreatedBy' => array(
            'columns'           => array('created_by_id'),
            'refTableClass'     => 'System_Model_DbTable_Users',
            'refColumns'        => array('id'),
            'onDelete'          => self::CASCADE,
        )
    );

    protected $_typesArray = null;

    /**
     * Inserts a new response row.
     *
     * @see    parent::insert()
     * @param  array  $data  Column-value pairs.
     * @return mixed         The primary key of the row inserted.
     */
    public function insert(array $data)
    {
        $data = $this->_setDate($data, 'created_at');
        $data = $this->_setActor($data, 'created_by_id');

        $text = $data["title"] . " " . $data["content"];
        $text = Singular_Loader::get("System_Service_Morphy")->getBaseForm($text);

        $data['search_index'] = $text;

        return parent::insert($data);
    }

    /**
     * Updates existing response rows.
     *
     * @param  array        $data  Column-value pairs.
     * @param  array|string $where An SQL WHERE clause, or an array of SQL WHERE clauses.
     * @return int          The number of rows updated.
     */
    public function update(array $data, $where)
    {
        $data = $this->_setDate($data, 'updated_at');
        $data = $this->_setActor($data, 'updated_by_id');
        parent::update($data, $where);
    }

    /**
     * Returns response row by id
     *
     * @param  int $id
     * @return System_Model_Response|null
     */
    public function findById($id)
    {        
        $select = $this->select();
        $select->where('id=?', $id);
        $response = $this->fetchRow($select);
        if ($response instanceof System_Model_Response) {
            $loggedUser = Singular_Runtime::extract('loggedUser');
            if ($loggedUser->canSeeResponse($response)) {
                return $response;
            } else {
                return false;
            }
        }
        return null;
    }

    /**
     * Returns responses by id's
     *
     * @param  array $ids
     * @return Zend_Db_Table_Rowset
     */
    public function findByIds($ids)
    {
        $select = $this->select();
        $select->where('id IN (?)', $ids);
        $responses = $this->fetchAll($select);
        if ($responses->valid()) {
            $loggedUser = Singular_Runtime::extract('loggedUser');
            $data = array();
            foreach ($responses as $response) {
                if ($loggedUser->canSeeResponse($response)) {
                    $data[] = $response->toArray();
                }
            }
            $responses = new System_Model_Rowset(array(
                'table'    => $this,
                'rowClass' => 'System_Model_Response',
                'readOnly' => $select->isReadOnly(),
                'data'     => $data,
                'stored'   => true
            ));
        }
        return $responses;
    }

    /**
     * Returns response by given slug
     *
     * @param  string $slug
     * @return System_Model_Response
     */
    public function findBySlug($slug)
    {
        $select = $this->select();
        $select->where('slug=?', $slug);
        $response = $this->fetchRow($select);
        if ($response instanceof System_Model_Response) {
            $loggedUser = Singular_Runtime::extract('loggedUser');
            if ($loggedUser->canSeeResponse($response)) {
                return $response;
            } else {
                return false;
            }
        }
        return null;
    }

    /**
     * Returns top responses
     *
     * @param  string $type
     * @param  int $limit
     * @return Zend_Db_Table_Rowset
     */
    public function findTop($type, $limit = 5)
    {
        $select = $this->getSelect();
        $topResponses = Singular_Core::_("settings")->getSetting("top_responses");

        if (
            'divided_by_type' == Singular_Core::_("settings")
                ->getSetting("homepage_responses_display_variant")
        ) {
            $limit = Singular_Core::_("settings")->getSetting("homepage_responses_items_num");
        }
        
        $where = array(
            'type'   => $type,
            'status' => $topResponses,
        );

        if ($type == self::RESPONSE_TYPE_ANY) {
            unset($where['type']);
        }

        $this->prepareSelect($select, $where);
        $select->where("{$this->getTableName()}.order > 0");
        $select->limit($limit);
        $select->reset('order');
        $select->order("{$this->getTableName()}.order DESC");
        
        switch ($topResponses) {
            case 'popular':
                $select->order("vote DESC");
                break;
            case 'new':
                $select->order("{$this->getTableName()}.created_at DESC");
                break;
            case 'updated':
                $select->order("updated DESC");
                break;
        }
        return $this->fetchAll($select);
    }
    
    /**
     * Returns responses by given criteria
     *
     * @param  array|null           $where
     * @param  string|array         $order
     * @param  int                  $limit
     * @param  Zend_Db_Table_Select $select
     *
     * @return Zend_Db_Table_Rowset
     */
    public function findAll($where = null, $order = null, $limit = null, $select = null)
    {
        if (!$select instanceof Zend_Db_Table_Select) {
            $select = $this->getSelect();
        }

        $tableName = $this->getTableName();
        if (is_array($where) && isset($where['status']) && $where['status'] == 'new') {
            $select->where("$tableName.created_at > ?", date("Y-m-d H:i:s", strtotime("-7 day last Monday")));
        }
        if ($order) {
            $select->order($order);
        } else {
            $select->order("created_at DESC");
        }
        if ($limit) {
            $select->limit($limit);
        }

        return $this->fetchAll($select);
    }

    /**
     * Prepares select for responses fetching
     *
     * @param  Zend_Db_Select $select
     * @param  array $where
     * @param  mixed $order
     * @return Zend_Db_Select
     * @throws Zend_Db_Exception
     */
    public function prepareSelect($select, $where, $order = null)
    {
        if (!($select instanceof Zend_Db_Select)) {
            throw new Zend_Db_Exception('$select must be instance of Zend_Db_Select');
        }
        $tableName = $this->getTableName();

        /* date */
        if (!empty($where['start_date'])) {
            $select->where("$tableName.created_at >= ?", date("Y-m-d H:i:s", strtotime($where['start_date'])));
        }
        if (!empty($where['end_date'])) {
            $select->where("$tableName.created_at <= ?", date("Y-m-d H:i:s", strtotime($where['end_date'] . "24:00:00")));
        }
        if (!empty($where['date'])) {
            switch ($where['date']) {
                case 'today' :
                    $select->where("$tableName.created_at > ?",
                        date("Y-m-d H:i:s", strtotime("-1 day")));
                    break;
                case 'yesterday' :
                    $select->where("$tableName.created_at > ?",
                        date("Y-m-d H:i:s", strtotime("-2 day")));
                    $select->where("$tableName.created_at < ?",
                        date("Y-m-d H:i:s", strtotime("-1 day")));
                    break;
                case 'this_week' :
                    $select->where("$tableName.created_at > ?",
                        date("Y-m-d H:i:s", strtotime("last Monday")));
                    break;
                case 'last_week' :
                    $select->where("$tableName.created_at > ?",
                        date("Y-m-d H:i:s", strtotime("-7 day last Monday")));
                    $select->where("$tableName.created_at < ?",
                        date("Y-m-d H:i:s", strtotime("last Monday")));
                    break;
                case 'this_month' :
                    $select->where("$tableName.created_at > ?",
                        date("Y-m-d H:i:s", mktime("00", "00", "00", date('m'), "01", date('Y'))));
                    break;
                case 'last_month' :
                    $time = strtotime("last month");
                    $select->where("$tableName.created_at > ?",
                        date("Y-m-d H:i:s", mktime("00", "00", "00", date('m', $time), "01", date('Y', $time))));
                    $select->where("$tableName.created_at < ?",
                        date("Y-m-d H:i:s", mktime("00", "00", "00", date('m'), "01", date('Y'))));
                    break;
            }
        }
        if (!empty($where['created_by_id'])) {
            $select->where("$tableName.created_by_id=?", $where['created_by_id']);
        }
        if (!empty($where['type']) && $where['type'] != 'all') {
            $select->where("type=?", $where['type']);
        }

        /* status */
        if (!empty($where['status'])) {
            if ($where['status'] == 'all') {
                $where['status'] = 'popular';
            }
            switch ($where['status']) {
                case 'has_answer':
                    $answersTableName = Singular_Loader::get('System_Model_DbTable_ResponsesAnswers')->getTableName();
                    $select->joinInner(array('a' => $answersTableName),
                        "$tableName.id=a.response_id",
                        array()
                    );
                    break;
                case 'new':
                    if (empty($order)) {
                        $order['order'] = 'created_at';
                        $order['sort'] = 0;
                    }
                    break;
                case 'updated':
                    $responsesAnswersDbName = Singular_Loader::get('System_Model_DbTable_ResponsesAnswers')->getTableName();
                    $selectResponsesAnswers = Singular_Loader::get('System_Model_DbTable_ResponsesAnswers')->getSelect();
                    $selectResponsesAnswers->reset("columns")
                                           ->reset("order")
                                           ->columns(array("$responsesAnswersDbName.response_id", "$responsesAnswersDbName.created_at AS answer_added"))
                                           ->order("$responsesAnswersDbName.created_at");
                    
                    $commentsDbName = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments')->getTableName();
                    $selectComments = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments')->getSelect();
                    $selectComments->reset("columns")
                                   ->reset("order")
                                   ->columns(array("$commentsDbName.response_id", "MAX($commentsDbName.created_at) AS last_comment_added"))
                                   ->group("$commentsDbName.response_id");
                    
                    $select->joinLeft(array("a_tmp" => $selectResponsesAnswers),
                                    "a_tmp.response_id=$tableName.id",
                                    array()
                            );
                    $select->joinLeft(array("c" => $selectComments),
                                    "c.response_id=$tableName.id",
                                    array()
                            );
                    $select->columns(array(
                        new Zend_Db_Expr("CASE WHEN last_comment_added>answer_added OR answer_added IS NULL THEN last_comment_added ELSE answer_added END AS updated")));
                    $order = array();
                    $order['order'] = 'updated';
                   
                    break;
                case 'popular':
                    $votesTableName = Singular_Loader::get('Resources_Model_DbTable_Votes')
                        ->getTableName();
                    $select->setIntegrityCheck(false);
                    $select->joinLeft(array('v' => $votesTableName),
                        "$tableName.id=v.object_id",
                        array(new Zend_Db_Expr('SUM(ABS(v.value)) AS vote'))
                    );
                    $select->where("$tableName.type=v.object");
                    
                    $group = $select->getPart("group");
                    if ( !in_array("$tableName.id", $group) ) {
                        $select->group("$tableName.id");
                    };

                    if (!isset($order) || !isset($order['order'])
                        || !$order['order']
                    ) {
                        $order['order'] = 'votes';
                        $order['sort'] = 0;
                    }
                    break;
                default:
                    $select->where("status=?", $where['status']);
                    break;
            }
        }

        /* without comments */
        if (!empty($where['without_comments']) && $where['without_comments']) {
            $cTableName = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments')
                ->getTableName();
            if (!array_key_exists('c', $select->getPart('from'))) {    
                $select->joinLeft(array('c' => $cTableName),
                    "$tableName.id=c.response_id",
                    array()
                );
            }
            $select->where("c.id IS NULL");
        }

        /* no official answer */
        if (isset($where['no_official_answer']) && !empty($where['no_official_answer'])) {
            $aTableName = Singular_Loader::get('System_Model_DbTable_ResponsesAnswers')
                ->getTableName();
            $select->joinLeft(array('a' => $aTableName),
                "$tableName.id=a.response_id",
                array()
            );
            $select->where("a.id IS NULL");
        }

        /* pay attention */
        if (isset($where['pay_attention']) && $where['pay_attention']
            || isset($order['order']) && $order['order'] == 'pay_attention'
        ) {
            /* statuses */
            $statuses = Singular_Runtime::extract("statuses");
            $slugs = array();
            $sql = "";
            foreach ($statuses as $status) {
                if ($status->pay_attention) {
                    $slugs[] = $status->slug;
                }
            }
            if (!empty($slugs)) {
                $sql = $this->getAdapter()
                    ->quoteInto("OR $tableName.status IN (?)", $slugs);
            }

            /* answer */
            if (!isset($where['no_official_answer']) || (isset($where['no_official_answer']) && empty($where['no_official_answer']))) {
                $aTableName = Singular_Loader::get('System_Model_DbTable_ResponsesAnswers')
                    ->getTableName();
                $select->joinLeft(array('a' => $aTableName),
                    "$tableName.id=a.response_id",
                    array()
                );
            }

            /* last comment */
            $commentsDbName = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments')->getTableName();
            $uTableName = Singular_Loader::get('System_Model_DbTable_Users')->getTableName();
            $selectComments = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments')->getSelect();
            $selectComments->reset("columns")
                           ->setIntegrityCheck(false)
                           ->reset("order")
                           ->columns(array("$commentsDbName.response_id", "MAX($commentsDbName.created_at) AS last_comment_added"))
                           ->group("$commentsDbName.response_id");
            
            $tmpSelect = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments')->getSelect();
            $tmpSelect->reset("columns")
                      ->setIntegrityCheck(false)
                      ->reset("order")
                      ->columns(array("$commentsDbName.response_id"))
                      ->joinInner(array("tmp" => $selectComments),
                                 "$commentsDbName.response_id=tmp.response_id AND $commentsDbName.created_at=tmp.last_comment_added",
                                 array());
            $tmpSelect->joinLeft(array("u" => $uTableName),
                        "{$commentsDbName}.created_by_id=u.id",
                        array("u.role"));

            /** @var $rolesDb System_Model_DbTable_Roles */
            $rolesDb = Singular_Loader::get('System_Model_DbTable_Roles');
            $roles = $rolesDb->findRolesSlugsByResourcesDeny(array(
                'object:can_access_admin', 'object:representative'
            ));

            $select->joinLeft(array("tmp" => $tmpSelect),
                    "tmp.response_id={$tableName}.id",
                    array());

            $select->columns(array(
                'pay_attention' => "CASE WHEN tmp.role IN (" . $this->getAdapter()->quote($roles) .  ") OR a.id IS NULL $sql THEN 1 ELSE 0 END"
            ));
            
            if (isset($where['pay_attention']) && $where['pay_attention']) {
                $select->having("pay_attention=1");
            } else {
                $select->reset("order");
                $select->order("pay_attention DESC");
            }
        }

        /* search text */
        if (!empty($where['text'])) {
            $words = Singular_Loader::get("System_Service_Morphy")
                ->getAllForm($where['text']);
            if (empty($words)) {
                $words = $where['text'];
            }
            $fulltextFilter = new System_Filter_FulltextSearch();
            $words = $fulltextFilter->filter($words);
            $from = $select->getPart('from');
            if (!isset($from['a'])) {
                $rAnswerTableName = Singular_Loader::get('System_Model_DbTable_ResponsesAnswers')
                    ->getTableName();
                $select->joinLeft(
                    array('a' => $rAnswerTableName),
                    "{$this->getTableName()}.id=a.response_id",
                    array()
                );
            }

            $select->columns(array(
                "MATCH ({$this->getTableName()}.title, {$this->getTableName()}.content, {$this->getTableName()}.search_index, {$this->getTableName()}.tags) AGAINST ({$this->getAdapter()->quote($words)}) + MATCH(a.answer) AGAINST ({$this->getAdapter()->quote($words)}) AS rel"
            ))
                ->where("MATCH ({$this->getTableName()}.title, {$this->getTableName()}.content, {$this->getTableName()}.search_index, {$this->getTableName()}.tags) AGAINST ({$this->getAdapter()->quote($words)}) > 0 OR MATCH(a.answer) AGAINST ({$this->getAdapter()->quote($words)}) > 0")
                ->reset('order')
                ->order('rel DESC');
            //$order = null;

        }
        if (isset($where['user_filter'])) {
            switch ($where['user_filter']['type']) {
                case 'all':
                    $subscribersTableName = Singular_Loader::get('System_Model_DbTable_ResponsesSubscribers')
                        ->getTableName();
                    $select->joinLeft(
                        array("s" => $subscribersTableName),
                        "$tableName.id=s.response_id",
                        array()
                    );
                    $commentTableName = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments')
                        ->getTableName();
                    $select->joinLeft(
                        array("c" => $commentTableName),
                        "$tableName.id=c.response_id",
                        array()
                    );
                    $votesTableName = Singular_Loader::get('Resources_Model_DbTable_Votes')
                        ->getTableName();
                    $select->joinLeft(
                        array("v" => $votesTableName),
                        "$tableName.id=v.object_id AND $tableName.type=v.object",
                        array()
                    );
                    $group = $select->getPart("group");
                    if ( !in_array("$tableName.id", $group) ) {
                        $select->group("$tableName.id");
                    };
                    $select->where("s.user_id='{$where['user_filter']['user_id']}' OR $tableName.created_by_id={$where['user_filter']['user_id']} OR c.created_by_id={$where['user_filter']['user_id']} OR (v.created_by_id={$where['user_filter']['user_id']} AND $tableName.created_by_id<>{$where['user_filter']['user_id']})");
                    break;
                case 'created':
                    $select->where("$tableName.created_by_id=?", $where['user_filter']['user_id']);
                    break;
                case 'following':
                    $subscribersTableName = Singular_Loader::get('System_Model_DbTable_ResponsesSubscribers')
                        ->getTableName();
                    $select->joinLeft(
                        array("s" => $subscribersTableName),
                        "$tableName.id=s.response_id",
                        array()
                    );
                    $group = $select->getPart("group");
                    if ( !in_array("$tableName.id", $group) ) {
                        $select->group("$tableName.id");
                    };
                    $select->where("s.user_id=?", $where['user_filter']['user_id']);
                    break;
                case 'commented':
                    $commentTableName = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments')
                        ->getTableName();
                    $select->joinLeft(
                        array("c" => $commentTableName),
                        "$tableName.id=c.response_id",
                        array()
                    );
                    $select->where("c.created_by_id=?", $where['user_filter']['user_id']);
                    $group = $select->getPart("group");
                    if ( !in_array("$tableName.id", $group) ) {
                        $select->group("$tableName.id");
                    };
                    break;
                case 'voted':
                    $votesTableName = Singular_Loader::get('Resources_Model_DbTable_Votes')
                        ->getTableName();
                    $select->joinLeft(
                        array("v" => $votesTableName),
                        "$tableName.id=v.object_id AND $tableName.type=v.object",
                        array()
                    );
                    $select->where("v.created_by_id=?", $where['user_filter']['user_id']);
                    $select->where("$tableName.created_by_id<>?", $where['user_filter']['user_id']);
                    break;
            }
        }

        if (isset($order)) {
            if (is_array($order)) {
                if (isset($order["order"]) && $order["order"]) {
                    if (isset($order["sort"]) && $order['sort']) {
                        $sort = "ASC";
                    } else {
                        $sort = "DESC";
                    }
                    switch ($order["order"]) {
                        case 'votes':
                            $select->reset("order");
                            if (!isset($where['status'])
                                || $where['status'] != 'popular'
                            ) {
                                $votesTableName = Singular_Loader::get('Resources_Model_DbTable_Votes')
                                    ->getTableName();
                                $select->setIntegrityCheck(false);
                                $select->joinLeft(array('v' => $votesTableName),
                                    "$tableName.id=v.object_id",
                                    array(new Zend_Db_Expr('SUM(v.value) AS vote'))
                                );
                                $select->where("$tableName.type=v.object");
                                $group = $select->getPart("group");
                                if ( !in_array("$tableName.id", $group) ) {
                                    $select->group("$tableName.id");
                                };
                            }
                            $select->order("vote $sort");
                            break;
                        case 'updated':
                            $select->reset("order");
                            $select->order("updated DESC");
                            break;
                        default :
                            if (in_array($order["order"], $this->getCols())) {
                                $select->reset("order");
                                $select->order("$tableName.{$order["order"]} $sort");
                            }
                            break;
                    }
                }
            } else {
                $select->order($order);
            }
        }

        Singular_Event::dispatch('afterPrepareSelect', $select);
        return $select;
    }

    /**
     * Return fetched paginator rows.
     *
     * @param  array $where
     * @param  int $page
     * @param  mixed $order
     * @param  int $limit
     * @return Zend_Db_Table_Rowset|null
     */
    public function getPaginator($where, $page, $order = null, $limit = null)
    {
        if (!$where instanceof Zend_Db_Select) {
            $select = $this->getSelect();
            $select->distinct(true);
            $this->prepareSelect($select, $where, $order);
        } else {
            $select = $where;
        }
        if (empty($limit)) {
            $limit = self::LIMIT;
        }

        return $this->getPaginatorRows($select, null, null, $page, $limit);
    }

    /**
     * Returns last user's response
     *
     * @param  int $userId
     * @return System_Model_Response|null
     */
    public function findLastByUserId($userId)
    {
        $select = $this->getSelect();
        $select->where("{$this->getTableName()}.created_by_id=?", $userId);
        $select->order("{$this->getTableName()}.created_at DESC");
        $select->limit(1);

        return $this->fetchRow($select);
    }

    /**
     * Counts how many responses has posted by given user id
     *
     * @param  int $userId
     * @return int
     */
    public function countByUserId($userId)
    {
        return $this->count(array('created_by_id' => $userId));
    }

    /**
     * Counts responses by given criteria
     *
     * @param  array $where
     * @return int
     */
    public function count($where = null)
    {
        $select = $this->getSelect();
        if (is_array($where)) {
            foreach ($where as $cond => $value) {
                if ($value !== null) {
                    $select->where($cond . '=?', $value);
                }
            }
        } else if (!empty($where) && is_string($where)) {
            $select->where($where);
        }
        $select->reset('columns')
               ->reset('order')
               ->reset('group');
        
        $select->columns(array('count' => new Zend_Db_Expr("COUNT(DISTINCT({$this->getTableName()}.id))")));
        Singular_Event::dispatch('responseCountAfter', $select);
        $result = $this->fetchRow($select);

        return $result['count'];
    }

    /**
     * Returns prepared select object
     *
     * @see    parent::getSelect()
     * @return Zend_Db_Table_Select
     */
    public function getSelect()
    {
        $select = parent::getSelect();
        if (System_Service_Environment::getInstance()->isAdmin()) {
            $select->where("{$this->getTableName()}.deleted_by_id <> 0");
        } else {
            $select->where("{$this->getTableName()}.deleted_by_id=?", 0);
        }
        
        if ($this->_typesArray === null) {
            $types = Singular_Runtime::extract('types');
            if (null != $types) $this->_typesArray = $types->findValuesBySlug();
        }
        $select->where("{$this->getTableName()}.type IN (?)", $this->_typesArray);
        
        $select->order("{$this->getTableName()}.created_at DESC");

        return $select;
    }

    /**
     * Counts responses (by one selected type) by given criteria
     *
     * @param  string|array $where
     * @return int
     */
    public function countByType($where = null, $type = null)
    {
        $select = $this->getSelect();
        $select->reset('where');
        if(!empty($type)) {
            $select->where('type=?', $type);
        } else {
            if ($this->_typesArray === null) {
                $types = Singular_Runtime::extract('types');
                $this->_typesArray = $types->findValuesBySlug();
            }
            $select->where("{$this->getTableName()}.type IN (?)", $this->_typesArray);

            $select->order("{$this->getTableName()}.created_at DESC");
        }
        if(!empty($where)) {
            $select->where($where);
        }        
        $select->reset('columns')
               ->reset('order')
               ->reset('group');
        
        $select->columns(array('count' => new Zend_Db_Expr("COUNT(DISTINCT({$this->getTableName()}.id))")));
        $result = $this->fetchRow($select);

        return $result['count'];
    }
    
    /**
     * Summ votes of responses by given criteria
     *
     * @param  string|array $where
     * @return int
     */
    public function sumViews($where)
    {
        $select = $this->getSelect();
        $select->reset('where');
        
        if(!empty($where)) {
            $select->where($where);
        }        
        $select->reset('columns')
               ->reset('order')
               ->reset('group');
        
        $select->columns(array('sum' => new Zend_Db_Expr("SUM(views)")));
        $result = $this->fetchRow($select);

        return $result['sum'];
    }       
}
