<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * subscribers table model
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 */
class System_Model_DbTable_ResponsesSubscribers extends System_Model_DbTable_Abstract
{
    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'subscribers';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'id';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass = 'System_Model_ResponseSubscriber';

    /**
     * Reference map
     *
     * @var array
     */
    protected $_referenceMap = array(
        'Response' => array(
            'columns'       => array('response_id'),
            'refTableClass' => 'System_Model_DbTable_Responses',
            'refColumns'    => array('id'),
            'onDelete'      => self::CASCADE
        ),
        'User' => array(
            'columns'       => array('user_id'),
            'refTableClass' => 'System_Model_DbTable_Users',
            'refColumns'    => array('id'),
            'onDelete'      => self::CASCADE
        )        
    );

    /**
     * Returns subscribers by user_id and response_id
     *
     * @param  int $userId
     * @param  int $responseId
     * @return System_Model_ResponseSubscriber
     */
    public function findByUserIdAndResponseId($userId, $responseId)
    {
        $select = $this->getSelect()
            ->where('user_id=?', $userId)
            ->where('response_id=?', $responseId);
        return $this->fetchRow($select);
    }

    /**
     * Returns users by response_id
     *
     * @param System_Model_Response $response
     * @return System_Model_Rowset
     */
    public function findUsersByResponse($response)
    {
        $usersDb = Singular_Loader::get('System_Model_DbTable_Users');
        $select = $usersDb->getSelect();
        $select->joinLeft(array('s' => $this->getTableName()),
            "{$usersDb->getTableName()}.id=s.user_id",
            array());
        $select->where("s.response_id=?", $response->id);
        $allUsers = $usersDb->fetchAll($select);
        $users = new System_Model_Rowset(array(
            'table' => Singular_Loader::get('System_Model_DbTable_Users'),
            'rowClass' => 'System_Model_User'
        ));

        /* valid */
        if ($allUsers->valid()) {
            foreach ($allUsers as $user) {
                $configValues = $user->getConfigValues();
                if (isset($configValues['notifications_created'])
                    && $configValues['notifications_created']
                    && isset($configValues['notifications_posted_comments'])
                    && $configValues['notifications_posted_comments']
                ) {
                    $users->add($user);
                } elseif (!isset($configValues['notifications_created'])
                    || !$configValues['notifications_created']
                ) {
                    if ($user->id != $response->created_by_id) {
                        $users->add($user);
                    }
                } elseif (!isset($configValues['notifications_posted_comments'])
                    || !$configValues['notifications_posted_comments']
                ) {
                    $commentsDb = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
                    $items = $commentsDb->count(array(
                        'response_id' => $response->id,
                        'created_by_id' => $user->id
                    ));
                    if (!$items) {
                        $users->add($user);
                    }
                }
            }
        }
        
        return $users;
    }

    /**
     * Deletes subscriber by user_id and response_id
     *
     * @param  int $userId
     * @param  int $responseId
     * @return int          The number of rows deleted.
     */
    public function deleteByUserIdAndResponseId($userId, $responseId)
    {
        $where = array(
            'user_id=?' => $userId,
            'response_id=?' => $responseId
        );
        return $this->delete($where);
    }

    /**
     * Returns total number of subscribers by response_id
     *
     * @param  int $responseId
     * @return int
     */
    public function countByResponseId($responseId)
    {
        return $this->count(array('response_id' => $responseId));
    }

    /**
     * Returns total number of subscribers by user_id
     *
     * @param  int $userId
     * @return int
     */
    public function countByUserId($userId)
    {
        return $this->count(array('user_id' => $userId));
    }

    /**
     * Returns total number of subscribers
     *
     * @param  array|null $where
     * @return string
     */
    public function count($where = null)
    {
        $select = $this->getAdapter()->select();
        if (is_array($where)) {
            foreach ($where as $cond => $value) {
                if ($value) {
                    $select->where('rs.' . $cond . '=?', $value);
                }
            }
        }
        $select->from(array('rs' => $this->getTableName()), array('count' => new Zend_Db_Expr('COUNT(rs.id)')));

        return $this->getAdapter()->fetchOne($select);
    }
}
