<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * settings table model
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 */
class System_Model_DbTable_Settings extends System_Model_DbTable_Abstract
{
    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'settings';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'id';

    /**
     * Returns all settings
     * when $extendedFormat = true returns in raw format
     *
     * @param  bool $extendedFormat
     * @return mixed
     */
    public function getSettings($extendedFormat = false)
    {
        return (false === (bool)$extendedFormat)
            ? $this->getSettingsKeyVal()
            : $this->getSettingsExtended();
    }

    /**
     * Returns all settings in raw format
     *
     * @return array
     */
    public function getSettingsExtended()
    {
        $settings = $this->fetchAll();
        if (null !== $settings) {
            $settings = $settings->toArray();
        }
        return $settings;
    }

    /**
     * Returns all settings in key=>val format
     *
     * @return array
     */
    public function getSettingsKeyVal()
    {
        $select = $this->getAdapter()->select();
        $select->from($this->_name, array('key', 'value'));
        return $this->getAdapter()->fetchPairs($select);
    }

    /**
     * Returns all settings for specific module
     *
     * @param  string $moduleName
     * @param  null|string $sectionName
     * @return Zend_Db_Table_Rowset
     */
    public function getModuleSettings($moduleName, $sectionName = null)
    {
        $select = $this->getAdapter()->select();
        $select->from($this->_name);
        $select->where('module = ?', $moduleName);
        if (is_string($sectionName)) {
            $select->where('section = ?', $sectionName);
        }
        $select->order(array('module ASC', 'section'));
        return $this->getAdapter()->fetchAll($select);
    }

    /**
     * Returns setting by key
     * when $asArray = false returns result as Zend_Db_Table_Row
     *
     * @param  string $keyName
     * @param  bool $asArray
     * @return null
     */
    public function getSetting($keyName, $asArray = true)
    {
        $setting = $this->fetchRow($this->select()->where('`key` = ?', $keyName));
        return (true === (bool)$asArray)
            ? ($setting !== null)
                ? $setting->toArray()
                : null
            : $setting;
    }

    /**
     * Changes setting value by key
     *
     * @param  string $keyName
     * @param  mixed $value
     * @return void
     */
    public function setSettingValue($keyName, $value)
    {
        $setting = $this->getSetting($keyName, false);
        $setting->value = $value;
        $setting->save();

        /* clean cache */
        $this->cleanCache();
    }

    /**
     * Changes setting row fields data by key
     *
     * @param  string $keyName
     * @param  array $data
     * @return void
     */
    public function changeSetting($keyName, $data)
    {
        $setting = $this->getSetting($keyName, false);
        /* prevent overwrite setting name */
        if (array_key_exists('key', $data)) {
            $data['key'] = $keyName;
        }
        $setting->setFromArray($data);
        $setting->save();

        /* clean cache */
        $this->cleanCache();
    }

    /**
     * Deletes setting by key
     *
     * @param string $name
     */
    public function deleteSetting($name)
    {
        $this->deleteRowByField('key', $name);

        /* clean cache */
        $this->cleanCache();
    }

    /**
     * Deletes all settings by module name
     * when $setting contains section name method will delete
     * only this section's settings.
     *
     * @param  string $moduleName
     * @param  null|string $section
     * @return void
     */
    public function deleteModuleSettings($moduleName, $section = null)
    {
        $where['module = ?'] = $moduleName;
        if (is_string($section)) {
            $where['section = ?'] = $section;
        }
        $this->delete($where);

        /* clean cache */
        $this->cleanCache();
    }

    /**
     * Adds module settings
     *
     * @param  array $settings
     * @throws Singular_Exception
     * @return void
     */
    public function addModuleSettings($settings)
    {
        if (!is_array($settings)) {
            throw new Singular_Exception('Settings must be an array.');
        }

        /* check for multi-settings */
        if (is_array($settings[0])) {
            foreach ($settings as $setting) {
                $this->insert($setting);
            }
        } else {
            /* if is a single setting */
            $this->insert($settings);
        }

        /* clean cache */
        $this->cleanCache();
    }

    /**
     * Returns settings by search criteria
     *
     * @param  array $where
     * @return Zend_Db_Table_Rowset|null
     */
    public function findByAll($where)
    {
        $select = $this->getSelect();
        foreach ($where as $column => $value) {
            $select->where("$column=?", $value);
        }
        return $this->fetchAll($select);
    }
}
