<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * users table model
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 */
class System_Model_DbTable_Users extends System_Model_DbTable_Abstract
{
    /**
     * Default roles
     */
    const ADMIN = 'administrator';
    const MODER = 'moderator';
    const USER  = 'user';

    /**
     * Default fetch limit
     */
    const LIMIT = 20;

    /**
     * Cache users
     */
    protected $_users = array();

    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'users';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'id';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass = 'System_Model_User';

    /**
     * Dependent Tables
     *
     * @var array
     */
    protected $_dependentTables = array(
        'System_Model_DbTable_UsersConfigs',
        'System_Model_DbTable_Responses',
        'System_Model_DbTable_ResponsesAnswers',
        'System_Model_DbTable_ResponsesSubscribers',
        'Resources_Model_DbTable_ResponsesComments',
        'Resources_Model_DbTable_Votes'
    );

    /**
     * Inserts a new user row.
     *
     * @see    parent::insert()
     * @param  array  $data  Column-value pairs.
     * @return mixed         The primary key of the row inserted.
     */
    public function insert (array $data)
    {
        /* add time */
        if (empty($data['created_at'])) {
            $data['created_at'] = date("Y-m-d H:i:s");
        }
        if (empty($data['role'])) {
            $data['role'] = 'user';
        }
        if (empty($data['is_active'])) {
            $data['is_active'] = 1;
        }
        if (empty($data['hidden'])) {
            $data['hidden'] = 0;
        }
        return parent::insert($data);
    }

    /**
     * Updates existing user rows.
     *
     * @param  array        $data  Column-value pairs.
     * @param  array|string $where An SQL WHERE clause, or an array of SQL WHERE clauses.
     * @return int          The number of rows updated.
     */
    public function update (array $data, $where)
    {
        /* add time */
        $data = $this->_setDate($data, 'updated_at');
        return parent::update($data, $where);
    }

    /**
     * Deletes user by id
     *
     * @param  int $id
     * @return int The number of rows deleted.
     */
    public function deleteById ($id)
    {
        $this->deletePhotoByUserId($id);
        return parent::delete('id=' . $id);
    }

    /**
     * Returns user by id
     *
     * @param  int $id
     * @return System_Model_User
     */
    public function findById ($id)
    {
        if (!isset($this->_users[$id])) {
            $select = $this->getSelect();
            $select->where("id=?", $id);
            $this->_users[$id] = $this->fetchRow($select);
        }
        return $this->_users[$id];
    }

    /**
     * Returns user by id
     *
     * @param  int $id
     * @return System_Model_User
     */
    public function findActiveById ($id)
    {
        $select = $this->getSelect();
        $select->where("id=?", $id);
        $select->where("is_active=1");
        return $this->fetchRow($select);
    }
    
    /**
     * Returns user by api key
     *
     * @param  string $apiKey
     * @return System_Model_User
     */
    public function findActiveByApiKey ($apiKey)
    {
        $select = $this->getSelect();
        $select->where("api_key = ?", $apiKey);
        $select->where("is_active = 1");
        return $this->fetchRow($select);
    }

    /**
     * Returns multiple users by ids
     *
     * @param  array $ids
     * @return Zend_Db_Table_Rowset|null
     */
    public function findByIds ($ids)
    {
        $select = $this->getSelect();
        $select->where("id IN (?)", $ids);
        return $this->fetchAll($select);
    }

    /**
     * Returns user by email
     *
     * @param  string $email
     * @return Zend_Db_Table_Rowset|null
     */
    public function findByEmail ($email)
    {
        $emails = array($email);
        return $this->findByEmails($emails)->current();
    }

    /**
     * Returns multiple users by emails
     *
     * @param  array $emails
     * @return Zend_Db_Table_Rowset|null
     */
    public function findByEmails ($emails)
    {
        $select = $this->getSelect();
        $select->where("email IN (?)", $emails);
        return $this->fetchAll($select);
    }

    /**
     * Returns multiple users by roles
     *
     * @param  array|string $roles
     * @return Zend_Db_Table_Rowset|null
     */
    public function findByRoles ($roles)
    {
        if (is_string($roles)) {
            $roles = array($roles);
        }
        $select = $this->getSelect();
        $select->where("{$this->getTableName()}.hidden=0");
        $select->where("{$this->getTableName()}.role IN (?)", $roles);
        $select->order('created_at ASC');

        return $this->fetchAll($select);
    }

    /**
     * Returns multiple users by privileges
     *
     * @param  array|string $privileges
     * @return Zend_Db_Table_Rowset|null
     */
    public function findByPrivileges ($privileges)
    {
        $select    = $this->getSelect();
        $tableName = $this->getTableName();
        $select->where("{$tableName}.hidden=0");
        $pTableName = Singular_Loader::get('System_Model_DbTable_Privileges')
            ->getTableName();
        $select->joinLeft(
            array("p" => $pTableName),
            "$tableName.role=p.roles",
            array()
        );
        if (is_string($privileges)) {
            $privileges = array($privileges);
        }
        if (is_array($privileges)) {
            $select->where("p.resources IN (?)", $privileges);
        }
        $select->where("p.type='allow'");
        $select->group("$tableName.id");

        return $this->fetchAll($select);
    }

    /**
     * Returns 6 most active users
     *
     * @return Zend_Db_Table_Rowset|null
     */
    public function findMostActive ()
    {
        $select = $this->getSelect();
        $tableName = $this->getTableName();
        $select->where("{$tableName}.hidden=0");
        $responseDbName = Singular_Loader::get('System_Model_DbTable_Responses')->getTableName();
        $selectResponses = Singular_Loader::get('System_Model_DbTable_Responses')->getSelect();
        $commentsDbName = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments')->getTableName();
        $selectComments = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments')->getSelect();

        $selectResponses->reset("columns")->reset("order")
                ->columns(array("$responseDbName.created_by_id", new Zend_Db_Expr("COUNT(*) AS sum")))
                ->group("$responseDbName.created_by_id");

        $selectComments->reset("columns")->reset("order")
                ->columns(array("$commentsDbName.created_by_id", "COUNT(*) AS sum"))
                ->group("$commentsDbName.created_by_id");

        $roles = Singular_Loader::get('System_Model_DbTable_Roles')
                ->findRolesSlugsByResourcesDeny(array(
            System_Model_DbTable_Resources::ADMIN_ACCESS,
            System_Model_DbTable_Resources::OFFICIAL_REPRESENTATIVE
        ));

        $select->columns(array("sum" => "(r.sum) + (c.sum)"))
                ->joinLeft(
            array("r" => $selectResponses),
            "r.created_by_id={$this->getTableName()}.id",
            array()
        )
                ->joinLeft(
            array("c" => $selectComments),
            "c.created_by_id={$this->getTableName()}.id",
            array()
        )
                ->order("sum DESC")
                ->where("{$this->getTableName()}.role IN (?)", $roles)
                ->limit(6)
                ->setIntegrityCheck(false);

        return $this->fetchAll($select);
    }

    /**
     * Returns 4 newly registered users
     *
     * @return Zend_Db_Table_Rowset|null
     */
    public function findNewlyRegistered ()
    {
        $select = $this->getSelect();
        $select->order('created_at DESC');
        $select->where("{$this->getTableName()}.hidden=0");
        $select->limit(4);

        return $this->fetchAll($select);
    }

    /**
     * Returns user by hidden
     *
     * @param  null
     * @return Zend_Db_Table_Rowset|null
     */
    public function findAnonymous ()
    {
        $select = $this->select();
        $select->where('hidden=1');
        $select->order('id ASC');
        $select->limit(1);

        return $this->fetchRow($select);
    }

    /**
     * Returns user id by hidden
     *
     * @param  null
     * @return array
     */
    public function findAnonymousIds ()
    {
        $select = $this->select();
        $select->reset('columns');
        $select->from($this->getTableName(), array('id'));
        $select->where('hidden=1');
        $select->order('id ASC');
        return $this->getAdapter()->fetchCol($select);
    }
    
    /**
     * Return fetched paginator rows
     *
     * @param  array $where
     * @param  int $page
     * @param  int $rowCount
     * @return mixed
     */
    public function getPaginator($where, $page, $rowCount = null)
    {
        $tableName = $this->getTableName();
        $select = $this->getSelect();
        $select->where("{$tableName}.hidden=0");

        if (!empty($where['search'])) {
            $select->where('email LIKE ? OR full_name LIKE ?', "%" .$where['search'] . "%");
        }
        if (!empty($where['role']) && $where['role'] != 'all') {
            $select->where('role=?', $where['role']);
        }
        if (!empty($where['sort'])) {

            $rTableName = Singular_Loader::get('System_Model_DbTable_Responses')
                    ->getTableName();
            $cTableName = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments')
                    ->getTableName();
            $vTableName = Singular_Loader::get('Resources_Model_DbTable_Votes')
                    ->getTableName();
            switch ($where['sort']) {
                case 'created_most':
                    $select->joinLeft(
                        array("r" => $rTableName),
                        "$tableName.id=r.created_by_id",
                        array("count" => "COUNT(r.id)")
                    );
                    $select->setIntegrityCheck(false);
                    $select->group("$tableName.id");
                    $select->order("count DESC");
                    break;
                case 'commented_most':
                    $select->joinLeft(
                        array("c" => $cTableName),
                        "$tableName.id=c.created_by_id",
                        array("count" => "COUNT(c.id)")
                    );
                    $select->setIntegrityCheck(false);
                    $select->group("$tableName.id");
                    $select->order("count DESC");
                    break;
                case 'top_rated':
                    $selectResponses = Singular_Loader::get('System_Model_DbTable_Responses')
                            ->getSelect();
                    $selectResponses->reset("columns")
                            ->columns(array("COUNT(v.id)"))
                            ->joinLeft(
                        array("v" => $vTableName),
                        "$rTableName.id=v.object_id",
                        array()
                    )
                            ->where("v.object=$rTableName.type")
                            ->where("$rTableName.created_by_id=$tableName.id");

                    $selectComments = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments')
                            ->getSelect();
                    $selectComments->reset("columns")
                            ->joinLeft(
                        array("v" => $vTableName),
                        "$cTableName.id=v.object_id",
                        array()
                    )
                            ->columns(array("COUNT(v.id)"))
                            ->where("v.object='comment'")
                            ->where("$cTableName.created_by_id=$tableName.id");

                    $select->columns(array("sum" => "($selectResponses) + ($selectComments)"))
                            ->order("sum DESC")
                            ->limit(6)
                            ->setIntegrityCheck(false);
                    break;
                default :
                    $select->order("created_at DESC");
                    break;
            }
        }

        $limit = $rowCount ? $rowCount : self::LIMIT;
        return $this->getPaginatorRows($select, null, null, $page, $limit);
    }

    /**
     * Returns total number of users by given condition
     *
     * @param  array|null $where
     * @return int
     */
    public function count ($where = null)
    {
        $select = $this->getAdapter()->select();
        $select->where('u.hidden=0');

        if (is_array($where)) {
            foreach ($where as $cond => $value) {
                if ($value) {
                    $select->where('u.' . $cond . '=?', $value);
                }
            }
        } else if (!empty($where) && is_string($where)) {
            $select->where($where);
        }
        $select->from(array('u' => $this->getTableName()), array('count' => new Zend_Db_Expr('COUNT(u.id)')));

        return $this->getAdapter()->fetchOne($select);
    }
}
