<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * responses row class
 *
 * @uses       System_Model_Row
 * @category   System
 * @package    System_Model
 */
class System_Model_Response extends System_Model_Row
{
    protected $_comments = null;

    protected $_bestComment = null;

    protected $_countComments = null;

    protected $_status = null;

    protected $_statuses = null;

    protected $_isCommented = false;

    protected $_isSubscribed = array();

    protected $_votes = null;

    protected $_userVote = array();

    protected $_guestVote = array();

    protected $_votesSum = null;

    protected $_resentActivity = null;

    protected $_stopFollowStatusModify = false;

    protected $_tableClass = 'System_Model_DbTable_Responses';


    /**
     * Allows pre-insert logic to be applied to row.
     * Subclasses may override this method.
     *
     * @see    parent::_insert()
     * @return void
     */
    protected function _insert()
    {
        $this->slug = $this->createSlug('slug', $this->title);
        if (empty($this->created_by_id)) {
            $loggedUser = Singular_Runtime::extract("loggedUser");
            if ($loggedUser instanceof System_Model_User) {
                $this->created_by_id = $loggedUser->id;
            }
        }
        parent::_insert();
    }

    /**
     * Allows post-delete logic to be applied to row.
     * Subclasses may override this method.
     *
     * @see    parent::_postDelete()
     * @return void
     */
    protected function _postDelete()
    {
        parent::_postDelete();

        // Remove relative votes
        $votesDb = Singular_Loader::get("Resources_Model_DbTable_Votes");
        $votes   = $votesDb->fetchAll("object_id={$this->id} AND object='{$this->type}'");
        if ($votes->valid()) {
            foreach ($votes as $vote) {
                $vote->delete();
            }
        }

        // Remove relative attachments
        if (Singular_Module::getInstance()->moduleExists('files')) {
            $attachments = Singular_Loader::get('Files_Model_DbTable_Files')->findByObjectId($this);
            /** @var Files_Model_File $attachment */
            foreach ($attachments as $attachment) {
                $attachment->delete();
            }
        }
        
        $this->_cleanDefaultCache(array('responses'));
    }

    /**
     * Allows post-insert logic to be applied to row.
     * Subclasses may override this method.
     *
     * @see    parent::_postInsert()
     * @return void
     */
    protected function _postInsert()
    {
        /* subscribe */
        $this->subscribe($this->created_by_id);

        /* add vote */
        $vote = new Resources_Model_Vote();
        $vote->object = $this->getObject();
        $vote->object_id = $this->id;
        $vote->value = 1;
        $vote->save();

        /* log */
        $log = new System_Model_Log();
        $log->save(array(
            'object'           => $this->getObject(),
            'object_id'        => $this->id,
            'action'           => System_Model_DbTable_Logs::CREATED,
            'title'            => $this->title,
            'comment'          => $this->content,
            'created_by_id'    => $this->getAuthor()->id,
            'created_by_name'  => $this->getAuthor()->getName(false),
            'created_by_email' => $this->getAuthor()->email,
        ));

        parent::_postInsert();
        
        $this->_cleanDefaultCache(array('responses'));
    }

    /**
     * Allows post-update logic to be applied to row.
     * Subclasses may override this method.
     *
     * @return void
     */
    protected function _postUpdate()
    {
        $cacheTags = array('responses');
        if ( $this->isModified('title') 
                || $this->isModified('type')
                || $this->isModified('tags')
                || $this->isModified('content') ) {
            //log
            $log = new System_Model_Log();
            $log->save(array(
                'object'    => $this->getObject(),
                'object_id' => $this->id,
                'action'    => System_Model_DbTable_Logs::UPDATED,
                'title'     => $this->title,
                'comment'   => $this->content
            ));
        }
        if ($this->isModified('status') && !$this->getStopFollowStatusModify()) {
            //log
            $log = new System_Model_Log();
            $log->save(array(
                'object'    => $this->getObject(),
                'object_id' => $this->id,
                'action'    => System_Model_DbTable_Logs::CHANGE_STATUS,
                'title'     => $this->title,
                'comment'   => $this->getStatus()->title
            ));
        }
        if ($this->isModified('type')) {
            
            $statuses = Singular_Runtime::extract("statuses")
                            ->findItemsByField('type', $this->type);
            if ($statuses->valid()) {
                $status = $statuses->findByField('is_default', 1);
                if (!$status instanceof System_Model_ResponseStatus) {
                    $status = $statuses->getRow(0);
                }
                $tStatus = $status->slug;
            } else {
                $tStatus = '';
            }
            $this->getTable()->update(array(
                    'status' => $tStatus
                ), array(
                    'id=?' => $this->id
                ));
            
            $votesDb = Singular_Loader::get("Resources_Model_DbTable_Votes");
            $votesDb->update(array(
                    'object' => $this->type
                ), array(
                    'object_id=?' => $this->id,
                    'object=?'    => $this->_cleanData['type'],
                ));
            
            $logDb = Singular_Loader::get("System_Model_DbTable_Logs");
            $logDb->update(array(
                    'object' => $this->type
                ), array(
                    'object_id=?' => $this->id,
                    'object=?'    => $this->_cleanData['type'],
                ));
            
        }
        
        parent::_postUpdate();
        
        $this->_cleanDefaultCache($cacheTags);
    }

    /**
     * Clean default cache
     *
     * @return void
     */
    protected function _cleanDefaultCache($cacheTags)
    {
        $cacheTags[] = $this->type . '_' . $this->id;
        $cacheTags[] = 'breadcrumbs';
        $cache = Singular_Runtime::extract('cachemanager')->getCache('default');
        $cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG, $cacheTags);
        if (class_exists('Supercache', false)) {
            SuperCache::getInstance()->getCache()->clean(Zend_Cache::CLEANING_MODE_ALL);
        }
    }

    /**
     * If $flag is true prevents following response status modification.
     *
     * @param bool $flag
     */
    public function setStopFollowStatusModify($flag = true)
    {
        $this->_stopFollowStatusModify = (bool)$flag;
    }

    /**
     * Returns response status modification follow flag.
     *
     * @return bool
     */
    public function getStopFollowStatusModify()
    {
        return $this->_stopFollowStatusModify;
    }

    /**
     * Returns ISO-formatted `created_at` field
     *
     * @return Zend_Date
     */
    public function getCreatedAt()
    {
        return new Zend_Date($this->created_at, Zend_Date::ISO_8601);
    }

    /**
     * Returns ISO-formatted `updated_at` field
     *
     * @return Zend_Date
     */
    public function getUpdatedAt()
    {
        return new Zend_Date($this->updated_at, Zend_Date::ISO_8601);
    }

    /**
     * Returns employees
     *
     * @return Zend_Db_Table_Rowset
     */
    public function getEmployees()
    {
        $usersDb = Singular_Loader::get('System_Model_DbTable_Users');
        $responsesCommentsDb = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
        $select = $usersDb->getSelect();
        $select->joinLeft(
            array('rc' => $responsesCommentsDb->getTableName()),
            $usersDb->getTableName() . '.id = rc.created_by_id',
            array()
        );
        $select->where("rc.response_id=?", $this->id);
        $select->where("role='administrator' OR role='official-representative'");
        $select->group($usersDb->getTableName() . '.id');

        $users = $usersDb->fetchAll($select);
        
        if ($this->hasAnswer()) {
            $user = $this->getAnswer()->getAuthor();
            $add = true;
            foreach ($users as $u) {
                if ($u->id == $user->id) {
                    $add = false;
                }
            }
            $users->rewind();
            if ($add) {
                $users->add($user);
            }
        }
        return $users;
    }

    /**
     * Returns response author
     *
     * @return System_Model_User
     */
    public function getAuthor()
    {
        if (!isset($this->_properties['CreatedBy'])) {
            $users = Singular_Loader::get('System_Model_DbTable_Users');
            $result = $users->findById($this->created_by_id);
            if ($result instanceof System_Model_User) {
                $this->_properties['CreatedBy'] = $result;
            } else {
                $this->_properties['CreatedBy'] = new System_Model_User();
            }
        }
        return $this->_properties['CreatedBy'];
    }

    /**
     * Returns response deleter
     *
     * @return System_Model_User
     */
    public function getDeleter()
    {
        if(is_null($this->_properties['DeletedBy']) && isset($this->deleted_by_id)){
            $result = $this->findSystem_Model_DbTable_UsersByResponseDeleter()->current();
            if($result instanceof System_Model_User){
                $this->_properties['DeletedBy'] = $result;
            }
        }
        return $this->_properties['DeletedBy'];
    }

    /**
     * Returns response statuses
     *
     * @return array
     */
    public function getStatuses()
    {
        if (!isset($this->_statuses)) {
            $this->_statuses = Singular_Runtime::extract("statuses")
                                ->findItemsByField("type", $this->type);
        }
        return $this->_statuses;
    }

    /**
     * Returns response status
     *
     * @return string
     */
    public function getStatus()
    {
        if (!isset($this->_status)) {
            $statuses = $this->getStatuses();
            if ($statuses instanceof System_Model_Rowset) {
                $this->_status = $statuses->findBySlug($this->status);
                $statuses->rewind();
            }
        }
        return $this->_status;
    }

    /**
     * Checks response for status supporting.
     *
     * @return bool
     */
    public function isSupportStatuses()
    {
        return (bool) $this->getDefaultStatus();
    }
    
    /**
     * Returns response default status slug
     *
     * @return obj
     */
    public function getDefaultStatus()
    {
        $statuses = Singular_Loader::get('System_Model_DbTable_ResponsesStatuses')->findByType($this->type);
        foreach ($statuses as $status) {
            if ($status->is_default) {
                return $status;
            }
        }
        return false;
    }        

    /**
     * Returns response type
     *
     * @return string
     */
    public function getType()
    {
        if (!isset($this->_properties['type'])) {
            $this->_properties['type'] = Singular_Runtime::extract('types')
                                            ->findBySlug($this->type);
        }
        return $this->_properties['type'];
    }

    /**
     * Returns response tags
     *
     * @return array
     */
    public function getTags()
    {
        if (!isset($this->_properties['tags'])) {
            $this->_properties['tags'] = array_map('trim', explode(",", $this->tags));
            $this->_properties['tags'] = array_diff($this->_properties['tags'], array(''));
        }
        return $this->_properties['tags'];
    }
    
    /**
     * Sets tags
     *
     * @param array $tags
     *
     * @return Zend_Db_Table_Row This Zend_Db_Table_Row object.
     */
    public function setTags(array $tags)
    {
        $tags = array_map('trim', $tags);
        $tags = array_diff($tags, array(''));
        $this->_properties['tags'] = $tags;
        
        $this->tags = implode(",", $tags);
        
        return $this;
    }

    /**
     * Returns response comments
     *
     * @return Zend_Db_Table_Rowset
     */
    public function getComments($vote = 'all')
    {
        if ($this->_comments === null) {
            $comments = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
            $this->_comments = $comments->findByResponseId($this->id, $vote);
        }
        return $this->_comments;
    }

    /**
     * Returns best comment
     *
     * @return Resources_Model_ResponseComment
     */
    public function getBestComment()
    {
        if ($this->_bestComment === null) {
            $commentsDb = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
            $select = $commentsDb->getSelect();
            $select->where("{$commentsDb->getTableName()}.response_id=?", $this->id);
            $select->where("{$commentsDb->getTableName()}.is_best=?", 1);
            $this->_bestComment = $commentsDb->fetchRow($select);
        }
        return $this->_bestComment;
    }

    public function hasNegativeComments()
    {
        $commentsDb = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
        $select = $commentsDb->getSelect();
        $select->reset('columns');
        $select->setIntegrityCheck(false);
        $select->columns('COUNT(*) AS count');
        $select->where("{$commentsDb->getTableName()}.response_id=?", $this->id);
        $select->where("{$commentsDb->getTableName()}.is_negative=?", 1);
        $result = $commentsDb->getAdapter()->fetchRow($select);
        return (bool) $result['count'];
    }
    
    /**
     * Returns last comment
     *
     * @return Resources_Model_ResponseComment
     */
    public function getLastComment()
    {
        $commentsDb = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
        return $commentsDb->findLastByResponseId($this->id);
    }

    /**
     * Count comments
     *
     * @return int
     */
    public function countComments()
    {
        if ($this->_countComments === null) {
            $comments = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
            $this->_countComments = $comments->countByResponseId($this->id);
        }
        return $this->_countComments;
    }

    /**
     * Retuns log
     *
     * @return System_Model_Rowset
     */
    public function getResentActivity()
    {
        if (is_null($this->_resentActivity)) {
            $logDb = Singular_Loader::get('System_Model_DbTable_Logs');
            $this->_resentActivity = $logDb->findByObject($this);
        }
        return $this->_resentActivity;        
    }
    
    public function getVotes()
    {
        if (is_null($this->_votes)) {
            $votes = Singular_Loader::get('Resources_Model_DbTable_Votes');
            $this->_votes = $votes->findByObject($this);
        }
        return $this->_votes;
    }

    public function getVotesSum()
    {
        if(is_null($this->_votesSum)){
            $votes = Singular_Loader::get('Resources_Model_DbTable_Votes');
            $this->_votesSum = (object) $votes->getRatingByObject($this);
        }
        return $this->_votesSum;
    }

    public function getGuestVote($pole)
    {
        if (is_null($pole)) {
            $key = 'NULL';
        } else {
            $key = $pole;
        }
        if (!isset($this->_guestVote[$key])) {
            $votesDb = Singular_Loader::get('Resources_Model_DbTable_Votes');
            $vote = $votesDb->findByObjectAndUserId($this, 0, $pole);
            $this->_guestVote[$key] = $vote;
        }
        return $this->_guestVote[$key];
    }

    public function getUserVote($userId = null)
    {
        if (is_null($userId)) {
            $auth = Zend_Auth::getInstance();
            if ($auth->hasIdentity()) {
                $userId = $auth->getIdentity()->id;
            } else {
                return null;
            }
        }
        if (!isset($this->_userVote[$userId])) {
            $votesDb = Singular_Loader::get('Resources_Model_DbTable_Votes');
            $vote = $votesDb->findByObjectAndUserId($this, $userId);
            $this->_userVote[$userId] = $vote;
        }        
        return  $this->_userVote[$userId];
    }

    public function isVoted($userId = null, $pole = null)
    {
        if (is_null($userId)) {
            $auth = Zend_Auth::getInstance();
            if ($auth->hasIdentity()) {
                $userId = $auth->getIdentity()->id;
            } elseif (Singular_Core::_("Settings")->getSetting("anonymous_votes")) {
                $cookie = Zend_Controller_Front::getInstance()->getRequest()
                            ->getCookie('votes', '{}');
                $cookie = Zend_Json_Decoder::decode($cookie);
                if (isset($cookie['response'][$this->id])) {
                    $vote = $this->getGuestVote($pole);
                    if ($vote instanceof Resources_Model_Vote) {
                        if (isset($pole)) {
                            if ($pole > 0 && $cookie['response'][$this->id] > 0) {
                                return true;
                            } elseif ($pole < 0 && $cookie['response'][$this->id] < 0) {
                                return true;
                            } else {
                                return false;
                            }
                        }
                        return true;
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
                return true;
            } else {
                return false;
            }
        }
        
        $vote = $this->getUserVote($userId);
        if ($vote instanceof Resources_Model_Vote) {
            if (isset($pole)) {
                if ($pole > 0 && $vote->value > 0) {
                    return true;
                } elseif ($pole < 0 && $vote->value < 0) {
                    return true;
                } else {
                    return false;
                }
            }
            return true;
        } else {
            return false;
        }
    }

    public function isPayAttention()
    {
        if (key_exists('pay_attention', $this)) {
            return (bool) $this->pay_attention;
        }
        $statuses = $this->getStatuses()->findItemsByField("pay_attention", 1);
        foreach ($statuses as $status) {
            if ($status->pay_attention && $this->status == $status->slug) {
                return true;
            }
        }

        if (!$this->hasAnswer()) {
            return true;
        }

        $comment = $this->getLastComment();
        if ($comment instanceof Resources_Model_ResponseComment) {
            $user = $comment->getAuthor();
            if ($user instanceof System_Model_User) {
                if (!$user->isAllowed('can_access_admin')
                    && !$user->isAllowed('representative')) {
                    return true;
                }
            }
        }
    }

    public function hasAnswer()
    {
        $answer = $this->getAnswer();
        if ($answer instanceof System_Model_ResponseAnswer) {
            return true;
        } else {
            return false;
        }
    }

    public function canEdit($user = null)
    {
        $result = $this->isAllowed('manage_responses') || $this->isAllowed('can_access_admin');
        if (!$result) {
            $loggedUser = Singular_Runtime::extract("loggedUser");
            if ($loggedUser->id == $this->created_by_id) {
                $createdAt = strtotime($this->created_at);
                if ($createdAt > (time() - 60*60)) {
                    $result = true;
                }
            }
        }
        $data = new stdClass();
        $data->response = $this;
        $data->result = $result;
        Singular_Event::dispatch('canEditResponse', $data);
        return $data->result;
    }

    public function canDelete($user = null)
    {
        $result = $this->isAllowed('manage_responses') || $this->isAllowed('can_access_admin');
        
        $data = new stdClass();
        $data->response = $this;
        $data->result = $result;
        Singular_Event::dispatch('canDeleteResponse', $data);
        return $data->result;
    }

    public function canChangeStatus($user = null)
    {
        $result = $this->isAllowed('manage_responses') || $this->isAllowed('can_access_admin');
        
        $data = new stdClass();
        $data->response = $this;
        $data->result = $result;
        Singular_Event::dispatch('canChangeResponseStatus', $data);
        return $data->result;
    }

    public function canCommented($user = null)
    {
        $result = Zend_Auth::getInstance()->hasIdentity() && !$this->lock_comments;
        
        $data = new stdClass();
        $data->response = $this;
        $data->result = $result;
        Singular_Event::dispatch('canCommented', $data);
        return $data->result;
    }

    public function canLockComments($user = null)
    {
        $result = $this->isAllowed('manage_responses') || $this->isAllowed('can_access_admin');
        
        $data = new stdClass();
        $data->response = $this;
        $data->result = $result;
        Singular_Event::dispatch('canLockComments', $data);
        return $data->result;
    }

    public function canAddOfficialAnswer($user = null)
    {
        $result = $this->isAllowed('can_access_admin') || $this->isAllowed('representative');
        
        $data = new stdClass();
        $data->response = $this;
        $data->result = $result;
        Singular_Event::dispatch('canAddOfficialAnswer', $data);
        return $data->result;
    }

    public function getAnswer()
    {
        if(!isset($this->_properties['Answer'])){
            $answers = Singular_Loader::get('System_Model_DbTable_ResponsesAnswers');
            $result = $answers->findByResponseId($this->id);
            if($result instanceof System_Model_ResponseAnswer){
                $this->_properties['Answer'] = $result;
            }else{
                $this->_properties['Answer'] = false;
            }
        }
        return $this->_properties['Answer'];
    }

    /**
     * Deletes Response Official answer.
     *
     * @return bool
     */
    public function removeAnswer()
    {
        $answer = $this->getAnswer();
        if ($answer instanceof System_Model_ResponseAnswer) {
            return $answer->delete();
        } else {
            return false;
        }
    }

    public function getRepliesSum()
    {
        $commentsDb = Singular_Loader::get("Resources_Model_DbTable_ResponsesComments");
        $commentsTableName = $commentsDb->getTableName();
        $select = $commentsDb->getSelect();
        $select->setIntegrityCheck(false);
        $select->reset("columns");
        $select->columns(array("u.id"));
        $select->where("$commentsTableName.response_id=?", $this->id);
        $usersTableName = Singular_Loader::get("System_Model_DbTable_Users")->getTableName();
        $select->joinLeft(
                array("u" => $usersTableName),
                "u.id=$commentsTableName.created_by_id",
                array()
        );
        $select->group("u.id");
        $select->where("u.role='" . System_Model_DbTable_Roles::OFFICIAL_REPRESENTATIVE . "' OR u.role='" . System_Model_DbTable_Roles::ADMINISTRATOR . "'");
        $result = $this->getTable()->getAdapter()->fetchAll($select);
        $items = count($result);
        if ($this->hasAnswer()) {
            $items++;
        }
        return $items;
    }

    public function getParticipants()
    {
        $commentsDb = Singular_Loader::get("Resources_Model_DbTable_ResponsesComments");
        $commentsTableName = $commentsDb->getTableName();
        $select = $commentsDb->getSelect();
        $select->reset("columns");
        $select->columns(array("$commentsTableName.created_by_id"));
        $select->where("$commentsTableName.response_id=?", $this->id);
        $select->where("$commentsTableName.created_by_id <> ?", $this->created_by_id);
        if ($this->hasAnswer()) {
            $answerAuthorId = $this->getAnswer()->getAuthor()->id;
            $select->where("$commentsTableName.created_by_id <> ?", $answerAuthorId);
        }
        $select->group("$commentsTableName.created_by_id");
        $participantsIds = $this->getTable()->getAdapter()->fetchAll($select);
        $participantsIds[] = $this->created_by_id;

        if ($this->hasAnswer() && $this->getAnswer()->created_by_id != $this->created_by_id) {
            $participantsIds[] = $this->getAnswer()->created_by_id;
        }

        $usersDb = Singular_Loader::get("System_Model_DbTable_Users");
        $users = $usersDb->findByIds($participantsIds);

        return $users;
    }

    public function getParticipantsSum()
    {
        $items = 1;
        if ($this->hasAnswer() && $this->getAnswer()->created_by_id != $this->created_by_id) {
            $items++;
        }
        $commentsDb = Singular_Loader::get("Resources_Model_DbTable_ResponsesComments");
        $commentsTableName = $commentsDb->getTableName();
        $subSelect = $commentsDb->getSelect();
        $subSelect->reset("columns");
        $subSelect->columns(array("$commentsTableName.id"));
        $subSelect->where("$commentsTableName.response_id=?", $this->id);
        $subSelect->where("$commentsTableName.created_by_id <> ?", $this->created_by_id);
        if ($this->hasAnswer()) {
            $answerAuthorId = $this->getAnswer()->getAuthor()->id;
            $subSelect->where("$commentsTableName.created_by_id <> ?", $answerAuthorId);
        }
        $subSelect->group("$commentsTableName.created_by_id");
        $select = "SELECT COUNT(rc.id) AS count FROM ($subSelect) AS rc WHERE 1";
        $result = $this->getTable()->getAdapter()->fetchAll($select);
        $items += $result[0]['count'];
        return $items;
    }

    public function getLastReply()
    {
        if (!isset($this->_properties['lastReply'])) {
            $comment = $this->getLastComment();
            if ($comment instanceof Resources_Model_ResponseComment) {
                if ($this->hasAnswer() &&
                        $this->getAnswer()->getCreatedAt()->getTimestamp()
                            > $comment->getCreatedAt()->getTimestamp()) {
                    $this->_properties['lastReply'] = $this->getAnswer();
                } else {
                    $this->_properties['lastReply'] = $comment;
                }
            } else {
                if ($this->hasAnswer()) {
                    $this->_properties['lastReply'] = $this->getAnswer();
                } else {
                    $this->_properties['lastReply'] = $this;
                }
            }
        }
        return $this->_properties['lastReply'];
    }

    public function getUrl()
    {
        $router = Zend_Controller_Front::getInstance()->getRouter();
        return $router->assemble(
            array(
                 'slug' => $this->slug
            ), 'responses_show'
        );
    }

    public function isSubscribed($userId)
    {
        if ($userId) {
            if (!isset($this->_isSubscribed[$userId])) {
                $responsesSubscribers = Singular_Loader::get('System_Model_DbTable_ResponsesSubscribers');
                $subscriber = $responsesSubscribers->findByUserIdAndResponseId($userId, $this->id);
                if ($subscriber instanceof System_Model_ResponseSubscriber) {
                    $this->_isSubscribed[$userId] = true;
                } else {
                    $this->_isSubscribed[$userId] = false;
                }
            }
            return $this->_isSubscribed[$userId];
        }
        return false;
    }

    public function subscribe($userId)
    {
        if (empty($userId)) {
            return false;
        }
        $responsesSubscribers = Singular_Loader::get('System_Model_DbTable_ResponsesSubscribers');
        $subscriber = $responsesSubscribers->findByUserIdAndResponseId($userId, $this->id);
        if (!($subscriber instanceof System_Model_ResponseSubscriber)) {
            $subscriber = new System_Model_ResponseSubscriber();
            $subscriber->response_id = $this->id;
            $subscriber->user_id = $userId;
            return $subscriber->save();
        } else {
            return true;
        }
    }

    public function unSubscribe($userId)
    {
        $responsesSubscribers = Singular_Loader::get('System_Model_DbTable_ResponsesSubscribers');
        $responsesSubscribers->deleteByUserIdAndResponseId($userId, $this->id);
    }

    public function recovery()
    {
        $this->deleted_by_id = 0;
        $this->deleted_at = null;
        $result = $this->save();
        if ($result) {
            $log = new System_Model_Log();
            $log->save(array(
                'object'    => $this->getObject(),
                'object_id' => $this->id,
                'action'    => System_Model_DbTable_Logs::RECOVERY,
                'title'     => $this->title,
                'comment'   => $this->content
            ));
        }
        return $result;
    }

    public function trash()
    {
        $this->deleted_by_id = Singular_Runtime::extract("loggedUser")->id;
        $this->deleted_at = date("Y-m-d H:i:s");
        $result = $this->save();
        if ($result) {
            $log = new System_Model_Log();
            $log->save(array(
                'object'    => $this->getObject(),
                'object_id' => $this->id,
                'action'    => System_Model_DbTable_Logs::MOVED_TO_TRASH,
                'title'     => $this->title,
                'comment'   => $this->content
            ));
        }
        return $result;
    }
    
    /**
     * Switch response status. Return the primary key value(s), as an
     * associative array if the key is compound, or a scalar if the key is
     * single-column.
     * 
     * @param string $status 
     * @return mixed
     */
    public function switchStatus ($status)
    {
        if ($this->status == $status) {
            return false;
        }
        // Switch status
        Singular_Event::dispatch('beforeSwitchResponseStatus', $this);
        $this->status = $status;
        $save = $this->save();
        Singular_Event::dispatch('afterSwitchResponseStatus', $this);
        return $save;
    }
    
    /**
     * Sets all data in the row from an array.
     *
     * @param  array $data
     * @return System_Model_DbTable_Response
     */
    public function setFromArray(array $data)
    {
        if (isset($data['tags']) && is_array($data['tags']) ) {
            $this->setTags($data['tags']);
            unset($data['tags']);
        }
        parent::setFromArray($data);
    }
    
    /**
     * Returns the column/value data as an array.
     *
     * @return array
     */
    public function toArray()
    {
        $data = parent::toArray();
        if (is_string($data['tags'])) {
            $data['tags'] = $this->getTags();
        }
        return $data;
    }
    
    /**
     * Saves the properties to the database.
     *
     * @param array|null $data 
     */
    public function save($data = null)
    {
        if (is_array($data) && !empty($data)) {
            $loggedUser = Singular_Runtime::extract('loggedUser');
            if ($loggedUser->isGuest()) {
                $anonymousResponses = Singular_Core::_('settings')->getSetting('anonymous_responses');
                if ($anonymousResponses) {
                    /* add user */
                    if (!empty($data['notify_email'])) {
                        $user = Singular_Loader::get('System_Model_DbTable_Users')
                                    ->findByEmail($data['notify_email']);
                        if ( !($user instanceof System_Model_User) ) {
                            $user = new System_Model_User();
                            $translate = Singular_Runtime::extract('Zend_Translate');
                            $userData = array(
                                'full_name' => $translate->translate('Anonymous'),
                                'email'     => $data['notify_email'],
                            );
                            if (!$user->register($userData)) {
                                return false;
                            }
                        }
                        $data['created_by_id'] = $user->id;
                    } else {
                        $anonymousUser = Singular_Loader::get('System_Model_DbTable_Users')
                                            ->findAnonymous();
                        $data['created_by_id'] = $anonymousUser->id;
                    }
                } else {
                    return false;
                }
            }
        }

        return parent::save($data);
    }

    /**
     * Is Response type has votes?
     *
     * @return bool
     */
    public function hasVotesType()
    {
        if (!Singular_Runtime::inStorage('types')) {
            return false;
        }
        $types = Singular_Runtime::extract('types');
        if (!$types instanceof Zend_Db_Table_Rowset_Abstract || !count($types)) {
            return false;
        }
        foreach ($types as $type) {
            if ($type->has_votes && $type->slug == $this->type) {
                return true;
            }
        }
        return false;
    }

    /**
     * Cleans cache.
     *
     * @param array $additionalTags
     * @param bool  $onlyTags
     */
    public static function cleanCache($additionalTags = array(), $onlyTags = false)
    {
        $tags = array('response', 'responses', Singular_Loader::get(__CLASS__)->getTable()->getTableName());
        if (is_array($additionalTags) && count($additionalTags)) {
            if ($onlyTags) {
                $tags = $additionalTags;
            } else {
                $tags = $tags + $additionalTags;
            }
        }
        Singular_Core::_('cache')->removeAllByTags($tags);
    }
}
