<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * response_types row class
 *
 * @uses       System_Model_Row
 * @category   System
 * @package    System_Model
 */
class System_Model_Role extends System_Model_Row
{
    /**
     * Name of the class of the Zend_Db_Table_Abstract object.
     *
     * @var string
     */
    protected $_tableClass = 'System_Model_DbTable_Roles';

    /**
     * Roles constants
     */
    const ADMIN                   = 'administrator';
    const OFFICIAL_REPRESENTATIVE = 'official-representative';
    const PEOPLE_MANAGER          = 'people-manager';
    const RESPONSE_MANAGER        = 'response-manager';
    const USER                    = 'user';

    /**
     * Magic method __tostring
     *
     * @return string
     */
    public function  __toString()
    {
        if ($this->is_protected) {
            $translate = Singular_Runtime::extract('Zend_Translate');
            return $translate->translate($this->name);
        } else {
            return $this->name;
        }
    }

    /**
     * Allows pre-insert logic to be applied to row.
     * Subclasses may override this method.
     *
     * @see    parent::_insert()
     * @return void
     */
    public function  _insert()
    {
        $this->role = $this->createSlug('role', $this->name);
        parent::_insert();
    }

    /**
     * Allows post-delete logic to be applied to row.
     * Subclasses may override this method.
     *
     * @return void
     */
    public function _postDelete()
    {
        $privilegesDb = Singular_Loader::get('System_Model_DbTable_Privileges');
        $privileges = $privilegesDb->findByRole($this->role);
        if ($privileges->valid()) {
            foreach($privileges as $privilege) {
                $privilege->delete();
            }
        }
    }

    /**
     * Returns role name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Checks if specified resource is allowed for current role
     *
     * @param  string $resource
     * @return bool
     */
    public function isAllowed($resource)
    {
        $acl = Singular_Runtime::extract("acl");
        if ($acl->isAllowed($this->role, Singular_Acl::RESOURCE_TYPE_OBJECT . Singular_Acl::TYPE_DIVIDER . $resource)) {
            return true;
        }
        return false;
    }

    /**
     * Retirns total number of user assigned to current role
     *
     * @return int
     */
    public function countUsers()
    {
        $users = Singular_Loader::get('System_Model_DbTable_Users');
        return $users->count(array('role' => $this->role));
    }

    /**
     * Returns resources names by current role
     *
     * @see    System_Model_DbTable_Privileges::findResourcesNameByRole()
     * @return array
     */
    public function getResourcesNameByPrivileges()
    {
        $privilegesDb = Singular_Loader::get('System_Model_DbTable_Privileges');
        return $privilegesDb->findResourcesNameByRole($this->role);
    }

    /**
     * Saves the properties to the database.
     *
     * This performs an intelligent insert/update, and reloads the
     * properties with fresh data from the table on success.
     *
     * @param  array|null $data
     * @return mixed The primary key value(s), as an associative array if the
     *     key is compound, or a scalar if the key is single-column.
     */
    public function save($data = null)
    {
        $result = parent::save($data);
        if ($result) {
            $this->savePrivileges((array)$data['resource']);
        }
        return $result;
    }

    /**
     * Saves privileges
     *
     * @param  array $resources
     * @return void
     */
    public function savePrivileges($resources)
    {
        if (is_array($resources)) {
            $privilegesDb = Singular_Loader::get('System_Model_DbTable_Privileges');
            $privileges = $privilegesDb->findByRole($this->role);
            if ($privileges->valid()) {
                foreach($privileges as $privilege) {
                    if (in_array($privilege->resources, $resources)) {
                        $privilege->type = System_Model_Privilege::ALLOW;
                    } else {
                        $privilege->type = System_Model_Privilege::DENY;
                    }
                    $privilege->save();
                    $resources = array_diff($resources, array($privilege->resources));
                }
            }
            if (!empty($resources)) {
                foreach($resources as $resource) {
                    $privilege = new System_Model_Privilege();
                    $privilege->type = System_Model_Privilege::ALLOW;
                    $privilege->roles = $this->role;
                    $privilege->resources = $resource;
                    $privilege->save();
                }
            }
        }
    }

    /**
     * Cleans cache.
     *
     * @param array $additionalTags
     * @param bool  $onlyTags
     */
    public static function cleanCache($additionalTags = array(), $onlyTags = false)
    {
        $tags = array('role', 'roles', Singular_Loader::get(__CLASS__)->getTable()->getTableName());
        if (is_array($additionalTags) && count($additionalTags)) {
            if ($onlyTags) {
                $tags = $additionalTags;
            } else {
                $tags = $tags + $additionalTags;
            }
        }
        Singular_Core::_('cache')->removeAllByTags($tags);
    }
}
