<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Service
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Api service
 *
 * @category   System
 * @package    System_Service
 */
class System_Service_Api
{    
    /**
     * Generate api key
     *
     * @param  int $length
     * @return string
     */
    public static function generate($length = 32)
    {
        $chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
        $size = strlen($chars);
        $str = '';
        for ($i = 0; $i < $length; $i++) {
            $str .= $chars[rand(0, $size - 1)];
        }
        return $str;
    }
    
    /**
     * Convert array to xml
     *
     * @param  array $data
     * @param  SimpleXMLElement $xml
     * @return string
     */
    public static function xml ($data, $xml = null)
    {
        $data = self::_sanitizeData($data);
        if (null === $xml) {
            $xml = simplexml_load_string('<?xml version="1.0" encoding="UTF-8"?><data />');
        }

        foreach ($data as $key => $value) {
            if (is_numeric($key)) {
                $key = "item$key";
            }
            if (is_array($value)) {
                $node = $xml->addChild($key);
                self::xml($value, $node);
            } else {
                if (is_numeric($value) && 0 === $value) $value = '0';
                $view = Zend_Layout::getMvcInstance()->getView();
                $xml->addChild($key, "<![CDATA[{$view->escape($value)}]]>");
            }
        }
        return str_replace(
            array('&lt;![CDATA[', ']]&gt;'),
            array('<![CDATA[', ']]>'),
            $xml->asXML()
        );
    }
    
    /**
     * Convert array to json
     *
     * @param  array $data
     * @return string
     */
    public static function json ($data)
    {
        return Zend_Json::encode(self::_sanitizeData($data));
    }

    private function _sanitizeData ($data)
    {
        foreach ($data as $key => $val) {
            if (is_array($val) || $val instanceof ArrayAccess || $val instanceof Countable) {
                if (empty($val) || (1 == count($val) && '' === current($val))) {
                    $data[$key] = '';
                } else {
                    $data[$key] = self::_sanitizeData($val);
                }
            } elseif (is_numeric($val)) {
                $data[$key] = (int)$val;
            } elseif (is_object($val)) {
                $data[$key] = (string)$val;
            }
        }
        return $data;
    }
    
    /**
     * Build comments list
     * 
     * @param System_Model_Rowset $comments
     * @return array
     */
    public static function commentsListing ($comments)
    {
        // Empty
        if (null === $comments) {
            return array();
        }
        
        // Rowset or Row
        $rows = $comments instanceof Resources_Model_ResponseComment
              ? array($comments) : $comments;
        
        // Build listing
        $items = array();
        foreach ($rows as $comment) {
            $item = array(
                'id'          => $comment->id,
                'content'     => $comment->content,
                'response_id' => $comment->response_id,
                'created_at'  => $comment->getCreatedAt(),
                'author'      => self::usersListing($comment->getAuthor()),
                'is_best'     => $comment->is_best,
                'is_negative' => $comment->is_negative,
                'votes'       => (array) $comment->getVotesSum(),
            );
            if ($comments instanceof Resources_Model_ResponseComment) {
                return $item;
            }
            $items[] = $item;
        }
        return $items;
    }
    
        
    /**
     * Build users list
     * 
     * @param System_Model_Rowset $users
     * @return array
     */
    public static function usersListing ($users)
    {
        if (null === $users) {
            return array();
        }
        
        $rows = $users instanceof System_Model_User
              ? array($users) : $users;

        // Getting models
        $votes = Singular_Loader::get('Resources_Model_DbTable_Votes');
        $responses = Singular_Loader::get('System_Model_DbTable_Responses');
        $responsesComments = Singular_Loader::get(
            'Resources_Model_DbTable_ResponsesComments'
        );
        $subscriptions = Singular_Loader::get(
            'System_Model_DbTable_ResponsesSubscribers'
        );
        
        $loggedUser = Singular_Runtime::extract('loggedUser');
        $items = array();
        foreach ($rows as $user) {
            $configs = $user->getConfigValues();
            if (isset($configs['photo_type'])
                && $configs['photo_type'] == System_Model_UserConfig::GRAVATAR) {
                $avatar = $user->gravatar_email;
            } else {
                $avatar = $user->getAvatarSrc();
            }
            $item = array(
                'id'            => $user->id,
                'name'          => $user->getName(),
                'joined'        => $user->getCreatedAt(),
                'rating'        => $votes->getRatingForUser($user),
                'responses'     => $responses->countByUserId($user->id),
                'comments'      => $responsesComments->countByUserId($user->id),
                'role'          => $user->getRole()->name,
                'avatar'        => $avatar,
                'last_activity' => $user->getLastActivity(),
                'subscriptions' => $subscriptions->countByUserId($user->id),
                'votes'         => $votes->findByResponsesAndUserId($user->id),
            );
            if ($loggedUser->canManagePeople()
                || $loggedUser->isAdmin()) {
                $item['email'] = $user->email;
            }
            if ($loggedUser->isAdmin()) {
                $item['api_key'] = $user->api_key;
            }
            if ($users instanceof System_Model_User) {
                return $item;
            }
            $items[] = $item;
        }
        return $items;
    }
    
    /**
     * Build responses list
     * 
     * @param System_Model_Rowset|System_Model_Response $responses
     * @return array
     */
    public static function responsesListing ($responses)
    {
        // Empty
        if (null === $responses) {
            return array();
        }
        
        // Getting required models
        $subscribers = Singular_Loader::get(
            'System_Model_DbTable_ResponsesSubscribers'
        );
        
        // Rowset or Row
        $rows = $responses instanceof System_Model_Response
              ? array($responses) : $responses;
        
        // Build listing
        $items = array();
        foreach ($rows as $response) {
            $bestComment = $response->getBestComment();
            $bestComment = $bestComment ? $bestComment->toArray() : null;
            $lastComment = $response->getLastComment();
            $lastComment = $lastComment ? $lastComment->toArray() : null;
            $item = array(
                'id'                 => $response->id,
                'title'              => $response->title,
                'slug'               => $response->slug,
                'type'               => (string) $response->getType(),
                'content'            => $response->content,
                'tags'               => $response->getTags(),
                'status'             => (string) $response->getStatus(),
                'created_at'         => (string) $response->getCreatedAt(),
                'lock_comments'      => $response->lock_comments,
                'employees'          => self::usersListing($response->getEmployees()),
                'participants_count' => $response->getParticipantsSum(),
                'participants'       => self::usersListing($response->getParticipants()),
                'replies_count'      => $response->getRepliesSum(),
                'subscribers_count'  => $subscribers->countByResponseId($response->id),
                'comments_count'     => $response->countComments(),
                'last_comment'       => $lastComment,
                'best_comment'       => $bestComment,
                'votes'              => (array) $response->getVotesSum(),
                'author'             => self::usersListing($response->getAuthor()),
                'answer'             => $response->getAnswer(),
            );
            if ($responses instanceof System_Model_Response) {
                return $item;
            }
            $items[] = $item;
        }
        return $items;
    }
}
