<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Component
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @category   Singular
 * @package    Singular_Component
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
abstract class Singular_Component_Abstract
{
    /**
     * Default component scripts directory
     */
    const COMPONENT_SCRIPTS_DIR = 'components';

    /**
     * Default extension for view scripts
     */
    const VIEW_EXT = '.phtml';

    /**
     * Zend_View
     *
     * @var Zend_View_Interface
     */
    public $view = null;

    /**
     * Computed view script name
     *
     * @var string
     */
    private $_viewScript = null;

    /**
     * Component options
     *
     * @var array
     */
    private $_options = array();

    /**
     * Flag for component run
     *
     * @var bool
     */
    private $_runnableFlag = true;

    private $_viewPathsOriginal = array();

    private $_currModule = null;

    /**
     * Class constructor
     *
     * @param string $module
     * @param array $options
     */
    public final function __construct($module, $options = array())
    {
        $this->_initView();
        $this->setOptions($options);
        $this->_modifyViewPaths(strtolower($module));
    }

    /**
     * Init method
     *
     * Used for initialize component
     *
     * @return void
     */
    public function init()
    {
    }

    /**
     * Set options
     *
     * @param array $options
     * @return Singular_Component
     */
    public function setOptions($options = array())
    {
        if ($options instanceof Zend_Config) {
            $this->_options = $options->toArray();
        } elseif (is_array($options)) {
            foreach ($options as $oName => $oValue) {
                $this->setOption($oName, $oValue);
            }
        } else {
            throw new Singular_Component_Exception(
                'Incorrect option type. Array|Zend_Config is needed. '
                . ucfirst(gettype($options)) . ' was given.');
        }

        return $this;
    }

    /**
     * Set option
     * 
     * @throws Singular_Component_Exception
     * @param  string $option
     * @param  mixed $value
     * @return Singular_Component_Abstract
     */
    public function setOption ($option, $value)
    {
        if ('' == $option) {
            throw new Singular_Component_Exception('Option name is not set.');
        }
        $this->_options[$option] = $value;

        return $this;
    }

    /**
     * Get component options
     *
     * @return array
     */
    public function getOptions ()
    {
        return $this->_options;
    }

    /**
     * Get option by key
     * 
     * @param  string $option
     * @return array|null
     */
    public function getOption ($option)
    {
        if ($this->hasOption($option)) {
            return $this->_options[$option];
        }
        return null;
    }

    /**
     * Returns true if options is exists
     *
     * @param $option
     *
     * @return null
     */
    public function hasOption($option)
    {
        return array_key_exists($option, $this->_options);
    }

    /**
     * Set view script template name
     *
     * @param string $script
     * @param bool $useCompDir
     * @return Singular_Component
     */
    public function setViewScript($script, $useCompDir = true)
    {
        if (substr($script, -strlen(self::VIEW_EXT)) != self::VIEW_EXT) {
            $script .= self::VIEW_EXT;
        }
        if (true === (bool) $useCompDir) {
            $script = self::COMPONENT_SCRIPTS_DIR . DS . $script;
        }
        $this->_viewScript = $script;

        return $this;
    }

    /**
     * Get view script template name
     *
     * @return null|string
     */
    public function getViewScript()
    {
        return $this->_viewScript;
    }

    /**
     * Clear view script template name
     *
     * @return Singular_Component_Abstract
     */
    public function clearViewScript()
    {
        $this->_viewScript = null;

        return $this;
    }

    /**
     * Initialize view object
     *
     * @return void
     */
    protected function _initView()
    {
        $layout = Zend_Layout::getMvcInstance();
        $this->setView($layout->getView());
    }

    /**
     * Set view object
     *
     * @param  Zend_View_Interface $view
     * @return void
     */
    public function setView(Zend_View_Interface $view)
    {
        $this->view = $view;
    }

    /**
     * Get view object
     *
     * @return Zend_View_Interface
     */
    public function getView()
    {
        return $this->view;
    }

    /**
     * Assign view variable into template
     *
     * @param  string $spec
     * @param  mixed|null $value
     * @return Singular_Component
     */
    public function assignViewVar($spec, $value = null)
    {
        $this->view->assign($spec, $value);

        return $this;
    }

    /**
     * Component runnable flag
     *
     * @param  bool $flag
     * @return void
     */
    protected function _setRunnable($flag = false)
    {
        $this->_runnableFlag = (bool) $flag;
    }

    /**
     * Get runnable component flag
     *
     * @return bool
     */
    public function isRunnable()
    {
        return (bool) $this->_runnableFlag;
    }

    /**
     * Modifies current view script paths dependently module component
     * 
     * @param  string|null $moduleName
     * @return void
     */
    private function _modifyViewPaths ($moduleName = null)
    {
        $this->_viewPathsOriginal = $this->getView()->getScriptPaths();
        $this->_currModule = Zend_Controller_Front::getInstance()
                ->getRequest()
                ->getModuleName();

        $replacements = array(
            '/modules/' . $this->_currModule => '/modules/' . strtolower($moduleName),
            '\modules\\' . $this->_currModule => '\modules\\' . strtolower($moduleName),
            '/modules\\' . $this->_currModule => '/modules\\' . strtolower($moduleName)
        );

        $paths = $this->_replaceRecursive(
            $this->_viewPathsOriginal,
            $replacements
        );

        $this->getView()->setScriptPath(array_reverse($paths));
    }

    /**
     * Restore orgignal view paths
     *
     * @return void
     */
    private function _restoreViewPaths ()
    {
        $this->getView()->setScriptPath(array_reverse($this->_viewPathsOriginal));
    }

    /**
     * Help method
     * used by _modifyViewPaths()
     *
     * @param  array|string $search
     * @param  array $replacements
     * @return array|string
     */
    private function _replaceRecursive ($search, $replacements = array())
    {
        if (is_array($search)) {
            $result = array();
            foreach ($search as $key => $val) {
                $result[strtr($key, $replacements)] = $this->_replaceRecursive(
                    $val,
                    $replacements
                );
            }
            return $result;
        } else {
            return strtr($search, $replacements);
        }
    }

    /**
     * Component runner/renderer
     *
     * @return string
     */
    public final function run()
    {
        $this->init();
        
        if (!$this->isRunnable()) {
            return;
        }

        Singular_Event::dispatch('componentRender', $this);
        
        $this->_setRunnable(false);

        if (null !== $this->getViewScript()) {
            $output = $this->view->render($this->getViewScript());
            $outputObj = new stdClass();
            $outputObj->output = $output;
            $outputObj->component = $this;
            if ('object' != gettype($output)) unset($output);
            $this->_restoreViewPaths();
            Singular_Event::dispatch('componentRenderAfter', $outputObj);
            return $outputObj->output;
        }
    }
}