<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Controller
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Zend_Controller_Action
 * @category   Singular
 * @package    Singular_Controller
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Controller_Action extends Zend_Controller_Action
{
    /**
     * Autoresponder Singular_Controller_Action_Helper_Autoresponder object
     *
     * @var Singular_Controller_Action_Helper_Autoresponder
     */
    public $autoresponder = null;

    /**
     * Scaffolding switch flag
     * 
     * @var bool
     */
    public $useScaffolding = false;

    /**
     * Scaffolding action helper object
     *
     * @var Singular_Controller_Action_Helper_Scaffolding
     */
    public $scaffolding = null;

    /**
     * Active model name or Singular_Db_Table_Abstract object
     *
     * @var string|Singular_Db_Table_Abstract
     */
    public $activeModel = null;

    /**
     * Class constructor
     *
     * The request and response objects should be registered with the
     * controller, as should be any additional optional arguments; these will be
     * available via {@link getRequest()}, {@link getResponse()}, and
     * {@link getInvokeArgs()}, respectively.
     *
     * When overriding the constructor, please consider this usage as a best
     * practice and ensure that each is registered appropriately; the easiest
     * way to do so is to simply call parent::__construct($request, $response,
     * $invokeArgs).
     *
     * After the request, response, and invokeArgs are set, the
     * {@link $_helper helper broker} is initialized.
     *
     * Finally, {@link init()} is called as the final action of
     * instantiation, and may be safely overridden to perform initialization
     * tasks; as a general rule, override {@link init()} instead of the
     * constructor to customize an action controller's instantiation.
     *
     * @param Zend_Controller_Request_Abstract $request
     * @param Zend_Controller_Response_Abstract $response
     * @param array $invokeArgs Any additional invocation arguments
     */
    public function __construct(Zend_Controller_Request_Abstract $request,
        Zend_Controller_Response_Abstract $response, array $invokeArgs = array())
    {
        $this->setRequest($request)
             ->setResponse($response)
             ->_setInvokeArgs($invokeArgs);
        $this->_helper = new Zend_Controller_Action_HelperBroker($this);
        if (Singular_Core::_('Environment')->isAdminRequest()) {
            $this->_initAdminAccess();
        }
        $this->_initScaffolding();
        $this->autoresponder = $this->_helper->autoresponder();
        $this->init();
    }

    /**
     * Initialize scaffolding
     *
     * @return void
     */
    protected function _initScaffolding ()
    {
        if (!$this->useScaffolding) return;
        
        if (null === $this->scaffolding) {
            $this->scaffolding = $this->_helper->scaffolding();
        }
    }

    /**
     * Initialize admin access
     *
     * @throws Singular_Acl_Exception
     * @return void
     */
    protected function _initAdminAccess ()
    {
        // check access to admin area
        if (!$this->_helper->acl()->objectIsAllowed('can_access_admin')) {
            throw new Singular_Acl_Exception('Access deny');
        }
    }

    /**
     * gotoIndex() method
     *
     * @param  string|null $message
     * @param  string $notifyType
     * @return void
     */
    protected function gotoIndex ($message = null, $notifyType = 'info')
    {
        if (is_string($message)) {
            $message = $this->view->t($message);

            switch ($notifyType) {
                case 'info':
                case 'error':
                case 'success':
                break;

                default:
                    $notifyType = 'info';
                break;
            }

            $this->view->notify()->{$notifyType}($message);
        }

        $this->_helper->redirector()->gotoSimpleAndExit('index');
    }

    /**
     * Return true if request maked from xmlHttpRequest
     *
     * @return bool
     */
    public function isAjax ()
    {
        return $this->getRequest()->isXmlHttpRequest();
    }

    /**
     * Page title setter
     *
     * @param  string $text
     * @param  bool $useTranslator
     * @return void
     */
    public function title ($text, $useTranslator = true)
    {
        if (true === (bool) $useTranslator) {
            $text = $this->view->t($text);
        }

        $this->view->pageTitle = $text;
        $this->view->headTitle($text);
    }

    /**
     * Render template into variable
     *
     * @param  string $template
     * @return string
     */
    public function fetchTemplate ($template = null)
    {
        if (null === $template) {
            $template = $this->getViewScript();
        }
        
        $ext = '.' . $this->viewSuffix;
        
        if (substr($template, -strlen($ext)) != $ext) {
            $template .= $ext;
        }

        $layout = Zend_Layout::getMvcInstance();
        /** @var $view Zend_View */
        $view = $layout->getView();

        return $view->render($template);
    }

    /**
     * Instantly render template
     *
     * @param  string $template
     * @param  null|string $name
     * @param  bool $resetBody
     * @return void
     */
    public function template ($template, $name = null, $resetBody = false)
    {
        if (!strpos($template, ".{$this->viewSuffix}")) {
            $template .= ".{$this->viewSuffix}";
        }

        if ($resetBody) {
            $this->getResponse()->setBody(
                $this->view->render($template),
                $name
            );

            $this->_helper->viewRenderer->setNeverRender(true);

            return;
        }

        $this->renderScript($template, $name);
    }

    /**
     * Scaffolding list action
     *
     * @return void
     */
    public function listAction ()
    {
        if ((bool) $this->useScaffolding) $this->scaffolding->operate();
        else $this->_disabledScaffoldingException();
    }

    /**
     * Scaffolding get action
     *
     * @return void
     */
    public function getAction ()
    {
        if ((bool)$this->useScaffolding) $this->scaffolding->operate();
        else $this->_disabledScaffoldingException();
    }

    /**
     * Scaffolding add action
     *
     * @return void
     */
    public function addAction ()
    {
        if ((bool)$this->useScaffolding) $this->scaffolding->operate();
        else $this->_disabledScaffoldingException();
    }

    /**
     * Scaffolding edit action
     *
     * @return void
     */
    public function editAction ()
    {
        if ((bool)$this->useScaffolding) $this->scaffolding->operate();
        else $this->_disabledScaffoldingException();
    }

    /**
     * Scaffolding delete action
     *
     * @return void
     */
    public function deleteAction ()
    {
        if ((bool)$this->useScaffolding) $this->scaffolding->operate();
        else $this->_disabledScaffoldingException();
    }

    /**
     * Exception for scaffolding
     *
     * @throws Zend_Controller_Action_Exception
     * @return void
     */
    private function _disabledScaffoldingException ()
    {
        throw new Zend_Controller_Action_Exception('Scaffolding is disabled');
    }
}
