<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Singular_Controller_Action_Helper
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Autoresponder action helper
 *
 * @example
 * Sample autoresponder output in JSON format:
 *  {
 *   "response": {
 *       "server_time": "08.06.2011 13:37:15",
 *       "module": "system",
 *       "controller": "structure",
 *       "action": "expand",
 *       "response_type": "success",
 *       "code": 0,
 *       "message": "Undefined",
 *       "output": "<h1>Sample output data</h1>",
 *       "params": {
 *           "param1": 'value1',
 *           "param2": 'value2'
 *           }
 *       }
 *   }
 *
 * @uses       Zend_Controller_Action_Helper_Abstract
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Singular_Controller_Action_Helper
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Controller_Action_Helper_Autoresponder extends Zend_Controller_Action_Helper_Abstract
{
    const RESPONSE_TYPE_ERROR   = 'error';
    const RESPONSE_TYPE_SUCCESS = 'success';

    const DEFAULT_ERROR_MESSAGE   = 'Error';
    const DEFAULT_SUCCESS_MESSAGE = 'Success';

    const DEFAULT_CODE = 0;

    const FORMAT_JSON       = 'json';
    const FORMAT_XML        = 'xml';
    const FORMAT_SERIALIZED = 'serialized';

    /**
     * Output format
     * supports: json, xml, serialized
     *
     * @var string
     */
    protected $_format = self::FORMAT_JSON;

    /**
     * Current output format
     *
     * @var null
     */
    protected $_currFormat = null;

    /**
     * Content type
     *
     * @var string
     */
    protected $_contentType = 'text/plain';

    /**
     * Capture view variables flag
     *
     * @var bool
     */
    protected $_captureViewVars = true;

    /**
     * Enabled flag
     * can be changed statically by calling: self::switchOff() or self::switchOn()
     * or in actions by calling:
     *      $this->autoresponder->disable();
     *      $this->autoresponder->enable();
     *
     * @var bool
     */
    protected static $_isEnabled = true;

    /**
     * Force render flag
     *
     * @see Singular_Controller_Action_Helper_Autoresponder::setForceRender()
     * @var bool
     */
    protected $_forceRender = false;

    protected static $_data
        = array(
            'response' => array(
                'server_time'   => null,
                'module'        => null,
                'controller'    => null,
                'action'        => null,
                'response_type' => self::RESPONSE_TYPE_SUCCESS,
                'code'          => self::DEFAULT_CODE,
                'message'       => 'Undefined',
                'output'        => null,
                'params'        => array()
            )
        );

    /**
     * Use this helper as a method;
     *
     * @return void
     */
    public function direct ()
    {
        self::$_data['response']['module']     = $this->getRequest()->getModuleName();
        self::$_data['response']['controller'] = $this->getRequest()->getControllerName();
        self::$_data['response']['action']     = $this->getRequest()->getActionName();

        $currFormat = $this->getRequest()->getParam('format', null);

        switch ($currFormat) {
            case self::FORMAT_JSON:
            case self::FORMAT_XML:
            case self::FORMAT_SERIALIZED:
                $this->_currFormat = $currFormat;
                break;
        }

        return $this;
    }

    /**
     * Switch off autoresponder
     * static implementation
     *
     * @static
     * @return void
     */
    public static function switchOff ()
    {
        self::$_isEnabled = false;
    }

    /**
     * Switch on autoresponder
     * static implementation
     *
     * @static
     * @return void
     */
    public static function switchOn ()
    {
        self::$_isEnabled = true;
    }

    /**
     * Switch off autoresponder
     *
     * @return void
     */
    public function disable ()
    {
        self::$_isEnabled = false;
    }

    /**
     * Switch on autoresponder
     *
     * @return void
     */
    public function enable ()
    {
        self::$_isEnabled = true;
    }

    /**
     * Return true if autoresponder can be used
     *
     * @return bool
     */
    public function isEnabled ()
    {
        return (bool) self::$_isEnabled;
    }

    /**
     * Is error getter
     *
     * @return bool
     */
    public function isError ()
    {
        return (self::RESPONSE_TYPE_ERROR == self::$_data['response']['response_type']) ? true : false;
    }

    /**
     * Is success getter
     * 
     * @return bool
     */
    public function isSuccess ()
    {
        return (self::RESPONSE_TYPE_SUCCESS == self::$_data['response']['response_type']) ? true : false;
    }

    /**
     * Adds parameter with specified value that will be shown in autoresponder response
     *
     * @example
     * "params": {
     *     "param1": 'value1',
     *     "param2": 'value2'
     * }
     *
     * @param  string $param
     * @param  string|mixed $value
     * @return Singular_Controller_Action_Helper_Autoresponder
     */
    public function setParam ($param, $value)
    {
        self::$_data['response']['params'][$param] = $value;

        return $this;
    }

    /**
     * The same as setParam() but allows multiple params as array
     *
     * @param  array $data
     * @return Singular_Controller_Action_Helper_Autoresponder
     */
    public function setParams ($data)
    {
        if (is_array($data) && !empty($data)) {
            foreach ($data as $param => $value) {
                $this->setParam($param, $value);
            }
        }

        return $this;
    }

    /**
     * Clears all params
     *
     * @return Singular_Controller_Action_Helper_Autoresponder
     */
    public function clearParams ()
    {
        self::$_data['response']['params'] = array();

        return $this;
    }

    /**
     * Sets "error" response type
     * supports message and code parameters
     *
     * @param  string $message
     * @param  int $code
     * @return Singular_Controller_Action_Helper_Autoresponder
     */
    public function setError ($message = self::DEFAULT_ERROR_MESSAGE, $code = self::DEFAULT_CODE)
    {
        self::$_data['response']['response_type'] = self::RESPONSE_TYPE_ERROR;
        self::$_data['response']['code'] = $code;
        self::$_data['response']['message'] = $this->_actionController->view->t($message);

        return $this;
    }

    /**
     * Sets "seccess" response type
     * supports message and code parameters
     *
     * @param  string $message
     * @param  int $code
     * @return Singular_Controller_Action_Helper_Autoresponder
     */
    public function setSuccess ($message = self::DEFAULT_SUCCESS_MESSAGE, $code = self::DEFAULT_CODE)
    {
        self::$_data['response']['response_type'] = self::RESPONSE_TYPE_SUCCESS;
        self::$_data['response']['code'] = $code;
        self::$_data['response']['message'] = $this->_actionController->view->t($message);

        return $this;
    }

    /**
     * Defines autoresponder output format
     * supports: json(default), xml, serialized(php native serialized)
     *
     * @param  string $format
     * @return Singular_Controller_Action_Helper_Autoresponder
     */
    public function setFormat ($format)
    {
        if ($format != self::FORMAT_JSON && $format != self::FORMAT_XML && $format != self::FORMAT_SERIALIZED) {
            return;
        }

        $this->_format = $format;

        return $this;
    }

    /**
     * Sets output body
     * may be used for example to render template
     *
     * @param  string $content
     * @return Singular_Controller_Action_Helper_Autoresponder
     */
    public function setOutput ($content)
    {
        if (is_string($content) || is_numeric($content)) {
            self::$_data['response']['output'] = $content;
        }

        return $this;
    }

    /**
     * Gets output body
     *
     * @return string
     */
    public function getOutput ()
    {
        return self::$_data['response']['output'];
    }

    /**
     * Sets capture view variables
     * if true - all defined view variables will be captured into "params" section
     *
     * @param  bool $flag
     * @return Singular_Controller_Action_Helper_Autoresponder
     */
    public function setCaptureViewVars ($flag)
    {
        $this->_captureViewVars = (bool) $flag;

        return $this;
    }

    /**
     * Sets force render autoresponder response
     * if true - force render even if no xmlHttpRequest request has detected
     *
     * @param  bool $flag
     * @return void
     */
    public function setForceRender ($flag)
    {
        $this->_forceRender = (bool) $flag;
    }

    /**
     * Gets force render on|off flag
     *
     * @return bool
     */
    public function getForceRender ()
    {
        return $this->_forceRender;
    }

    /**
     * Removes module/controller/action params in response
     * may be used in security reasons
     *
     * @return Singular_Controller_Action_Helper_Autoresponder
     */
    public function removeMvcParams ()
    {
        unset(self::$_data['response']['module']);
        unset(self::$_data['response']['controller']);
        unset(self::$_data['response']['action']);

        return $this;
    }

    /**
     * Post dispatch processing
     *
     * Execute postDispatch callback for autoresponder, if available
     *
     * @throws Zend_Controller_Action_Exception
     * @return void
     */
    public function postDispatch ()
    {
        if (!$this->isEnabled() || null !== $this->getRequest()->getParam('no_responder', null)) {
            return;
        }

        // Render automatically when xmlHttpRequest is detected or manually in action by calling setForceRender(true)
        if ($this->getRequest()->isXmlHttpRequest() || true === $this->getForceRender()) {
            Zend_Controller_Action_HelperBroker::getStaticHelper('noRender')->direct();

            $this->getResponse()->setBody($this->render());
            $this->getResponse()->setHeader('Content-type', $this->_contentType, true);
        }
    }

    /**
     * Converts response array to xml format
     *
     * @param  array $data
     * @param  string $rootNodeName
     * @param  null|array $xml
     * @return string
     */
    protected function _toXml ($data, $rootNodeName = 'response', &$xml = null)
    {
        // turn off compatibility mode as simple xml throws a wobbly if you don't.
        if (ini_get('zend.ze1_compatibility_mode') == 1) ini_set('zend.ze1_compatibility_mode', 0);
        if (is_null($xml)) $xml = simplexml_load_string("<?xml version='1.0' encoding='utf-8'?><$rootNodeName />");

        // loop through the data passed in.
        foreach ($data as $key => $value) {
            $numeric = false;

            // no numeric keys in our xml please!
            if (is_numeric($key)) {
                $numeric = 1;
                $key = $rootNodeName;
            }

            // delete any char not allowed in XML element names
            $key = preg_replace('/[^a-z0-9\-\_\.\:]/i', '', $key);

            // if there is another array found recrusively call this function
            if (is_array($value)) {
                $node = (is_array($value))
                        || $numeric
                        ? $xml->addChild($key)
                        : $xml;

                // recrusive call.
                if ($numeric) $key = 'anon';
                $this->_toXml($value, $key, $node);
            } else {

                // add single node.
                $value = htmlentities($value);
                $xml->addChild($key, $value);
            }
        }

        // pass back as XML
        return $xml->asXML();
    }

    /**
     * Renders response in specified format
     * render() can be used manually to render response into variable
     * by default called from self::postDispatch()
     *
     * @throws Zend_Controller_Action_Exception
     * @return string
     */
    public function render ()
    {
        if (!$this->isEnabled() || null !== $this->getRequest()->getParam('no_responder', null)) {
            return;
        }

        /* capture view variables */
        if ($this->_captureViewVars) {
            $viewRenderer = Zend_Controller_Action_HelperBroker::getStaticHelper('viewRenderer');
            /** @var $view Zend_View */
            $view = $viewRenderer->view;
            
            if ($view instanceof Zend_View_Interface) {
                if (method_exists($view, 'getVars')) {
                    $vars = $view->getVars();
                    self::$_data['response']['params'] = array_merge(self::$_data['response']['params'], $vars);
                } else {
                    throw new Zend_Controller_Action_Exception('View does not implement the getVars() method needed to encode the view into JSON');
                }
            }
        }

        $this->_format = (null !== $this->_currFormat) ? $this->_currFormat : $this->_format;

        /* server time */
        $serverTime = new Zend_Date();
        self::$_data['response']['server_time'] = $serverTime->toString();

        /* output format */
        switch ($this->_format) {
            case self::FORMAT_JSON:
                $this->_contentType = 'application/json;charset=utf-8';
                $output = Zend_Json::encode(self::$_data, true);
                break;

            case self::FORMAT_XML:
                $this->_contentType = 'text/xml;charset=utf-8';
                $output = $this->_toXml(self::$_data['response']);
                break;
            case self::FORMAT_SERIALIZED:
                $this->_contentType = 'application/php';
                $output = serialize(self::$_data);
                break;
        }

        return $output;
    }
}
