<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Singular_Controller_Action_Helper
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Authorize action helper
 *
 * @uses       Zend_Controller_Action_Helper_Abstract
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Singular_Controller_Action_Helper
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Controller_Action_Helper_Mail
    extends Zend_Controller_Action_Helper_Abstract
{
    /**
     * Mail object
     * 
     * @var Singular_Mail
     */
    protected $_mail = null;
    
    /**
     * Db Table
     * 
     * @var Singular_Db_Table
     */
    protected $_templatesTable = null;
    
    /**
     * Mail settings
     * 
     * @var Zend_Config
     */
    protected $_settings = null;
    
    /**
     * Proxy method
     *
     * @param array|string $to
     * @param string $tpl
     * @param array $replacements
     * @return void
     */
    public function direct ($to, $tpl = null, $replacements = null)
    {
        $this->_initMail();
        
        if (null === $tpl) {
            return $this->_simpleMail($to);
        }
        
        // find template
        $template = $this->_templatesTable->getRowByField('slug', $tpl);
        if (!$template instanceof
            Singular_Core_System_Model_EmailTemplate
        ) {
            throw new Singular_Exception('Email template not found.');
        }
        
        // add recipients
        if (!is_array($to)) {
            $to = array($to);
        }
        foreach ($to as $email) {
            $this->_mail->addTo($email);
        }
        
        // get vars
        $variables = $template->getVariables();
        
        // make body
        $body = $template->getBody();
        foreach ($variables as $var) {
            $body = str_replace(":$var", $replacements[$var], $body);
        }
        $this->_mail->setBodyHtml($body);
        
        // make subject
        $subject = $template->getSubject();
        foreach ($variables as $var) {
            $subject = str_replace(":$var", $replacements[$var], $subject);
        }
        
        // send!
        $this->_mail->setSubject($subject);
        $this->_mail->send();
        $this->_clearData();
    }
    
    /**
     * Clear message
     * 
     * @return void
     */
    protected function _clearData ()
    {
        $this->_mail->clearSubject();
        $this->_mail->clearRecipients();
    }
    
    /**
     * Mail object initialization
     * 
     * @return void
     */
    protected function _initMail ()
    {
        // load templates table
        if (null === $this->_templatesTable) {
            $this->_templatesTable = Singular_Loader
                ::get('Singular_Core_System_Model_DbTable_EmailTemplates');
        }
        
        // load mailing setting
        if (null === $this->_settings) {
            $this->_settings = Singular_Core::_('settings')
                ->getModuleSettings('system', 'mailing');
        }
        
        // init mail
        if (null === $this->_mail) {
            
            // As Bulk?
            $returnPath = null;
            if ($this->_settings->mailing_settings_bulk->value) {
                $from = $this->_getFrom();
                $returnPath = $from['email'];
            }
            
            $config = array();
            $connectionType = $this->_settings->mailing->value;
            
            // smtp trnsport
            if ('smtp' == $connectionType) {
                
                // authenticate?
                if ('yes' == $this->_settings->mailing_smtp_authenticate->value
                ) {
                    $config['auth'] = 'login';
                    $config['username']
                        = $this->_settings->mailing_smtp_username->value;
                    $config['password']
                        = $this->_settings->mailing_smtp_password->value;
                    if ('off' != $this->_settings->mailing_smtp_security->value
                    ) {
                        $config['ssl']
                            = $this->_settings->mailing_smtp_security->value;
                    }
                }
                
                // setup port
                $config['port'] = $this->_settings->mailing_smtp_port->value;
                $transport = new Zend_Mail_Transport_Smtp(
                    $this->_settings->mailing_smtp_host->value, $config
                );
                
            // sendmail transport
            } else {
                $params = !ini_get('safe_mode') && $returnPath ? sprintf(
                    $this->_settings->mailing_native_options->value,
                    $returnPath) : '';
                
                $transport = new Zend_Mail_Transport_Sendmail($params);
            }
            
            // set as default transport
            Singular_Mail::setDefaultTransport($transport);
            
            // make object
            $this->_mail = new Singular_Mail('utf-8');
            $this->_mail->addHeader('X-MailGenerator', 'Singular CMS');
            if ($returnPath) {
                $this->_mail->setReturnPath($returnPath);
            }
            
            // set from
            $from = $this->_getFrom();
            $this->_mail->setFrom($from['email'], $from['name']);
        }
    }
    
    /**
     * Send simple message
     * 
     * @param array $data
     * @return void 
     */
    protected function _simpleMail (array $data)
    {
        $this->_mail->addTo($data['recipient']);
        $this->_mail->setSubject($data['subject']);
        $this->_mail->setBodyHtml($data['body']);
        $this->_mail->send();
        $this->_clearData();
    }
    
    /**
     * Get from part
     * 
     * @return array
     */
    protected function _getFrom ()
    {
        $from = array(
            'email' => $this->_settings->mailing_settings_from_email->value,
            'name'  => $this->_settings->mailing_settings_from_name->value
        );
        
        if (!$from['email']) {
            $from['email'] = Singular_Core::_('settings')
                ->getSetting('site_admin_email');
        }
        
        return $from;
    }
}
