<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Singular_Controller_Action_Helper
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Scaffolding action helper
 *
 * @uses       Zend_Controller_Action_Helper_Abstract
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Singular_Controller_Action_Helper
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Controller_Action_Helper_Scaffolding extends Zend_Controller_Action_Helper_Abstract
{
    /**
     * Unfiltered RAW data
     *
     * @var array
     */
    protected $_data = null;

    /**
     * Use this helper as a method;
     *
     * @return Singular_Controller_Action_Helper_Scaffolding
     */
    public function direct ()
    {
        $this->_initActiveModel();
        
        return $this;
    }

    /**
     * Initialize active model as data provider
     *
     * @throws Zend_Controller_Action_Exception
     * @return void
     */
    protected function _initActiveModel ()
    {
        $aM = $this->getActiveModel();

        if (is_string($aM)) {
            if (!class_exists($aM)) {
                try {
                    Zend_Loader::loadClass($aM);
                } catch (Exception $e) {
                    throw new Zend_Controller_Action_Exception('Incorrect name for activeModel: ' . $e->getMessage());
                }
            }
            
            $this->setActiveModel(new $aM());
        }
    }

    /**
     * Gets active model
     *
     * @return Singular_Db_Table_Abstract
     */
    public function getActiveModel ()
    {
        return $this->_actionController->activeModel;
    }

    /**
     * Sets active model
     *
     * @throws Zend_Controller_Action_Exception
     * @param  Singular_Db_Table_Abstract $model
     * @return Singular_Controller_Action_Helper_Scaffolding
     */
    public function setActiveModel (Singular_Db_Table_Abstract $model)
    {
        if (!$this->isProperlyActiveModel($model)) {
            throw new Zend_Controller_Action_Exception('Active model must be an instance of Singular_Db_Table_Abstract');
        }

        $this->_actionController->activeModel = $model;

        return $this;
    }

    /**
     * Checks active model instance
     *
     * properly active model must be instance of Singular_Db_Table_Abstract
     *
     * @param  null|Singular_Db_Table_Abstract $model
     * @return bool
     */
    public function isProperlyActiveModel ($model = null)
    {
        $aM = (null !== $model) ? $model : $this->_actionController->activeModel;

        return $aM instanceof Singular_Db_Table_Abstract;
    }

    /**
     * Process specified action operation directly
     *
     * actions are supported: list, get, add, edit, delete
     *
     * @param  null|string $action
     * @return void
     */
    public function operate ($action = null)
    {
        $action = (null === $action) ? $this->getRequest()->getActionName() : $action;
        $id = $this->getId();
        $data = $this->getFilteredData();

        switch ($action) {
            case 'list':
                // get params
                $orderBy = $this->getRequest()->getParam('order_by', $this->_detectPrimary());
                $orderType = $this->getRequest()->getParam('order_type', 'DESC');
                $order = $orderBy . ' ' . strtoupper($orderType);
                $count = $this->getRequest()->getParam('count', 500);
                $offset = $this->getRequest()->getParam('offset', null);

                // fetch result
                $result = $this->getActiveModel()->getRows($order, $count, $offset);
                    
                // store result
                $this->getRequest()->setParam('result', $result);
                $this->_actionController->view->result = $result;
                break;

            case 'get':
                if (null === $id) {
                    return $this->_returnResponse('error', 'Couldn\'t get data');
                }

                // fetch result
                $result = $this->getActiveModel()->getRowByField($this->_detectPrimary(), $id);
                $result = (null !== $result) ? $result->toArray() : null;
                    
                // store result
                $this->getRequest()->setParam('result', $result);
                $this->_actionController->view->result = $result;
                break;

            case 'add':
                if (empty($data)) {
                    return $this->_returnResponse('error', 'Data couldn\'t be added');
                }

                // insert
                $this->getActiveModel()->insert($data);
                $this->_returnResponse('success', 'Data has been successfully added');
                break;

            case 'edit':
                if (null === $id && empty($data)) {
                    $this->_returnResponse('error', 'Data couldn\'t be updated');
                }

                // update
                $this->getActiveModel()->updateRowByField($this->_detectPrimary(), $id, $data);
                $this->_returnResponse('success', 'Data has been successfully updated');
                break;

            case 'delete':
                if (null === $id) {
                    $this->_returnResponse('error', 'Data couldn\'t be deleted');
                }

                // delete
                $this->getActiveModel()->deleteRowByField($this->_detectPrimary(), $id);
                $this->_returnResponse('success', 'Data has been successfully deleted');
                break;
        }
    }

    /**
     * Gets request data parameters for using in specified operation
     *
     * automatically detects current request type
     * supports: get and post also maked with xmlHttpRequest(ajax)
     *
     * @return array
     */
    public function getData ()
    {
       if (null !== $this->_data) {
           return $this->_data;
       }

        // get request
        $request = $this->getRequest();

        if ($request->isPost()) {
            $this->_data = $request->getPost();
        } elseif ($request->isGet()) {
            $this->_data = $request->getQuery();
        } else {
            $this->_data = array();
        }

        return $this->_data;
    }

    /**
     * Filters request data parameters
     * according db table fields are provided by specified active model
     *
     * proxies self::_filterFields() method
     *
     * @see    Singular_Controller_Action_Helper_Scaffolding::getFilteredData()
     * @return array
     */
    public function getFilteredData ()
    {
        return $this->_filterFields($this->getData());
    }

    /**
     * Return db table primary id field name
     *
     * @return string
     */
    public function getId ()
    {
        $data = $this->getData();
        $primary = $this->_detectPrimary();

        return (isset($data[$primary])) ? $data[$primary] : null;
    }

    /**
     * Detects primary key according db table schema
     *
     * @return string
     */
    protected function _detectPrimary ()
    {
        $primary = $this->getActiveModel()->info('primary');
        
        return (!empty($primary)) ? current($primary) : null;
    }

    /**
     * Filters request data parameters
     *
     * @throws Zend_Controller_Action_Exception
     * @param  array $fields
     * @return array
     */
    protected function _filterFields ($fields)
    {
        if (!is_array($fields)) {
            throw new Zend_Controller_Action_Exception('Input parameter \"$fileds\" must be an array. ' . gettype($fields) . ' was given.');
        }

        $original = $this->getActiveModel()->info('cols');
        
        foreach ($fields as $field => $value) {
            if (!in_array($field, $original) || is_numeric($field)) {
                unset($fields[$field]);
            }
        }

        // remove primary key if exists
        $primary = $this->_detectPrimary();
        if (null !== $primary) {
            if (array_key_exists($primary, $fields)) unset($fields[$primary]);
        }

        return $fields;
    }

    /**
     * Returns response
     *
     * @param  string|null $type
     * @param  string|null $message
     * @return void
     */
    protected function _returnResponse ($type = null, $message = null)
    {
        $isAjax = $this->getRequest()->isXmlHttpRequest();
        $view = $this->_actionController->view;

        switch ($type) {
            case 'success':
                if ($isAjax) {
                    $this->_actionController->autoresponder->setSuccess($view->t($message));
                } else {
                    $view->notify()->success($view->t($message));
                }
                break;
            
            case 'error':
                if ($isAjax) {
                    $this->_actionController->autoresponder->setError($view->t($message));
                } else {
                    $view->notify()->error($view->t($message));
                }
                break;
        }
    }
}
