<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Auth core resource accessor
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_Auth
{
    /**
     * @static
     * @var Zend_Auth_Adapter_DbTable
     */
    private static $_authAdapter = null;

    /**
     * Guest role data.
     *
     * @var array
     */
    private $_guestRoleData = array(
        'role'      => 'guest',
        'full_name' => 'Guest',
        'email'     => 'guest@localhost'
    );

    /**
     * Class constructor
     */
    public function __construct()
    {
        if (null === self::$_authAdapter) {
            self::setupAdapter();
        }
    }

    /**
     * Gets auth adapter
     *
     * @see    Singular_Core_Auth::$_authAdapter
     * @return null|Zend_Auth_Adapter_DbTable
     */
    public function getAdapter ()
    {
        return self::$_authAdapter;
    }

    /**
     * Logins user with specified identity and credential
     *
     * most used case is authenticate user via form
     *
     * @param  string $identity
     * @param  string $credential
     * @param  null|string $successRedirectURI
     * @return bool
     */
    public function login ($identity, $credential, $successRedirectURI = null)
    {
        $result = $this->authenticate($identity, $credential);

        if ($result->isValid()) {
            $baseUrl = Zend_Controller_Front::getInstance()->getBaseUrl();
            $successRedirectURI = (null == $successRedirectURI)
                    ? $baseUrl. '/'
                    : $successRedirectURI;

            $this->_getRedirector()->gotoUrlAndExit($successRedirectURI);

            return true;
        }

        return false;
    }

    /**
     * Logout current logged in user
     *
     * @uses   Singular_Core_Auth::clearIdentity()
     * @param  null|string $redirectURI
     * @return void
     */
    public function logout ($redirectURI = null)
    {
        return $this->clearIdentity($redirectURI);
    }

    /**
     * Authenticate user with specified identity and credential
     *
     * most used case is authenticate user inline in script
     *
     * @param string $identity
     * @param string $credential
     * @return Zend_Auth_Result
     */
    public function authenticate ($identity, $credential)
    {
        $auth = Zend_Auth::getInstance();
        $adapter = $this->getAdapter();
        $adapter->setIdentity($identity)
                ->setCredential(self::passwordHash($credential));

        $config = Singular_Runtime::extract('config');
        $isActiveCol = $config->resources->auth->columns->is_active;
        $isActiveAllowVal = $config->resources->auth->allow_value;

        /**
         * @see APPLICATION_PATH/configs/application.ini -> resources.auth
         */
        if (null != $isActiveCol && null != $isActiveAllowVal) {
            $adapter->getDbSelect()->where("{$isActiveCol} = ?", $isActiveAllowVal);
        }

        $authEvent = (object) array('auth' => $auth, 'adapter' => $adapter);

        Singular_Event::dispatch('beforeAuth', $authEvent);
        
        $result = $auth->authenticate($adapter);

        if ($result->isValid()) {
            $auth->getStorage()->write($adapter->getResultRowObject());
            Singular_Event::dispatch('afterAuth', $authEvent);
        }
        
        return $result;
    }

    /**
     * Gets logged in user identity data
     *
     * @param  bool $asArray
     * @return mixed|null
     */
    public function getIdentity ($asArray = false)
    {
        $returnType = (false === (bool) $asArray) ? 'object' : 'array';
        $identity = Zend_Auth::getInstance()->getIdentity();
        settype($identity, $returnType);

        return $identity;
    }
    
    /**
     * Gets logged in user identity data as db row
     * 
     * @param void
     * @return Singular_Core_System_Model_User
     */
    public function getLiveIdentity ()
    {
        $userId = $this->getIdentity()->id;
        
        if ($userId) {
            return Singular_Loader::get('Singular_Core_System_Model_DbTable_Users')->find($userId)->current();
        }
    }

    /**
     * Returns true only if an identity is available
     * 
     * @return bool
     */
    public function hasIdentity ()
    {
        return Zend_Auth::getInstance()->hasIdentity();
    }

    /**
     * Clears authentication identity
     *
     * @param null $redirectURI
     * @return void
     */
    public function clearIdentity ($redirectURI = null)
    {
        $auth = Zend_Auth::getInstance();
        $auth->clearIdentity();
        $auth->getStorage()->clear();

        $baseUrl = Zend_Controller_Front::getInstance()->getRequest()->getBaseUrl();
        $redirectURI = (is_string($redirectURI))
                ? $baseUrl . $redirectURI
                : $baseUrl . '/';
        $this->_getRedirector()
                ->gotoUrl($redirectURI);
    }

    /**
     * Gets generated password hash
     *
     * @param  string $password
     * @return string
     */
    public function makePassword ($password)
    {
        return self::passwordHash($password);
    }

    /**
     * Returns Logged User.
     *
     * @return System_Model_User
     */
    public function getLoggedUser()
    {
        $loggedUser = null;
        if (Singular_Runtime::inStorage('loggedUser')) {
            $loggedUser = Singular_Runtime::extract('loggedUser');
            if (!$loggedUser instanceof System_Model_User) {
                $loggedUser = null;
            } else {
                if (null == $loggedUser->email) {
                    $loggedUser->setFromArray($this->_guestRoleData);
                }
            }
        }

        if (null === $loggedUser) {
            if ($this->hasIdentity()) {
                $loggedUser = new System_Model_User();
                $loggedUser->setFromArray($this->getIdentity(true));
            }
        }

        if (null === $loggedUser) {
            $loggedUser = new System_Model_User();
            $loggedUser->setFromArray($this->_guestRoleData);
        }

        return $loggedUser;
    }

    /**
     * Returns filtered Logged User data without any security-dependent fields.
     *
     * @return array
     */
    public function getPublicLoggedUserData()
    {
        $userData = $this->getLoggedUser()->toArray();
        $params   = array(
            'id' => null, 'full_name' => null, 'email' => null, 'photo' => null, 'role' => null, 'is_trusted'
        );

        return array_uintersect_assoc($userData, $params, function () {
            return false;
        });
    }

    /**
     * Gets redirector helper
     *
     * @uses   Zend_Controller_Action_Helper_Redirector
     * @return Zend_Controller_Action_Helper_Abstract
     */
    protected function _getRedirector ()
    {
        return Zend_Controller_Action_HelperBroker::getStaticHelper('redirector');
    }

    /**
     * Setup authentication adapter
     *
     * @uses   Zend_Auth_Adapter_DbTable
     * @static
     * @param  string|null $tableName
     * @param  string|null $identityColumn
     * @param  string|null $credentialColumn
     * @return void
     */
    public static function setupAdapter ($tableName = null, $identityColumn = null, $credentialColumn = null)
    {
        $config = Singular_Runtime::extract('config');

        // get parameters for DbTable adapter
        $tableName = (null == $tableName)
                ? $config->resources->auth->table_name : $tableName;
        $identityColumn = (null == $identityColumn)
                ? $config->resources->auth->columns->identity : $identityColumn;
        $credentialColumn = (null == $credentialColumn)
                ? $config->resources->auth->columns->credential : $credentialColumn;

        $adapter = new Zend_Auth_Adapter_DbTable(
            Singular_Db_Table_Abstract::getDefaultAdapter()
        );
        $tablePrefix = $config->resources->db->params->prefix;

        $adapter->setTableName($tablePrefix . $tableName)
                ->setIdentityColumn($identityColumn)
                ->setCredentialColumn($credentialColumn);

        self::$_authAdapter = $adapter;

        // store in runtime
        Singular_Runtime::store('auth_adapter', self::$_authAdapter, true);
    }

    /**
     * Password hash generator
     *
     * @static
     * @param  string $password
     * @return string
     */
    public static function passwordHash ($password)
    {
        $password = strtolower($password);
        
        return md5(
            str_repeat(
                md5($password) . strrev($password) . sha1($password),
                strlen($password)
            )
        );
    }
}
