<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Cache core resource accessor
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_Cache
{
    /**
     * @var Zend_Cache_Manager
     */
    public $manager = null;

    /**
     * Class constructor
     */
    public function __construct()
    {
        if (null === $this->manager) {
            $this->manager = Singular_Runtime::extract('cachemanager');
        }
    }

    /**
     * Get cache manager
     *
     * @return Zend_Cache_Manager
     */
    public function getManager ()
    {
        return $this->manager;
    }

    /**
     * Fetch the named cache object, or instantiate and return a cache object
     * using a named configuration template
     *
     * @see Zend_Cache_Core::getCache
     * @throws Singular_Core_Exception
     * @param string $name
     * @return Zend_Cache_Core
     */
    public function getCache ($name)
    {
        if ($this->hasCache($name)) {
            return $this->manager->getCache($name);
        }

        throw new Singular_Core_Exception("Can't find cache \"{$name}\" in Zend_Cache_Manager");
    }

    /**
     * Check if the Cache Manager contains the named cache object, or a named
     * configuration template to lazy load the cache object
     *
     * @see Zend_Cache_Core::hasCache
     * @param string $name
     * @return bool
     */
    public function hasCache ($name)
    {
        return $this->manager->hasCache($name);
    }

    /**
     * Disables all caches in cachemanager
     *
     * @return void
     */
    public function disableAll ()
    {
        $caches = $this->manager->getCaches();
        foreach ($caches as $name => $cache) {
            if ($cache instanceof Zend_Cache_Core) {
                $cache->setOption('caching', false);
                $cache->setBackend(new Zend_Cache_Backend_BlackHole());
                $cache->clean(Zend_Cache::CLEANING_MODE_ALL);
                $this->manager->setCache($name, $cache);
            }
        }
    }

    /**
     * Removes all cache by given id
     *
     * @see Zend_Cache_Core::remove
     * @param int $id
     */
    public function removeAllById ($id)
    {
        $caches = $this->manager->getCaches();
        /** @var $cache Zend_Cache_Core */
        foreach ($caches as $cache) {
            $cache->remove($id);
        }
    }

    /**
     * Removes all cache by tags
     *
     * @param array  $tags
     * @param string $mode
     */
    public function removeAllByTags($tags, $mode = Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG)
    {
        $caches = $this->manager->getCaches();
        /** @var $cache Zend_Cache_Core */
        foreach ($caches as $cache) {
            $cache->clean($mode, $tags);
        }
    }

    /**
     * Cleans all cache
     *
     * @return void
     */
    public function cleanAll ()
    {
        $this->_clean('all');
    }

    /**
     * Cleans only expired cache
     *
     * @return void
     */
    public function cleanOld ()
    {
        $this->_clean('old');
    }

    /**
     * Cleans cache by given mode
     *
     * @see Zend_Cache_Core::clean
     * @param string $mode
     * @return void
     */
    protected function _clean ($mode = 'all')
    {
        $caches = $this->manager->getCaches();

        /** @var $cache Zend_Cache_Core */
        foreach ($caches as $cache) {
            $cache->clean($mode);
            $cache->setOption('caching', false);
        }
        if (class_exists('Supercache', false)) {
            $sc = SuperCache::getInstance();
            $sc->getCache()->clean(Zend_Cache::CLEANING_MODE_ALL);
            $sc->stop();
        }
    }
}
