<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Settings core resource accessor
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_Settings
{
    /**
     * Settings constants
     */
    const SETTINGS_ALL_FORMATTED = 'formatted';
    const SETTINGS_KEY_VAL       = 'key_val';
    const SETTINGS_BOTH          = 'both';

    /**
     * Settings model
     * 
     * @var Singular_Core_System_Model_DbTable_Settings
     */
    private $_model = null;

    /**
     * Settings
     *
     * @var array
     */
    protected static $_settings = null;

    /**
     * Cache key
     *
     * @var string
     */
    public static $cacheKey = __CLASS__;

    /**
     * Class constructor
     */
    public function __construct()
    {
        $this->_setModel();
        $this->_prepareSettings();
    }

    /**
     * Retrieve settings model
     *
     * @return Singular_Core_System_Model_DbTable_Settings
     */
    public function getModel ()
    {
        return $this->_model;
    }

    /**
     * Sets settings model
     *
     * @return void
     */
    protected function _setModel ()
    {
        $this->_model = new Singular_Core_System_Model_DbTable_Settings();
    }

    /**
     * Formats settings array as module->section->key structure
     *
     * @param  array $settings
     * @return array
     */
    protected function _formatSettingsStructure ($settings)
    {
        if (is_array($settings)) {
            $formatArr = array();
            $keyValArr = array();
            foreach ($settings as $s) {
                if (is_array($s)) {
                    $module = $s['module'];
                    $section = $s['section'];
                    $keyValArr[$s['key']] = $s;
                    unset($s['module'], $s['section']);
                    $formatArr[$module][$section][$s['key']] = $s;
                }
            }
            return array(
                self::SETTINGS_ALL_FORMATTED => $formatArr,
                self::SETTINGS_KEY_VAL => $keyValArr
            );
        }
    }

    /**
     * Prepares settings data
     *
     * @return void
     */
    private function _prepareSettings ()
    {
        if (null === self::$_settings) {
            if (Singular_Runtime::inStorage('cachemanager')) {
                $cache = Singular_Runtime::extract('cachemanager')->getCache('default');
                if (!$data = $cache->load(self::$cacheKey)) {
                    $settingsRaw = $this->getModel()->getSettingsExtended();
                    $data = $this->_formatSettingsStructure($settingsRaw);
                    $cache->save($data, self::$cacheKey, array($this->getModel()->getTableName()));
                }
            } else {
                $settingsRaw = $this->getModel()->getSettingsExtended();
                $data = $this->_formatSettingsStructure($settingsRaw);
            }
            
            $this->setSettings($data);
        }
    }

    /**
     * Cleans settings cache
     * 
     * @return void
     */
    private function _cleanCache ()
    {
        if (Singular_Runtime::inStorage('cachemanager')) {
            $cache = Singular_Runtime::extract('cachemanager')->getCache('database');
            $cache->remove(self::$cacheKey);
        }

        self::$_settings = null;
    }

    /**
     * Sets settings array
     *
     * @throws Singular_Core_Exception
     * @param  array $settings
     * @return void
     */
    public function setSettings ($settings)
    {
        if (is_array($settings)) {
            self::$_settings = $settings;
        } else {
            throw new Singular_Core_Exception("Settings must be an array.\n" . ucfirst(gettype($settings)) . ' was given.');
        }
    }

    /**
     * Retrieve settings
     *
     * @return Zend_Config
     */
    public function getSettings ()
    {
        return $this->_getSettings();
    }

    /**
     * Retrieve setting by key
     *
     * @param  string $name
     * @param  bool $fullData
     * @return array|string
     */
    public function getSetting ($name, $fullData = false)
    {
        $settings = $this->_getSettings(self::SETTINGS_KEY_VAL);
        if (isset($settings->$name)) {
            return (true === (bool) $fullData)
                    ? $settings->$name
                    : $settings->$name->value;
        }
    }

    /**
     * Returns true if setting key is exists
     *
     * @param  string $name
     *
     * @return bool
     */
    public function hasSetting($name)
    {
        $settings = $this->_getSettings(self::SETTINGS_KEY_VAL);
        return isset($settings->$name);
    }

    /**
     * Retrieve settings by module name
     *
     * @throws Singular_Core_Exception
     * @param  sttring $moduleName
     * @param  null|string $section
     * @return Zend_Config
     */
    public function getModuleSettings ($moduleName, $section = null)
    {
        $settings = $this->_getSettings(self::SETTINGS_ALL_FORMATTED);
        if (isset($settings->$moduleName)) {
            if (null !== $section) {
                if (isset($settings->$moduleName->$section)) {
                    return $settings->$moduleName->$section;
                } else {
                    throw new Singular_Core_Exception("Section name \"{$section}\" is not exists.");
                }
            }
            
            return $settings->$moduleName;
        }
        throw new Singular_Core_Exception("Module name \"{$moduleName}\" is not exists.");
    }

    /**
     * Retrieve all available settings
     *
     * @return Zend_Config
     */
    public function getAllSettings ()
    {
        return $this->_getSettings(self::SETTINGS_KEY_VAL);
    }

    /**
     * Changes setting
     *
     * if $data an array
     * @see    Singular_Core_System_Model_DbTable_Settings::changeSetting()
     *         if $data is an string
     * @see    Singular_Core_System_Model_DbTable_Settings::setSettingValue()
     * @throws Singular_Core_Exception
     * @param  string        $name     Setting name
     * @param  array|string  $data     Data array according settings table row fields
     * @return bool
     */
    public function changeSetting ($name, $data)
    {
        if (!$this->hasSetting($name)) {
            return false;
        }

        /* check data value datatype */
        if (is_array($data)) {
            $this->getModel()->changeSetting($name, $data);
        } else {
            $error = false;
            if (is_object($data)) {
                if (method_exists($data, 'toString')) {
                    $data = $data->toString();
                } else $error = true;
            } elseif (is_resource($data)) $error = true;

            /* error */
            if ($error) {
                throw new Singular_Core_Exception(
                    'Invalid argument: $data must be string|integer|array|object::toString() ' . ucfirst(gettype($data)) . ' was given.'
                );
            }

            /* apply setting value */
            $this->getModel()->setSettingValue($name, $data);
        }
        
        /* clean cache */
        $this->_cleanCache();

        return true;
    }

    /**
     * Deletes setting by key
     * 
     * @param  string $name
     * @return void
     */
    public function deleteSetting ($name)
    {
        $this->getModel()->deleteSetting($name);

        // clean cache
        $this->_cleanCache();
    }

    /**
     * Deletes settings by module name
     *
     * if $section is defined - deletes settings that are in specified section
     *
     * @param  string $moduleName
     * @param  null|string $section
     * @return void
     */
    public function deleteModuleSettings ($moduleName, $section = null)
    {
        $this->getModel()->deleteModuleSettings($moduleName, $section);

        // clean cache
        $this->_cleanCache();
    }

    /**
     * Adds settings
     *
     * @param  array $settings
     * @return void
     */
    public function addSettings ($settings)
    {
        $this->getModel()->addModuleSettings($settings);

        // clean cache
        $this->_cleanCache();
    }

    /**
     * Retrieve settings in determined format according $settingsType value
     *
     * @param  string $settingsType
     * @return Zend_Config
     */
    protected function _getSettings ($settingsType = self::SETTINGS_ALL_FORMATTED)
    {
        switch ($settingsType) {
            case self::SETTINGS_BOTH:
                self::$_settings;
            break;
            case self::SETTINGS_KEY_VAL:
                return new Zend_Config(self::$_settings[self::SETTINGS_KEY_VAL]);
            break;
            default:
            case self::SETTINGS_ALL_FORMATTED:
                return new Zend_Config(self::$_settings[self::SETTINGS_ALL_FORMATTED]);
            break;
        }
    }
}
