<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @subpackage Singular_Core_System_Model_DbTable
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Singular_Db_Table_Abstract
 * @throws     Singular_Exception
 * @category   Singular
 * @package    Singular_Core
 * @subpackage Singular_Core_System_Model_DbTable
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_System_Model_DbTable_Settings
    extends Singular_Db_Table_Abstract
{
    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'settings';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'id';

    /**
     * Retrieve settings
     *
     * @param  bool $extendedFormat
     * @return array|Zend_Db_Table_Rowset_Abstract
     */
    public function getSettings ($extendedFormat = false)
    {
        return (false === (bool) $extendedFormat)
                ? $this->getSettingsKeyVal()
                : $this->getSettingsExtended();
    }

    /**
     * Retrieve settings as raw rowset array
     *
     * @return array
     */
    public function getSettingsExtended ()
    {
        $select = $this->select()->order('id ASC');
        $settings = $this->fetchAll($select);
        if (null !== $settings) {
            $settings = $settings->toArray();
        }

        return $settings;
    }

    /**
     * Retrieve settings as key=>val array
     * where key - setting name, val - setting value
     *
     * @return array
     */
    public function getSettingsKeyVal ()
    {
        $select = $this->getAdapter()->select();
        $select->from($this->_name, array('key', 'value'));

        return $this->getAdapter()->fetchPairs($select);
    }

    /**
     * Retrieve settings by module name
     * 
     * @param  string $moduleName
     * @param  null $sectionName
     * @return array
     */
    public function getModuleSettings ($moduleName, $sectionName = null)
    {
        $select = $this->getAdapter()->select();
        $select->from($this->_name);
        $select->where('module = ?', $moduleName);
        if (is_string($sectionName)) {
            $select->where('section = ?', $sectionName);
        }
        $select->order(array('module ASC', 'section'));

        return $this->getAdapter()->fetchAll($select);
    }

    /**
     * Retrieve setting by key
     *
     * @param  string $keyName
     * @param  bool $asArray
     * @return array|string
     */
    public function getSetting ($keyName, $asArray = true)
    {
        $setting = $this->fetchRow($this->select()->where('`key` = ?', $keyName));

        return (true === (bool) $asArray)
            ? ($setting !== null)
                ? $setting->toArray()
                : null
            : $setting;
    }

    /**
     * Changes setting value
     *
     * @param  string $key
     * @param  string $value
     * @return void
     */
    public function setSettingValue ($key, $value)
    {
        $setting = $this->getSetting($key, false);
        $setting->value = $value;
        $setting->save();
        
        // clean cache
        $this->cleanCache();
    }

    /**
     * Changes setting data following row fields structure in "settings" table
     *
     * @param  string $key
     * @param  array $data
     * @return void
     */
    public function changeSetting ($key, $data)
    {
        $setting = $this->getSetting($key, false);

        // prevent overwrite setting name
        if (array_key_exists('key', $data)) {
            $data['key'] = $key;
        }
        
        $setting->setFromArray($data);
        $setting->save();
        
        // clean cache
        $this->cleanCache();
    }

    /**
     * Deletes setting by key name
     *
     * @param  string $key
     * @return void
     */
    public function deleteSetting ($key)
    {
        $this->deleteRowByField('key', $key);

        // clean cache
        $this->cleanCache();
    }

    /**
     * Deletes settings by module name
     *
     * if $section is defined - deletes settings that are in specified section
     * 
     * @param  string $moduleName
     * @param  string|null $section
     * @return void
     */
    public function deleteModuleSettings ($moduleName, $section = null)
    {
        $where['module = ?'] = $moduleName;
        if (is_string($section)) {
            $where['section = ?'] = $section;
        }
        $this->delete($where);

        // clean cache
        $this->cleanCache();
    }

    /**
     * Adds module settings following row fields structure in "settings" table
     *
     * @throws Singular_Exception
     * @param  array $settings
     * @return void
     */
    public function addModuleSettings ($settings)
    {
        if (!is_array($settings)) {
            throw new Singular_Exception('Settings must be an array.');
        }
        
        // check for multi-settings
        if (isset($settings[0]) && is_array($settings[0])) {
            foreach ($settings as $setting) {
                $this->insert($setting);
            }
        } else {
            // if is a single setting
            $this->insert($settings);
        }

        // clean cache
        $this->cleanCache();
    }
}
