<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Taxonomy core resource accessor
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_Taxonomy
{
    const NODE_STRUCTURE_CHILDREN = 'children';

    const IS_PROTECTED_FIELD = 'is_protected';
    const DEPTH_FIELD        = 'depth';

    /**
     * Taxonomy model
     * 
     * @var Singular_Core_System_Model_DbTable_Taxonomy
     */
    private static $_model = null;

    /**
     * Class constructor
     */
    public function __construct()
    {
        if (null === self::$_model) {
            $this->_setModel();
        }
    }

    /**
     * Retrieve taxonomy model
     *
     * @return Singular_Core_System_Model_DbTable_Taxonomy
     */
    public function getModel()
    {
        return self::$_model;
    }

    /**
     * Taxonomy model setter
     * 
     * @return void
     */
    protected function _setModel()
    {
        self::$_model = new Singular_Core_System_Model_DbTable_Taxonomy();
    }

    /**
     * Adds one or more nodes into taxonomy
     *
     * $nodes must be an array in the right key=>val format according taxonomy table row fields
     * if $objectiveNodeId is defined - appends nodes to specified node id
     *
     * @param  array $nodes
     * @param  null|int $objectiveNodeId
     * @return void
     */
    public function addNodeStructure ($nodes, $objectiveNodeId = null)
    {
        if (is_array($nodes)) {
            if (array_key_exists(self::NODE_STRUCTURE_CHILDREN, $nodes)) {
                $insertData = $nodes;
                unset($insertData[self::NODE_STRUCTURE_CHILDREN]);
                $nodeId = $this->getModel()->insert(
                    $insertData,
                    $objectiveNodeId,
                    Singular_Db_Table_NestedSet::LAST_CHILD);
                $this->addNodeStructure(
                    $nodes[self::NODE_STRUCTURE_CHILDREN],
                    $nodeId);
            } else {
                foreach ($nodes as $nodeKey => $nodeVal) {
                    if (is_array($nodeVal)) {
                        $this->addNodeStructure($nodeVal, $objectiveNodeId);
                    } else {
                        $this->getModel()->insert(
                            $nodes,
                            $objectiveNodeId,
                            Singular_Db_Table_NestedSet::LAST_CHILD);
                        break;
                    }
                }
            }
        }
    }

    /**
     * Retrieve node tree in specified format if $format is defined
     *
     * if (int) $parentId is defined - fetch will be started from $parentId value and lower
     *
     * @param  null|int $parentId
     * @param  bool $format
     * @return array
     */
    public function getNodeTree ($parentId = null, $format = false)
    {
        $where = null;
        if (is_numeric($parentId)) {
            $node = $this->getNodeById($parentId);
            $where = $this->getModel()
                    ->getAdapter()
                    ->quoteInto(Singular_Db_Table_NestedSet::LEFT_TBL_ALIAS
                                . '.lft >= ?',
                                $node['lft'], Zend_Db::INT_TYPE);
            $where .= $this->getModel()
                    ->getAdapter()
                    ->quoteInto(' AND '
                                . Singular_Db_Table_NestedSet::LEFT_TBL_ALIAS
                                . '.rgt <= ?',
                                $node['rgt'], Zend_Db::INT_TYPE);
        }
        $result = $this->getModel()->cache_getTree($where);
        
        $result = (null !== $result)
                ? $result->toArray()
                : $result;

        if (true === (bool) $format && null !== $result) {
            $result = $this->formatNodeStructure($result);
        }

        return $result;
    }

    /**
     * Retrieve node by id
     *
     * @param  int $id
     * @return array|null
     */
    public function getNodeById ($id)
    {
        return $this->getNodeByField('id', $id);
    }

    /**
     * Retrieve node where specified $field is field name in row and $value is it's value
     *
     * @param  string $field
     * @param  string|int|float $value
     * @return array|null|Zend_Db_Table_Row_Abstract
     */
    public function getNodeByField ($field, $value)
    {
        $type = (is_numeric($value) || is_int($value))
                ? Zend_Db::INT_TYPE
                : null;
        
        $where = $this->getModel()
                ->getAdapter()
                ->quoteInto("{$field} = ?", $value, $type);
        $result = $this->getModel()->fetchRow($where, 'parent_id');

        return (null !== $result) ? $result->toArray() : $result;
    }

    /**
     * Adds node with specefied row fields values
     *
     * @param  array $data
     * @param  int|null $objectiveNodeId
     * @param  string $position
     * @return int
     */
    public function addNode ($data, $objectiveNodeId = null,
        $position = Singular_Db_Table_NestedSet::LAST_CHILD)
    {
        $result = $this->getModel()->insert($data, $objectiveNodeId, $position);
        $this->getModel()->cleanCache();

        return $result;
    }

    /**
     * Deletes node by id
     *
     * @param  int $id
     * @param  bool $recursive
     * @return array|bool
     */
    public function deleteNode ($id, $recursive = true)
    {
        $node = $this->getNodeById($id);
        $result = (1 != $node[self::IS_PROTECTED_FIELD])
                ? $this->getModel()->deleteNode($id, $recursive)
                : false;
        $this->getModel()->cleanCache();

        return $result;
    }

    /**
     * Updates node data
     *
     * @param  array $data
     * @param  int $id
     * @return int
     */
    public function updateNodeData ($data, $id)
    {
        if (isset($data['lft'])) unset($data['lft']);
        if (isset($data['rgt'])) unset($data['rgt']);
        $where = $this->getModel()
                ->getAdapter()
                ->quoteInto('id = ?', $id, Zend_Db::INT_TYPE);
        $result = $this->getModel()->update($data, $where);
        $this->getModel()->cleanCache();
        
        return $result;
    }

    /**
     * Moves node with $id into node with $objectiveNodeId
     *
     * @param  int $id
     * @param  int $objectiveNodeId
     * @param  string $position
     * @return bool|mixed
     */
    public function moveNode ($id, $objectiveNodeId,
        $position = Singular_Db_Table_NestedSet::LAST_CHILD)
    {
        $node = $this->getNodeById($id);
        $result = (1 != $node[self::IS_PROTECTED_FIELD])
                ? $this->getModel()
                        ->updateNode(array(), $id, $objectiveNodeId, $position)
                : false;
        $this->getModel()->cleanCache();

        return $result;
    }

    /**
     * Apply format to node RAW tree
     *
     * @param  array $data
     * @return array
     */
    public function formatNodeStructure ($data = array())
    {
        if (empty($data) || !is_array($data)) {
            $nodes = $this->getNodeTree();
        }
        else {
            $nodes = $data;
        }

        $result = array();
        $stackLevel = 0;

        if (count($nodes) > 0) {
            // Node Stack. Used to help building the hierarchy
            $stack = array();

            foreach ($nodes as $node) {
                $node[self::NODE_STRUCTURE_CHILDREN] = array();

                // Number of stack items
                $stackLevel = count($stack);

                // Check if we're dealing with different levels
                while ($stackLevel > 0 && $stack[$stackLevel - 1][self::DEPTH_FIELD] >= $node[self::DEPTH_FIELD]) {
                    array_pop($stack);
                    $stackLevel--;
                }

                // Stack is empty (we are inspecting the root)
                if ($stackLevel == 0) {
                    // Assigning the root node
                    $i = count($result);

                    // $result[$i] = $item;
                    $result[$i] = $node;
                    $stack[] =& $result[$i];
                }
                else {
                    // Add node to parent
                    $i = count($stack[$stackLevel - 1][self::NODE_STRUCTURE_CHILDREN]);

                    $stack[$stackLevel - 1][self::NODE_STRUCTURE_CHILDREN][$i] = $node;
                    $stack[] =& $stack[$stackLevel - 1][self::NODE_STRUCTURE_CHILDREN][$i];
                }
            }
        }

        return $result;
    }

    /**
     * Converts node tree into JSON format
     *
     * @param  array $data
     * @param  bool $prettyPrint
     * @return string
     */
    public function toJson ($data, $prettyPrint = false)
    {
        $data = $this->formatNodeStructure($data);
        $result = Zend_Json::encode($data);
        if (true === (bool) $prettyPrint) {
            $result = Zend_Json::prettyPrint($result, array("indent" => " "));
        }

        return $result;
    }
}
