<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Settings core resource accessor
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_UserSettings
{
    /**
     * Cache key
     */
    const CACHE_KEY = 'user_settings_';
    
    /**
     * Settings model
     * 
     * @var Singular_Core_System_Model_DbTable_UserSettings
     */
    protected $_model = null;

    /**
     * Settings
     *
     * @var array
     */
    protected static $_settings = array();

    /**
     * Class constructor
     */
    public function __construct ()
    {
        $this->_model = new Singular_Core_System_Model_DbTable_UserSettings();
    }
    
    /**
     * Get value by name and user
     * 
     * @param string $name
     * @param Singular_Core_System_Model_User $user
     * @return mixed 
     */
    public function getValue ($name, $user = null)
    {
        if ($user instanceof Singular_Core_System_Model_User) {
            self::$_settings[$user->id] = $this->_getUserSettings($user);
            if ($this->hasValue($name, $user)) {
                return self::$_settings[$user->id][$name];
            }
        }
        
        return $this->_getDefaultValue($name);
    }
    
    /**
     * User has value?
     * 
     * @param string $name
     * @param Singular_Core_System_Model_User $user
     * @return bool
     */
    public function hasValue ($name, Singular_Core_System_Model_User $user = null)
    {
        if (!$user instanceof Singular_Core_System_Model_User) {
            return false;
        }
        
        self::$_settings[$user->id] = $this->_getUserSettings($user);
        
        return isset(self::$_settings[$user->id][$name]);
    }
    
    /**
     * Change setting
     * 
     * @param string $name
     * @param mixed $value
     * @param Singular_Core_System_Model_User $user
     * @return bool
     */
    public function setValue ($name, $value, Singular_Core_System_Model_User $user)
    {
        $userSetting = $this->_model->findByUserAndName($user, $name);

        if ($userSetting instanceof Singular_Core_System_Model_UserSetting) {
            $userSetting->value = $value;
        } else {
            $userSetting = $this->_model->createRow(array(
                'value' => $value,
                'user_id' => $user->id,
                'name' => $name,
            ));
        }
        
        $save = $userSetting->save();
        if ($save) {
            $this->_cleanCache($user);
            return true;
        }
        
        return false;
    }
    
    /**
     * Remove setting
     * 
     * @param string $name
     * @param Singular_Core_System_Model_User $user
     * @return bool
     */
    public function removeValue ($name, Singular_Core_System_Model_User $user)
    {
        $userSetting = $this->_model->findByUserAndName($user, $name);
        
        if ($userSetting instanceof Singular_Core_System_Model_UserSetting) {
            $delete = $userSetting->delete();
            if ($delete) {
                $this->_cleanCache($user);
                return true;
            }
        }
        
        return false;
    }
    
    /**
     * Remove all settings by name
     * 
     * @param string $name 
     * @return bool
     */
    public function removeByName ($name)
    {
        $rowset = $this->_model->findByName($name);
        if ($rowset instanceof Zend_Db_Table_Rowset) {
            $removed = $this->_model->removeByName($name);
            if ($removed) {
                foreach ($rowset as $row) {
                    $this->_cleanCache($row->user_id);
                }
                return true;
            }
        }
        
        return false;
    }
    
    /**
     * Remove all settings by user
     * 
     * @param Singular_Core_System_Model_User $user
     * @return bool
     */
    public function removeByUser (Singular_Core_System_Model_User $user)
    {
        $removed = $this->_model->removeByUser($user);
        
        if ($removed) {
            $this->_cleanCache($user);
            return true;
        }
        
        return false;
    }
    
    /**
     * Fetch user settings
     * 
     * @param Singular_Core_System_Model_User $user
     * @return array 
     */
    protected function _getUserSettings (Singular_Core_System_Model_User $user)
    {
        // Memory settings
        if (isset(self::$_settings[$user->id]) && !empty(self::$_settings)) {
            return self::$_settings[$user->id];
        }
        
        // Cached settings
        $cachedSettings = $this->_getFromCache($user);
        if (is_array($cachedSettings)) {
            return $cachedSettings;
        }
        
        // Settings from database
        $settings = $this->_model->findByUser($user);
        if (is_array($settings) && count($settings)) {
            $this->_setToCache($settings, $user);
            return $settings;
        }
        
        // Default empty array
        return array();
    }
    
    /**
     * Gets variables from cache instance
     * 
     * @param Singular_Core_System_Model_User $user
     * @return array
     */
    protected function _getFromCache (Singular_Core_System_Model_User $user)
    {
        $cacheId = self::CACHE_KEY . $user->id;
        return Singular_Runtime::extract('cachemanager')
            ->getCache('default')->load($cacheId);
    }
    
    /**
     * Sets variables to cache instance
     * 
     * @param array $settings
     * @param Singular_Core_System_Model_User $user
     * @return bool
     */
    protected function _setToCache ($settings, Singular_Core_System_Model_User $user)
    {
        $cacheId = self::CACHE_KEY . $user->id;
        return Singular_Runtime::extract('cachemanager')
            ->getCache('default')->save($settings, $cacheId);
    }

    /**
     * Cleans settings cache
     * 
     * @param Singular_Core_System_Model_User|int $user
     * @return void
     */
    private function _cleanCache ($user)
    {
        if ($user instanceof Singular_Core_System_Model_User) {
            $cacheId = self::CACHE_KEY . $user->id;
        } else {
            $cacheId = self::CACHE_KEY . $user;
        }
        
        Singular_Runtime::extract('cachemanager')->getCache('default')
            ->remove($cacheId);

        self::$_settings = array();
    }
    
    /**
     * Get default system option
     * 
     * @param string $name
     * @return mixed
     */
    protected function _getDefaultValue ($name)
    {
        return Singular_Core::_('Settings')->getSetting($name);
    }
}
