<?php
/**
 * Singular
 *
 * LICENSE
 *
 * The SOFTWARE PRODUCT is protected by copyright treaties,
 * as well as other intellectual property laws and treaties.
 * The SOFTWARE PRODUCT is licensed, not sold.
 * Software.
 * You may install and use one copy of the SOFTWARE PRODUCT
 * for each license purchased or if a site license then you
 * may install any number of copies at the site where the
 * site license is defined.
 * Your rights to use the SOFTWARE PRODUCT are not transferable.
 *
 * @category   Singular
 * @package    Singular_Db
 * @subpackage Table
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */

 /**
 * @category   Singular
 * @package    Singular_Controller
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */
class Singular_Db_Table_Abstract extends Zend_Db_Table_Abstract
{
    /**
     * Cache prefix constant
     *
     * Using for dynamic methods that must be cached
     */
    const CACHE_METHOD_PREFIX = 'cache_';

    /**
     * DB table prefix name
     *
     * @var string|null
     */
    private $_tablePrefix = null;
    
    /**
     * Cache object
     *
     * @var Zend_Cache
     */
    protected $_cache = null;
    
    /**
     * Classname for row
     *
     * @var string
     */
    protected $_rowClass = 'Singular_Db_Table_Row';

     /**
     * Constructor.
     *
     * Reload parent constructor
     *
     * @param  mixed $config Array of user-specified config options, or just the Db Adapter.
     */
    public function __construct($config = array())
    {
        // get DB table prefix from config
        $ini = $prefix = Singular_Runtime::extract('config');
        
        if (isset($ini->resources->db->params->prefix)) {
            $prefix = $ini->resources->db->params->prefix;
            $this->_name = $prefix . $this->_name;
            $this->_setTablePrefix($prefix);
        }
        
        // Init cache
        $this->_initCache();

        parent::__construct($config);
    }

    /**
     * Magic class method __call for dynamically 'cache' prefix methods
     *
     * @param  string $name
     * @param  array $arguments
     * @return mixed|null
     */
    public function __call($name, $arguments)
    {
        if ((substr($name, 0, strlen(self::CACHE_METHOD_PREFIX))) == self::CACHE_METHOD_PREFIX) {
            $clearMethod = str_replace(self::CACHE_METHOD_PREFIX, '', $name);
            if (method_exists($this, $clearMethod)) {
                $tableName = $this->getTableName();
                $cacheKey  = "{$tableName}_{$clearMethod}_" . md5(var_export($arguments, true));
                if (!$data = $this->_getCache()->load($cacheKey)) {
                    $data = call_user_func_array(array($this, $clearMethod), $arguments);
                    $this->_getCache()->save(
                        $data, $cacheKey,
                        array(
                             $tableName,
                             get_called_class(),
                             $clearMethod
                        )
                    );
                }

                return $data;
            }
        }
        
        trigger_error("Method \"{$name}\" does not exist and was not trapped in __call()", E_USER_ERROR);
    }

    /**
     * Initialize cache for models
     *
     * @return void
     */
    protected function _initCache ()
    {
        if (Singular_Runtime::inStorage('cachemanager')) {
            $this->_cache = Singular_Runtime::extract('cachemanager')->getCache('database');
        }
    }

    /**
     * @return null|Zend_Cache
     */
    protected function _getCache ()
    {
        return $this->_cache;
    }

    /**
     * Clean cache wrapper
     *
     * @param mixed|string|array $tags
     * @return void
     */
    public function cleanCache ($tags = null)
    {
        $_tags = array();
        if (is_array($tags)) {
            $_tags = $tags;
        } elseif (is_string($tags)) {
            $_tags[] = $tags;
        } else {
            $_tags[] = $this->getTableName();
        }
        $this->_cleanCacheByTags($_tags);
    }

    /**
     * Clean cache matching given tags
     *
     * @param  array $tags
     * @return void
     */
    protected function _cleanCacheByTags (array $tags)
    {
        if (Singular_Runtime::inStorage('cachemanager')) {
            $this->_getCache()->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG, $tags);
        }
    }

    /**
     * Return row by id
     *
     * @param int $id
     * @return Zend_Db_Table_Row_Abstract
     */
    public function getRowById ($id)
    {
        $where = $this->getAdapter()->quoteInto('id = ?', $id, Zend_Db::INT_TYPE);
        $result = $this->fetchRow($where);
        if (true === empty($result)) {
            throw new Zend_Exception('Row with id = ' . $id . ' not fount');
        }
        
        return $result;
    }

    /**
     * Return row by field name
     *
     * @param string $field
     * @param mixed $value
     * @return Zend_Db_Table_Row_Abstract
     */
    public function getRowByField ($field, $value)
    {
        $where = $this->getAdapter()->quoteInto("`{$field}` = ?", $value);
        $result = $this->fetchRow($where);
        if (true === empty($result)) {
            return null;
        }

        return $result;
    }

    /**
     * Delete row by id
     *
     * @param int $id
     * @return int
     */
    public function deleteRowById ($id)
    {
        $where = $this->getAdapter()->quoteInto('id = ?', $id, Zend_Db::INT_TYPE);

        return $this->delete($where);
    }

    /**
     * Delete row by field name
     *
     * @param string $field
     * @param mixed $value
     * @return int
     */
    public function deleteRowByField ($field, $value)
    {
        $where = $this->getAdapter()->quoteInto("`{$field}` = ?", $value);

        return $this->delete($where);
    }

    /**
     * Update row by id
     *
     * @param array $data
     * @param int $id
     * @return int
     */
    public function updateRowById ($data, $id)
    {
        $where = $this->getAdapter()->quoteInto('id = ?', $id, Zend_Db::INT_TYPE);

        return $this->update($data, $where);
    }

    /**
     * Update row by field name
     *
     * @param string $field
     * @param mixed $value
     * @param array $data
     * @return int
     */
    public function updateRowByField ($field, $value, $data)
    {
        $where = $this->getAdapter()->quoteInto("`{$field}` = ?", $value);

        return $this->update($data, $where);
    }

    /**
     * Getting all rows from current table
     *
     * @param mixed $order
     * @return array
     */
    public function getRows ($order = 'id', $count = null, $offset = null)
    {
        /* get select object */
        $select = $this->select()->order($order);

        /* define count/offset */
        if (null !== $count) {
            $select->limit($count, $offset);
        }

        /* fetch data */
        return $this->fetchAll($select, $count, $offset);
    }

    /**
     * Getting rows via paginator
     *
     * @param null|Zend_Db_Select
     * @param mixed $order
     * @param string $direction
     * @param int $pageNumber
     * @param int $itemCountPerPage
     * @return Zend_Paginator
     */
    public function getPaginatorRows ($select = null, $order = 'id', $direction = Zend_Db_Select::SQL_ASC,
        $pageNumber = 1, $itemCountPerPage = 10)
    {
        // getting select object
        if (!$select instanceof Zend_Db_Select) {
            // compute direction param
            $direction = (null != $direction) ? $direction : Zend_Db_Select::SQL_ASC;
            // concat order with direction
            if (null == $order) {
                $order = current($this->info('primary')) . ' ' . $direction;
            } else {
                $order = $order . ' ' . $direction;
            }

            // make select query object (Zend_Db_Select)
            $select = $this->select()->order($order);
        }

        // make paginator
        $paginator = new Zend_Paginator(
            new Singular_Paginator_Adapter_DbTableSelect($select)
        );

        // paginator settings
        $pageRange = Singular_Core::_('Settings')->getSetting('paginator_page_range');
        $paginator->setCurrentPageNumber((int) $pageNumber);
        $paginator->setItemCountPerPage($itemCountPerPage);
        $paginator->setPageRange($pageRange);

        // return
        return $paginator;
    }

    /**
     * Return DB  table name of current model
     * ($clearPrefix = true means without table prefix)
     *
     * @param bool $clearPrefix
     * @return string
     */
    public function getTableName ($clearPrefix = false)
    {
        if (true === $clearPrefix) {
            return str_replace($this->getTablePrefix(), '', $this->_name);
        } else {
            return $this->_name;
        }
    }

    /**
     * Return table prefix
     *
     * @return string
     */
    public function getTablePrefix ()
    {
        return $this->_getTablePrefix();
    }

    /**
     * @param  null|string $name
     * @return void
     */
    protected function _setTablePrefix ($name = null)
    {
        $this->_tablePrefix = $name;
    }

    /**
     * @return string
     */
    protected function _getTablePrefix ()
    {
        return $this->_tablePrefix;
    }

    /**
     * Called by parent table's class during delete() method.
     *
     * @param  string $parentTableClassname
     * @param  array  $primaryKey
     * @return int    Number of affected rows
     */
    public function _cascadeDelete($parentTableClassname, array $primaryKey)
    {
        $this->_setupMetadata();
        $rowsAffected = 0;
        foreach ($this->_getReferenceMapNormalized() as $map) {
            if ($map[self::REF_TABLE_CLASS] == $parentTableClassname && isset($map[self::ON_DELETE])) {
                switch ($map[self::ON_DELETE]) {
                    case self::CASCADE:
                        $where = array();
                        for ($i = 0; $i < count($map[self::COLUMNS]); ++$i) {
                            $col = $this->_db->foldCase($map[self::COLUMNS][$i]);
                            $refCol = $this->_db->foldCase($map[self::REF_COLUMNS][$i]);
                            $type = $this->_metadata[$col]['DATA_TYPE'];
                            $where[] = $this->_db->quoteInto(
                                $this->_db->quoteIdentifier($col, true) . ' = ?',
                                $primaryKey[$refCol], $type);
                        }
                        
                        $toDelete = $this->fetchAll($where);
                        foreach ($toDelete as $row) {
                            $rowsAffected += $row->delete();
                        }
                        break;
                    default:
                        // no action
                        break;
                }
            }
        }
        return $rowsAffected;
    }
}
