<?php
abstract class Singular_Event_Listener_Abstract
{
    /**
     * Event execution priority
     */
    const PRIORITY_HIGH   = 2;
    const PRIORITY_NORMAL = 1;

    /**
     * Binded listeners
     *
     * @var array
     */
    private static $_listeners = array();

    /**
     * View object
     *
     * @var Zend_View_Interface
     */
    public static $view = null;

    /**
     * Returns view object.
     *
     * @return Zend_View_Interface
     */
    public static function getView ()
    {
        if (!self::$view instanceof Zend_View_Interface) {
            self::$view = Zend_Layout::getMvcInstance()->getView();
        }
        return self::$view;
    }

    /**
     * Returns true if event has one or more aggregates.
     *
     * @static
     * @param  string $name
     * @return bool
     */
    public static function hasEvent ($name)
    {
        return array_key_exists(strtolower($name), self::$_listeners);
    }

    /**
     * Returns all aggregates attached on specific event.
     *
     * @static
     * @param  string|null $event
     * @return array|null    Array of aggregates or null if no aggregates has attached on event
     */
    public static function getAggregates ($event = null)
    {
        if (null === $event) return self::$_listeners;
        return (self::hasEvent($event)) ? self::$_listeners[strtolower($event)] : null;
    }

    /**
     * Sets event listeners.
     *
     * May be useful for manual setting listeners from cache.
     *
     * @static
     * @param  array $listeners Proper array with event aggregates
     * @return bool
     * @throws Singular_Event_Exception
     */
    public static function setListeners ($listeners)
    {
        if (is_array($listeners)) {
            if (current(current($listeners)) instanceof stdClass) {
                self::$_listeners = $listeners;
                return true;
            }
        }
        throw new Singular_Event_Exception(
            'invalid argument: $listeners must be proper array with keys as event names and values'
                    . ' with stClass instances contained aggregates.'
        );
    }

    /**
     * Attaches aggregate on specific event.
     *
     * @static
     * @param  string $event            Event name
     * @param  array|object $aggregate  An array(<className>, <methodName>)
     *                                  or an object which has <event>/on<Event> method for non-static calls only
     * @param  int $priority            1 - normal, 2 - high
     * @throws Singular_Event_Exception If invalid $aggregate argument has passed
     */
    public static function attachAggregate ($event, $aggregate, $priority = self::PRIORITY_NORMAL)
    {
        $callback = new stdClass();
        if (is_array($aggregate) && (2 == count($aggregate))) {
            $callback->class = $aggregate[0]; $callback->method = $aggregate[1];
        } elseif (is_object($aggregate)) {
            if (method_exists($aggregate, $event)) {
                $callback->class = $aggregate; $callback->method = $event;
            } elseif (method_exists($aggregate, $method = ('on' . ucfirst($event)))) {
                $callback->class = $aggregate; $callback->method = $method;
            }
        } else {
            throw new Singular_Event_Exception(
                'Invalid argument: $aggregate must be an array($object, $method)'
                        . 'or an object which has <event> or on<Event> method'
            );
        }

        $event = strtolower($event);
        if (self::PRIORITY_HIGH == $priority) {
            self::$_listeners[$event][key(self::$_listeners[$event]) - 1] = $callback;
            ksort(self::$_listeners[$event]);
        } else {
            self::$_listeners[$event][] = $callback;
        }
    }

    /**
     * Must be implemented in inherited classes.
     *
     * @static
     * @abstract
     * @return mixed
     */
    public static function bindListeners (){}
}
