<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Loader
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @category   Singular
 * @package    Singular_Loader
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Loader_Autoloader
{
    /**
     * Singleton static thing
     *
     * @var Singular_Loader_Autoloader
     */
    private static $instance = null;

    /**
     * Cache key
     *
     * Should be unique per project using this, if multiple projects on the same server.
     *
     * @var string
     */
    protected $cacheKey = 'autoloader_cache';

    /**
     * @var array
     */
    protected $classPaths = array();

    /**
     * @var string
     */
    protected static $_classFileSuffix = '.php';

    /**
     * Cache instance
     *
     * @var Zend_Cache
     */
    protected $cacheBackend = null;

    /**
     * Cached result
     *
     * @var array
     */
    public static $cache = null;

    /**
     * Class constructor
     *
     * Explicitly here so you are aware that it doesn't have to be a singleton
     */
    public function __construct ()
    {
    }

    /**
     * Well, if you want, you can use it as a singleton....
     *
     * @return Singular_Loader_Autoloader $singleton instance
     */
    public static function getInstance ()
    {
        if (self::$instance == null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    /**
     * You might need this if running the same code base twice on the same server but
     * under different directories - e.g. dev and live. In which case, specify a local per project key.
     *
     * @param  $string
     * @return void
     */
    public function setCacheKey ($string)
    {
        $this->cacheKey = $string;
    }

    /**
     * Sets the paths to search in when looking for a class.
     *
     * @param  array $paths
     * @return void
     **/
    public function setClassPaths ($paths)
    {
        $this->classPaths = $paths;
    }

    /**
     * Adds a path to search in when looking for a class.
     *
     * @param string $path
     * @return void
     **/
    public function addClassPath ($path)
    {
        if (!is_array($path)) {
            $path = array($path);
        }
        foreach ($path as $item) {
            $this->classPaths[] = $item;
        }
    }

    /**
     * Set the Zend_Cache instance to use.
     *
     *     <code>
     *     $autoloader->setCache($cache);
     *     </code>
     *
     * @param  Zend_Cache_Frontend $cache
     * @return void
     **/
    public function setCache ($cache)
    {
        $this->cacheBackend = $cache;
    }

    /**
     * Return caching backend used; or create new one if none already defined
     *
     * @param  bool $loadDefault
     * @return Zend_Cache_Core
     */
    public function getCache ($loadDefault = true)
    {
        if ($this->cacheBackend == null && $loadDefault) {
            $this->cacheBackend = Zend_Cache::factory('Core', 'Apc', array('automatic_serialization' => true));
        }
        return $this->cacheBackend;
    }

    /**
     * Sets the suffix to append to a class name in order to get a file name
     * to look for
     *
     * @static
     * @param  string $suffix - $className . $suffix = filename.
     * @return void
     **/
    public static function setClassFileSuffix ($suffix)
    {
        self::$_classFileSuffix = $suffix;
    }

    /**
     * Given a map of class names to absolute file paths, try and load the class.
     * If we get to including the file, we ensure that the class_exists after doing
     * so, otherwise we'll return false.
     *
     * Also return false if we don't know where the class ($name) could be.
     *
     * @param  string $name the name of the class we wish to load.
     * @return bool
     */
    protected function tryAndLoadClass ($name)
    {
        if (isset(self::$cache[$name])) {
            include self::$cache[$name];
            if (class_exists($name) || interface_exists($name)) {
                return true;
            }
        }
        return false;
    }

    /**
     * Autoload a class
     *
     * @static
     * @param  string $className
     * @return bool
     */
    public static function autoload ($className)
    {
        if (class_exists($className) || interface_exists($className)) return true;
        $instance = self::getInstance();
        return $instance->loadClass($className);
    }

    /**
     * Returns true if the class file was found and included, false if not.
     *
     * @param  string $className
     * @return boolean
     **/
    public function loadClass ($className)
    {
        /** @var $cache Zend_Cache_Core */
        $cache = $this->getCache(false);
        if (null == self::$cache && null !== $cache) {
            self::$cache = $cache->load($this->cacheKey);
        }

        if (!self::$cache) {
            self::$cache = array();
        }

        $status = $this->tryAndLoadClass($className);
        if ($status) {
            /* class loaded */
            return true;
        }

        foreach ($this->classPaths as $path) {
            /* scan for file */
            $realPath = $this->_searchClass($className, $path);
            if ($realPath !== false) {
                self::$cache[$className] = $realPath;
                break;
            }
        }

        if (null !== $cache) $cache->save(self::$cache, $this->cacheKey);

        $status = $this->tryAndLoadClass($className);

        return $status;
    }

    /**
     * Doing search for class transmitted to autoloader
     *
     * @param  string $className    Autoloaded class name
     * @param  string $directory    Directory search root path
     * @return bool|string          Returns class file path if loaded successfully
     */
    protected function _searchClass ($className, $directory)
    {
        $pathParts = explode('_', $className);
        if (is_dir($directory) && is_readable($directory)) {
            $pathPartsNum = count($pathParts);
            $ppI = 0;
            foreach ($pathParts as $part) {
                $found = false;

                if ($pathPartsNum == ++$ppI) {
                    $part .= self::$_classFileSuffix;
                }

                $partVariations = array(
                    $part,
                    strtolower($part),
                    ucfirst(strtolower($part))
                );

                // Converts camel-cased parts to dashed for searching in segment like: */foo-module
                $pattern     = array('#(?<=(?:[A-Z]))([A-Z]+)([A-Z][A-z])#', '#(?<=(?:[a-z0-9]))([A-Z])#');
                $replacement = array('\1-\2', '-\1');
                $dashed      = preg_replace($pattern, $replacement, $part);
                if (!in_array($dashed, $partVariations)) {
                    $partVariations[] = strtolower($dashed);
                }

                switch ($part) {
                    case 'Controller':
                        $partVariations[] = 'controllers';
                        break;
                    case 'Model':
                        $partVariations[] = 'models';
                        break;
                    case 'Lib':
                        $partVariations[] = 'library';
                        break;
                    case 'Service':
                        $partVariations[] = 'services';
                        break;
                    case 'Form':
                        $partVariations[] = 'forms';
                        break;
                    case 'Helper':
                        $partVariations[] = 'helpers';
                        break;
                    case 'Filter':
                        $partVariations[] = 'filters';
                        break;
                    case 'Handler':
                        $partVariations[] = 'handlers';
                        break;
                    case 'Mapper':
                        $partVariations[] = 'mappers';
                        break;
                }

                foreach ($partVariations as $var) {
                    if (!is_dir($directory) || !file_exists($directory . DS . $var)) {
                        continue;
                    }
                    $directory .= DS . $var;
                    $found = true;
                }

                if (!$found) {
                    return false;
                }
            }

            if (is_file($directory)) {
                return $directory;
            }
        }

        return false;
    }

    /**
     * Register this class with SPL as an autoloader.
     *
     * @return void
     */
    public function register ()
    {
        spl_autoload_register(array(__CLASS__, 'autoload'));
        spl_autoload_unregister(array('Zend_Loader_Autoloader', 'autoload'));
    }
}
