<?php

class Singular_Module extends ArrayObject
{
    const MODULE_FILE = 'Module.php';

    const MODULE_SETTINGS_FORM = 'Settings.php';

    /**
     * @var Singular_Module
     */
    static private $_instance = null;

    /**
     * Class constructor
     */
    public function __construct ()
    {
        $this->setFlags(ArrayObject::ARRAY_AS_PROPS);

        $this->_initModules();
    }

    /**
     * Prevent object cloning
     */
    final private function  __clone ()
    {
    }

    /**
     * Returns new or existing Singleton instance
     * 
     * @return Singular_Module
     */
    public static function getInstance ()
    {
        if (null === self::$_instance) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }

    /**
     * Return current module name
     *
     * @static
     * @return string
     */
    public static function getCurrentModule ()
    {
        return Zend_Controller_Front::getInstance()
                ->getRequest()->getModuleName();
    }

    /**
     * Initializes all active installed modules
     * 
     * @return void
     */
    private function _initModules ()
    {
        $modules = $this->_loadModules();
        foreach ($modules as $name => $module) {
            // add module
            $this->offsetSet($name, $module);
        }

        Singular_Event::dispatch('settings', $this);
    }

    /**
     * Recursive loads all active modules files
     *
     * @param  array|null $modules
     * @return void
     */
    private function _loadModules ($modules = null)
    {
        if (null === $modules) {
            $modules = Singular_Runtime::extract('active_modules');
        }

        $loadedModules = array();

        foreach ($modules as $module) {
            $moduleFilePath = APPLICATION_PATH . DS
                              . 'modules' . DS
                              . $module . DS . self::MODULE_FILE;
            $moduleClass    = Singular_Stdlib_Module::formatName($module) . '_Module';
            
            if (!file_exists($moduleFilePath)) {
                continue;
            }

            // load module file/class
            require_once $moduleFilePath;
            if (!class_exists($moduleClass, false)) {
                continue;
            }

            /** @var $moduleInstance Singular_Module_Abstract */
            $moduleInstance = new $moduleClass($module);

            if (!$moduleInstance instanceof Singular_Module_Abstract) {
                continue;
            }

            // try to load module settings form (if exists)
            $settings = $this->_loadModuleSettings($moduleInstance);
            if ($settings instanceof Singular_Module_Settings_Container) {
                $settings->populateSupplyBlockWithModuleData();
                $moduleInstance->getSettings()->addShortcut('settings', $settings);
            }

            $loadedModules[$module] = $moduleInstance;
        }

        return $loadedModules;
    }

    /**
     * Loads module settings container
     *
     * @param Singular_Module_Abstract $moduleClass
     * @return null|Singular_Module_Settings_Container
     */
    protected function _loadModuleSettings (Singular_Module_Abstract $moduleClass)
    {
        $moduleName = $moduleClass->getModuleName();
        $settingsPath = APPLICATION_PATH . DS
                        . 'modules' . DS
                        . $moduleName . DS
                        . 'forms' . DS . self::MODULE_SETTINGS_FORM;
        
        if (file_exists($settingsPath)) {
            require_once $settingsPath;

            $formClass = Singular_Stdlib_Module::formatName($moduleName) . '_Form_Settings';
            if ('Singular_Form' != get_parent_class($formClass)) return null;

            $moduleContainer = $moduleClass->getContainer();
            $shortcut = new Singular_Module_Settings_Container();
            $shortcut->setModuleClass($moduleClass);
            $shortcut->setName(
                ($moduleContainer->getName() != null
                        ? $moduleContainer->getName()
                        : ucfirst($moduleName))
            );
            $moduleIcon = $moduleClass->view->imageSrc('brand/icon.png', $moduleName);
            if (file_exists(ROOT_PATH . $moduleIcon)) {
                $shortcut->setIcon($moduleIcon);
            } else {
                $shortcut->setIcon($moduleClass->view->imageSrc('admin/icons/module-ico.png', null));
            }
            $shortcut->setSection(Singular_Module_Settings_Container::DEFAULT_MODULE_SECTION)
                    ->setModule('system')
                    ->setController('admin-settings')
                    ->setAction('module')
                    ->addParam('name', $moduleName);

            return $shortcut;
        }

        return null;
    }

    /**
     * Gets module directory names
     * 
     * @return array
     */
    private function _getModuleDirectoryStructure ()
    {
        $modules = array();

        $moduleDir = new DirectoryIterator(MODULES_PATH);
        foreach ($moduleDir as $module) {
            if ($module->isDir() && !$module->isDot()) {
                $modules[] = $module->getFileInfo()->getFilename();
            }
        }

        return $modules;
    }

    /**
     * Return module by name
     * return null if module is not exists
     *
     * @param string $name
     * @return Singular_Module_Abstract|null
     */
    public function getModule ($name)
    {
        if ($this->moduleExists($name)) {
            return $this->offsetGet($name);
        }

        return null;
    }

    /**
     * Return all loaded modules
     *
     * @return array
     */
    public function getModules ()
    {
        return $this->getArrayCopy();
    }

    /**
     * Gets all installed active modules
     * 
     * @return array
     */
    public function getActiveModules ()
    {
        return $this->getArrayCopy();
    }

    /**
     * Gets all installed active modules
     *
     * @return array
     */
    public function getInstalledModules ()
    {
        $model = new Singular_Core_System_Model_DbTable_Modules();
        $installed = $model->getAdapter()->fetchPairs(
            $model->select(true)->columns(array('name', 'active'))
        );

        return $this->_loadModules(
            array_merge($this->getAutoloadedModules(), $installed)
        );
    }

    /**
     * Gets all modules that are not installed
     *
     * @return array
     */
    public function getNotInstalledModules ()
    {
        $model = new Singular_Core_System_Model_DbTable_Modules();
        $installed = $model->getAdapter()->fetchPairs(
            $model->select(true)->columns(array('name', 'active'))
        );
        
        $modules = array_diff(
            $this->_getModuleDirectoryStructure(),
            $installed,
            $this->getAutoloadedModules()
        );

        return $this->_loadModules($modules);
    }

    /**
     * Gets all modules that are not active
     * 
     * @return array
     */
    public function getInactiveModules ()
    {
        $inactive = new Singular_Core_System_Model_DbTable_Modules();
        $inactive = $inactive->getInactiveModules()->toArray();

        $modules = array();
        foreach ($inactive as $iaModule) {
            $modules[] = $iaModule['name'];
        }

        return $this->_loadModules($modules);
    }

    public function getAutoloadedModules ()
    {
        $autoloadedModules = Singular_Core::_('Config')->resources->autoloader->zend->modules;
        if (null !== $autoloadedModules) {
            return $autoloadedModules->toArray();
        }

        return array();
    }

    /**
     * Checks for module has loaded
     *
     * @param string $name
     * @return bool
     */
    public function moduleExists ($name)
    {
        return $this->offsetExists($name);
    }
}
