<?php
class Singular_Module_Abstract
{
    /**
     * @var Zend_View
     */
    public $view = null;

    /**
     * Module name
     *
     * @var string
     */
    protected $_moduleName = null;

    /**
     * Module information container
     *
     * @var Singular_Module_Container
     */
    private $_container;

    /**
     * Settings
     *
     * @var Singular_Module_Settings
     */
    private $_settings;

    /**
     * System module flag
     *
     * @var bool
     */
    private $_isSystemModule = false;

    /**
     * Module forms path
     *
     * @var string
     */
    private $_formsPath;

    /**
     * Class name of the default settings form class
     *
     * @var string
     */
    private $_defaultSettingsFormClass;

    /**
     * Class constructor
     * 
     * @param string $moduleName
     */
    final public function __construct ($moduleName)
    {
        $this->_setModuleName($moduleName);

        if ('system' == $moduleName) {
            $this->setIsSystemModule(true);
        }

        $this->view = Zend_Layout::getMvcInstance()->getView();
        $this->_initModuleContainer();
        $this->_initSettings();
        $this->_initSettingsFormsParams();
        $this->init();
        $this->_injectModuleClass();
    }

    /**
     * Call magic method
     *
     * @throws Singular_Module_Exception
     * @param string $name Container method name
     * @param array $args Container arguments
     * @return mixed
     */
    public function __call ($name, $args)
    {
        if (!method_exists($this->_container, $name)) {
            throw new Singular_Module_Exception('Error: Call to undefined method ' . __CLASS__ . "::{$name}().");
        }

        return call_user_func_array(array($this->_container, $name), $args);
    }

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @return void
     */
    public function init ()
    {
    }

    /**
     * Init module container
     *
     * @return void
     */
    private function _initModuleContainer ()
    {
        $this->_container = new Singular_Module_Container($this);
    }

    private function _initSettings ()
    {
        $this->_settings = new Singular_Module_Settings($this);
    }

    private function _initSettingsFormsParams ()
    {
        // set module form path
        $this->_formsPath = APPLICATION_PATH . DS
                            . 'modules' . DS
                            . $this->getModuleName() . DS
                            . 'forms';

        // set form class name
        $this->_defaultSettingsFormClass = ucfirst($this->getModuleName()) . '_Form_Settings';
    }

    /**
     * Gets module container
     *
     * @return Singular_Module_Container
     */
    public function getContainer ()
    {
        return $this->_container;
    }

    /**
     * Gets settings
     *
     * @return Singular_Module_Settings
     */
    public function getSettings ()
    {
        return $this->_settings;
    }

    /**
     * Sets system module flag
     *
     * @param boolean $flag
     * @return void
     */
    public function setIsSystemModule ($flag)
    {
        $this->_isSystemModule = (bool) $flag;
    }

    /**
     * Gets system module boolean value
     * true - yes, false - no
     *
     * @return boolean
     */
    public function isSystemModule ()
    {
        return $this->_isSystemModule;
    }

    private function _setModuleName ($moduleName)
    {
        $this->_moduleName = $moduleName;
    }

    public function getModuleName ()
    {
        return $this->_moduleName;
    }

    /**
     * If module has default settings form - true
     * else/default - false
     * 
     * @return bool
     */
    public function hasSettingsForm ()
    {
        $formFile = $this->_formsPath . DS . Singular_Module::MODULE_SETTINGS_FORM;

        // check for default settings form exists
        if (file_exists($formFile)) {
            require_once $formFile;

            if ('Singular_Form' == get_parent_class($this->_defaultSettingsFormClass)) {
                return true;
            }
        }

        return false;
    }

    public function getSettingsForm ()
    {
        if ($this->hasSettingsForm()) {
            return new $this->_defaultSettingsFormClass();
        }

        return false;
    }

    private function _injectModuleClass ()
    {
        // inject into settings container
        if ($this->getSettings()->hasShortcuts()) {
            $scNames = $this->getSettings()->getShortcutNames();

            foreach ($scNames as $scName) {
                if (!$this->getSettings()->getShortcut($scName)->hasModuleClass()) {
                    $this->getSettings()->getShortcut($scName)->setModuleClass($this);
                }
            }
        }
    }

}
