<?php

class Singular_Module_Container extends Singular_Module_ModuleClassExtender
{
    const RELEASE_DATE_FORMAT = Zend_Date::DATETIME_LONG;

    const CHANGELOG_DATE_FORMAT = Zend_Date::DATETIME_LONG;

    const VERSION_HIGHER = 'higher';

    const VERSION_LOWER = 'lower';

    const VERSION_EQUAL = 'equal';

    private $_container = array();

    public function __construct (Singular_Module_Abstract $moduleClass)
    {
        $this->_setModuleClass($moduleClass);
        $this->_initContainer();
    }

    /**
     * Call magic method
     *
     * @throws Singular_Module_Exception
     * @param  string $name Container method name
     * @param  array $args Container arguments
     * @return Singular_Module_Container
     */
    public function __call ($name, $args)
    {
        return $this;
    }

    public function __set ($name, $value)
    {
        if (!$this->_isParamKeyExists($name)) {
            throw new Singular_Module_Exception("Invalid module container param key \"{$name}\"");
        }

        $this->_container[$name] = $value;
    }

    public function __get ($name)
    {
        return ($this->_isParamKeyExists($name)) ? $this->_container[$name] : null;
    }

    protected function _isParamKeyExists ($key)
    {
        return array_key_exists($key, $this->_container);
    }

    private function _initContainer ()
    {
        $this->_container = array(
            'name'         => null,
            'description'  => null,
            'version'      => null,
            'author'       => null,
            'email'        => null,
            'url'          => null,
            'release_date' => null,
            'path'         => null,
            'dependencies' => array(),
            'changelog'    => array(),
            'icon'         => null,
            'route'        => null,
            'actions'      => array(),
        );
    }

    public function setName ($value)
    {
        $this->name = $value;

        return $this;
    }

    public function getName ()
    {
        return $this->name;
    }

    public function setDescription ($value)
    {
        $this->description = $value;

        return $this;
    }

    public function getDescription ()
    {
        return $this->description;
    }

    public function setVersion ($value)
    {
        $this->version = $value;

        return $this;
    }

    public function getVersion ()
    {
        return $this->version;
    }

    public function setAuthor ($value)
    {
        $this->author = $value;

        return $this;
    }

    public function getAuthor ()
    {
        return $this->author;
    }

    public function setEmail ($value)
    {
        $this->email = $value;

        return $this;
    }

    public function getEmail ($asHtml = false)
    {
        if ((bool) !$asHtml) {
            return $this->email;
        }

        $view = $this->getModuleClass()->view;

        return (null != $this->email)
                ? $view->link('mailto:' . $this->email, $view->truncate($this->email, 30))
                : null;
    }

    public function setUrl ($value)
    {
        $this->url = $value;

        return $this;
    }

    public function getUrl ($asHtml = false, $attribs = array())
    {
        if ((bool)!$asHtml) {
            return $this->url;
        }

        $view = $this->getModuleClass()->view;
        $attribs['target'] = '_blank';
        
        return (null != $this->url)
                ? $view->link($this->url, $view->truncate($this->url, 30), $attribs)
                : null;
    }

    public function setReleaseDate ($value)
    {
        $this->release_date = new Zend_Date($value);

        return $this;
    }

    public function getReleaseDate ()
    {
        /** @var $date Zend_Date */
        $date = $this->release_date;

        if (!$date instanceof Zend_Date) return null;

        return $date->toString(self::RELEASE_DATE_FORMAT);
    }

    public function setPath ($value)
    {
        $this->path = $value;

        return $this;
    }

    public function getPath ()
    {
        return $this->path;
    }
    
    public function setIcon ($value)
    {
        $this->icon = $value;
        
        return $this;
    }
    
    public function getIcon ()
    {
        return $this->icon;
    }
    
    public function setRoute ($value)
    {
        $this->route = $value;
        
        return $this;
    }
    
    public function getRoute ()
    {
        return $this->route;
    }
    
    public function setActions ($value)
    {
        $this->actions = $value;
        
        return $this;
    }
    
    public function getActions ()
    {
        return $this->actions;
    }
    
    public function hasActions ()
    {
        return (bool) count($this->actions);
    }

    public function addDependency ($module, $version = null, $req = self::VERSION_HIGHER)
    {
        array_unshift($this->_container['dependencies'],
                      $this->_dependencyEntry($module, $version, $req));

        return $this;
    }

    public function getDependencies ()
    {
        return $this->dependencies;
    }

    private function _dependencyEntry ($module, $version = null, $req = self::VERSION_HIGHER)
    {
        $values = array(
            self::VERSION_HIGHER,
            self::VERSION_EQUAL,
            self::VERSION_LOWER
        );
        if (!in_array($req, $values)) {
            $req = self::VERSION_HIGHER;
        }

        return array(
            'module'  => $module,
            'version' => $version,
            'req'     => $req
        );
    }

    public function addChangelog ($date, $text, $author = null)
    {
        array_unshift($this->_container['changelog'],
                      $this->_changelogEntry($date, $text, $author));

        return $this;
    }

    public function getChangelog ()
    {
        return $this->changelog;
    }

    private function _changelogEntry ($date, $text, $author = null)
    {
        /** @var $date Zend_Date */
        $date = new Zend_Date($date);
        
        return array(
            'date'   => $date->toString(self::CHANGELOG_DATE_FORMAT),
            'text'   => $text,
            'author' => $author
        );
    }
}
