<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Module
 * @subpackage Singular_Module_Settings
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

 /**
 * @category   Singular
 * @package    Singular_Module
 * @subpackage Singular_Module_Settings
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Module_Settings_Container
    extends Singular_Module_ModuleClassExtender
{
    /**
     * Default module settings
     */
    const DEFAULT_MODULE_SECTION = 'modules';

    /**
     * Predefined sections
     *
     * @var array
     */
    protected static $_defaultSections = array(
        'general'                    => 'General',
        'tools'                      => 'Tools',
        self::DEFAULT_MODULE_SECTION => 'Modules',
        'other'                      => 'Other'
    );

    /**
     * User defined sections
     *
     * @var array
     */
    protected static $_definedSections = array();

    /**
     * Settings container
     *
     * @var array
     */
    private $_container = array();

    /**
     * @var Singular_Module_Settings_SupplyBlock
     */
    private $_supplyBlock = null;

    /**
     * Init
     */
    public function __construct ()
    {
        $this->_initContainer();
    }

    /**
     * Set param
     * 
     * @param string $name
     * @param mixed $value 
     * @return void
     */
    public function __set ($name, $value)
    {
        if (!$this->_hasParam($name)) {
            throw new Singular_Module_Exception("Invalid module container param \"{$name}\"");
        }

        $this->_container[$name] = $value;
    }

    /**
     * Get param
     * 
     * @param string $name
     * @return mixed 
     */
    public function __get ($name)
    {
        return ($this->_hasParam($name)) ? $this->_container[$name] : null;
    }

    /**
     * Check param exists
     * 
     * @param string $key
     * @return bool 
     */
    protected function _hasParam ($key)
    {
        return array_key_exists($key, $this->_container);
    }

    /**
     * @param Singular_Module_Abstract $class
     * @return void
     */
    public function setModuleClass (Singular_Module_Abstract $class)
    {
        $this->_setModuleClass($class);
    }

    /**
     * Init container
     * 
     * @param void
     * @return void
     */
    private function _initContainer ()
    {
        $this->_container = array(
            'name'       => null,
            'section'    => 'general',
            'icon'       => null,
            'module'     => null,
            'controller' => null,
            'action'     => null,
            'route'      => null,
            'resource'   => null,
            'params'     => array()
        );
    }

    /**
     * Set name
     * 
     * @param string $value
     * @return Singular_Module_Settings_Container 
     */
    public function setName ($value)
    {
        $this->name = $value;

        return $this;
    }

    /**
     * Get name
     * 
     * @param void
     * @return string 
     */
    public function getName ()
    {
        return $this->name;
    }

    /**
     * Set section
     * 
     * @param string $value
     * @return Singular_Module_Settings_Container 
     */
    public function setSection ($value)
    {
        if (!$this->hasSection($value)) {
            throw new Singular_Module_Exception("Section \"{$value}\" is not exists. Use defineSection() method before calling " . __METHOD__ . '().');
        }

        $this->section = $value;

        return $this;
    }

    /**
     * Get section
     * 
     * @param void
     * @return string 
     */
    public function getSection ()
    {
        $sections = $this->getSections();
        
        return array($this->section => $sections[$this->section]);
    }

    /**
     * Get default sections
     * 
     * @param void
     * @return array 
     */
    public function getDefaultSections ()
    {
        return self::$_defaultSections;
    }

    /**
     * Adds a new defined section
     *
     * @throws Singular_Module_Exception
     * @param string $name
     * @param string $title
     * @return void
     */
    public function defineSection ($name, $title)
    {
        if (is_numeric($name)) {
            throw new Singular_Module_Exception("Section name must be not numeric.");
        }
        if ($this->hasDefinedSection($name)) {
            return $this;
        }

        self::$_definedSections[$name] = $title;

        return $this;
    }

    /**
     * Return all defined sections
     *
     * @param void
     * @return array
     */
    public function getDefinedSections ()
    {
        return self::$_definedSections;
    }

    /**
     * Checks particular defined section exists or not
     *
     * @param string $name
     * @return bool
     */
    public function hasDefinedSection ($name)
    {
        return isset(self::$_definedSections[$name]);
    }

    /**
     * Checks particular section exists or not
     * between default and defined sections
     *
     * @param string $name
     * @return bool
     */
    public function hasSection ($name)
    {
        return array_key_exists($name, $this->getSections());
    }

    /**
     * Get all sections
     * (default + defined)
     *
     * @param void
     * @return array
     */
    public function getSections ()
    {
        return array_merge(self::$_defaultSections, self::$_definedSections);
    }

    /**
     * Sets path to the icon for settings shortcut
     *
     * @param string $value
     * @return Singular_Module_Settings_Container
     */
    public function setIcon ($value)
    {
        $this->icon = $value;

        return $this;
    }

    /**
     * Get icon
     * 
     * @param void
     * @return string 
     */
    public function getIcon ()
    {
        return $this->icon;
    }
    
    /**
     * Set ACL resource
     * 
     * @param string $resource
     * @return Singular_Module_Settings_Container
     */
    public function setResource ($resource)
    {
        $this->resource = $resource;
        
        return $this;
    }
    
    /**
     * Get ACL resource
     * 
     * @param void
     * @return string
     */
    public function getResource ()
    {
        return $this->resource;
    }
    
    /**
     * Visible?
     * 
     * @param void
     * @return bool
     */
    public function isVisible ()
    {
        /** @var $view Zend_View */
        $view = $this->getModuleClass()->view;
        
        if ($this->resource) {
            return $view->acl()->objectIsAllowed($this->resource)
                ? true : false;
        }
        
        return true;
    }

    /**
     * Set module
     * 
     * @param string $value
     * @return Singular_Module_Settings_Container 
     */
    public function setModule ($value)
    {
        $this->module = $value;

        return $this;
    }

    /**
     * Get module
     * 
     * @param void
     * @return string
     */
    public function getModule ()
    {
        return $this->module;
    }

    /**
     * Set controller
     * 
     * @param string $value
     * @return Singular_Module_Settings_Container 
     */
    public function setController ($value)
    {
        $this->controller = $value;

        return $this;
    }

    /**
     * Get controller
     * 
     * @param void
     * @return string
     */
    public function getController ()
    {
        return $this->controller;
    }

    /**
     * Set action
     * 
     * @param string $value
     * @return Singular_Module_Settings_Container 
     */
    public function setAction ($value)
    {
        $this->action = $value;

        return $this;
    }

    /**
     * Get action
     * 
     * @param void
     * @return string
     */
    public function getAction ()
    {
        return $this->action;
    }

    /**
     * Set route
     * 
     * @param string $value
     * @return Singular_Module_Settings_Container 
     */
    public function setRoute ($value)
    {
        $this->route = $value;

        return $this;
    }

    /**
     * Get route
     * 
     * @param void
     * @return string
     */
    public function getRoute ()
    {
        return $this->route;
    }

    /**
     * Set params
     * 
     * @param array $params
     * @return Singular_Module_Settings_Container 
     */
    public function setParams ($params)
    {
        $this->params = $params;
        
        return $this;
    }

    /**
     * Add new param
     * 
     * @param string $key
     * @param mixed $value
     * @return Singular_Module_Settings_Container 
     */
    public function addParam ($key, $value)
    {
        $this->_container['params'][$key] = $value;

        return $this;
    }

    /**
     * Get all params
     * 
     * @param void
     * @return array 
     */
    public function getParams ()
    {
        return $this->params;
    }

    /**
     * Remove param
     * 
     * @param string $name 
     * @return void
     */
    public function removeParam ($name)
    {
        if (array_key_exists($name, $this->params)) {
            unset($this->_container['params'][$name]);
        }
    }

    /**
     * Clear all params
     * 
     * @param void
     * @return void
     */
    public function clearParams ()
    {
        $this->params = array();
    }

    /**
     * Set supply block
     * 
     * @param Singular_Module_Settings_SupplyBlock $block
     * @return Singular_Module_Settings_Container 
     */
    public function setSupplyBlock (Singular_Module_Settings_SupplyBlock $block)
    {
        if (!$block instanceof Singular_Module_Settings_SupplyBlock) {
            throw new Singular_Module_Exception("Supply block must an instance of Singular_Module_Settings_SupplyBlock. " . gettype($block) . ' was given.');
        }

        $this->_supplyBlock = $block;

        return $this;
    }

    /**
     * Has supply block?
     * 
     * @param void
     * @return bool
     */
    public function hasSupplyBlock ()
    {
        return $this->_supplyBlock instanceof Singular_Module_Settings_SupplyBlock;
    }

    /**
     * Remove supply block
     * 
     * @param void
     * @return void
     */
    public function removeSupplyBlock ()
    {
        $this->_supplyBlock = null;
    }

    /**
     * Get supply block
     * 
     * @param void
     * @return Singular_Module_Settings_SupplyBlock
     */
    public function getSupplyBlock ()
    {
        return $this->_supplyBlock;
    }

    /**
     * Assemble url
     * 
     * @param void
     * @return string 
     */
    public function getHref ()
    {
        $front = Zend_Controller_Front::getInstance();
        
        $route = $this->getRoute();
        if (!empty($route)) {
            $url = $front->getRouter()->assemble($this->params, $route, true);
        } else {
            $params = $this->params;
            $params['module'] = (null != $this->module)
                    ? $this->module
                    : $this->getModuleClass()->getModuleName();
            $params['controller'] = (null != $this->controller)
                    ? $this->controller
                    : 'admin';
            $params['action'] = (null != $this->action)
                    ? $this->action
                    : $front->getDefaultAction();

            $url = $front->getRouter()->assemble($params, 'default');
        }

        return $url;
    }

    /**
     * Populate supply block with module data
     * 
     * @param void
     * @return void
     */
    public function populateSupplyBlockWithModuleData ()
    {
        $mContainer = $this->getModuleClass()->getContainer();
        $supplyBlock = ($this->hasSupplyBlock())
                ? $this->getSupplyBlock()
                : new Singular_Module_Settings_SupplyBlock();
        $view = $this->getModuleClass()->view;

        $supplyBlock->setTitle(
            (null != (string) $mContainer->getName())
                    ? $mContainer->getName() . ((null != (string) $mContainer->getVersion())
                            ? ' ' . $mContainer->getVersion()
                            : '')
                    : ucfirst($this->getModuleClass()->getModuleName())
        );
        $supplyBlock->setText(
            (null != (string) $mContainer->getDescription())
                    ? $mContainer->getDescription()
                    : $view->t('No description')
        );

        $info = array(
            $view->t('Module') => (null != (string) $mContainer->getName())
                    ? $mContainer->getName()
                    : null,
            $view->t('Version') => (null != (string) $mContainer->getVersion())
                    ? $mContainer->getVersion()
                    : null,
            $view->t('Release date') => (null != (string) $mContainer->getReleaseDate())
                    ? $mContainer->getReleaseDate()
                    : null,
            $view->t('Author') => (null != (string) $mContainer->getAuthor())
                    ? $mContainer->getAuthor()
                    : null,
            $view->t('Email') => (null != (string) $mContainer->getEmail())
                    ? $mContainer->getEmail(true)
                    : null,
            $view->t('Homepage') => (null != (string) $mContainer->getUrl())
                    ? $mContainer->getUrl(true)
                    : null
        );

        foreach ($info as $infoKey => $infoVal) {
            if (null === $infoVal) unset($info[$infoKey]);
        }

        $supplyBlock->setInfo($info);

        $this->setSupplyBlock($supplyBlock);
    }

    /**
     * Make self hask
     * 
     * @param void
     * @return string
     */
    public function getHash ()
    {
        return spl_object_hash($this);
    }
}
