<?php

class Singular_Navigation extends Zend_Navigation
{
    /**
     * Root element
     */
    const MENU_ROOT_ID = 'root';
    
    /**
     * Finded pages collection
     * 
     * @var Zend_Navigation_Container|null 
     */
    protected static $_menuTypePages = null;

    /**
     * Adds a page to the container
     *
     * This method will inject the container as the given page's parent by
     * calling {@link Zend_Navigation_Page::setParent()}.
     *
     * @param  Zend_Navigation_Page|array|Zend_Config $page  page to add
     * @param  string $parentId                              parent ID
     * @return Zend_Navigation_Container                     fluent interface,
     *                                                       returns self
     * @throws Zend_Navigation_Exception                     if page is invalid
     */
    public function addPage ($page, $parentId = null)
    {        
        if ($page instanceof Zend_Navigation_Page || $page instanceof Zend_Config) {
            $page = $page->toArray();
        }
        
        if (!isset($page['route'])) {
            $page['route'] = 'default';
        }
        
        if (!isset($page['group'])) {
            $page['group'] = 'no_group';
        }
        
        $page['parent_id'] = $parentId;
        
        if ($parentId) {
            $parentPage = $this->findBy('id', $parentId);
            $page = Singular_Navigation_Page::factory($page);
            $parentPage->addPage($page);
        } else {
            return parent::addPage($page);
        }

        return $this;
    }
    
    /**
     * Gets menu root page
     *
     * @return Zend_Navigation_Page
     */
    public function getRootPage ()
    {
        return $this->findById(self::MENU_ROOT_ID);
    }
    
    /**
     * Groups elements
     * 
     * @param Zend_Navigation_Container $container
     * @return array
     */
    public function groupFormat ($container)
    {
        if (!$container instanceof Zend_Navigation_Container) {
            throw new Singular_Navigation_Exception('Invalid container was given. Must be instance of Zend_Navigation_Container.');
        }
        
        $formatted = array();
        /** @var $page Zend_Navigation_Page */
        foreach ($container as $page) {
            if (!$this->isRestricted($page)) {
                continue;
            }
            if (isset($page->no_group) && true === (bool) $page->no_group) {
                continue;
            }
            if (!isset($page->group)) {
                throw new Singular_Navigation_Exception('Can\'t apply group formatting. Page instance needs to contain required "group" parameter. Set "no_group" parameter to true if you want to skip this exception.');
            }
            $formatted[$page->group][] = $page;
        }

        // Moving system group to the first stack
        if (isset($formatted['system'])) {
            $systemGroup = $formatted['system'];
            unset($formatted['system']);
            array_unshift($formatted, $systemGroup);
        }

        return $formatted;
    }
    
    /**
     * Available page?
     * 
     * @param Zend_Navigation_Page $page
     * @return bool
     */
    public function isRestricted (Zend_Navigation_Page $page)
    {
        $view = Zend_Layout::getMvcInstance()->getView();
        if (!$page->isVisible() || !$view->navigation()->accept($page)) {
            return false;
        }

        return true;
    }
    
    /**
     * Fetch menu by type
     * 
     * @param string $menuType
     * @return Zend_Navigation
     */
    public function getMenuByType ($menuType)
    {
        $active = Zend_Layout::getMvcInstance()->getView()->navigation()->findActive($this);
        self::$_menuTypePages = new Singular_Navigation();

        if (empty($active)) {
            $active = array(
                'page' => Zend_Layout::getMvcInstance()->getView()->navigation()->getContainer()->findBy('id', self::MENU_ROOT_ID)->setActive(true),
                'depth' => 0,
            );
        }
        
        if ($menuType == $active['page']->menu_type && $active['depth'] > 0) {
            self::$_menuTypePages->addPages($active['page']->getParent()->getPages());
        } else {
            $down = $this->_liftDown($active['page']);

            if (null !== $down) {
                $this->_findMenuType($down, $menuType);
            }
        }
        
        return self::$_menuTypePages;
    }
    
    /**
     * Finds the most bottom page element
     * 
     * @param Zend_Navigation_Page $page
     * @return Zend_Navigation_Page
     */
    private function _liftDown ($page)
    {        
        if ($page->hasPages()) {
            $pages = $page->getPages();
            foreach ($pages as $_page) {
                $down = $this->_liftDown($_page);
            }
        } else {
            $down = $page;
        }
        
        return $down;
    }
    
    /**
     * Finds menu type pages
     * 
     * @param Zend_Navigation_Page $downPage
     * @return void
     */
    private function _findMenuType (Zend_Navigation_Page $downPage, $menuType)
    {
        $parent = $downPage->getParent();
        if (null !== $parent && $parent instanceof Zend_Navigation_Page) {
            $children = $parent->getPages();
            foreach ($children as $child) {
                if ($child->menu_type == $menuType) {
                    self::$_menuTypePages->addPage($child);
                }
            }
            $this->_findMenuType($parent, $menuType);
        }
    }
}
