<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Sidebar_Container
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Sidebar component
 *
 * @uses       Singular_Sidebar_Container
 * @category   Singular
 * @package    Singular_Sidebar
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Sidebar extends Singular_Sidebar_Container
{
    /**
     * Sidebars storage
     * 
     * @var array
     */
    protected static $_storage = array();

    /**
     * Default sidebar name
     * 
     * @var string|null
     */
    protected static $_default = null;

    /**
     * Class constructor
     * 
     * @throws Singular_Sidebar_Exception
     * @param  string $name
     * @param  array|null $widgets
     */
    public function __construct ($name, $widgets = null)
    {
        parent::__construct();
        if (is_array($widgets)) {
            $this->addWidgets($widgets);
        } elseif (null !== $widgets) {
            throw new Singular_Sidebar_Exception(
                'Invalid argument: $widgets must be an array or null'
            );
        }

        self::addSidebar($name, $this);

        if (!self::hasDefault()) {
            self::setDefault($name);
        }
    }

    /**
     * Sets default sidebar
     * 
     * @static
     * @throws Singular_Sidebar_Exception
     * @param  string $name
     * @return void
     */
    public static function setDefault ($name)
    {
        if (!is_string($name)) {
            throw new Singular_Sidebar_Exception(
                'Invalid argument: $name must be an string'
            );
        }

        if (!self::hasSidebar($name)) {
            throw new Singular_Sidebar_Exception(
                "Invalid argument: there is no registered sidebar with name {$name}"
            );
        }

        self::$_default = $name;
    }

    /**
     * Gets default sidebar
     * 
     * @static
     * @return Singular_Sidebar_Container
     */
    public static function getDefault ()
    {
        if (self::hasDefault()) {
            return self::getSidebar(self::$_default);
        }

        return null;
    }

    /**
     * Checks wherever default sidebar has set or not
     *
     * true  - has set
     * false - has not set
     *
     * @static
     * @return bool
     */
    public static function hasDefault ()
    {
        return (null !== self::$_default);
    }

    /**
     * Removes default sidebar reference
     * 
     * @return void
     */
    public function removeDefault ()
    {
        self::$_default = null;
    }

    /**
     * Render sidebar implementation
     * 
     * @return string
     */
    public function renderSidebar ()
    {
        if ($this->hasWidgets()) {
            $segments = array();

            $this->rewind();

            while ($this->valid()) {
                /** @var $widget Singular_Widget_Abstract */
                $widget = $this->current();
                $segments[] = $widget->render();
                $this->next();
            }

            if (count($segments) > 0) {
                return implode($this->getSeparator(), $segments);
            }
        }

        return;
    }

    /**
     * Store initialized sidebar instance with name $name
     *
     * @static
     * @throws Singular_Sidebar_Exception
     * @param  string $name
     * @param  Singular_Sidebar_Container $sidebar
     * @return void
     */
    public static function addSidebar ($name, Singular_Sidebar_Container $sidebar)
    {
        if (!is_string($name)) {
            throw new Singular_Sidebar_Exception(
                'Invalid argument: $name must be an string'
            );
        }
        if (self::hasSidebar($name)) {
            throw new Singular_Sidebar_Exception(
                "Can't add sidebar: sidebar with name \"{$name}\" has already exists"
            );
        }
        if (!$sidebar instanceof Singular_Sidebar_Container) {
            throw new Singular_Sidebar_Exception(
                'Invalid argument: $sidebar must be an instance of Singular_Sidebar_Container'
            );
        }

        self::$_storage[$name] = $sidebar;
    }

    /**
     * Gets sidebar by given name
     * 
     * @static
     * @throws Singular_Sidebar_Exception
     * @param  string $name
     * @return Singular_Sidebar_Container
     */
    public static function getSidebar ($name)
    {
        if (!self::$_storage[$name]) {
            throw new Singular_Sidebar_Exception(
                "Can't get sidebar: sidebar with name \"{$name}\" is not exists"
            );
        }

        return self::$_storage[$name];
    }

    /**
     * Check if sidebar with given name was initialized
     * 
     * @static
     * @param  string $name
     * @return bool
     */
    public static function hasSidebar ($name)
    {
        return ((array_key_exists($name, self::$_storage)
            && self::$_storage[$name] instanceof Singular_Sidebar_Container));
    }

    /**
     * Removes sidebar by given name
     * 
     * @static
     * @param  string $name
     * @return void
     */
    public static function removeSidebar ($name)
    {
        if (self::hasSidebar($name)) {
            unset(self::$_storage[$name]);
        }
    }
}
