<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Sidebar
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Sidebar container
 *
 * @abstract
 * @uses       ArrayIterator
 * @category   Singular
 * @package    Singular_Sidebar
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
abstract class Singular_Sidebar_Container extends ArrayIterator
{
    /**
     * View object
     *
     * @var Zend_View_Interface
     */
    public $view = null;
    
    /**
     * Partial view script to use for rendering widget
     *
     * @var null|array
     */
    protected $_partial = null;

    /**
     * Separator
     * 
     * @var string|null
     */
    protected $_separator = "<br />";

    /**
     * Sidebar enable flag
     *
     * true (default) - enabled
     * false          - disabled
     *
     * @var bool
     */
    protected $_enabled = true;

    /**
     * Widgets paths
     * 
     * @var array
     */
    protected static $_paths = array();

    /**
     * Class constructor
     */
    public function __construct ()
    {
        $this->_initWidgetsPaths();
        $this->_initView();
    }

    /**
     * Initializes Zend_View
     *
     * @return void
     */
    private function _initView ()
    {
        if (null === $this->view) {
            $this->view = Zend_Layout::getMvcInstance()->getView();
        }
    }

    /**
     * Adds multiple widgets
     * 
     * @param  array $widgets
     * @return void
     */
    public function addWidgets ($widgets)
    {
        if (is_array($widgets) && !empty($widgets)) {
            foreach ($widgets as $widget) {
                $this->addWidget($widget);
            }
        }
    }

    /**
     * Adds widget
     * 
     * @throws Singular_Sidebar_Exception
     * @param  Singular_Widget_Abstract $widget
     * @param  string $id
     * @param  array|null $options
     * @param  int|null $offset
     * @return Singular_Sidebar_Container
     */
    public function addWidget ($widget, $id = null, $options = null, $offset = null)
    {
        if (is_string($widget)) {
            $widget = $this->_getWidgetByName($widget);
        }
        
        if (!$widget instanceof Singular_Widget_Abstract) {
            throw new Singular_Sidebar_Exception(
                'Invalid widget: $widget must be an instance of Singular_Widget_Abstract ' .
                'or valid string with widget name'
            );
        }

        // set id
        if (null !== $id) {
            $widget->setId($id);
        }

        // Check for widget is already in container
        if ($this->hasWidget($widget->getId())) {
            throw new Singular_Sidebar_Exception(
                "Unable to add widget with id \"{$widget->getId()}\". " .
                'Widget with such id already exists'
            );
        }

        // set options
        if (null !== $options) {
            $widget->setOptions($options);
        }

        // set position offset
        if (is_int($offset)) {
            $widget->offsetSet($offset);
        }
        
        $this->offsetSet($widget->getId(), $widget);

        return $this;
    }

    public function getWidget ($id)
    {
        $this->rewind();

        while ($this->valid()) {
            /** @var $widget Singular_Widget_Abstract */
            $widget = $this->current();
            if ($id == $widget->getId()) {
                return $widget;
            }

            $this->next();
        }

        return null;
    }

    public function hasWidget ($id)
    {
        $this->rewind();

        while ($this->valid()) {
            /** @var $widget Singular_Widget_Abstract */
            $widget = $this->current();
            if ($id == $widget->getId()) {
                return true;
            }

            $this->next();
        }

        return false;
    }


    public function removeWidget ($id)
    {
        $this->rewind();

        while ($this->valid()) {
            /** @var $widget Singular_Widget_Abstract */
            $widget = $this->current();
            if ($id == $widget->getId()) {
                $this->offsetUnset($this->key());
                break;
            }
            
            $this->next();
        }

        return $this;
    }

    /**
     * Gets all widgets
     * 
     * @return array
     */
    public function getAllWidgets ()
    {
        return $this->getArrayCopy();
    }

    /**
     * Check whether sidebar contains one or more widgets
     * 
     * @return bool
     */
    public function hasWidgets ()
    {
        return $this->count() > 0;
    }

    /**
     * Gets widgets separator
     *
     * @return null|string
     */
    public function getSeparator ()
    {
        return $this->_separator;
    }

    /**
     * Sets widgets separator
     * 
     * @throws Singular_Sidebar_Exception
     * @param  string|null $value
     * @return void
     */
    public function setSeparator ($value)
    {
        if (!is_string($value) || !is_null($value)) {
            throw new Singular_Sidebar_Exception(
                'Invalid argument: $value must be an sting or null'
            );
        }

        $this->_separator = $value;
    }

    /**
     * Sets enable flag
     * 
     * @param  bool $flag
     * @return Singular_Sidebar_Container
     */
    public function setEnable ($flag)
    {
        $this->_enabled = (bool) $flag;
        return $this;
    }

    /**
     * Gets enable flag
     * 
     * @return bool
     */
    public function isEnabled ()
    {
        return $this->_enabled;
    }

    /**
     * Sets which partial view script to use for rendering sidebar
     *
     * @param  string|array $partial             partial view script or null. If
     *                                           an array is given, it is
     *                                           expected to contain two values;
     *                                           the partial view script to use,
     *                                           and the module where the script
     *                                           can be found.
     * @return Singular_Sidebar_Container        fluent interface, returns self
     */
    public function setPartial ($partial)
    {
        if (null === $partial || is_string($partial) || is_array($partial)) {
            $this->_partial = $partial;
        }

        return $this;
    }

    /**
     * Returns partial view script to use for rendering sidebar
     *
     * @return string|array|null
     */
    public function getPartial ()
    {
        return $this->_partial;
    }

    /**
     * Initializes default widgets paths
     * 
     * @return void
     */
    protected function  _initWidgetsPaths ()
    {
        $paths = array(
            'Singular_Widget' => LIBRARY_PATH . '/Singular/Widget'
        );

        $modules = Singular_Runtime::extract('active_modules');

        foreach ($modules as $module) {
            $namespace = ucfirst($module) . '_Widget';
            $path      = MODULES_PATH . "/{$module}/widgets";
            $paths[$namespace] = $path;
        }

        self::$_paths = array_reverse($paths);
    }

    private function _getWidgetByName ($name)
    {
        $widget = null;

        foreach (self::$_paths as $ns => $path) {
            $filepath = $path . "/{$name}.php";
            $class    = $ns . "_{$name}";
            if (!class_exists($class)) {
                if (file_exists($filepath)) {
                    require_once $filepath;
                    Zend_Loader::loadClass($class);
                }
            }

            $widget = new $class;
            break;
        }

        return $widget;
    }

    /**
     * Gets all registered widgets paths
     * 
     * @return array
     */
    public function getWidgetsPaths ()
    {
        return self::$_paths;
    }

    /**
     * Adds multiple widgets paths
     * 
     * @throws Singular_Sidebar_Exception
     * @param  array $paths
     * @return Singular_Sidebar_Container
     */
    public function addWidgetsPaths ($paths)
    {
        if (!is_array($paths)) {
            throw new Singular_Sidebar_Exception(
                'Invalid argument: $paths mus be an array'
            );
        }

        foreach ($paths as $path) {
            if (isset($path['namespace']) && isset($path['path'])) {
                $this->addPath($path);
            }
        }

        return $this;
    }

    /**
     * Adds widgets path
     *
     * @throws Singular_Sidebar_Exception
     * @param  string|array $namespace
     * @param  string|null $path
     * @return Singular_Sidebar_Container
     */
    public function addWidgetsPath ($namespace, $path = null)
    {
        if (is_array($namespace)
            && isset($namespace['namespace'])
            && isset($namespace['path'])) {
            $path      = $namespace['path'];
            $namespace = $namespace['namespace'];
        }


        if (!is_string($namespace)) {
            throw new Singular_Sidebar_Exception(
                'Invalid argument: $namespace must be an string or valid array'
            );
        }
        if ( !is_string($path)) {
            throw new Singular_Sidebar_Exception(
                'Invalid argument: $path must be an string or null if $namespace is valid array'
            );
        }

        self::$_paths = array($namespace => $path) + self::$_paths;

        return $this;
    }

    /**
     * Reset widgets paths
     * 
     * @return Singular_Sidebar_Container
     */
    public function resetWidgetsPaths ()
    {
        $this->_initWidgetsPaths();
        return $this;
    }

    /**
     * Renders the sidebar by invoking the partial view helper
     *
     * The container will simply be passed on as a model to the view script
     * as-is, and will be available in the partial script as 'sidebar', e.g.
     * <code>echo 'Number of pages: ', count($this->container);</code>.
     *
     * @param  string|array             $partial     [optional] partial view
     *                                               script to use. Default is to
     *                                               use the partial registered
     *                                               in the helper. If an array
     *                                               is given, it is expected to
     *                                               contain two values; the
     *                                               partial view script to use,
     *                                               and the module where the
     *                                               script can be found.
     * @return string                                helper output
     */
    public function renderPartial ($partial = null)
    {
        if (null === $partial) {
            $partial = $this->getPartial();
        }

        if (empty($partial)) {
            $e = new Zend_View_Exception(
                'Unable to render sidebar: No partial view script provided'
            );
            $e->setView($this->view);
            throw $e;
        }

        $model = array(
            'sidebar' => $this
        );

        if (is_array($partial)) {
            if (count($partial) != 2) {
                $e = new Zend_View_Exception(
                    'Unable to render menu: A view partial supplied as '
                    . 'an array must contain two values: partial view '
                    . 'script and module where script can be found'
                );
                $e->setView($this->view);
                throw $e;
            }

            return $this->view->partial($partial[0], $partial[1], $model);
        }

        return $this->view->partial($partial, null, $model);
    }

    /**
     * Render sidebar abstract implementation
     *
     * @abstract
     * @return void
     */
    public abstract function renderSidebar ();

    /**
     * Renders sidebar
     *
     * If a partial view is registered in the helper, the sidebar will be rendered
     * using the given partial script. If no partial is registered, the sidebar
     * will be rendered by renderSidebar() internal method.
     *
     * @see renderPartial()
     * @see renderSidebar()

     * @return string
     */
    public function render ()
    {
        if ($partial = $this->getPartial()) {
            return $this->renderPartial($partial);
        } else {
            return $this->renderSidebar();
        }
    }
}
