<?php
 
abstract class Singular_Taxonomy extends Singular_Core_Taxonomy
{
    const CLUSTER_FIELD_EMPTY = null;

    /**
     * @var Singular_Taxonomy_Cluster_Structure
     */
    protected $_structure = null;

    /**
     * @var Singular_Taxonomy_ActiveRecord
     */
    protected $_activeRecord = null;

    /**
     * @var bool
     */
    protected $_allowEmptyCluster = false;

    /**
     * @var bool
     */
    protected $_checkClusterDataIntegrity = true;

    /**
     * @var bool
     */
    protected $_mergeClusterWithStructure = true; 

    /**
     * @var array
     */
    private $_validPositions = array(
        Singular_Db_Table_NestedSet::FIRST_CHILD,
        Singular_Db_Table_NestedSet::LAST_CHILD,
        Singular_Db_Table_NestedSet::NEXT_SIBLING,
        Singular_Db_Table_NestedSet::PREV_SIBLING,
    );

    /**
     * @var bool
     */
    private $_useClusterSearchExact = false;

    /**
     * @var bool
     */
    private $_useClusterSearchRegexp = false;

    /**
     * @var string
     */
    protected $_moduleName = null;

    /**
     * Singular_Taxonomy constructor
     *
     * @final final
     */
    final public function __construct ()
    {
        parent::__construct();

        $this->_initActiveRecord();
        $this->init();
    }

    public abstract function init ();

    /**
     * @return void
     */
    protected function _initActiveRecord ()
    {
        if (null === $this->_activeRecord) {
            $this->_activeRecord = new Singular_Taxonomy_ActiveRecord();
        }
    }

    /**
     * @throws Singular_Taxonomy_Exception
     * @param Singular_Taxonomy_ActiveRecord $activeRecord
     * @return void
     */
    public function setActiveRecord ($activeRecord)
    {
        if (!$activeRecord instanceof Singular_Taxonomy_ActiveRecord) {
            throw new Singular_Taxonomy_Exception('Invalid Active record, must be instance of Singular_Taxonomy_ActiveRecord.');
        }
        $this->_activeRecord = $activeRecord;

        return $this;
    }

    /**
     * @return Singular_Taxonomy_ActiveRecord
     */
    public function getActiveRecord ()
    {
        $this->_initActiveRecord();

        return $this->_activeRecord;
    }

    /**
     * Wrapper for getActiveRecord
     *
     * @return Singular_Taxonomy_ActiveRecord
     */
    public function activeRecord ()
    {
        return $this->getActiveRecord();
    }

    /**
     * @throws Singular_Taxonomy_Exception
     * @param array|Zend_Config|Singular_Taxonomy_Cluster_Structure $structure
     * @param bool $allowModifications
     * @return void
     */
    protected function _setClusterStructure ($structure, $allowModifications = false)
    {
        if ($structure instanceof Singular_Taxonomy_Cluster_Structure) {
            $this->_structure = $structure;
        } elseif ($structure instanceof Zend_Config || is_array($structure)) {
            $this->_structure = new Singular_Taxonomy_Cluster_Structure(
                $structure,
                $allowModifications);
        } else {
            throw new Singular_Taxonomy_Exception("Invalid cluster structure. \nStructure must be an instance of Zend_Config, Singular_Taxonomy_Cluster_Structure or has array type.");
        }
    }

    /**
     * @throws Singular_Taxonomy_Exception
     * @param bool $asArray
     * @return array|Zend_Config
     */
    public function getClusterStructure ($asArray = false)
    {
        /** @var $structure Singular_Taxonomy_Cluster_Structure */
        $structure = $this->_structure;
        if (null === $structure) {
            throw new Singular_Taxonomy_Exception('Cluster structure wasn\'t set.');
        }

        return (true === (bool) $asArray)
                ? $structure->getStructure(true)
                : $structure;
    }

    /**
     * @param array|Zend_Config|Singular_Taxonomy_Cluster_Data $cluster
     * @return void
     */
    public function setCluster ($cluster)
    {
        $this->getActiveRecord()->setCluster($cluster);
    }

    /**
     * @return Singular_Taxonomy_Cluster_Data
     */
    public function getCluster ()
    {
        return $this->getActiveRecord()->getCluster();
    }

    /**
     * Inserts module into taxonomy root
     *
     * @throws Singular_Taxonomy_Exception
     * @param null|Singular_Taxonomy_ActiveRecord $activeRecord
     * @return void
     */
    public function insertModule (Singular_Taxonomy_ActiveRecord $activeRecord = null)
    {
        $activeRecord = $this->_recognizeActiveRecord($activeRecord);

        $moduleName = $this->getModuleName();

        // Module name validation
        if (!$this->_isValidModuleName($moduleName)) {
            throw new Singular_Taxonomy_Exception("Invalid module name, not empty string needed. Use setModuleName() method to set module name.");
        }

        // Is module installed?
        if (!in_array($moduleName, Singular_Runtime::extract('active_modules'))) {
            throw new Singular_Taxonomy_Exception("Module \"$moduleName\" is not installed or is not active.");
        }

        // Does module already exists?
        if ($this->isModuleInTaxonomy($moduleName)) {
            throw new Singular_Taxonomy_Exception("Can't insert module \"$moduleName\". Such module already exists.");
        }

        /** @var $activeRecord Singular_Taxonomy_ActiveRecord */
        if (!is_string($activeRecord->getAdminRoute())) {
            $activeRecord->setAdminRoute('admin_taxonomy_type_module_default');
        }

        if (!is_string($activeRecord->getFrontendRoute())) {
            $activeRecord->setFrontendRoute('frontend_taxonomy_type_module_default');
        }

        $activeRecord->setModule($moduleName);
        $activeRecord->setType(Singular_Taxonomy_ActiveRecord::TAX_TYPE_MODULE);

        if ($this->isMergeClusterWithStructure()) {
            $activeRecord = $this->_mergeClusterWithStructure($activeRecord);
        }

        // Insert routine
        return $this->_insert($activeRecord);
    }

    /**
     * Inserts record into taxonomy
     *
     * Can be appended or prepended to any section or module
     * by their given $objectiveNodeId.
     *
     * @param int $objectiveNodeId
     * @param Singular_Taxonomy_ActiveRecord|null $activeRecord
     * @param string $position
     * @return void
     */
    public function insertRecord ($parentNodeId, Singular_Taxonomy_ActiveRecord $activeRecord = null,
        $position = Singular_Db_Table_NestedSet::FIRST_CHILD)
    {
        if (!$this->nodeIsExists($parentNodeId)) {
            throw new Singular_Taxonomy_Exception("Can't insert record, because parent node id \"$parentNodeId\" is not exists.");
        }

        $activeRecord = $this->_recognizeActiveRecord($activeRecord);

        $moduleName = $this->getModuleName();

        /** @var $activeRecord Singular_Taxonomy_ActiveRecord */
        if (!is_string($activeRecord->getAdminRoute())) {
            $activeRecord->setAdminRoute('admin_taxonomy_type_record_default');
        }

        if (!is_string($activeRecord->getFrontendRoute())) {
            $activeRecord->setFrontendRoute('frontend_taxonomy_type_record_default');
        }

        $activeRecord->setModule($moduleName);
        $activeRecord->setType(Singular_Taxonomy_ActiveRecord::TAX_TYPE_RECORD);

        // Wherever is need to merge cluster data with structure
        if ($this->isMergeClusterWithStructure()) {
            $activeRecord = $this->_mergeClusterWithStructure($activeRecord);
        }

        // Insert routine
        return $this->_insert($activeRecord, $parentNodeId, $position);
    }

    /**
     * Inserts section into taxonomy
     *
     * @param int $parentNodeId
     * @param Singular_Taxonomy_ActiveRecord|null $activeRecord
     * @param string string $position
     * @return int
     */
    public function insertSection ($parentNodeId, Singular_Taxonomy_ActiveRecord $activeRecord = null,
        $position = Singular_Db_Table_NestedSet::FIRST_CHILD)
    {
        $activeRecord = $this->_recognizeActiveRecord($activeRecord);

        $moduleName = $this->getModuleName();

        /** @var $activeRecord Singular_Taxonomy_ActiveRecord */
        if (!is_string($activeRecord->getAdminRoute())) {
            $activeRecord->setAdminRoute('admin_taxonomy_type_section_default');
        }

        if (!is_string($activeRecord->getFrontendRoute())) {
            $activeRecord->setFrontendRoute('frontend_taxonomy_type_section_default');
        }

        $activeRecord->setModule($moduleName);
        $activeRecord->setType(Singular_Taxonomy_ActiveRecord::TAX_TYPE_SECTION);

        // Insert routine
        return $this->_insert($activeRecord, $parentNodeId, $position);
    }

    /**
     * @param int|string $module
     * @return Singular_Taxonomy_ActiveRecord
     */
    public function getModule ($module)
    {
        if (is_int($module) || is_numeric($module)) {
            $findBy = 'id';
            settype($module, 'integer');
        } elseif (is_string($module)) {
            $findBy = 'module';
            settype($module, 'string');
        } else {
            throw new Singular_Taxonomy_Exception(
                "Module parameter must be an string|integer. " . gettype($module) . ' was given.'
            );
        }

        $entry = $this->getNodeByField($findBy, $module);

        if (null === $entry) {
            return null;
        }

        $this->setActiveRecord(
            new Singular_Taxonomy_ActiveRecord($entry, false, true)
        );

        unset($entry);

        return $this->getActiveRecord();
    }

    /**
     * @return array|null
     */
    public function getModules ()
    {
        $model = $this->getModel();
        $moduleType = $model->getAdapter()
                ->quote(Singular_Taxonomy_ActiveRecord::TAX_TYPE_MODULE);
        $where = $model->select()
                ->where("type = {$moduleType}");

        $modules = $this->getModel()->fetchAll($where);

        if (null !== $modules) {
            $modules = $modules->toArray();
        } else {
            return null;
        }

        foreach ($modules as $moduleIdx => $module) {
            $ar = new Singular_Taxonomy_ActiveRecord($module, false, true);
            unset($modules[$moduleIdx]);
            $modules[(string)$ar->getModule()] = $ar;
            unset($ar);
        }

        return $modules;
    }

    /**
     * Finds all taxonomy entries depending given conditions
     * 
     * @param array $where        OPTIONAL An SQL WHERE clause.
     * @param array $clusterWhere OPTIONAL An SQL xPath WHERE clause.
     * @param bool $getAsTree     OPTIONAL Whether to retrieve nodes as tree.
     * @param string|array $order OPTIONAL An SQL ORDER clause.
     * @param int $count          OPTIONAL An SQL LIMIT count.
     * @param int $offset         OPTIONAL An SQL LIMIT offset.
     * @return array|null
     */
    public function findAll ($where = null, $clusterWhere = null, $getAsTree = false, $order = null, $count = null,
        $offset = null)
    {
        if (null !== $clusterWhere) {
            $clusterWhere = $this->_clusterWhere($clusterWhere,
                (true === (bool) $getAsTree
                        ? Singular_Db_Table_NestedSet::LEFT_TBL_ALIAS . '.'
                          . Singular_Taxonomy_ActiveRecord::CLUSTER_COLUMN
                        : Singular_Taxonomy_ActiveRecord::CLUSTER_COLUMN)
            );
            $clusterWhere = implode(' ', $clusterWhere);

            $where[] = $clusterWhere;
        }

        $result = $this->getModel()
                ->fetchAll($where, $getAsTree, 'parent_id', $order, $count, $offset);

        if (null !== $result) {
            $result = $result->toArray();
            foreach ($result as $rowIdx => $row) {
                $result[$rowIdx] = new Singular_Taxonomy_ActiveRecord($row, false, true);
            }
        }

        return $result;
    }

    /**
     * Finds all taxonomy entries depending model module name ang given conditions
     *
     * @see self::findAll()
     * @param array $where        OPTIONAL An SQL WHERE clause.
     * @param array $clusterWhere OPTIONAL An SQL xPath WHERE clause.
     * @param bool $getAsTree     OPTIONAL Whether to retrieve nodes as tree.
     * @param string|array $order OPTIONAL An SQL ORDER clause.
     * @param int $count          OPTIONAL An SQL LIMIT count.
     * @param int $offset         OPTIONAL An SQL LIMIT offset.
     * @return array|null
     */
    public function findAllByThisModule ($where = null, $clusterWhere = null, $getAsTree = false, $order = null
        , $count = null, $offset = null)
    {
        $column = (true === (bool) $getAsTree)
                ? Singular_Db_Table_NestedSet::LEFT_TBL_ALIAS . '.' . 'module'
                : 'module';

        $where[] = "{$column} = '" . $this->getModuleName() . "'";

        return $this->findAll($where, $clusterWhere, $getAsTree , $order, $count, $offset);
    }

    /**
     * Finds one/first taxonomy entry depending given conditions
     *
     * @param array $where            OPTIONAL An SQL WHERE clause.
     * @param array $clusterWhere  OPTIONAL An SQL xPath WHERE clause.
     * @param string|array $order    OPTIONAL An SQL ORDER clause.
     * @return Singular_Taxonomy_ActiveRecord|null
     */
    public function findOne ($where = null, $clusterWhere = null, $order = null)
    {
        if (null !== $clusterWhere) {
            $clusterWhere = $this->_clusterWhere($clusterWhere);
            $clusterWhere = implode(' ', $clusterWhere);

            $where[] = $clusterWhere;
        }

        $result = $this->getModel()->fetchRow($where, 'parent_id', $order);

        if (null !== $result) {
            $result = $result->toArray();
            $result = new Singular_Taxonomy_ActiveRecord($result, false, true);
        }

        return $result;
    }

    public function expand ($nodeId, $order = null, $count = null, $offset = null)
    {
        $level = $this->getModel()->expandNode($nodeId, $order, $count, $offset);
        
        if (null !== $level) {
            $level = $level->toArray();

            foreach ($level as $rowIdx => $row) {
                $level[$rowIdx] = new Singular_Taxonomy_ActiveRecord($row, false, true);
            }
        }

        return $level;
    }

    public function update ($activeRecord = null)
    {
        $activeRecord = $this->_recognizeActiveRecord($activeRecord);

        $id                       = $activeRecord->id;
        $type                     = $activeRecord->getType();
        $allowEmptyClusterInitial = $this->getAllowEmptyCluster();

        switch ($type) {
            case Singular_Taxonomy_ActiveRecord::TAX_TYPE_MODULE:
                $this->setAllowEmptyCluster(true);
                Singular_Event::dispatch('taxonomyModuleUpdate', $activeRecord);
                break;
            case Singular_Taxonomy_ActiveRecord::TAX_TYPE_RECORD:
                $this->setAllowEmptyCluster(false);
                Singular_Event::dispatch('taxonomyRecordUpdate', $activeRecord);
                break;
            case Singular_Taxonomy_ActiveRecord::TAX_TYPE_SECTION:
                $this->setAllowEmptyCluster(true);
                Singular_Event::dispatch('taxonomySectionUpdate', $activeRecord);
                break;
            default:
                throw new Singular_Taxonomy_Exception("Incorrect taxonomy node type \"{$type}\".");
                break;
        }

        // Prepare update
        $activeRecord->prepareUpdate();

        // If is needed checks cluster data integrity
        if (true === $this->getCheckClusterDataIntegrity()) {
            try {
                $cluster = $activeRecord->getCluster(true);
                $this->_checkClusterDataIntegrity($cluster->toArray());
            } catch (Exception $e) {
                // If empty cluster is not allowed - re-throw exception
                if (!$this->getAllowEmptyCluster()) {
                    throw new Singular_Taxonomy_Exception('Empty cluster is not allowed: ' . $e->getMessage());
                }
            }
        }

        // Get record data for update
        $record = $activeRecord->getRecord(true, true);
        // Update routine
        $this->updateNodeData($record, $id);
        // Clear table cache
        $this->getModel()->cleanCache();
        // Restore _allowEmptyCluster value
        $this->setAllowEmptyCluster($allowEmptyClusterInitial);
    }

    /**
     * @param string $pathTraverse
     * @param string $column
     * @return string
     */
    private function _xpathExpr ($pathTraverse, $column = Singular_Taxonomy_ActiveRecord::CLUSTER_COLUMN)
    {
        if (false === stripos($pathTraverse, '/singular-cluster') &&
            false === stripos($pathTraverse, '//singular-cluster') &&
            false === stripos($pathTraverse, 'singular-cluster')) {
            if ('//' != substr($pathTraverse, 0, 2)) {
                $nodeRoot = ('/' != substr($pathTraverse, 0, 1)) ? '/' : '';
                $pathTraverse = '/singular-cluster'
                                . $nodeRoot
                                . $pathTraverse;
            }
        }

        return "EXTRACTVALUE({$column}, '{$pathTraverse}')";
    }

    private function _clusterWhere ($where, $column = Singular_Taxonomy_ActiveRecord::CLUSTER_COLUMN)
    {
        $where = (array) $where;
        $select = $this->getModel()->select();

        foreach ($where as $key => $val) {
            // is $key an int?
            if (is_int($key)) {
                // $val is the full condition
                $select->where($val);
                continue;
            } else {
                // $key is the condition with placeholder,
                // and $val is quoted into the condition
                $exact = $this->getUseClusterSearchExact();
                $regexp = $this->getUseClusterSearchRegexp();

                if (is_array($val)) {
                    if (array_key_exists('value', $val)) {
                        $exact = (array_key_exists('exact', $val))
                                ? (bool) $val['exact']
                                : $exact;

                        $regexp = (array_key_exists('use_regexp', $val))
                                ? (bool) $val['use_regexp']
                                : $regexp;

                        $isBool = (array_key_exists('is_bool', $val))
                                ? (bool) $val['is_bool']
                                : false;

                        $cs = $this->_clusterSearch($key, $val['value'], $exact, $regexp, $column);

                        (true === (bool) $isBool) // true = AND, false = OR
                                ? $select->where($cs)
                                : $select->orWhere($cs);

                        continue;
                    }
                }

                $select->where($this->_clusterSearch($key, $val, $exact, $regexp, $column));
            }
        }

        unset($where);

        return $select->getPart('where');
    }

    /**
     * @param string $pathWhere
     * @param string|int $value
     * @param bool $exact
     * @param bool $regexp
     * @return string
     */
    private function _clusterSearch ($pathWhere, $value, $exact = true, $regexp = false,
        $column = Singular_Taxonomy_ActiveRecord::CLUSTER_COLUMN)
    {
        $xpathCase = $this->_xpathExpr($pathWhere, $column);
        $comparison = (false === (bool) $regexp) ? 'LIKE' : 'REGEXP';
        $exact = (true === (bool) $regexp) ? true : $exact;
        $tor = (true === (bool) $exact) ? '' : '%';
        $value = $tor . trim($value) . $tor;
        $where = $this->getModel()
                ->getAdapter()
                ->quoteInto("{$xpathCase} {$comparison} ?", $value);

        return $where;
    }

    private function _fulltextSearchExpr ($columns, $words,
        $criteria = 'any', $inBooleanMode = true)
    {
        // Check criteria
        switch ($criteria) {
            case 'any':
            case 'all':
            case 'strict':
                break;
            default:
                $criteria = 'any';
                break;
        }

        // Process words
        if (is_array($words)) {
            foreach ($words as $wIdx => $word) {
                $word = trim($word);
                
                if (!is_string($word) || empty($word)) continue;

                switch ($criteria) {
                    case 'any':
                        $word = $word . '*';
                        break;
                    case 'all':
                        $word = "+(" . $word . ")";
                        break;
                    case 'strict':
                        $word = '+' . $word . '*';
                        break;
                }
                
                $words[$wIdx] = $word;
            }
        } elseif (is_string($words)) {
            $words = explode(' ', $words);

            foreach ($words as $wIdx => $word) {
                $word = trim($word);

                if (empty($word)) continue;

                $words[$wIdx] = $word = $word . '*';
            }
        } else {
            throw new Singular_Taxonomy_Exception("Invalid type for parameter \"words\", must be an array|string. " . gettype($words) . ' was given.');
        }

        // Process columns
        if (is_array($columns)) {
            foreach ($columns as $cIdx => $column) {
                $column = trim($column);
                $columns[$cIdx] = "`{$column}`";
            }
        } elseif (is_string($columns)) {
            $columns = array('`' . trim($columns) . '`');
        }

        $matchCase = 'MATCH (' . implode(', ', $columns) . ')';
        
        $againstCase = "AGAINST ('"
                       . implode(' ', $words)
                       . (true === (bool) $inBooleanMode) ? ' IN BOOLEAN MODE' : ''
                       . ')';

        return " {$matchCase} {$againstCase} ";
    }

    public function nodeIsExists ($objectiveNodeId)
    {
        return (null !== $this->getNodeById($objectiveNodeId))
                ? true
                : false;
    }

    public function isModuleInTaxonomy ($module)
    {
        $where = array(
            "`type` = '" . Singular_Taxonomy_ActiveRecord::TAX_TYPE_MODULE . "'",
            "`module` = '{$module}'"
        );
        $checkModule = $this->getModel()->fetchRow($where);

        if (null === $checkModule) {
            return false;
        }

        return true;
    }

    private function _insert ($activeRecord = null, $objectiveNodeId = null,
        $position = Singular_Db_Table_NestedSet::LAST_CHILD)
    {
        $activeRecord = $this->_recognizeActiveRecord($activeRecord);
        $type = $activeRecord->getType();
        $allowEmptyClusterInitial = $this->getAllowEmptyCluster();
        
        switch ($type) {
            case Singular_Taxonomy_ActiveRecord::TAX_TYPE_MODULE:
                $objectiveNodeId = null;
                $this->setAllowEmptyCluster(true);
                Singular_Event::dispatch('taxonomyModuleInsert', $activeRecord);
                break;
            case Singular_Taxonomy_ActiveRecord::TAX_TYPE_RECORD:
                $this->setAllowEmptyCluster(false);
                Singular_Event::dispatch('taxonomyRecordInsert', $activeRecord);
                break;
            case Singular_Taxonomy_ActiveRecord::TAX_TYPE_SECTION:
                $this->setAllowEmptyCluster(true);
                Singular_Event::dispatch('taxonomySectionInsert', $activeRecord);
                break;
            default:
                throw new Singular_Taxonomy_Exception("Incorrect taxonomy node type \"{$type}\".");
                break;
        }

        // Prepare fields
        $activeRecord->prepareInsert();

        // If is needed checks cluster data integrity
        if (true === $this->getCheckClusterDataIntegrity()) {
            try {
                $cluster = $activeRecord->getCluster(true);
                $this->_checkClusterDataIntegrity($cluster->toArray());
            } catch (Exception $e) {
                // If empty cluster is not allowed - re-throw exception
                if (!$this->getAllowEmptyCluster()) {
                    throw new Singular_Taxonomy_Exception('Empty cluster is not allowed: ' . $e->getMessage());
                }
            }
        }

        // Validate position
        if (!in_array($position, $this->_validPositions)) {
            throw new Singular_Taxonomy_Exception("Invalid node position \"{$position}\". Valid positions are: " . implode(', ', $this->_validPositions));
        }

        $record = $activeRecord->getRecord();
        unset($activeRecord);

        // Add record node into taxonomy
        $result = $this->addNode($record, $objectiveNodeId, $position);

        // Reset Active record data
        $this->activeRecord()->reset();
        // Clear table cache
        $this->getModel()->cleanCache();
        // Restore _allowEmptyCluster value
        $this->setAllowEmptyCluster($allowEmptyClusterInitial);

        return $result;
    }

    protected function _isValidModuleName ($module = null)
    {
        if (null === $module) {
            $module = $this->getModuleName();
        }

        if (is_string($module) && !empty($module) && !is_numeric($module)) {
            return true;
        }

        return false;
    }

    public function setAllowEmptyCluster ($flag)
    {
        $this->_allowEmptyCluster = (bool) $flag;
    }

    public function getAllowEmptyCluster ()
    {
        return (bool) $this->_allowEmptyCluster;
    }

    /**
     * @param bool $flag
     * @return Singular_Taxonomy
     */
    public function setCheckClusterDataIntegrity ($flag)
    {
        $this->_checkClusterDataIntegrity = (bool) $flag;

        return $this;
    }

    /**
     * @return bool
     */
    public function getCheckClusterDataIntegrity ()
    {
        return (bool) $this->_checkClusterDataIntegrity;
    }

    /**
     * @param bool $flag
     * @return Singular_Taxonomy
     */
    public function setUseClusterSearchExact ($flag)
    {
        $this->_useClusterSearchExact = (bool) $flag;

        return $this;
    }

    /**
     * @return bool
     */
    public function getUseClusterSearchExact ()
    {
        return (bool) $this->_useClusterSearchExact;
    }

    /**
     * @param bool $flag
     * @return Singular_Taxonomy
     */
    public function setUseClusterSearchRegexp ($flag)
    {
        $this->_useClusterSearchRegexp = (bool) $flag;

        return $this;
    }

    /**
     * @return bool
     */
    public function getUseClusterSearchRegexp ()
    {
        return (bool) $this->_useClusterSearchRegexp;
    }
    
    protected function _checkClusterDataIntegrity ($clusterData, $structureStack = null)
    {
        $structure = (null === $structureStack)
                ? $this->getClusterStructure(true)
                : $structureStack;

        $currStack = $structure;

        static $stackPath = array();

        $stackPathIndex = count($stackPath);

        foreach ($clusterData as $param => $value) {
            $stackPath[$stackPathIndex] = $param;
            if (array_key_exists($param, $currStack)) {
                if (is_array($value)) {
                    if (empty($value) || is_bool($value)) {
                        continue;
                    } else {
                        $this->_checkClusterDataIntegrity($value, $currStack[$param]);
                    }
                }
            } else {
                throw new Singular_Taxonomy_Exception("Field \"{$param}\" was not defined in cluster structure. \n" . 'Data stack path: ' . implode(' > ', $stackPath));
            }
        }
    }

    private function _recognizeActiveRecord ($activeRecord)
    {
        if (!$activeRecord instanceof Singular_Taxonomy_ActiveRecord) {
            $activeRecord = $this->getActiveRecord();
        }

        if (is_array($activeRecord) || $activeRecord instanceof Zend_Config) {
            $activeRecord->populateSafety($activeRecord);
        }

        return $activeRecord;
    }

    public function setMergeClusterWithStructure ($flag)
    {
        $this->_mergeClusterWithStructure = (bool) $flag;
    }

    public function isMergeClusterWithStructure ()
    {
        return (bool) $this->_mergeClusterWithStructure;
    }

    private function _mergeClusterWithStructure ($activeRecord = null)
    {
        /** @var $activeRecord Singular_Taxonomy_ActiveRecord */
        $activeRecord = $this->_recognizeActiveRecord($activeRecord);
        /** @var $cluster Singular_Taxonomy_Cluster_Data */
        $cluster = $activeRecord->getCluster(true);
        $structure = $this->getClusterStructure(true);

        $clusterData = $cluster->toArray();

        // Merge cluster with structure
        $cluster->fromArray($structure, true);
        $cluster->fromArray($clusterData, false);

        $activeRecord->setCluster($cluster);

        return $activeRecord;
    }

    public function populateCluster ($data)
    {
        $this->getActiveRecord()->populateSafety($data);
    }

    /**
     * @param string $name
     * @return void
     */
    public function setModuleName ($name)
    {
        if (!is_string($name)) {
            return;
        }
        $this->_moduleName = strtolower($name);
    }

    /**
     * @return null|string
     */
    public function getModuleName ()
    {
        return $this->_moduleName;
    }
}
