<?php

class Singular_Taxonomy_ActiveRecord
{
    const TAX_TYPE_MODULE = 'module';

    const TAX_TYPE_RECORD = 'record';

    const TAX_TYPE_SECTION = 'section';

    const CLUSTER_COLUMN = 'cluster';

    const OPERATION_MODE_INSERT = 'insert';

    const OPERATION_MODE_UPDATE = 'update';

    /**
     * Valid types
     * 
     * @var array
     */
    private $_validTypes = array(
        self::TAX_TYPE_MODULE,
        self::TAX_TYPE_RECORD,
        self::TAX_TYPE_SECTION
    );

    private $_restrictedFields = array(
        'id', 'lft', 'rgt'
    );

    private $_requiredFields = array(
        'name',
        'type',
        'module',
        'admin_route',
        'frontend_route'
    );

    /**
     * @var array
     */
    protected $_record = null;

    /**
     * @var Singular_Taxonomy_Cluster_Data
     */
    protected $_cluster = null;

    public function __construct ($data = null, $safety = false, $allowEmptyCluster = false)
    {
        $this->reset();
        if (is_array($data) && !empty($data)) {
            $this->populate($data, (bool) $safety, (bool) $allowEmptyCluster);
        }
    }

    public function __set ($name, $value)
    {
        $this->set($name, $value);
    }

    public function __get ($name)
    {
        return $this->get($name);
    }

    public function set ($name, $value)
    {
        if (!$this->has($name)) {
            $this->_fieldNotExistsException($name);
        } elseif (in_array($name, $this->_restrictedFields)) {
            throw new Singular_Taxonomy_Exception("Field \"{$name}\" can't set value directly. \nUse special methods instead.");
        }
        $this->_record[$name] = $value;
    }

    public function get ($name, $throwException = true)
    {
        if (!$this->has($name)) {
            if (true === (bool) $throwException) {
                $this->_fieldNotExistsException($name);
            }
            return null;
        }

        return $this->_record[$name];
    }

    public function has ($name)
    {
        if (!array_key_exists($name, $this->_record)) {
            return false;
        }

        return true;
    }

    public function getRecord ($clearRestrictedFields = true, $clearNullFields = false)
    {
        $record = $this->_record;

        if (true === (bool) $clearRestrictedFields) {
            $record = $this->clearRestrictedFields($record);
        }

        if (true === (bool) $clearNullFields) {
            foreach ($record as $field => $value) {
                $value = trim($value);
                if ('' == $value) {
                    unset($record[$field]);
                }
            }
        }
        
        if ($record['updated_at'] == '0000-00-00 00:00:00') {
            $record['updated_at'] = date('Y-m-d H:i:s');
        }
        
        return $record;
    }

    protected function _mergeRecord ($data)
    {
        $this->_record = array_merge($this->_record, $data);
    }

    public function reset ()
    {
        $taxData = array(
            'id'                 => null,
            'lft'                => null,
            'rgt'                => null,
            'name'               => null,
            'type'               => null,
            'module'             => null,
            'alias'              => null,
            'resource'           => null,
            'admin_route'        => null,
            'frontend_route'     => null,
            self::CLUSTER_COLUMN => null,
            'custom_field'       => null,
            'created_at'         => null,
            'updated_at'         => null,
            'created_by'         => null,
            'updated_by'         => null,
            'is_visible'         => 1,
            'is_protected'       => 0
        );

        $this->_record = $taxData;
        $this->_cluster = null;
    }

    public function getId ()
    {
        return $this->get('id');
    }

    public function getLeft ()
    {
        return $this->get('lft');
    }

    public function getRight ()
    {
        return $this->get('rgt');
    }

    public function setName ($name)
    {
        if (!is_string($name) || empty($name)) {
            $this->_fieldIncorrectTypeException($name);
        }

        $this->set('name', $name);

        return $this;
    }

    public function getName ()
    {
        return $this->get('name');
    }

    public function setType ($type)
    {
        if (!in_array($type, $this->_validTypes)) {
            throw new Singular_Taxonomy_Exception("Invalid value \"{$type}\" for type filed. Valid values are: " . implode(', ', $this->_validTypes));
        }
        $this->set('type', $type);

        return $this;
    }

    public function getType ()
    {
        return $this->get('type');
    }

    public function setModule ($module)
    {
        if (!is_string($module) && !empty($module)) {
            throw new Singular_Taxonomy_Exception('Module must be an string. ' .gettype($module) . ' was given.');
        }
        $this->set('module', $module);

        return $this;
    }

    public function getModule ()
    {
        return $this->get('module');
    }

    public function setAlias ($alias)
    {
        if (!is_string($alias) || empty($alias)) {
            throw new Singular_Taxonomy_Exception('Alias must be an string. ' . gettype($module) . ' was given.');
        }

        $this->set('alias', $alias);

        return $this;
    }

    public function getAlias ()
    {
        return $this->get('alias');
    }

    public function setResource ($resource, $checkResourceIsRegistered = true)
    {
        if (!is_string($resource) && !empty($resource)) {
            throw new Singular_Taxonomy_Exception('Resource must be an string. ' . gettype($resource) . ' was given.');
        }

        if (true === (bool) $checkResourceIsRegistered) {
            /** @var $resourceCheck Singular_View_Helper_Acl */
            $acl = Zend_Layout::getMvcInstance()->getView()->acl();

            if (!$acl->hasResource($resource)) {
                throw new Singular_Taxonomy_Exception("Resource \"{$resource}\" was not found.");
            }
        }

        $this->set('resource', $resource);

        return $this;
    }

    public function getResource ()
    {
        return $this->get('resource');
    }

    public function setAdminRoute ($route)
    {
        if (!is_string($route) && !empty($route)) {
            throw new Singular_Taxonomy_Exception('Admin route must be an string. ' . gettype($route) . ' was given.');
        }
        $this->set('admin_route', $route);

        return $this;
    }

    public function getAdminRoute ()
    {
        return $this->get('admin_route');
    }

    public function setFrontendRoute ($route)
    {
        if (!is_string($route) && !empty($route)) {
            throw new Singular_Taxonomy_Exception('Frontend route must be an string. ' . gettype($route) . ' was given.');
        }
        $this->set('frontend_route', $route);

        return $this;
    }

    public function getFrontendRoute ()
    {
        return $this->get('frontend_route');
    }

    public function setCreatedAt ($value)
    {
        $this->set('created_at', $value);

        return $this;
    }

    public function setCreatedAtNow ()
    {
        $this->set('created_at', new Zend_Db_Expr('NOW()'));

        return $this;
    }

    public function getCreatedAt ()
    {
        return $this->get('created_at');
    }

    public function setUpdatedAt ($value)
    {
        $this->set('updated_at', $value);

        return $this;
    }

    public function setUpdatedAtNow ()
    {
        $this->set('updated_at', new Zend_Db_Expr('NOW()'));

        return $this;
    }

    public function setNeverUpdated ()
    {
        $this->set('updated_at', '0000-00-00 00:00:00');

        return $this;
    }

    public function getUpdatedAt ()
    {
        return $this->get('updated_at');
    }

    public function setCreatedBy ($userId)
    {
        $this->set('created_by', $userId);

        return $this;
    }

    public function setCreatedByCurrentUser ()
    {
        $this->set('created_by', Singular_Core::_('Auth')->getIdentity()->id);

        return $this;
    }

    public function getCreatedBy ()
    {
        return $this->get('created_by');
    }

    public function setUpdatedBy ($userId)
    {
        $this->set('updated_by', $userId);

        return $this;
    }

    public function setUpdatedByCurrentUser ()
    {
        $this->set('updated_by', Singular_Core::_('Auth')->getIdentity()->id);

        return $this;
    }

    public function getUpdatedBy ()
    {
        return $this->get('updated_by');
    }

    public function setIsVisible ($flag = true)
    {
        $this->set('is_visible', ($flag) ? 1 : 0);

        return $this;
    }

    public function getIsVisible ($asBoolean = true)
    {
        $isVisible = $this->get('is_visible');

        return (true === (bool) $asBoolean)
                ? (bool) $isVisible
                : (int) $isVisible;
    }

    public function setIsProtected ($flag = true)
    {
        $this->set('is_protected', ($flag) ? 1 : 0);

        return $this;
    }

    public function getIsProtected ($asBoolean = true)
    {
        $isVisible = $this->get('is_protected');

        return (true === (bool) $asBoolean)
                ? (bool) $isVisible
                : (int) $isVisible;
    }

    /**
     * @throws Singular_Taxonomy_Exception
     * @param array|Zend_Config|Singular_Taxonomy_Cluster_Data $cluster
     * @return 
     */
    public function setCluster ($cluster, $allowEmptyCluster = false)
    {
        if (true === (bool) $allowEmptyCluster && is_null($cluster)) {
            return $this;
        }

        if ($cluster instanceof Zend_Config || is_array($cluster)) {
            $cluster = new Singular_Taxonomy_Cluster_Data($cluster);
        } elseif ($cluster instanceof Singular_Taxonomy_Cluster_Data) {
            $this->_cluster = $cluster;
            return $this;
        } elseif (is_string($cluster) && strstr($cluster, '<?xml')) {
            $cluster = new Singular_Taxonomy_Cluster_Data($cluster, true);
        } else {
            throw new Singular_Taxonomy_Exception(
                "Invalid cluster type. \nCluster must be an instance of Zend_Config, Singular_Taxonomy_Cluster_Data or has array type."
            );
        }
        
        $this->_cluster = $cluster;

        return $this;
    }

    public function getCluster ($throwException = false)
    {
        if (true === (bool) $throwException) {
            if (!$this->_cluster instanceof Singular_Taxonomy_Cluster_Data) {
                throw new Singular_Taxonomy_Exception('Invalid cluster object. Cluster must be an instance of Singular_Taxonomy_Cluster_Data. ' . gettype($this->_cluster) . ' was given.');
            }
        }

        return $this->_cluster;
    }

    public function setCustomFieldValue ($value)
    {
        $this->set('custom_field', $value);

        return $this;
    }

    public function getCustomFieldValue ()
    {
        return $this->get('custom_field');
    }

    public function populateSafety ($data)
    {
        $this->populate($data, true);

        return $this;
    }

    public function populate ($data, $safety = false, $allowEmptyCluster = false)
    {
        if ($data instanceof Zend_Config) {
            $data = $data->toArray();
        } elseif (!is_array($data)) {
            throw new Singular_Taxonomy_Exception('Invalid data type. Data must be an array or an instance of Zend_Config. ' . gettype($data) . ' was given.');
        }

        if (true === $safety) {
            $data = $this->clearRestrictedFields($data);
        }

        $this->_populate($data, $allowEmptyCluster);

        return $this;
    }

    private function _populate ($data, $allowEmptyCluster = false)
    {
        foreach ($data as $field => $value) {
            // Delete incorrect fields
            if (!$this->has($field)) {
                unset($data[$field]);
            }

            switch ($field) {
                case 'name':
                    if (empty($value) || !is_string($value)) {
                        $this->_fieldIncorrectTypeException($field);
                    }
                    break;
                case 'cluster':
                    $this->setCluster($value, $allowEmptyCluster);

                    if (true === (bool) $allowEmptyCluster && empty($value)) {
                        continue;
                    }
                        
                    $data[$field] = $this->getCluster()->toXml();
                    break;
                case 'created_at':
                case 'updated_at':
                    $validator = new Zend_Validate_Date('YYYY-MM-dd HH:mm:ss');
                        
                    if (!$validator->isValid($value)
                        && null !== $value
                        && '0000-00-00 00:00:00' != $value) {
                        throw new Singular_Taxonomy_Exception("Invalid datetime value for field \"{$field}\".");
                    }
                    break;
                case 'type':
                    if (!in_array($value, $this->_validTypes)) {
                        throw new Singular_Taxonomy_Exception("Invalid value \"{$value}\" for type filed. Valid values are: " . implode(', ', $this->_validTypes));
                    }
                    break;
            }
        }
        
        // Merge input data with current record data
        $this->_mergeRecord($data);
    }

    public function prepareInsert ()
    {
        if (null === $this->get('created_at')) {
            $this->setCreatedAtNow();
        }

        $this->setNeverUpdated();
        
        if (null === $this->get('created_by')) {
            $this->setCreatedByCurrentUser();
        }

        if (null === $this->get('updated_by')) {
            $this->setUpdatedByCurrentUser();
        }

        // Update cluster xml value
        if (null !== $this->getCluster()) {
            $this->set('cluster', $this->getCluster(true)->toXml());
        }

        $this->_checkRequiredFields();

        return $this;
    }

    public function prepareUpdate ()
    {
        if (null === $this->get('created_at')) {
            $this->setCreatedAtNow();
        }

        $this->setUpdatedAtNow();
        $this->setUpdatedByCurrentUser();

        if (null === $this->get('created_by')) {
            $this->setCreatedByCurrentUser();
        }

        // Update cluster xml value
        if (null !== $this->getCluster()) {
            $this->set('cluster', $this->getCluster(true)->toXml());
        }

        $this->_checkRequiredFields();

        return $this;
    }

    public function clearRestrictedFields ($data)
    {
        foreach ($data as $field => $value) {
            if (in_array($field, $this->_restrictedFields)) {
                unset($data[$field]);
            }
        }

        return $data;
    }

    protected function _checkRequiredFields ()
    {
        foreach ($this->getRecord() as $field => $value) {
            if (in_array($field, $this->_requiredFields) && empty($value)) {
                throw new Singular_Taxonomy_Exception("Required field \"$field\" must be not empty.");
            }
        }
    }

    private function _fieldIncorrectTypeException ($name)
    {
        throw new Singular_Taxonomy_Exception("Field \"{$name}\" must be an string and not empty.");
    }

    private function _fieldNotExistsException ($name)
    {
        throw new Singular_Taxonomy_Exception("Incorrect field name. \nField {$name} is not exists.");
    }
}
