<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Update
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Zend_Loader_Autoloader_Resource
 * @category   Singular
 * @package    Singular_Update
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Update
{
    /**
     * Default update server URL
     */
    const DEFAULT_UPDATE_SERVER_URL = 'http://sglr-update/upd/v1';

    /**
     * Valid XML needle for checking response RAW data
     */
    const VALID_XML_NEEDLE = '<?xml';

    /**
     * User-defined ypdate server URL
     *
     * @var null
     */
    private static $_updateServerUrl = null;

    /**
     * Update drivers
     *
     * @var array
     */
    private static $_drivers = array();

    public static function run ($options = array())
    {
        if (!is_array($options) || empty($options)) {
            // default options
            $options = array(
                'drivers' => array(
                    'Core' => array(
                        'enable' => true
                    ),
                    'Modules' => array(
                        'enable' => true
                    ),
                    'Product' => array(
                        'enable' => true
                    )
                )
            );
        }

        foreach ($options as $option => $value) {
            switch ($option) {
                case 'drivers':
                    if (is_array($value)) {
                        foreach ($value as $driverName => $driverOptions) {
                            $driverClass = 'Singular_Update_Driver_' . ucfirst($driverName);
                            Zend_Loader::loadClass($driverClass);

                            if (isset($driverOptions['enable']) && (bool) $driverOptions['enable'] === false) {
                                continue;
                            }

                            $driver = new $driverClass($driverOptions);
                            self::addDriver($driverName, $driver);
                        }
                    }
                break;
            }
        }

        $response = self::_sendRequest();

        return Singular_Stdlib_XmlToArray::toArray($response->getBody());
    }

    public static function setUpdateServerUrl ($url)
    {
        if (!self::isValidUpdateServerUrl($url)) {
            /*throw new Singular_Update_Exception(
                "There is no update server found at: $url"
            );*/
        }

        self::$_updateServerUrl = $url;
    }

    public static function getUpdateServerUrl ()
    {
        return (null !== self::$_updateServerUrl)
                ? self::$_updateServerUrl : self::DEFAULT_UPDATE_SERVER_URL;
    }

    public static function isValidUpdateServerUrl ($url)
    {
        /** client */
        $client = self::_getClient($url);

        /** error flag */
        $error = false;

        /** making request to update server */
        try {
            $client->setParameterPost('test_mode', 1);
            $response = $client->request('POST');
            if ($response->isSuccessful()) {
                if (!self::_isValidResponse($response)) {
                    /** error */
                    $error = true;
                }
            } else {
                /** error */
                $error = true;
            }
        } catch (Exception $e) {
            /** error */
            $error = true;
        }

        return (!$error) ? true : false;
    }

    /**
     * Gets http client
     *
     * @static
     * @param  string|null $url
     * @return Zend_Http_Client
     */
    protected static function _getClient ($url = null)
    {
        if (null === $url) {
            $url = self::getUpdateServerUrl();
        }

        if (!substr($url, 0, 7) == 'http://') {
            $url = 'http://' . $url;
        }

        /** initialize http client */
        $client = new Zend_Http_Client($url, array(
            'keepalive' => true
        ));

        return $client;
    }

    /**
     * Validates update server response
     *
     * @static
     * @param  Zend_Http_Response $response
     * @return bool
     */
    protected static function _isValidResponse (Zend_Http_Response $response)
    {
        $body = $response->getBody();

        if (0 !== stripos($body, self::VALID_XML_NEEDLE)) {
            return false;
        }

        try {
            $data = Singular_Stdlib_XmlToArray::toArray($body);
            if (!isset($data['response']['info']['server_version'])) {
                return false;
            }
        } catch (Exception $e) {
            return false;
        }

        return true;
    }

    /**
     * Builds and sends request
     *
     * @static
     * @return Zend_Http_Response
     */
    protected static function _sendRequest ()
    {
        $request = array();

        /** @var $driver Singular_Update_Driver */
        foreach (self::$_drivers as $driver) {
            if ($driver instanceof Singular_Update_Driver) {
                $request = array_merge_recursive($request, $driver->getRequestData());
            }
        }

        $client = self::_getClient();
        $client->setRawData(http_build_query($request));

        return $client->request('POST');
    }

    public static function addDriver ($name, $instance)
    {
        if (!is_string($name)) {
            throw new Singular_Update_Exception(
                'Invalid argument: $name must be an string'
            );
        }

        if (!$instance instanceof Singular_Update_Driver) {
            throw new Singular_Update_Exception(
                'Invalid argument: $instance must be an instance of Singular_Update_Driver'
            );
        }

        self::$_drivers[ucfirst($name)] = $instance;
    }

    public static function getDriver ($name)
    {
        $name = ucfirst($name);

        if (!isset(self::$_drivers[$name])) {
            throw new Singular_Update_Exception("Driver $name is not exists");
        }

        return self::$_drivers[$name];
    }
}
