<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_View
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Concrete class for handling view scripts.
 *
 * @category   Singular
 * @package    Singular_View
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_View extends Zend_View_Abstract
{
    /**
     * Whether or not to use streams to mimic short tags
     * @var bool
     */
    private $_useViewStream = false;

    /**
     * Whether or not to use stream wrapper if short_open_tag is false
     * @var bool
     */
    private $_useStreamWrapper = false;

    /**
     * @var Zend_Cache_Core
     */
    private $_cache = null;

    /**
     * Script file name to execute
     *
     * @var string
     */
    private $_file = null;

    /**
     * Debug flag
     *
     * @var bool
     */
    private static $_debug = false;

    /**
     * Constructor
     *
     * Register Zend_View_Stream stream wrapper if short tags are disabled.
     *
     * @param  array $config
     * @return void
     */
    public function __construct($config = array())
    {
        $this->_useViewStream = (bool) ini_get('short_open_tag') ? false : true;
        if ($this->_useViewStream) {
            if (!in_array('zend.view', stream_get_wrappers())) {
                require_once 'Zend/View/Stream.php';
                stream_wrapper_register('zend.view', 'Zend_View_Stream');
            }
        }

        if (array_key_exists('useStreamWrapper', $config)) {
            $this->setUseStreamWrapper($config['useStreamWrapper']);
        }

        parent::__construct($config);
    }

    /**
     * Directly assigns a variable to the view script.
     *
     * Checks first to ensure that the caller is not attempting to set a
     * protected or private member (by checking for a prefixed underscore); if
     * not, the public member is set; otherwise, an exception is raised.
     *
     * @param string $key The variable name.
     * @param mixed  $val The variable value.
     *
     * @return void
     * @throws Zend_View_Exception if an attempt to set a private or protected
     * member is detected
     */
    public function __set($key, $val)
    {
        if ('_' != substr($key, 0, 1)) {
            /* Event "viewVarSet" */
            $callbackData = new stdClass();
            $callbackData->view  = $this;
            $callbackData->name  = $key;
            $callbackData->value = $val;
            Singular_Event::dispatch('viewVarSet', $callbackData);
            $key = $callbackData->name;
            $val = $callbackData->value;

            $this->$key = $val;
            return;
        }

        //require_once \'Zend/View/Exception.php';
        $e = new Zend_View_Exception('Setting private or protected class members is not allowed');
        $e->setView($this);
        throw $e;
    }

    /**
     * Accesses a helper object from within a script.
     *
     * If the helper class has a 'view' property, sets it with the current view
     * object.
     *
     * @param string $name The helper name.
     * @param array $args The parameters for the helper.
     * @return string The result of the helper output.
     */
    public function __call($name, $args)
    {
        // is the helper already loaded?
        $helper = $this->getHelper($name);
        return $this->_executeCallback($helper, $name, $args);
    }

    /**
     * Executes callback
     *
     * call_user_func_array function is relatively slow (as of PHP 5.3.3)
     * and if you are calling a method with a known number of parameters
     * it is much faster to call it this way: $class->{$method}($param1, $param2);
     *
     * @param  object $c
     * @param  string $m
     * @param  array $p
     * @return mixed
     */
    protected function _executeCallback ($c, $m, $p)
    {
        switch(count($p)) {
            case 0: return $c->{$m}(); break;
            case 1: return  $c->{$m}($p[0]); break;
            case 2: return $c->{$m}($p[0], $p[1]); break;
            case 3: return $c->{$m}($p[0], $p[1], $p[2]); break;
            case 4: return $c->{$m}($p[0], $p[1], $p[2], $p[3]); break;
            case 5: return $c->{$m}($p[0], $p[1], $p[2], $p[3], $p[4]); break;
            default: return call_user_func_array(array($c, $m), $p);  break;
        }
    }

    /**
     * Set flag indicating if stream wrapper should be used if short_open_tag is off
     *
     * @param  bool $flag
     * @return Zend_View
     */
    public function setUseStreamWrapper($flag)
    {
        $this->_useStreamWrapper = (bool) $flag;
        return $this;
    }

    /**
     * Should the stream wrapper be used if short_open_tag is off?
     *
     * @return bool
     */
    public function useStreamWrapper()
    {
        return $this->_useStreamWrapper;
    }

    public static function setDebug ($flag = true)
    {
        self::$_debug = (bool) $flag;
    }

    public static function isDebugOn ()
    {
        return self::$_debug;
    }

    /**
     * Returns view cache
     *
     * @return Zend_Cache_Core
     */
    protected function _getCache ()
    {
        if (null === $this->_cache) {
            $cacheDir = APPLICATION_PATH . '/temp/cache/view';
            if (!is_readable($cacheDir)) mkdir($cacheDir, 0777, true);
            $this->_cache = Zend_Cache::factory('Core',
                'File',
                array(
                    'lifetime' => 7200, // cache lifetime of 2 hours
                    'automatic_serialization' => true
                ),
                array(
                    'cache_dir' => $cacheDir
                ));
        }

        return $this->_cache;
    }

    /**
     * Processes a view script and returns the output.
     *
     * @param string $name The script name to process.
     * @return string The script output.
     */
    public function render ($name)
    {
        // find the script file name using the parent private method
        $this->_file = $this->_script($name);

        if (self::isDebugOn()) {
            echo '<div style="border:1px solid red;margin:5px;"><strong style="font-size:10px;color:white;background:red;">' . $this->_file . '</strong><br />';
        }

        /* Event "viewRender" */
        $callbackData = new stdClass();
        $callbackData->view         = $this;
        $callbackData->templateName = $name;
        $callbackData->templatePath = $this->_file;
        Singular_Event::dispatch('viewRender', $callbackData);

        ob_start();
        $this->_run($this->_file);
        $content = (!self::isDebugOn()) ? ob_get_clean() : (ob_get_clean() . '</div>');

        /* Event "viewRenderAfter" */
        $callbackData->content = $content;
        Singular_Event::dispatch('viewRenderAfter', $callbackData);

        $content = $callbackData->content;

        return $this->_filter($content); // filter output
    }

    /**
     * Includes the view script in a scope with only public $this variables.
     *
     * @param string The view script to execute.
     */
    protected function _run()
    {
        if ($this->_useViewStream && $this->useStreamWrapper()) {
            include 'zend.view://' . func_get_arg(0);
        } else {
            include func_get_arg(0);
        }
    }

    /**
     * Applies the filter callback to a buffer.
     *
     * @param string $buffer The buffer contents.
     * @return string The filtered buffer.
     */
    private function _filter ($buffer)
    {
        // loop through each filter class
        foreach ($this->getFilters() as $name) {
            // load and apply the filter class
            $filter = $this->getFilter($name);
            $buffer = call_user_func(array($filter, 'filter'), $buffer);
        }

        // done!
        return $buffer;
    }
}
