<?php
/**
 * Singular
 *
 * LICENSE
 *
 * The SOFTWARE PRODUCT is protected by copyright treaties,
 * as well as other intellectual property laws and treaties.
 * The SOFTWARE PRODUCT is licensed, not sold.
 * Software.
 * You may install and use one copy of the SOFTWARE PRODUCT
 * for each license purchased or if a site license then you
 * may install any number of copies at the site where the
 * site license is defined.
 * Your rights to use the SOFTWARE PRODUCT are not transferable.
 *
 * @category   Singular
 * @package    Singular_View
 * @subpackage Helper
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */

/**
 * @category Singular
 * @package Singular_View
 * @subpackage Singular_View_Helper
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */
class Singular_View_Helper_Block extends Zend_View_Helper_Abstract
{
    /**
     * Config filename
     */
    const CONFIG_FILENAME = 'blocks.ini';

    /**
     * Block type names
     */
    const TYPE_TEMPLATE = 'template';
    const TYPE_ACTION = 'action';
    const TYPE_COMPONENT = 'component';

    /**
     * Cache tag
     */
    const CACHE_TAG = 'blocks';

    /**
     * Blocks config
     * 
     * @var array
     */
    protected static $_config = null;

    /**
     * Paths for searching config files
     * 
     * @var array
     */
    protected static $_configPaths = array();

    /**
     * Cache object
     *
     * @var Zend_Cache
     */
    protected static $_cache = null;

    /**
     * Get rendered block by given block name
     *
     * @see blocks.ini
     * @throws Singular_Exception
     * @param string|null $block
     * @return Singular_View_Helper_Block|string
     */
    public function block ($block = null)
    {
        // Initialize config if it wasn't yet
        if (null === self::$_config) {
            $this->_initConfig();
        }

        // Return self if $block has null value
        if ($block === null) {
            return $this;
        }

        // Check for block config presence 
        if ($this->isExists($block)) {
            $blockConfig = self::$_config[$block];
            if (isset($blockConfig['type'])) {
                return $this->render($block, $blockConfig);
            }
            throw new Singular_Exception("Option \"type\" for block \"{$block}\" was missed.");
        }
        throw new Singular_Exception("Block \"{$block}\" was not found on paths: " . implode(PATH_SEPARATOR, self::$_configPaths));
    }

    /**
     * Render block according given params
     *
     * @throws Singular_Exception
     * @param  string $block
     * @param  array $config
     * @return string
     */
    public function render ($block, $config)
    {
        $params = (isset($config['params']) && is_array($config['params']))
                ? $config['params'] : array();

        // check for cache necessity
        $useCache = false;
        if (isset($config['cache']) && true === (bool) $config['cache']) {
            $cacheKey = "block_{$block}";
            if ($output = $this->_getCache()->load($cacheKey)) {
                return $output;
            }
            $useCache = true;
        }
        
        // render block according specified type
        switch ($config['type']) {
            case self::TYPE_TEMPLATE:
                if (!isset($config['template'])) {
                    throw new Singular_Exception("Option \"template\" for block \"{$block}\" was missed.");
                }
                $this->view->assign($params);
                $output = $this->view->render($config['template']);
                break;
            case self::TYPE_ACTION:
                if (!isset($config['module']) || !isset($config['controller']) || !isset($config['action'])) {
                    throw new Singular_Exception("Options \"module, controller, action\" for block \"{$block}\" was missed.");
                }
                $output = $this->view->action(
                    $config['action'],
                    $config['controller'],
                    $config['module'],
                    $params
                );
                break;
            case self::TYPE_COMPONENT:
                if (!isset($config['component']) || !isset($config['module'])) {
                    throw new Singular_Exception("Options \"component, module\" for block \"{$block}\" was missed.");
                }
                $output = Singular_Component::run($config['component'], $config['module'], $params);
                break;
        }

        // save to cache if is necessary
        if (true === $useCache) {
            $this->_getCache()->save($output, $cacheKey, array(self::CACHE_TAG));
        }
        
        return $output;
    }

    /**
     * Check for presence block entry in config
     *
     * @param string $block
     * @return bool
     */
    public function isExists ($block)
    {
        return isset(self::$_config[$block]);
    }

    /**
     * Clean block cache
     * Remove cache by cache id if block name is specified,
     * or clean cache for all blocks when $block = null
     *
     * @param string|null $block
     * @return void
     */
    public function cleanCache ($block = null)
    {
        if (is_string($block)) {
            $this->_getCache()->remove("block_{$block}");
            return;
        }
        $this->_getCache()->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG,
                                  array(self::CACHE_TAG));
    }

    /**
     * Initialize config from blocks.ini files
     *  
     * @return void
     */
    protected function _initConfig ()
    {
        // merging config
        $blocksConfig = array();
        foreach (self::$_configPaths as $path) {
            $path = $path . DS . self::CONFIG_FILENAME;
            
            if (file_exists($path)) {
                $config = new Zend_Config_Ini($path);
                $blocksConfig = array_merge_recursive($blocksConfig, $config->toArray());
            }
        }
        
        self::$_config = $blocksConfig;
    }

    public static function addPaths ($paths)
    {
        $paths = (array) $paths;

        foreach ($paths as $path) {
            self::addPath($path);
        }
    }

    public static function addPath ($path)
    {
        if (!in_array($path, self::$_configPaths)) {
            self::$_configPaths[] = $path;
        }
    }

    /**
     * Get cache adapter
     * 
     * @return Zend_Cache
     */
    protected function _getCache ()
    {
        if (null === self::$_cache) {
            self::$_cache = Singular_Runtime::extract('cachemanager')->getCache('default');
        }

        return self::$_cache;
    }
}
