<?php
/**
 * Singular
 *
 * LICENSE
 *
 * The SOFTWARE PRODUCT is protected by copyright treaties,
 * as well as other intellectual property laws and treaties.
 * The SOFTWARE PRODUCT is licensed, not sold.
 * Software.
 * You may install and use one copy of the SOFTWARE PRODUCT
 * for each license purchased or if a site license then you
 * may install any number of copies at the site where the
 * site license is defined.
 * Your rights to use the SOFTWARE PRODUCT are not transferable.
 *
 * @category   Singular
 * @package    Singular_View
 * @subpackage Helper
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */

/**
 * @category Singular
 * @package Singular_View
 * @subpackage Singular_View_Helper
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */
class Singular_View_Helper_Section extends Zend_View_Helper_Abstract
{
    /**
     * Config filename
     */
    const CONFIG_FILENAME = 'sections.ini';

    /**
     * Cache tag
     */
    const CACHE_TAG = 'sections';

    /**
     * Section config
     *
     * @var array
     */
    protected static $_config = null;

    /**
     * Paths for searching config files
     *
     * @var array
     */
    protected static $_configPaths = array();

    /**
     * Cache object
     *
     * @var Zend_Cache
     */
    protected static $_cache = null;

    /**
     * Get rendered section by given section name
     *
     * @see sections.ini
     * @throws Singular_Exception
     * @param string|null $section
     * @return Singular_View_Helper_Section|string
     */
    public function section ($section = null)
    {
        // Initialize config if it wasn't yet
        if (null === self::$_config) {
            $this->_initConfig();
        }

        // Return self if $section has null value
        if ($section === null) {
            return $this;
        }

        // Check for section config presence
        if ($this->isExists($section)) {
            $sectionConfig = self::$_config[$section];
            if (isset($sectionConfig['blocks']) && is_array($sectionConfig['blocks'])) {
                return $this->render($section, $sectionConfig);
            }
            return null;
        }
        throw new Singular_Exception("Section \"{$section}\" was not found on paths: " . implode(PATH_SEPARATOR, self::$_configPaths));
    }

    /**
     * Render section according given params
     *
     * @throws Singular_Exception
     * @param  string $section
     * @param  array $config
     * @return string
     */
    public function render($section, $config)
    {
        // Check for cache necessity
        $useCache = false;
        if (isset($config['cache']) && true === (bool)$config['cache']) {
            $cacheKey = "section_{$section}";
            if ($output = $this->_getCache()->load($cacheKey)) {
                return $output;
            }
            $useCache = true;
        }

        // Adding blocks to output
        $output = null;
        ksort($config['blocks']);
        foreach ($config['blocks'] as $block) {
            $output .= $this->view->block($block) . PHP_EOL;
        }

        // Save to cache if is necessary
        if (true === $useCache) {
            $this->_getCache()->save($output, $cacheKey, array(self::CACHE_TAG));
        }

        return $output;
    }

    /**
     * Check for presence section entry in config
     *
     * @param string $section
     * @return bool
     */
    public function isExists($section)
    {
        return isset(self::$_config[$section]);
    }

    /**
     * Clean section cache
     * Remove cache by cache id if section name is specified,
     * or clean cache for all sections when $section = null
     *
     * @param string|null $section
     * @return void
     */
    public function cleanCache($section = null)
    {
        if (is_string($section)) {
            $this->_getCache()->remove("section_{$section}");
            return;
        }
        $this->_getCache()->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG,
                                  array(self::CACHE_TAG));
    }

    /**
     * Initialize config from sections.ini files
     *
     * @return void
     */
    protected function _initConfig()
    {
        // Merging config
        $sectionsConfig = array();
        foreach (self::$_configPaths as $path) {
            $path = $path . DS . self::CONFIG_FILENAME;
            
            if (file_exists($path)) {
                $config = new Zend_Config_Ini($path);
                $sectionsConfig = array_merge_recursive($sectionsConfig, $config->toArray());
            }
        }

        self::$_config = $sectionsConfig;
    }

    public static function addPaths ($paths)
    {
        $paths = (array) $paths;

        foreach ($paths as $path) {
            self::addPath($path);
        }
    }

    public static function addPath ($path)
    {
        if (!in_array($path, self::$_configPaths)) {
            self::$_configPaths[] = $path;
        }
    }

    /**
     * Get cache adapter
     *
     * @return Zend_Cache
     */
    protected function _getCache()
    {
        if (null === self::$_cache) {
            self::$_cache = Singular_Runtime::extract('cachemanager')->getCache('default');
        }

        return self::$_cache;
    }
}
