<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Widget
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Sidebar widget abstract class
 *
 * @abstract
 * @category   Singular
 * @package    Singular_Widget
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
abstract class Singular_Widget_Abstract
{
    /**
     * View object
     *
     * @var Zend_View_Interface
     */
    public $view = null;

    /**
     * If widget preRender() method has already executed
     * 
     * @var bool
     */
    private $_preRender = false;

    /**
     * Widget offset position in sidebar container
     * 
     * @var int
     */
    protected $_offset = 0;

    /**
     * Partial view script to use for rendering widget
     * 
     * @var null|array
     */
    protected $_partial = null;

    /**
     * Widget variables
     *
     * Can be used for storing init() result during rendering
     *
     * @var array
     */
    private $_vars = array();

    /**
     * Widget options
     * 
     * @var array
     */
    private $_options = array();

    final public function __construct ($options = array())
    {
        $this->setOptions($options);
        $this->_initView();
        $this->init();
    }

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     * 
     * @abstract
     * @return void
     */
    public abstract function init ();

    /**
     * Runs before the widget has been rendered
     * 
     * @return void
     */
    public function preRender ()
    {
    }

    /**
     * Executes once preRender() method
     *
     * @see    preRender()
     * @return void
     */
    protected function _execPreRender ()
    {
        if (!$this->_preRender) {
            $this->_preRender = true;
            $this->preRender();
        }
    }

    /**
     * Initializes Zend_View
     * 
     * @return void
     */
    private function _initView ()
    {
        if (null === $this->view) {
            $this->view = Zend_Layout::getMvcInstance()->getView();
        }
    }

    /**
     * Set the View object
     *
     * @param  Zend_View_Interface $view
     * @return Zend_View_Helper_Abstract
     */
    public function setView (Zend_View_Interface $view)
    {
        $this->view = $view;
        return $this;
    }

    /**
     * Set options
     * 
     * @throws Singular_Widget_Exception
     * @param  array $options
     * @param  bool $overwrite
     * @return Singular_Widget_Abstract
     */
    public function setOptions ($options, $overwrite = true)
    {
        if (!is_array($options)) {
            throw new Singular_Widget_Exception(
                'Invalid argument: $options must be an array'
            );
        }

        if ($overwrite) {
            $this->_options = $options;
        } else {
            foreach ($options as $name => $value) {
                $this->setOption($name, $value);
            }
        }

        return $this;
    }

    /**
     * Add options
     * 
     * @param  array $options
     * @return void
     */
    public function addOptions ($options)
    {
        $this->setOptions($options, false);
    }

    /**
     * Set option
     * 
     * @throws Singular_Widget_Exception
     * @param  string $name
     * @param  mixed $value
     * @return Singular_Widget_Abstract
     */
    public function setOption ($name, $value)
    {
        if (!is_string($name) || is_numeric($name)) {
            throw new Singular_Widget_Exception(
                'Invalid argument: $name must be an string'
            );
        }
        
        $this->_options[$name] = $value;

        return $this;
    }

    /**
     * Gets option
     * 
     * @param  string $name
     * @return null
     */
    public function getOption ($name)
    {
        return ($this->hasOption($name)) ? $this->_options[$name] : null;
    }

    /**
     * Gets all options
     * 
     * @return array
     */
    public function getAllOptions ()
    {
        return $this->_options;
    }

    /**
     * If option is exists
     * 
     * @param  string $name
     * @return bool
     */
    public function hasOption ($name)
    {
        return array_key_exists($name, $this->_options);
    }

    /**
     * Returns a property, or null if it doesn't exist
     *
     * Magic overload for enabling <code>$widget->propname</code>.
     *
     * @param  string $name
     * @return null
     */
    public function __get ($name)
    {
        return (isset($this->_vars[$name])) ? $this->_vars[$name] : null;
    }

    /**
     * Sets a custom property
     *
     * Magic overload for enabling <code>$widget->propname = $value</code>.
     * 
     * @throws Singular_Widget_Exception
     * @param  string $name
     * @param  string $value
     * @return void
     */
    public function __set ($name, $value)
    {
        if (!is_string($name) || empty($name)) {
            throw new Singular_Widget_Exception(
                'Invalid argument: $name must be a non-empty string');
        }
        
        $this->_vars[$name] = $value;
    }

    /**
     * Checks if a property is set
     *
     * Magic overload for enabling <code>isset($widget->propname)</code>.
     *
     * Returns true if the property is native (id, class, title, etc), and
     * true or false if it's a custom property (depending on whether the
     * property actually is set).
     * 
     * @param  string $name
     * @return bool
     */
    public function __isset ($name)
    {
        return isset($this->_vars[$name]);
    }

    /**
     * Sets a custom variable
     *
     * @throws Singular_Widget_Exception
     * @param  string $name
     * @param  string $value
     * @return Singular_Widget_Abstract
     */
    public function setVar ($name, $value)
    {
        if (!is_string($name)) {
            throw new Singular_Widget_Exception(
                'Var $name must be an string. ' .
                gettype($name) . ' was given'
            );
        }

        $this->$name = $value;

        return $this;
    }

    /**
     * Gets all variables
     * 
     * @return array
     */
    public function getAllVars ()
    {
        return $this->_vars;
    }

    /**
     * Gets variable by name
     *
     * @param  string $name
     * @return null
     */
    public function getVar ($name)
    {
        return ($this->hasVar($name)) ? $this->$name : null;
    }

    /**
     * Checks if a variable is set
     *
     * @param  string $name
     * @return bool
     */
    public function hasVar ($name)
    {
        return isset($this->$name);
    }

    /**
     * Gets widget id
     * 
     * @return string
     */
    public function getId ()
    {
        if (!isset($this->id)) {
            $this->id = spl_object_hash($this);
        }

        return $this->id;
    }

    /**
     * Sets widget id
     * 
     * @throws Singular_Widget_Exception
     * @param  string $value
     * @return void
     */
    public function setId ($value)
    {
        if (!is_string($value)) {
            throw new Singular_Widget_Exception(
                'Id $value must be an string. ' .
                gettype($value) . ' was given'
            );
        }
        $this->id = $value;
    }

    /**
     * Sets widget offset position
     * 
     * @param  int $position
     * @return void
     */
    public function offsetSet ($position)
    {
        if (is_int($position)) {
            $this->_offset = $position;
        }
    }

    /**
     * Gets widget offset position
     * 
     * @return int
     */
    public function offsetGet ()
    {
        return $this->_offset;
    }

    /**
     * Sets which partial view script to use for rendering widget
     *
     * @param  string|array $partial             partial view script or null. If
     *                                           an array is given, it is
     *                                           expected to contain two values;
     *                                           the partial view script to use,
     *                                           and the module where the script
     *                                           can be found.
     * @return Singular_Widget_Abstract          fluent interface, returns self
     */
    public function setPartial ($partial)
    {
        if (null === $partial || is_string($partial) || is_array($partial)) {
            $this->_partial = $partial;
        }

        return $this;
    }

    /**
     * Returns partial view script to use for rendering widget
     *
     * @return string|array|null
     */
    public function getPartial ()
    {
        return $this->_partial;
    }

    /**
     * Renders the widget by invoking the partial view helper
     *
     * The container will simply be passed on as a model to the view script
     * as-is, and will be available in the partial script as 'widget', e.g.
     * <code>echo 'Number of pages: ', count($this->container);</code>.
     *
     * @param  string|array             $partial     [optional] partial view
     *                                               script to use. Default is to
     *                                               use the partial registered
     *                                               in the helper. If an array
     *                                               is given, it is expected to
     *                                               contain two values; the
     *                                               partial view script to use,
     *                                               and the module where the
     *                                               script can be found.
     * @return string                                helper output
     */
    public function renderPartial ($partial = null)
    {
        $this->_execPreRender();
        
        if (null === $partial) {
            $partial = $this->getPartial();
        }

        if (empty($partial)) {
            $e = new Zend_View_Exception(
                'Unable to render widget: No partial view script provided'
            );
            $e->setView($this->view);
            throw $e;
        }

        $model = array(
            'widget' => (object) $this->getAllVars()
        );

        if (is_array($partial)) {
            if (count($partial) != 2) {
                $e = new Zend_View_Exception(
                    'Unable to render menu: A view partial supplied as '
                    . 'an array must contain two values: partial view '
                    . 'script and module where script can be found'
                );
                $e->setView($this->view);
                throw $e;
            }

            return $this->view->partial($partial[0], $partial[1], $model);
        }

        return $this->view->partial($partial, null, $model);
    }

    /**
     * Render widget abstract implementation
     * 
     * @abstract
     * @return void
     */
    public abstract function renderWidget ();

    /**
     * Renders widget
     *
     * If a partial view is registered in the helper, the widget will be rendered
     * using the given partial script. If no partial is registered, the widget
     * will be rendered by renderWidget() internal method.
     *
     * @see renderPartial()
     * @see renderWidget()

     * @return string
     */
    public function render ()
    {
        $this->_execPreRender();
        
        if ($partial = $this->getPartial()) {
            return $this->renderPartial($partial);
        } else {
            return $this->renderWidget();
        }
    }
}
