<?php

final class SuperCache
{
    const CACHE_DISABLED = 'wc_off';

    protected static $_instance;

    protected $_cacheDir;

    protected $_runnable = true;

    protected static $_initiated = false;

    protected $_tags = array();

    protected $_baseUrl = null;

    protected $_onAir = true;

    /**
     * Cache object
     *
     * @var Zend_Cache_Frontend_Page
     */
    private $_cache;

    private function __construct ()
    {
        $this->_cacheDir = APPLICATION_PATH . '/temp/cache/_superCache';
        if (!$this->_cacheAllowed()) $this->_runnable = false;
    }

    /**
     * Protect from cloning
     */
    private function __clone () {}

    /**
     * Protect from unserialize
     */
    private function __wakeup () {}

    /**
     * Returns singleton
     *
     * @return SuperCache
     */
    public static function getInstance ()
    {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    public function run ()
    {
        $this->_initCache();
        $this->_intelligentClean();
        if (!$this->isRunnable()) return;
        $this->_startCache();
        $this->_runnable = false;
    }

    public function stop ()
    {
        if ($cache = $this->getCache() instanceof Zend_Cache_Frontend_Page)
            $this->getCache()->cancel();
    }

    public function cleanMatchingAnyTag ($tags = array())
    {
        if (!self::$_initiated) $this->_initCache();
        $this->getCache()->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG, $tags);
    }

    public function cleanMatchingOnlyTag ($tags = array())
    {
        if (!self::$_initiated) $this->_initCache();
        $this->getCache()->clean(Zend_Cache::CLEANING_MODE_MATCHING_TAG, $tags);
    }

    public function setBaseUrl ($url)
    {
        $segments = parse_url($url);
        if (isset($segments['path'])) {
            $url = rtrim($segments['path'], '/');
            if ('/' == $url) $url = '';
        } else {
            $url = '';
        }

        $this->_baseUrl = $url;
        return $this;
    }

    public function getBaseUrl ()
    {
        return $this->_baseUrl;
    }

    public function isOnAir ()
    {
        return $this->_onAir;
    }

    /**
     * Returns cache object
     *
     * @return Zend_Cache_Frontend_Page
     */
    public function getCache ()
    {
        return $this->_cache;
    }

    public function getCurrentTags ()
    {
        return array('homepage');//$this->_makeTags();
    }

    public function isRunnable ()
    {
        return $this->_runnable;
    }

    private function _initCache ()
    {
        if (!file_exists($this->_cacheDir)) {
            mkdir($this->_cacheDir, 0777, true);
        }

        require_once 'Zend/Cache.php';
        require_once 'Zend/Cache/Core.php';
        require_once 'Zend/Cache/Frontend/Page.php';
        require_once 'Zend/Cache/Backend.php';
        require_once 'Zend/Cache/Backend/ExtendedInterface.php';
        require_once 'Zend/Cache/Backend/File.php';

        $frontendOptions = array(
            'lifetime'        => 86400,
            'debug_header'    => false,
            'default_options' => array(
                'cache'                          => false,
                'cache_with_get_variables'       => true,
                'cache_with_post_variables'      => false,
                'cache_with_session_variables'   => true,
                'cache_with_files_variables'     => true,
                'cache_with_cookie_variables'    => true,
                'make_id_with_cookie_variables'  => false,
                'make_id_with_session_variables' => false,
                'tags' => array('homepage')
                //'tags' => $this->_makeTags()
            ),
            'regexps' => array(
                // cache the homepage
                '^' . $this->getBaseUrl() . '/$' => array(
                    'cache' => true,
                ),
            )
        );

        $user = $this->_getUserIdentity();
        if (null !== $user) {
            $frontendOptions['default_options']['make_id_with_cookie_variables']  = true;
            $frontendOptions['default_options']['make_id_with_session_variables'] = true;
        }

        $backendOptions = array(
            'cache_dir' => $this->_cacheDir
        );

        $this->_cache = Zend_Cache::factory('Page', 'File', $frontendOptions, $backendOptions);
        self::$_initiated = true;
    }

    private function _startCache ()
    {
        $this->_onAir = $this->getCache()->start();
    }

    private function _cacheAllowed ()
    {
        if (
            isset($_GET[self::CACHE_DISABLED]) || isset($_POST[self::CACHE_DISABLED]) ||
            isset($_COOKIE[self::CACHE_DISABLED]) || isset($_SESSION[self::CACHE_DISABLED]) ||
            isset($_ENV[self::CACHE_DISABLED]) || isset($_GLOBALS[self::CACHE_DISABLED])
        ) {
            return false;
        }

        return true;
    }

    private function _deploySession ()
    {
        if (!isset($_COOKIE['PHPSESSID'])) {
            return null;
        }

        $sessPath = APPLICATION_PATH . '/temp/sessions/sess_' . $_COOKIE['PHPSESSID'];

        if (!$test = @fopen($sessPath, 'r')) {
            return null;
        }
        fclose($test);

        return $this->_decodeSessionData(file_get_contents($sessPath));
    }

    private function _decodeSessionData ($data)
    {
        $vars = preg_split('/([a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff^|]*)\|/',
            $data, -1, PREG_SPLIT_NO_EMPTY | PREG_SPLIT_DELIM_CAPTURE);
        for ($i = 0; count($vars) > $i; $i++) $result[$vars[$i++]] = unserialize($vars[$i]);

        return (isset($result)) ? $result : array();
    }

    protected function _getUserIdentity ()
    {
        $session = $this->_deploySession();
        if (!isset($session['Zend_Auth'])) return null;
        if (!isset($session['Zend_Auth']['storage'])) return null;
        if (!$session['Zend_Auth']['storage'] instanceof stdClass) return null;
        if(!is_numeric($session['Zend_Auth']['storage']->id)) return null;
        return $session['Zend_Auth']['storage'];
    }

    protected function _intelligentClean ()
    {
        if ('post' == strtolower($_SERVER['REQUEST_METHOD'])) {
            $this->cleanMatchingAnyTag($this->getCurrentTags());
        }
    }

    protected function _makeTags ()
    {
        if (empty($this->_tags)) {
            $parsed = parse_url($_SERVER['REQUEST_URI']);

            if ('/' == $parsed['path']) {
                return array('homepage');
            }

            // process path stack
            $tags = explode('/', $parsed['path']);
            unset($tags[0]);

            // process query stack
            if (isset($parsed['query'])) {
                $query = explode('&', $parsed['query']);
                foreach ($query as $q) {
                    $qPair = explode('=', $q);
                    $this->_stripTag($qPair[0], $tags);
                    if (isset($qPair[1])) $this->_stripTag($qPair[1], $tags);
                    ;
                }
            }

            foreach ($tags as $tIdx => $tag) {
                if (is_numeric($tag)) unset($tags[$tIdx]);
            }

            $this->_tags = $tags;
        }

        return $this->_tags;
    }

    private function _stripTag ($tag, &$tags)
    {
        $segments = preg_split('/( |-|_|\+)/', $tag, -1);
        foreach ($segments as $seg) {
            $tags[] = trim(preg_replace('/[^a-zA-Z0-9\s]/', '', $seg));
        }
    }
}
