<?php 
if (!defined('UPGRADE')) exit('Bad Request.');
date_default_timezone_set('GMT');

/**
 * This file is part of the UseResponse.
 * UseResponse - Brilliant Customer Support http://www.useresponse.com/
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 */
class Upgrade
{
    /**
     * Admin Email
     * 
     * @var string
     */
    protected $_adminEmail = null;

    /**
     * Admin Password
     * 
     * @var string
     */
    protected $_adminPassword = null;

    /**
     * Next version
     * 
     * @var string
     */
    protected $_nextVersion = null;
           
    /**
     * Results storage
     * 
     * @var array
     */
    protected $_results = array();
    
    /**
     * Errors counet
     * 
     * @var int
     */
    protected $_errors = 0;
    
    /**
     * Writable folders list
     * 
     * @var array
     */
    protected $_writableFolders = array(
        'application/configs',
        'application/sessions',
        'application/localization',
        'public/files',
        'public/files/avatars',
        'public/files/logo',
        'public/files/temp',
        'public/static',
        'cache',
        'work'
    );

    const VALIDATION_SUCCESS = 'accept';
    const VALIDATION_WARNING = 'warning';
    const VALIDATION_ERROR   = 'del';
    
    const LOGIN_TEMPLATE  = 'templates/login.php';
    const TEMPLATE        = 'templates/index.php';
    const TEMPLATE_CONFIG = 'templates/application.inc';
    
    const COLUMN_RIGHT = 'r';
    const COLUMN_LEFT  = 'l';
       
    /**
     * Magic wrapper :)
     * 
     * @return string
     */
    public function __toString ()
    {
        if (Zend_Auth::getInstance()->hasIdentity()) {
            $this->validateWritableFolders();
            return $this->render();
        } else {
            return $this->render(self::LOGIN_TEMPLATE);
        }
    }
    
    public function __construct()
    {
    }


    /**
     * Render installation
     *
     * @param  string $tpl
     *
     * @return string
     */
    public function render($tpl = self::TEMPLATE)
    {
        ob_start();
        require_once realpath(dirname(__FILE__)) . '/' . $tpl;
        return ob_get_clean();
    }
    
    /**
     * Get next version
     * 
     * @return string
     */
    public function getNextVersion()
    {
        if (!$this->_nextVersion) {
            $systemModule = new System_Module('system');
            $this->_nextVersion = $systemModule->getVersion();
        }
        return $this->_nextVersion;
    }
    
    /**
     * Make address with base URL
     * 
     * @param string $uri
     * @return string
     */
    public function baseUrl ($uri = '')
    {
        return $this->getBaseUrl() . '/' . $uri;
    }
    
    /**
     * Validate writable folders
     * 
     * @return void
     */
    public function validateWritableFolders ()
    {
        foreach ($this->_writableFolders as $folder) {
            $realPath = ROOT_PATH . '/' . $folder;
            if (is_writable($realPath)) {
                if (is_dir($realPath)) {
                    $tmpFile = $realPath . '/' . uniqid(mt_rand()) . '.tmp';
                    if (!($f = @fopen($tmpFile, 'w+'))) {
                        $this->addResult('Writable ' . $folder, self::VALIDATION_ERROR, null);
                    } else {
                        fclose($f);
                        unlink($tmpFile);
                        $this->addResult('Writable ' . $folder, self::VALIDATION_SUCCESS, null);
                    }
                } else {
                    $this->addResult('Writable ' . $folder, self::VALIDATION_SUCCESS, null);
                }
            } else {
                $this->addResult('Writable ' . $folder, self::VALIDATION_ERROR, null);
            }
        }
    }
    
    /**
     * Add result
     * 
     * @param  string $label
     * @param  string $status
     * @param  string $message
     * @param  string $alt
     *
     * @return void
     */
    public function addResult ($label, $status, $message = null, $alt = null)
    {
        $result = (object) array(
            'status'  => $status,
            'message' => $message,
            'label'   => $label,
            'alt'     => $alt,
        );
        
        if ($status == self::VALIDATION_ERROR) {
            $this->_errors++;
        } elseif ($status == self::VALIDATION_WARNING) {
            $this->_warnings++;
        }
        
        $this->_results[] = $result;
    }
    
    /**
     * Has validation errors?
     * 
     * @return bool
     */
    public function hasErrors ()
    {
        return (bool) $this->_errors;
    }
    
    /**
     * Has validation warnings?
     * 
     * @return bool
     */
    public function hasWarnings ()
    {
        return (bool) $this->_warnings;
    }

    /**
     * Login process
     * 
     * @return string
     */
    public function login()
    {
        $this->_adminEmail    = $_POST['email'];
        $this->_adminPassword = $_POST['password'];
        $validator = new Zend_Validate_EmailAddress();
        if (!$validator->isValid($this->_adminEmail)) {
            $this->addResult('email', self::VALIDATION_ERROR, 'Email is incorect!');
            return $this;
        }
        $validator = new Zend_Validate_NotEmpty();
        if (!$validator->isValid($this->_adminPassword)) {
            $this->addResult('email', self::VALIDATION_ERROR, 'Password is required!');
            return $this;            
        }
        if (Singular_Core::_('auth')->authenticate($this->_adminEmail, $this->_adminPassword)->isValid()) {
            $auth = Zend_Auth::getInstance()->getIdentity();
            if ($auth->role !== 'administrator') {
                Singular_Core::_('auth')->logout();
            }
        }
        return $this;
    }
    
    /**
     * Execute installation process
     * 
     * @return string
     */
    public function execute ()
    {
        if (isset($_POST['action']) && $_POST['action'] == 'upgrade') {
            if (isset($_POST['step'])) {
                return $this->upgrade($_POST['step']);
            }
        }
        return $this;
    }
    
    /**
     * Installation process
     * 
     * @param string
     * @return string
     */
    public function upgrade ($step)
    {        
        if (method_exists($this, $step)) {
            try {
                return call_user_func(array($this, $step));
            } catch (Exception $e) {
                if (!is_array($_SESSION['errors'])) {
                    $_SESSION['errors'] = array();
                }
                $_SESSION['errors'][] = $e->getMessage();
                return self::VALIDATION_ERROR;
            }
        }
        return self::VALIDATION_ERROR;
    }
    
    /**
     * Render installation result
     * 
     * @return string
     */
    public function renderComplete ()
    {
        if (isset($_SESSION['errors']) && is_array($_SESSION['errors'])) {
            $msg  = '<div class="text-area color2">';
            $msg .= '  <p>There were problems with system update!</p>';
            $msg .= '</div>';
            foreach ($_SESSION['errors'] as $message) {
                $msg .= '<p>' . $message . '</p>';
            }
            
            unset($_SESSION['errors']);
            return $msg;
            
        } else {
            return '
                <div class="text-area color1">
                    <p>System is successfully updated!</p>
                </div>
            ';
        }
    }
    
    /**
     * Make Base URL
     * 
     * @return string
     */
    public function getBaseUrl ()
    {
        $port = $_SERVER['SERVER_PORT'] ? $_SERVER['SERVER_PORT'] : 80;
        $isSecure = 'ON' == strtoupper(@$_SERVER['HTTPS']);
        $protocol = ($isSecure || 443 == $port) ? 'https://' : 'http://';
        
        if (80 != $port && 443 != $port) {
            return rtrim($protocol . dirname($_SERVER['HTTP_HOST'] . ':' . $port . $_SERVER['SCRIPT_NAME']), '/');
        } else {
            return rtrim($protocol . dirname($_SERVER['HTTP_HOST'] . $_SERVER['SCRIPT_NAME']), '/');
        }
    }
    
    public function isSubFolder ()
    {
        $baseUrl = trim(dirname($_SERVER['HTTP_HOST'] . $_SERVER['SCRIPT_NAME']), '/');
        return preg_match('/\//', $baseUrl);
    }
    
    /**
     * Upgrade default config
     * 
     * @return void
     */
    public function upgradeConfig ()
    {
        $oldConfig = ROOT_PATH . '/application/configs/application.ini';
        $newConfig = ROOT_PATH . '/application/configs/application.inc';
        if ( file_exists($oldConfig) && !file_exists($newConfig) ) {
            $config = parse_ini_file($oldConfig);
            $dbHost = $config['resources.db.params.host'];
            $dbName = $config['resources.db.params.dbname'];
            $dbUser = $config['resources.db.params.username'];
            $dbPass = $config['resources.db.params.password'];
            $dbPref = $config['resources.db.params.prefix'];
            $baseUrl = $config['resources.frontController.baseUrl'];

            $tpl = file_get_contents(ROOT_PATH . '/upgrade/application.inc');
            $configData = str_replace(
                array('{db_host}', '{db_name}', '{db_user}', '{db_pass}', '{db_pref}', '{base_url}'),
                array($dbHost, $dbName, $dbUser, $dbPass, $dbPref, $baseUrl),
                $tpl
            );
            $config = fopen($newConfig, 'w+');
            fwrite($config, $configData);
            fclose($config);
            chmod($newConfig, 0666);
        }

        return self::VALIDATION_SUCCESS;
    }
    
    /**
     * Upgrade database
     * 
     * @return void
     */
    public function upgradeDatabase ()
    {
        if (System_Service_Upgrade::upgradeDbConfig()) {
            return self::VALIDATION_SUCCESS;
        }
        return self::VALIDATION_ERROR;
    }
    
    public function cleanCache()
    {
        Singular_Core::_('Cache')->cleanAll();
        return self::VALIDATION_SUCCESS;
    }    
        
    /**
     * Final result
     * 
     * @return string
     */
    public function getFinalResult ()
    {
        if ($this->hasErrors()) {
            return '<div class="text-area color2"><p>There are some problems with environment! Please resolve issues to continue.</p></div>';
        } elseif ($this->hasWarnings()) {
            return '<div class="text-area color1"><p>UseResponse can be installed, but there are some recommendations.</p></div>';
        } else {
            return '<div class="text-area color1"><p>All checks are passed. UseResponse can be installed on this server!</p></div>';
        }
    }
    
    /**
     * Render item of message
     * 
     * @param string $message
     * @return string 
     */
    public function renderMessage ($message)
    {
        return $message ? '<span class="ico-small hint" title="'.$message.'"></span>' : '';
    }
    
    /**
     * Render item of result
     * 
     * @param object $result
     * @param string $message
     * @param string $color
     * @return string 
     */
    public function renderResult ($result, $message, $color)
    {
        $alt = '';
        $questionClass = '';
        if (isset($result->alt) && $result->alt) {
            $alt = '<span title="'.$result->alt.'">'.$result->label.'</span>';
            $questionClass = 'question';
        } else {
            $alt = $result->label;
        }
        
        return '<div class="'.$questionClass.' obj'.$color.'">'.$alt.$message.'<span class="ico-small '.$result->status.'"></span></div>';
    }
    
    /**
     * Render all results
     * 
     * @param string $column
     * @return string 
     */
    public function renderResults ($column)
    {
        $results = array();
        $center = ceil(count($this->_results) / 2);
        
        if ($column == self::COLUMN_LEFT) {
            $results = array_slice($this->_results, 0, $center);
        } else {
            $results = array_slice($this->_results, $center);
        }

        $out = array();
        foreach ($results as $key => $result) {
            $color = $key & 1 ? ' color-obj' : '';
            $message = $this->renderMessage($result->message);
            $out[] = $this->renderResult($result, $message, $color);
        }
        
        return implode(PHP_EOL, $out);
    }

    /**
     * Render item of field
     * 
     * @param string $type
     * @param string $name
     * @param string $defaulValue
     * @param string $validator
     * @param array $options
     * @return string 
     */
    public function renderField ($type, $name, $defaulValue = '', $validator = '', $options = null, $class = '')
    {
        if ($this->_isExecute) {
            switch ($validator) {
                case 'empty':
                    if (@empty($_POST[$name])) {
                        return '<input type="'.$type.'" name="'.$name.'" value="'.$_POST[$name].'" class="error" />';
                    } else {
                        return '<input type="'.$type.'" name="'.$name.'" value="'.$_POST[$name].'" />';
                    }
                break;
                case 'yes':
                    if (isset($_POST[$name]) && $_POST[$name]) {
                        $checked = $_POST[$name] ? 'checked="checked"' : '';
                        return '<label class="'.$options['class'].'"> '.$options['label'].' <input type="'.$type.'" name="'.$name.'" value="'.$_POST[$name].'" '.$checked.' /></label>';
                    } else {
                        return '<label class="'.$options['class'].' error"> '.$options['label'].' <input type="'.$type.'" name="'.$name.'" value="1" /></label>';
                    }
                break;
                case 'file':
                    if ($this->_isUploadedLicense) {
                        return '<input type="'.$type.'" name="'.$name.'" value="" class="inputFile" />';
                    } else {
                        return '<input type="'.$type.'" name="'.$name.'" value="" class="error inputFile" />';
                    }
                break;
                case 'email':
                    if (preg_match('#^[a-zA-Z0-9._-]+@[a-zA-Z0-9._-]+\.([a-zA-Z]{2,4})$#ui', $_POST[$name])) {
                        return '<input type="'.$type.'" name="'.$name.'" value="'.$_POST[$name].'" />';
                    } else {
                        return '<input type="'.$type.'" name="'.$name.'" value="'.$_POST[$name].'" class="error" />';
                    }
                break;
                case '':
                default:
                    return '<input type="'.$type.'" name="'.$name.'" value="'.$_POST[$name].'" />';
                break;
            }
        } else {
            if (!empty($options)) {
                return '<label class="'.$options['class'].'"> '.$options['label'].' <input type="'.$type.'" name="'.$name.'" value="'.$defaulValue.'" /></label>';
            } else {
                return '<input type="'.$type.'" name="'.$name.'" value="'.$defaulValue.'" class="'.$class.'" />';
            }
        }
    }
    
    /**
     * Render partial of template
     * 
     * @return string
     */
    public function partial ($name)
    {
        return include realpath(dirname(__FILE__)) . '/templates/' .$name;
    }
    
    /**
     * Password hash generator
     *
     * @param  string $password
     * @return string
     */
    protected function _passwordHash ($password)
    {
        $password = strtolower($password);
        return md5(
            str_repeat(
                md5($password) . strrev($password) . sha1($password),
                strlen($password)
            )
        );
    }
}
